------------------------------------------------------------------
--FCOItemSaver.lua
--Author: Baertram
--v0.8.6a
--[[
	Allows you to mark items so you know that you meant to save for some reason.
	Prevent items from beeing destroyed/extracted/traded/send or sold somehow.
	Including filters on/off/show only marked items inside inventories, crafting stations, banks, guild banks, guild stores, player 2 player trading, sending mail, fence, launder and craftbag
]]
------------------------------------------------------------------
--Global array with all data of this addon
FCOIS = {}
local FCOIS = FCOIS
--Get the current api version of the server, to distinguish code fragments
FCOIS.APIversion = GetAPIVersion()

--Addon variables
FCOIS.addonVars = {}
FCOIS.addonVars.addonVersionOptions 		= '0.8.6a' -- version shown in the settings panel
FCOIS.addonVars.addonVersionOptionsNumber	= 0.861
FCOIS.addonVars.gAddonName					= "FCOItemSaver"
FCOIS.addonVars.addonNameMenu				= "FCO ItemSaver"
FCOIS.addonVars.addonNameMenuDisplay		= "|c00FF00FCO |cFFFF00ItemSaver|r"
FCOIS.addonVars.addonAuthor 				= '|cFFFF00Baertram|r'
FCOIS.addonVars.addonVersion		   		= 0.10 -- Changing this will reset SavedVariables!
FCOIS.addonVars.gAddonLoaded				= false
FCOIS.addonVars.gPlayerActivated			= false
FCOIS.addonVars.gSettingsLoaded				= false

--Create the filter object for addon libFilters 2.0
FCOIS.libFilters = LibStub("LibFilters-2.0")
local libFilters = FCOIS.libFilters
--Initialize the libFilters filters
libFilters:InitializeLibFilters()

local LIBLA = LibStub:GetLibrary("LibLoadedAddons")
--Create the settings panel object of libAddonMenu 2.0
local LAM = LibStub('LibAddonMenu-2.0')
--The options panel of FCO ItemSaver
FCOIS.FCOSettingsPanel = nil
--Create the libMainMenu 2.0 object
local  LMM2 = LibStub("LibMainMenu-2.0")
LMM2:Init()

--Array for the mapping between variables and values
FCOIS.mappingVars = {}
--Local number variables
FCOIS.numVars = {}
--Local last variables
FCOIS.lastVars = {}
--Local override variables
FCOIS.overrideVars = {}
--Local counter variables
FCOIS.countVars = {}
--Local guild bank variables
FCOIS.guildBankVars = {}
FCOIS.guildBankVars.guildBankId = 0
--Local filter button variables
FCOIS.filterButtonVars = {}
--Local equipment variables
FCOIS.equipmentVars = {}
--Local icon variables
FCOIS.iconVars = {}
--Local custom menu vars
FCOIS.customMenuVars = {}
--Local context menu relating variables
FCOIS.contextMenuVars			= {}
--Local inventory additional button variables
FCOIS.invAdditionalButtonVars = {}
-- Local variables for other addons
FCOIS.otherAddons = {}
-- Local variables for improvement
FCOIS.improvementVars = {}
--Local pre chat color variables
FCOIS.preChatVars = {}

--Last item's markers (set by clicking the divider if enabled in the settings)
FCOIS.lastMarkedIcons			= nil

--Improvement re-marking of items
FCOIS.improvementVars.improvementBagId		= nil
FCOIS.improvementVars.improvementSlotIndex	= nil
FCOIS.improvementVars.improvementMarkedIcons = {}

--Uncolored "FCOIS" pre chat text for the chat output
FCOIS.preChatVars.preChatText = "FCOIS"
--Green colored "FCOIS" pre text for the chat output
FCOIS.preChatVars.preChatTextGreen = "|c22DD22"..FCOIS.preChatVars.preChatText.."|r "
--Red colored "FCOIS" pre text for the chat output
FCOIS.preChatVars.preChatTextRed = "|cDD2222"..FCOIS.preChatVars.preChatText.."|r "
--Blue colored "FCOIS" pre text for the chat output
FCOIS.preChatVars.preChatTextBlue = "|c2222DD"..FCOIS.preChatVars.preChatText.."|r "

FCOIS.customMenuVars.customMenuSubEntries	= {}
FCOIS.customMenuVars.customMenuCurrentCounter = 0
FCOIS.contextMenuVars.contextMenuIndex 		= -1

--The active filter panel Ids (filter panel Id = inventory types above!)
FCOIS.mappingVars.activeFilterPanelIds			= {
	[LF_INVENTORY] 					= true,
	[LF_BANK_WITHDRAW] 			   	= true,
	[LF_BANK_DEPOSIT]				= true,
	[LF_GUILDBANK_WITHDRAW] 	    = true,
	[LF_GUILDBANK_DEPOSIT]	    	= true,
	[LF_VENDOR_SELL] 				= true,
	[LF_SMITHING_DECONSTRUCT]  		= true,
	[LF_GUILDSTORE_SELL] 	 		= true,
	[LF_MAIL_SEND] 					= true,
	[LF_TRADE] 						= true,
	[LF_ENCHANTING_CREATION]		= true,
	[LF_ENCHANTING_EXTRACTION]		= true,
	[LF_SMITHING_IMPROVEMENT]		= true,
	[LF_FENCE_SELL] 				= true,
	[LF_FENCE_LAUNDER]				= true,
	[LF_ALCHEMY_CREATION]			= true,
    [LF_CRAFTBAG]					= true,
}

--The mapping array for libFilter inventory type to inventory backpack type
FCOIS.mappingVars.InvToInventoryType = {
	[LF_INVENTORY] 					= INVENTORY_BACKPACK,
	[LF_BANK_WITHDRAW] 				= INVENTORY_BANK,
	[LF_BANK_DEPOSIT]				= INVENTORY_BACKPACK,
	[LF_GUILDBANK_WITHDRAW] 		= INVENTORY_GUILD_BANK,
	[LF_GUILDBANK_DEPOSIT]    		= INVENTORY_BACKPACK,
	[LF_VENDOR_SELL] 				= INVENTORY_BACKPACK,
	[LF_SMITHING_DECONSTRUCT]  		= INVENTORY_BACKPACK,
	[LF_GUILDSTORE_SELL] 	 		= INVENTORY_BACKPACK,
	[LF_MAIL_SEND] 					= INVENTORY_BACKPACK,
	[LF_TRADE] 						= INVENTORY_BACKPACK,
	[LF_ENCHANTING_CREATION]		= INVENTORY_BACKPACK,
	[LF_ENCHANTING_EXTRACTION]		= INVENTORY_BACKPACK,
	[LF_SMITHING_IMPROVEMENT]		= INVENTORY_BACKPACK,
	[LF_FENCE_SELL] 				= INVENTORY_BACKPACK,
	[LF_FENCE_LAUNDER]				= INVENTORY_BACKPACK,
    [LF_CRAFTBAG]					= INVENTORY_CRAFT_BAG,
}

--Global variable to tell where the filtering is currently needed (Inventory, Bank, Crafting Station, Guild Bank, Guild Store, Mail, Trading, Vendor, Enchanting table, fence)
FCOIS.gFilterWhere		   		= LF_INVENTORY
--Global variable to tell which filter was clicked/used by chat command at last
FCOIS.lastVars.gLastFilterId               = {}
--variable to override the changed "split filters" settings at function UnregisterFilters()
FCOIS.overrideVars.gSplitFilterOverride		= false

--Available languages
FCOIS.numVars.languageCount = 7 --English, German, French, Spanish, Italian, Japanese, Russian
--Global: Count of available inventory filter types (LF_INVENTORY, LF_BANK_WITHDRAW, etc. -> see above)
FCOIS.numVars.gFCONumFilterInventoryTypes = 26 -- Updated from 16 to 26 due to libFilters 2.0
--Global value: Number of filters
FCOIS.numVars.gFCONumFilters			= 4

--Global value: Number of filter icons to choose by right click menu
FCOIS.numVars.gFCONumFilterIcons		= 12
--Possible icon IDs
--and possible context menus for the filter buttons: RESDECIMP and SELLGUILDINT
--[[
     1 = Lock symbol
     2 = Gear set 1
     3 = Research				(RES)
     4 = Gear set 2
     5 = Sell						(SELL)
     6 = Gear set 3
     7 = Gear set 4
     8 = Gear set 5
     9 = Deconstruction			(DEC)
    10 = Improvement			(IMP)
    11 = Sell at guild store		(GUILD)
    12 = Intriate					(INT)
]]

--Global value: Number of gear sets
FCOIS.numVars.gFCONumGearSets			= 5

--The icon index list
FCOIS.iconVars.iconsIndexList = {}

--Global value: Number of filter panels, where the filters can be active.
--As the panels for inventory, bank (deposit), guild bank (deposit), mail, trade
--and guild store (select items from inventory for selling) are the same,
--the number here differes from FCOIS.numVars.gFCONumFilterInventoryTypes!
--Different panels are:
--1. Inventories (all)
--2. Deconstruction
--3. Vendor
--4. Guild bank
--5. Bank
--6. Enchanting station extraction & creation
--7. Improvement
--8. Guild store
--9. Mail
--10. Trade
--11. Fence
--12. Launder
--13. Alchemy
--FCOIS.numVars.gFCONumFilterPanels		= 10

--Width and height for the icons
FCOIS.iconVars.gIconWidth							= 32
FCOIS.iconVars.gIconHeight             				= 32
FCOIS.equipmentVars.gEquipmentIconWidth 			= 20
FCOIS.equipmentVars.gEquipmentIconHeight			= 20
FCOIS.equipmentVars.gEquipmentArmorTypeIconHeight	= 16
FCOIS.equipmentVars.gEquipmentArmorTypeIconWidth 	= 16
--Width and height for the filter buttons
FCOIS.filterButtonVars.gFilterButtonWidth			= 24
FCOIS.filterButtonVars.gFilterButtonHeight     		= 24
--Left and top of the filter buttons
FCOIS.filterButtonVars.gFilterButtonTop				= 6
FCOIS.filterButtonVars.gFilterButtonLeft	   		= {
	-70,
    -44,
    -22,
      2,
}
--filter button offset Y at the improvement bottom
FCOIS.filterButtonVars.buttonOffsetYImprovement = 7

--Filter button offset on x axis, if InventoryGriView addon is active too
FCOIS.otherAddons.gGriedViewOffsetX			= 26
--Variables for the test if the Addon "Inventory Gridview" is enabled
FCOIS.otherAddons.GRIDVIEWBUTTON    		= "ZO_PlayerInventory_GridButton"
FCOIS.otherAddons.inventoryGridViewActive = false
--For the test, if the addon "Chat Merchant" is enabled
FCOIS.otherAddons.CHATMERCHANTBUTTON 		= "ZO_PlayerInventory_CMbutton"
FCOIS.otherAddons.chatMerchantActive 		= false
--For the test, if the addon "Research Assistant" is enabled
FCOIS.otherAddons.researchAssistantActive	= false
--For the test, if the addon "PotionMaker" is enabled
FCOIS.otherAddons.potionMakerActive		= false
--For the test, if the addon "Votans Settings Menu" is enabled
FCOIS.otherAddons.votansSettingsMenuActive= false
--For the test, if the addon "sousChef" is enabled
FCOIS.otherAddons.sousChefActive = false

FCOIS.ZOControlVars = {}
--Control names of ZO* standard controls etc.
FCOIS.ZOControlVars.INV				        = ZO_PlayerInventory
FCOIS.ZOControlVars.INV_NAME					= FCOIS.ZOControlVars.INV:GetName()
FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_ITEMS	= ZO_PlayerInventoryMenuBarButton1
FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_CRAFTBAG = ZO_PlayerInventoryMenuBarButton2
FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_CURRENCIES = ZO_PlayerInventoryMenuBarButton3
FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_QUICKSLOTS = ZO_PlayerInventoryMenuBarButton4
FCOIS.ZOControlVars.BACKPACK 		    		= ZO_PlayerInventoryBackpack
FCOIS.ZOControlVars.CRAFTBAG					= ZO_CraftBag
FCOIS.ZOControlVars.CRAFTBAG_NAME				= FCOIS.ZOControlVars.CRAFTBAG:GetName()
if FCOIS.APIversion <= 100015 then
	FCOIS.ZOControlVars.BACKPACK_BAG 				= ZO_PlayerInventoryBackpackContents
else
	--New since 1000016
	FCOIS.ZOControlVars.BACKPACK_BAG 				= ZO_PlayerInventoryListContents
end
FCOIS.ZOControlVars.VENDOR_SELL				= ZO_StoreWindowListSellToVendorArea
--FCOIS.ZOControlVars.VENDOR_SELL_NAME			= FCOIS.ZOControlVars.VENDOR_SELL:GetName()
--FCOIS.ZOControlVars.VENDOR_MENUBAR_BUTTON_SELL = ZO_StoreWindowMenuBarButton2
--FCOIS.ZOControlVars.FENCE						= ZO_Fence_Keyboard_WindowMenu
FCOIS.ZOControlVars.REPAIR_LIST				= ZO_RepairWindowList
FCOIS.ZOControlVars.BANK_INV					= ZO_PlayerBank
FCOIS.ZOControlVars.BANK_INV_NAME				= FCOIS.ZOControlVars.BANK_INV:GetName()
FCOIS.ZOControlVars.BANK			    		= ZO_PlayerBankBackpack
FCOIS.ZOControlVars.BANK_BAG		    		= ZO_PlayerBankBackpackContents
FCOIS.ZOControlVars.BANK_MENUBAR_BUTTON_WITHDRAW	= ZO_PlayerBankMenuBarButton1
FCOIS.ZOControlVars.BANK_MENUBAR_BUTTON_DEPOSIT = ZO_PlayerBankMenuBarButton2
FCOIS.ZOControlVars.bankSceneName				= "bank"
FCOIS.ZOControlVars.GUILD_BANK_INV 	    	= ZO_GuildBank
FCOIS.ZOControlVars.GUILD_BANK_INV_NAME		= FCOIS.ZOControlVars.GUILD_BANK_INV:GetName()
FCOIS.ZOControlVars.GUILD_BANK 	    		= ZO_GuildBankBackpack
FCOIS.ZOControlVars.GUILD_BANK_BAG    		= ZO_GuildBankBackpackContents
FCOIS.ZOControlVars.GUILD_BANK_MENUBAR_BUTTON_WITHDRAW	= ZO_GuildBankMenuBarButton1
FCOIS.ZOControlVars.GUILD_BANK_MENUBAR_BUTTON_DEPOSIT = ZO_GuildBankMenuBarButton2
FCOIS.ZOControlVars.guildBankSceneName		= "guildBank"
FCOIS.ZOControlVars.GUILD_STORE				= ZO_TradingHouse
--FCOIS.ZOControlVars.GUILD_STORE_MENUBAR_BUTTON_SEARCH = ZO_TradingHouseMenuBarButton1
--FCOIS.ZOControlVars.GUILD_STORE_MENUBAR_BUTTON_SELL = ZO_TradingHouseMenuBarButton2
--FCOIS.ZOControlVars.GUILD_STORE_MENUBAR_BUTTON_LIST = ZO_TradingHouseMenuBarButton3
--FCOIS.ZOControlVars.SMITHING_TOP_LEVEL		= ZO_SmithingTopLevel
FCOIS.ZOControlVars.DECONSTRUCTION_INV		= ZO_SmithingTopLevelDeconstructionPanelInventory
FCOIS.ZOControlVars.DECONSTRUCTION_INV_NAME	= FCOIS.ZOControlVars.DECONSTRUCTION_INV:GetName()
FCOIS.ZOControlVars.DECONSTRUCTION    		= ZO_SmithingTopLevelDeconstructionPanelInventoryBackpack
FCOIS.ZOControlVars.DECONSTRUCTION_BAG 		= ZO_SmithingTopLevelDeconstructionPanelInventoryBackpackContents
--FCOIS.ZOControlVars.DECONSTRUCTION_SLOT 		= ZO_SmithingTopLevelDeconstructionPanelSlotContainerExtractionSlot
FCOIS.ZOControlVars.DECONSTRUCTION_BUTTON_WEAPONS = ZO_SmithingTopLevelDeconstructionPanelInventoryTabsButton2
FCOIS.ZOControlVars.DECONSTRUCTION_BUTTON_ARMOR   = ZO_SmithingTopLevelDeconstructionPanelInventoryTabsButton1
FCOIS.ZOControlVars.REFINEMENT    			= ZO_SmithingTopLevelRefinementPanelInventoryBackpack
--FCOIS.ZOControlVars.SMITHING_MENUBAR_BUTTON_DECONSTRUCTION 		= ZO_SmithingTopLevelModeMenuBarButton3
FCOIS.ZOControlVars.IMPROVEMENT_INV  			= ZO_SmithingTopLevelImprovementPanelInventory
FCOIS.ZOControlVars.IMPROVEMENT_INV_NAME		= FCOIS.ZOControlVars.IMPROVEMENT_INV:GetName()
FCOIS.ZOControlVars.IMPROVEMENT    			= ZO_SmithingTopLevelImprovementPanelInventoryBackpack
FCOIS.ZOControlVars.IMPROVEMENT_BAG  			= ZO_SmithingTopLevelImprovementPanelInventoryBackpackContents
FCOIS.ZOControlVars.IMPROVEMENT_BOOSTER_CONTAINER = ZO_SmithingTopLevelImprovementPanelBoosterContainer
FCOIS.ZOControlVars.IMPROVEMENT_SLOT 			= ZO_SmithingTopLevelImprovementPanelSlotContainerImprovementSlot
FCOIS.ZOControlVars.IMPROVEMENT_BUTTON_WEAPONS = ZO_SmithingTopLevelImprovementPanelInventoryTabsButton2
FCOIS.ZOControlVars.IMPROVEMENT_BUTTON_ARMOR   = ZO_SmithingTopLevelImprovementPanelInventoryTabsButton1

--FCOIS.ZOControlVars.SMITHING_MENUBAR_BUTTON_IMPROVEMENT 			= ZO_SmithingTopLevelModeMenuBarButton4
FCOIS.ZOControlVars.LIST_DIALOG 	    		= ZO_ListDialog1List
FCOIS.ZOControlVars.MAIL_SEND					= ZO_MailSend
FCOIS.ZOControlVars.MAIL_SEND_NAME			= FCOIS.ZOControlVars.MAIL_SEND:GetName()
--FCOIS.ZOControlVars.MAIL_INBOX				= ZO_MailInbox
--FCOIS.ZOControlVars.MAIL_ATTACHMENTS			= ZO_MailSendAttachments
--FCOIS.ZOControlVars.MAIL_MENUBAR_BUTTON_SEND  = ZO_MainMenuSceneGroupBarButton2
FCOIS.ZOControlVars.PLAYER_TRADE				= ZO_Trade
--FCOIS.ZOControlVars.PLAYER_TRADE_NAME			= FCOIS.ZOControlVars.PLAYER_TRADE:GetName()
FCOIS.ZOControlVars.ENCHANTING				= ZO_Enchanting
FCOIS.ZOControlVars.ENCHANTING_STATION		= ZO_EnchantingTopLevelInventoryBackpack
FCOIS.ZOControlVars.ENCHANTING_STATION_NAME	= FCOIS.ZOControlVars.ENCHANTING_STATION:GetName()
FCOIS.ZOControlVars.ENCHANTING_STATION_BAG	= ZO_EnchantingTopLevelInventoryBackpackContents
--FCOIS.ZOControlVars.ENCHANTING_STATION_MENUBAR_BUTTON_CREATION    = ZO_EnchantingTopLevelModeMenuBarButton1
--FCOIS.ZOControlVars.ENCHANTING_STATION_MENUBAR_BUTTON_EXTRACTION  = ZO_EnchantingTopLevelModeMenuBarButton2
FCOIS.ZOControlVars.ALCHEMY_INV				= ZO_AlchemyTopLevelInventory
FCOIS.ZOControlVars.ALCHEMY_INV_NAME			= FCOIS.ZOControlVars.ALCHEMY_INV:GetName()
FCOIS.ZOControlVars.ALCHEMY_STATION			= ZO_AlchemyTopLevelInventoryBackpack
FCOIS.ZOControlVars.ALCHEMY_STATION_NAME		= FCOIS.ZOControlVars.ALCHEMY_STATION:GetName()
FCOIS.ZOControlVars.ALCHEMY_STATION_BAG		= ZO_AlchemyTopLevelInventoryBackpackContents
FCOIS.ZOControlVars.ALCHEMY_STATION_MENUBAR_BUTTON_CREATION = ZO_AlchemyTopLevelModeMenuBarButton1
FCOIS.ZOControlVars.ALCHEMY_STATION_MENUBAR_BUTTON_POTIONMAKER = ZO_AlchemyTopLevelModeMenuBarButton2
FCOIS.ZOControlVars.QUICKSLOT_CIRCLE  		= ZO_QuickSlotCircle
FCOIS.ZOControlVars.QUICKSLOT_LIST			= ZO_QuickSlotList
FCOIS.ZOControlVars.DestroyItemDialog    		= ESO_Dialogs["DESTROY_ITEM_PROMPT"]
FCOIS.ZOControlVars.RepairItemDialogName    	= "REPAIR_ITEM"
FCOIS.ZOControlVars.CHARACTER					= ZO_Character

--The last clicked enchanting panel top menu bar button
--FCOIS.lastVars.gLastEnchantingButton			= FCOIS.ZOControlVars.ENCHANTING_STATION_MENUBAR_BUTTON_CREATION
--FCOIS.lastVars.gLastSmithingButton				= FCOIS.ZOControlVars.SMITHING_MENUBAR_BUTTON_DECONSTRUCTION
FCOIS.lastVars.gLastBankButton					= FCOIS.ZOControlVars.BANK_MENUBAR_BUTTON_WITHDRAW
FCOIS.lastVars.gLastGuildBankButton   			= FCOIS.ZOControlVars.GUILD_BANK_MENUBAR_BUTTON_WITHDRAW
--FCOIS.lastVars.gLastGuildStoreButton			= FCOIS.ZOControlVars.GUILD_STORE_MENUBAR_BUTTON_SEARCH
--FCOIS.lastVars.gLastVendorButton				= FCOIS.ZOControlVars.VENDOR_MENUBAR_BUTTON_SELL
--FCOIS.lastVars.gLastMailButton         		= FCOIS.ZOControlVars.MAIL_MENUBAR_BUTTON_SEND
FCOIS.lastVars.gLastAlchemyButton				= FCOIS.ZOControlVars.ALCHEMY_STATION_MENUBAR_BUTTON_CREATION
FCOIS.lastVars.gLastSmithingDeconstructionSubFilterButton 	= FCOIS.ZOControlVars.DECONSTRUCTION_BUTTON_WEAPONS
FCOIS.lastVars.gLastSmithingImprovementSubFilterButton 	= FCOIS.ZOControlVars.IMPROVEMENT_BUTTON_WEAPONS
--FCOIS.lastVars.gLastEnchantingSubFilterButton 	= FCOIS.ZOControlVars.ENCHANTING_BUTTON_GLYPHS
FCOIS.lastVars.gLastInvButton					= FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_ITEMS

--The mapping array for filterPanelId to shown inventories
FCOIS.mappingVars.gFilterPanelIdToInv = {
	[LF_INVENTORY] 							= FCOIS.ZOControlVars.BACKPACK,
	[LF_CRAFTBAG] 							= FCOIS.ZOControlVars.CRAFTBAG,
    [LF_BANK_WITHDRAW] 							= FCOIS.ZOControlVars.BANK,
	[LF_BANK_DEPOSIT]						= FCOIS.ZOControlVars.BACKPACK,
	[LF_GUILDBANK_WITHDRAW] 			   		= FCOIS.ZOControlVars.GUILD_BANK,
	[LF_GUILDBANK_DEPOSIT]				= FCOIS.ZOControlVars.BACKPACK,
	[LF_VENDOR_SELL] 						= FCOIS.ZOControlVars.BACKPACK,
	[LF_SMITHING_DECONSTRUCT]  				= FCOIS.ZOControlVars.DECONSTRUCTION,
	[LF_GUILDSTORE_SELL] 	 		   		= FCOIS.ZOControlVars.BACKPACK,
	[LF_MAIL_SEND] 							= FCOIS.ZOControlVars.BACKPACK,
	[LF_TRADE] 				   		= FCOIS.ZOControlVars.BACKPACK,
    [LF_ENCHANTING_CREATION]			= FCOIS.ZOControlVars.ENCHANTING_STATION,
	[LF_ENCHANTING_EXTRACTION]	 		= FCOIS.ZOControlVars.ENCHANTING_STATION,
	[LF_SMITHING_IMPROVEMENT]					= FCOIS.ZOControlVars.IMPROVEMENT,
	[LF_FENCE_SELL]							= FCOIS.ZOControlVars.BACKPACK,
	[LF_FENCE_LAUNDER]						= FCOIS.ZOControlVars.BACKPACK,
    [LF_ALCHEMY_CREATION]						= FCOIS.ZOControlVars.ALCHEMY_STATION,
}

--The array for the texture names of each panel Id
FCOIS.mappingVars.gFilterPanelIdToTextureName = {
	[LF_INVENTORY] = FCOIS.ZOControlVars.INV_NAME .. "_FilterButton%sTexture",
	[LF_CRAFTBAG] = FCOIS.ZOControlVars.CRAFTBAG_NAME .. "_FilterButton%sTexture",
	[LF_SMITHING_DECONSTRUCT] = FCOIS.ZOControlVars.DECONSTRUCTION_INV_NAME .. "_FilterButton%sTexture",
	[LF_SMITHING_IMPROVEMENT] = FCOIS.ZOControlVars.IMPROVEMENT_INV_NAME .. "_FilterButton%sTexture",
	[LF_VENDOR_SELL] = FCOIS.ZOControlVars.INV_NAME .. "_FilterButton%sTexture",
	[LF_GUILDBANK_WITHDRAW] = FCOIS.ZOControlVars.GUILD_BANK_INV_NAME .. "_FilterButton%sTexture",
	[LF_GUILDBANK_DEPOSIT] = FCOIS.ZOControlVars.INV_NAME .. "_FilterButton%sTexture",
	[LF_GUILDSTORE_SELL] = FCOIS.ZOControlVars.INV_NAME .. "_FilterButton%sTexture",
	[LF_BANK_WITHDRAW] = FCOIS.ZOControlVars.BANK_INV_NAME .. "_FilterButton%sTexture",
	[LF_BANK_DEPOSIT] = FCOIS.ZOControlVars.INV_NAME .. "_FilterButton%sTexture",
	[LF_ENCHANTING_EXTRACTION] = FCOIS.ZOControlVars.ENCHANTING_STATION_NAME .. "_FilterButton%sTexture",
	[LF_ENCHANTING_CREATION] = FCOIS.ZOControlVars.ENCHANTING_STATION_NAME .. "_FilterButton%sTexture",
	[LF_MAIL_SEND] = FCOIS.ZOControlVars.INV_NAME .. "_FilterButton%sTexture",
	[LF_TRADE] = FCOIS.ZOControlVars.INV_NAME .. "_FilterButton%sTexture",
	[LF_FENCE_SELL] = FCOIS.ZOControlVars.INV_NAME .. "_FilterButton%sTexture",
	[LF_FENCE_LAUNDER] = FCOIS.ZOControlVars.INV_NAME .. "_FilterButton%sTexture",
	[LF_ALCHEMY_CREATION] = FCOIS.ZOControlVars.ALCHEMY_INV_NAME .. "_FilterButton%sTexture",
}

--The icons to choose from
FCOIS.textureVars = {}
FCOIS.textureVars.allGearSets  			= [[/esoui/art/crafting/smithing_tabicon_armorset_up.dds]]
FCOIS.textureVars.allGearSetsWidth		= 24
FCOIS.textureVars.allGearSetsHeight		= 24
FCOIS.textureVars.allResDecImp 			= [[/esoui/art/crafting/smithing_tabicon_weaponset_up.dds]]
FCOIS.textureVars.allResDecImpWidth		= 24
FCOIS.textureVars.allResDecImpHeight		= 24
FCOIS.textureVars.allSellGuildInt 		= [[/esoui/art/icons/item_generic_coinbag.dds]]
FCOIS.textureVars.allSellGuildIntWidth	= 20
FCOIS.textureVars.allSellGuildIntHeight	= 20
FCOIS.textureVars.MARKER_TEXTURES = {
	[1]  = [[/esoui/art/campaign/campaignbrowser_fullpop.dds]],
	[2]  = [[/esoui/art/inventory/inventory_tabicon_armor_disabled.dds]],
	[3]  = [[/esoui/art/crafting/smithing_tabicon_research_disabled.dds]],
	[4]  = [[/esoui/art/tradinghouse/tradinghouse_sell_tabicon_disabled.dds]],
	[5]  = [[/esoui/art/campaign/overview_indexicon_bonus_disabled.dds]],
	[6]  = [[/esoui/art/ava/tabicon_bg_score_disabled.dds]],
	[7]  = [[/esoui/art/guild/guild_rankicon_leader_large.dds]],
	[8]  = [[/esoui/art/lfg/lfg_healer_up.dds]],
	[9]  = [[/esoui/art/mounts/timer_icon.dds]],
    [10] = [[/esoui/art/crafting/alchemy_tabicon_solvent_up.dds]],
	[11] = [[/esoui/art/buttons/cancel_up.dds]],
    [12] = [[/esoui/art/buttons/info_up.dds]],
    [13] = [[/esoui/art/buttons/pinned_normal.dds]],
    [14] = [[/esoui/art/cadwell/cadwell_indexicon_gold_up.dds]],
    [15] = [[/esoui/art/cadwell/cadwell_indexicon_silver_up.dds]],
    [16] = [[/esoui/art/campaign/campaignbonus_keepicon.dds]],
    [17] = [[/esoui/art/icons/scroll_005.dds]],
    [18] = [[/esoui/art/campaign/campaignbrowser_columnheader_ad.dds]],
    [19] = [[/esoui/art/campaign/campaignbrowser_columnheader_dc.dds]],
    [20] = [[/esoui/art/campaign/campaignbrowser_columnheader_ep.dds]],
    [21] = [[/esoui/art/campaign/campaignbrowser_guild.dds]],
    [22] = [[/esoui/art/campaign/campaignbrowser_indexicon_normal_up.dds]],
    [23] = [[/esoui/art/campaign/overview_indexicon_scoring_up.dds]],
    [24] = [[/esoui/art/charactercreate/charactercreate_bodyicon_up.dds]],
    [25] = [[/esoui/art/characterwindow/gearslot_offhand.dds]],
    [26] = [[/esoui/art/characterwindow/gearslot_mainhand.dds]],
    [27] = [[/esoui/art/characterwindow/gearslot_costume.dds]],
    [28] = [[/esoui/art/chatwindow/chat_mail_up.dds]],
    [29] = [[/esoui/art/chatwindow/chat_notification_up.dds]],
    [30] = [[/esoui/art/crafting/alchemy_tabicon_reagent_up.dds]],
    [31] = [[/esoui/art/crafting/smithing_tabicon_refine_up.dds]],
    [32] = [[/esoui/art/deathrecap/deathrecap_killingblow_icon.dds]],
    [33] = [[/esoui/art/fishing/bait_emptyslot.dds]],
    [34] = [[/esoui/art/guild/guildhistory_indexicon_guildbank_up.dds]],
    [35] = [[/esoui/art/guild/guild_indexicon_member_up.dds]],
    [36] = [[/esoui/art/guild/tabicon_roster_up.dds]],
    [37] = [[/esoui/art/icons/poi/poi_dungeon_complete.dds]],
    [38] = [[/esoui/art/icons/poi/poi_groupinstance_complete.dds]],
    [39] = [[/esoui/art/icons/servicemappins/servicepin_magesguild.dds]],
    [40] = [[/esoui/art/icons/servicemappins/servicepin_fightersguild.dds]],
	[41] = [[/esoui/art/lfg/lfg_dps_up.dds]],
	[42] = [[/esoui/art/lfg/lfg_leader_icon.dds]],
	[43] = [[/esoui/art/lfg/lfg_tank_up.dds]],
	[44] = [[/esoui/art/lfg/lfg_veterandungeon_up.dds]],
	[45] = [[/esoui/art/lfg/lfg_normaldungeon_up.dds]],
	[46] = [[/esoui/art/progression/icon_dualwield.dds]],
	[47] = [[/esoui/art/progression/icon_firestaff.dds]],
	[48] = [[/esoui/art/progression/icon_bows.dds]],
	[49] = [[/esoui/art/progression/icon_2handed.dds]],
	[50] = [[/esoui/art/progression/icon_1handed.dds]],
	[51] = [[/esoui/art/progression/progression_tabicon_backup_inactive.dds]],
	[52] = [[/esoui/art/repair/inventory_tabicon_repair_disabled.dds]],
	[53] = [[/esoui/art/worldmap/selectedquesthighlight.dds]],
	[54] = [[/esoui/art/journal/journal_tabicon_cadwell_up.dds]],
	[55] = [[/esoui/art/crafting/enchantment_tabicon_deconstruction_disabled.dds]],
	[56] = [[/esoui/art/crafting/smithing_tabicon_improve_disabled.dds]],
    [57] = [[/esoui/art/bank/bank_tabicon_deposit_up.dds]],
    [58] = [[/esoui/art/currency/currency_gold.dds]],
    [59] = [[/esoui/art/guild/guild_bankaccess.dds]],
    [60] = [[/esoui/art/progression/progression_indexicon_guilds_up.dds]],
}
--Additional size for the textures, if the texture got a special size
-- Used at the filter buttons e.g.
FCOIS.textureVars.MARKER_TEXTURES_SIZE = {
    [58] = { width = 16, height = 16, offsetLeft = 1, offsetTop = -19, contextMenuOffsetLeft = 0, contextMenuOffsetTop = -19 },
}

--Arrays for the right click/context menu entries
FCOIS.localizationVars = {}
FCOIS.localizationVars.localizationAll 		= {}
FCOIS.localizationVars.fcois_loc 	 	    = {}
FCOIS.localizationVars.lTextMark 		    = {}
FCOIS.localizationVars.lTextDemark 	        = {}
FCOIS.localizationVars.contextEntries       = {}
FCOIS.localizationVars.lTextEquipmentMark   = {}
FCOIS.localizationVars.lTextEquipmentDemark = {}

FCOIS.settingsVars	= {}
FCOIS.settingsVars.settings			= {}
FCOIS.settingsVars.defaultSettings	= {}
FCOIS.settingsVars.firstRunSettings   = {}
FCOIS.settingsVars.defaults			= {}

FCOIS.markedItems = {}
for i = 1, FCOIS.numVars.gFCONumFilters, 1 do
	FCOIS.markedItems[i] = {}
end

FCOIS.dragAndDropVars = {}
FCOIS.dragAndDropVars.bag		= nil
FCOIS.dragAndDropVars.slot	= nil

FCOIS.preventerVars = {}
FCOIS.preventerVars.gLocalizationDone		= false
FCOIS.preventerVars.KeyBindingTexts		= false
FCOIS.preventerVars.gScanningInv	    	= false
--FCOIS.preventerVars.canUpdateInv 	   		= true
FCOIS.preventerVars.gFilteringBasics		= false
FCOIS.preventerVars.gActiveFilterPanel	= false
FCOIS.preventerVars.gNoCloseEvent 		= false
FCOIS.preventerVars.gAllowDestroyItem		= false
FCOIS.preventerVars.wasDestroyDone        = false
FCOIS.preventerVars.gItemSlotIsLocked 	= false
FCOIS.preventerVars.gCheckEquipmentSlots  = false
FCOIS.preventerVars.gUpdateMarkersNow		= false
FCOIS.preventerVars.gChangedGears			= false
FCOIS.preventerVars.gContextCreated		= {}
FCOIS.preventerVars.gGearSetFilterContextCreated = {}
FCOIS.preventerVars.gResDecImpFilterContextCreated = {}
FCOIS.preventerVars.gSellGuildIntFilterContextCreated = {}
FCOIS.preventerVars.askBeforeEquipDialogRetVal = false
FCOIS.preventerVars.gOverrideInvUpdateAfterMarkItem = false
FCOIS.preventerVars.doFalseOverride = false
FCOIS.preventerVars.newItemCrafted = false
FCOIS.preventerVars.ZOsPlayerItemLockEnabled = false -- implemented with API 1000015 by ZOs: Lock items in inventory. Items will get dataEntry "isPlayerLocked = true"

--The event handler array for OnMouseDoubleClick, Drag&Drop, etc.
FCOIS.eventHandlers = {}

--Table to map the FCOIS.settingsVars.settings filter state to the output text identifier
FCOIS.mappingVars.settingsFilterStateToText = {
	["true"]  = "on",
    ["false"] = "off",
    ["-99"]   = "onlyfiltered",
}

--Table to map iconId to relating filterId
FCOIS.mappingVars.iconToFilter = {
	[1]	= 1,
	[2]	= 2,
	[3]	= 3,
	[4]	= 2,
	[5]	= 4,
	[6]	= 2,
	[7]	= 2,
	[8]	= 2,
	[9]	= 3,
	[10]= 3,
    [11]= 4,
    [12]= 4,
}

--Table to map filterId to relating iconId
--As filters can have several icons only the main iconId of this filterId is maintained here
FCOIS.mappingVars.filterToIcon = {
    [1]	= 1,
    [2]	= 2,
    [3]	= 3,
    [4]	= 5,
}

--Table to map iconId to gearId
FCOIS.mappingVars.iconToGear = {
	[2] = 1,
	[4] = 2,
	[6] = 3,
	[7] = 4,
	[8] = 5,
}

--Table to map gearId to iconId
FCOIS.mappingVars.gearToIcon = {
	[1] = 2,
	[2] = 4,
	[3] = 6,
	[4] = 7,
	[5] = 8,
}

--Table to see if the icon belongs to a gear
FCOIS.mappingVars.iconIsGear = {
--Gears
	[2] = true,
	[4] = true,
	[6] = true,
	[7] = true,
	[8] = true,
--No gears
	[1] = false,
	[3] = false,
	[5] = false,
	[9] = false,
	[10]= false,
    [11]= false,
    [12]= false,
}

--Table to see if the icon is researchable
FCOIS.mappingVars.iconIsResearchable = {
	[2] = true,
	[3] = true,
	[4] = true,
	[6] = true,
	[7] = true,
	[8] = true,
	[9] = true,
	[10]= true,
    [12]= true,
}

--Mapping array for icon to research/deconstruction/improvement filter split
FCOIS.mappingVars.iconToResDecImp = {
	[3]  = 1,
	[9]  = 2,
	[10] = 3,
}

--Mapping array for research/deconstruction/improvement filter split to it's icon
FCOIS.mappingVars.resDecImpToIcon = {
	[1]  =  3,
	[2]  =  9,
	[3]  = 10,
}

--Mapping array for icon to sell/sell in guild store/intricate filter split
FCOIS.mappingVars.iconToSellGuildInt = {
    [5]  = 1,
    [11] = 2,
    [12] = 3,
}

--Mapping array for sell/sell in guild store/intricate filter split to it's icon
FCOIS.mappingVars.sellGuildIntToIcon = {
    [1] =  5,
    [2] = 11,
    [3] = 12,
}

--Table with the weapon types for the main&offhand checks
FCOIS.checkVars = {}
FCOIS.checkVars.weaponTypeCheckTable = {
   ["1hd"] = {
	[WEAPONTYPE_AXE] = true,
	[WEAPONTYPE_DAGGER] = true,
	[WEAPONTYPE_HAMMER] = true,
	[WEAPONTYPE_SWORD] = true,
   },
   ["2hd"] = {
	[WEAPONTYPE_TWO_HANDED_AXE] = true,
	[WEAPONTYPE_TWO_HANDED_HAMMER] = true,
	[WEAPONTYPE_TWO_HANDED_SWORD] = true,
   },
   ["2hdall"] = {
	[WEAPONTYPE_TWO_HANDED_AXE] = true,
	[WEAPONTYPE_TWO_HANDED_HAMMER] = true,
	[WEAPONTYPE_TWO_HANDED_SWORD] = true,
	[WEAPONTYPE_BOW] = true,
	[WEAPONTYPE_FIRE_STAFF] = true,
	[WEAPONTYPE_LIGHTNING_STAFF] = true,
	[WEAPONTYPE_FROST_STAFF] = true,
	[WEAPONTYPE_HEALING_STAFF] = true,
   },
   ["staff"] = {
	[WEAPONTYPE_FIRE_STAFF] = true,
	[WEAPONTYPE_LIGHTNING_STAFF] = true,
	[WEAPONTYPE_FROST_STAFF] = true,
	[WEAPONTYPE_HEALING_STAFF] = true,
   },
   ["sword"] = {
	[WEAPONTYPE_SWORD] = true,
   },
   ["swordall"] = {
	[WEAPONTYPE_SWORD] = true,
	[WEAPONTYPE_TWO_HANDED_SWORD] = true,
   },
   ["axe"] = {
	[WEAPONTYPE_AXE] = true,
   },
   ["axeall"] = {
	[WEAPONTYPE_AXE] = true,
	[WEAPONTYPE_TWO_HANDED_AXE] = true,
   },
   ["dagger"] = {
	[WEAPONTYPE_DAGGER] = true,
   },
   ["hammer"] = {
	[WEAPONTYPE_HAMMER] = true,
   },
   ["hammerall"] = {
	[WEAPONTYPE_HAMMER] = true,
	[WEAPONTYPE_TWO_HANDED_HAMMER] = true,
   },
   ["bow"] = {
	[WEAPONTYPE_BOW] = true,
   },
   ["shield"] = {
	[WEAPONTYPE_SHIELD] = true,
   },
   ["firestaff"] = {
	[WEAPONTYPE_FIRE_STAFF] = true,
   },
   ["lightningstaff"] = {
	[WEAPONTYPE_LIGHTNING_STAFF] = true,
   },
   ["froststaff"] = {
	[WEAPONTYPE_FROST_STAFF] = true,
   },
   ["healingstaff"] = {
	[WEAPONTYPE_HEALING_STAFF] = true,
   },
   ["rune"] = {
	[WEAPONTYPE_RUNE] = true,
   },
   }
--Table with allowed item traits for ornate items
FCOIS.checkVars.allowedOrnateItemTraits = {
	[ITEM_TRAIT_TYPE_ARMOR_ORNATE]   = true,
	[ITEM_TRAIT_TYPE_JEWELRY_ORNATE] = true,
	[ITEM_TRAIT_TYPE_WEAPON_ORNATE]  = true,
}
--Table with allowed item traits for intricate items
FCOIS.checkVars.allowedIntricateItemTraits = {
    [ITEM_TRAIT_TYPE_ARMOR_INTRICATE]   = true,
    [ITEM_TRAIT_TYPE_WEAPON_INTRICATE]  = true,
}
--Table with allowed item types for researching
FCOIS.checkVars.allowedResearchableItemTypes = {
   	[ITEMTYPE_ARMOR]			=
    {
    	allowed = true,
        isGlpyh = false,
    },
    [ITEMTYPE_WEAPON]   		= true,
    {
    	allowed = true,
        isGlpyh = false,
    },
	[ITEMTYPE_COSTUME]			=
    {
    	allowed = true,
        isGlpyh = false,
    },
	[ITEMTYPE_DISGUISE] 		=
    {
    	allowed = true,
        isGlpyh = false,
    },
    --Glyphs
 	[ITEMTYPE_GLYPH_ARMOR] 		=
    {
    	allowed = true,
        isGlpyh = true,
        allowedIcons = {
        	[9] = true,
        }
    },
	[ITEMTYPE_GLYPH_JEWELRY] 	=
    {
    	allowed = true,
        isGlpyh = true,
        allowedIcons = {
        	[9] = true,
        }
    },
	[ITEMTYPE_GLYPH_WEAPON] 	=
    {
    	allowed = true,
        isGlpyh = true,
        allowedIcons = {
        	[9] = true,
        }
    },
}

--Table with the allowed set item types
FCOIS.checkVars.allowedSetItemTypes = {
	[ITEMTYPE_ARMOR]	= true,
	[ITEMTYPE_WEAPON]	= true,
}

--Table with NOT allowed parent control names. These cannot use the new FCOItemSaver context menu entries
FCOIS.checkVars.notAllowedContextMenuParentControls = {
	["ZO_StoreWindowListContents"] = true,
	["ZO_BuyBackListContents"] = true,
	["ZO_PlayerInventoryQuestContents"] = true,
	["ZO_SmithingTopLevelImprovementPanel"] = true,
    ["ZO_SmithingTopLevelResearchPanelResearchLineListList"] = true,
	["ZO_SmithingTopLevelCreationPanelPatternListList"] = true,
	["ZO_SmithingTopLevelCreationPanelMaterialListList"] = true,
	["ZO_SmithingTopLevelCreationPanelStyleListList"] = true,
	["ZO_SmithingTopLevelCreationPanelTraitListList"] = true,
	--["ZO_AlchemyTopLevelInventoryBackpackContents"] = true,
	["ZO_MailInboxMessage"] = true,
	["ZO_MailSend"] = true,
    ["ZO_TradingHouseItemPaneSearchResultsContents"] = true,
    ["ZO_TradingHousePostedItemsList"] = true,
--		["ZO_QuickSlotListContents"] = true,
    ["ZO_InventoryWalletListContents"] = true,
}
--Table with NOT allowed control names. These cannot use the new FCOItemSaver context menu entries
FCOIS.checkVars.notAllowedContextMenuControls = {
	["ZO_SmithingTopLevelRefinementPanelSlotContainer"] = true,
	["ZO_SmithingTopLevelDeconstructionPanelSlotContainer"] = true,
	["ZO_AlchemyTopLevelSlotContainer"] = true,
    ["ZO_EnchantingTopLevelRuneSlotContainer"] = true,
    ["ZO_EnchantingTopLevelExtractionSlotContainer"] = true,
	["ZO_ApplyEnchantPanel"] = true,
	["ZO_SoulGemItemChargerPanel"] = true,
    ["ZO_SoulGemItemChargerPanelImprovementPreviewContainer"] = true,
}
--Table with allowed control names for the character equipment weapon and offhand weapon slots
FCOIS.checkVars.allowedCharacterEquipmentWeaponControlNames = {
	['ZO_CharacterEquipmentSlotsMainHand'] = true,
	['ZO_CharacterEquipmentSlotsOffHand'] = true,
	['ZO_CharacterEquipmentSlotsBackupMain'] = true,
	['ZO_CharacterEquipmentSlotsBackupOff'] = true,
   }
--Table with allowed control names for the character equipment jewelry
FCOIS.checkVars.allowedCharacterEquipmentJewelryControlNames = {
	['ZO_CharacterEquipmentSlotsNeck'] = true,
	['ZO_CharacterEquipmentSlotsRing1'] = true,
	['ZO_CharacterEquipmentSlotsRing2'] = true,
   }
--Table with weapon and jewelry slot names for equipment checks
FCOIS.checkVars.equipmentSlotsNames = {
    ["no_auto_mark"] = {
		["ZO_CharacterEquipmentSlotsCostume"] = true,
    }
}
--Table with all equipment slot names which can be updated with markes for the icons
--The index is the relating slotIndex of the bag BAG_WORN!
FCOIS.mappingVars.characterEquipmentSlotNameByIndex = {
	[0] = "ZO_CharacterEquipmentSlotsHead",
	[1] = "ZO_CharacterEquipmentSlotsNeck",
	[2] = "ZO_CharacterEquipmentSlotsChest",
	[3] = "ZO_CharacterEquipmentSlotsShoulder",
	[4] = "ZO_CharacterEquipmentSlotsMainHand",
	[5] = "ZO_CharacterEquipmentSlotsOffHand",
	[6] = "ZO_CharacterEquipmentSlotsBelt",
	[8] = "ZO_CharacterEquipmentSlotsLeg",
	[9] = "ZO_CharacterEquipmentSlotsFoot",
	[10] = "ZO_CharacterEquipmentSlotsCostume",
	[11] = "ZO_CharacterEquipmentSlotsRing1",
	[12] = "ZO_CharacterEquipmentSlotsRing2",
    [13] = "ZO_CharacterEquipmentSlotsPoison",
	[14] = "ZO_CharacterEquipmentSlotsBackupPoison",
	[16] = "ZO_CharacterEquipmentSlotsGlove",
	[20] = "ZO_CharacterEquipmentSlotsBackupMain",
	[21] = "ZO_CharacterEquipmentSlotsBackupOff",
}
--Table with the eqipment slot control names which are armor
FCOIS.mappingVars.characterEquipmentArmorSlots = {
	['ZO_CharacterEquipmentSlotsHead'] = true,
	['ZO_CharacterEquipmentSlotsChest'] = true,
	['ZO_CharacterEquipmentSlotsShoulder'] = true,
	['ZO_CharacterEquipmentSlotsBelt'] = true,
	['ZO_CharacterEquipmentSlotsLeg'] = true,
	['ZO_CharacterEquipmentSlotsFoot'] = true,
	['ZO_CharacterEquipmentSlotsGlove'] = true,
}

--The context menu for mark/unmark all
FCOIS.contextMenu 	 		= {}
FCOIS.contextMenu.bdSelectedLine 	= {}
--The context menu for the gear sets filter button
FCOIS.contextMenu.GearSetFilter 	= {}
FCOIS.contextMenu.GearSetFilter.bdSelectedLine = {}
--The context menu for the RESEARCH & DECONSTRUCTION & IMPORVEMENT filter button
FCOIS.contextMenu.ResDecImpFilter 	= {}
FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine = {}
--The context menu for the SELL & SELL IN GUILD STORE & INTRICATE  filter button
FCOIS.contextMenu.SellGuildIntFilter 	= {}
FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine = {}

--The table with the undo entries for last changes by context menu
FCOIS.contextMenuVars.undoMarkedItems = {}

--Context menu variables
FCOIS.contextMenuVars.maxWidth			= 275
FCOIS.contextMenuVars.maxHeight			= 561
FCOIS.contextMenuVars.entryHeight			= 20
FCOIS.contextMenuVars.maxCharactersInLine = 32
--Gear set filter split context menu variables
FCOIS.contextMenuVars.GearSetFilter	= {}
FCOIS.contextMenuVars.GearSetFilter.maxWidth		= 24
FCOIS.contextMenuVars.GearSetFilter.maxHeight		= 144
FCOIS.contextMenuVars.GearSetFilter.entryHeight	= 24
--Research/Deconstruction filter split context menu variables
FCOIS.contextMenuVars.ResDecImpFilter	= {}
FCOIS.contextMenuVars.ResDecImpFilter.maxWidth    = 24
FCOIS.contextMenuVars.ResDecImpFilter.maxHeight	= 96
FCOIS.contextMenuVars.ResDecImpFilter.entryHeight	= 24
--Sell/Guild sell/Intricate filter split context menu variables
FCOIS.contextMenuVars.SellGuildIntFilter	= {}
FCOIS.contextMenuVars.SellGuildIntFilter.maxWidth     = 24
FCOIS.contextMenuVars.SellGuildIntFilter.maxHeight    = 96
FCOIS.contextMenuVars.SellGuildIntFilter.entryHeight	= 24

--Mapping for the context menu button
FCOIS.mappingVars.contextMenuButtonClickedMenuToButton = {
	["gear"] 			= 2,
	["ResDecImp"] 		= 3,
	["SellGuildInt"] 	= 4,
}

--The textures for the button context menu, selected item
FCOIS.contextMenuVars.menuInfo =
{
	[MENU_TYPE_DEFAULT] =
    {
        backdropEdge       = "EsoUI/Art/miscellaneous/insethighlight_edge.dds",
        backdropCenter     = "EsoUI/Art/miscellaneous/insethighlight_center.dds",
        backdropInsets     = {16,16,-16,-16},
        backdropEdgeWidth  = 128,
        backdropEdgeHeight = 16,
    }
}
--Initialize the context menus & variables used
for i=1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
	if FCOIS.mappingVars.activeFilterPanelIds[i] == true then
		--Inventory button context menus
		FCOIS.preventerVars.gContextCreated[i] 				= false
	    FCOIS.contextMenu[i]					 				= nil
	    FCOIS.contextMenu.bdSelectedLine[i]					= nil
		--Gear sets filter button context menus
        FCOIS.preventerVars.gGearSetFilterContextCreated[i]	= false
	    FCOIS.contextMenu.GearSetFilter[i]	  	 			= nil
	    FCOIS.contextMenu.GearSetFilter.bdSelectedLine[i]		= nil
		--Research, Deconstruction, Improvement filter button context menus
        FCOIS.contextMenu.ResDecImpFilter[i]	  	 			= nil
	    FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[i]	= nil
		FCOIS.preventerVars.gResDecImpFilterContextCreated[i]	= false
        --Sell/Sell in guild store/intricate filter button context menus
        FCOIS.contextMenu.SellGuildIntFilter[i]	  	 			= nil
        FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[i]	= nil
        FCOIS.preventerVars.gSellGuildIntFilterContextCreated[i]	= false
		--Initialize the variable for the last choosen filter button
        FCOIS.lastVars.gLastFilterId[i]						= 1
    end
end

--The mapping table for context menu invoker buttons and their settings
FCOIS.invAdditionalButtonVars.playerInventoryFCOAdditionalOptionsButton = "ZO_PlayerInventoryButtonFCOISAdditionalOptions"
FCOIS.invAdditionalButtonVars.smithingTopLevelDeconstructionPanelInventoryButtonAdditionalOptions = "ZO_SmithingTopLevelDeconstructionPanelInventoryButtonFCOISAdditionalOptions"
FCOIS.invAdditionalButtonVars.smithingTopLevelImprovementPanelInventoryButtonAdditionalOptions = "ZO_SmithingTopLevelImprovementPanelInventoryButtonFCOISAdditionalOptions"
FCOIS.invAdditionalButtonVars.enchantingTopLevelInventoryButtonAdditionalOptions = "ZO_EnchantingTopLevelInventoryButtonFCOISAdditionalOptions"
FCOIS.invAdditionalButtonVars.playerBankWithdrawButtonAdditionalOptions = "ZO_PlayerBankWithdrawButtonFCOISAdditionalOptions"
FCOIS.invAdditionalButtonVars.guildBankFCOWithdrawButtonAdditionalOptions = "ZO_GuildBankWithdrawButtonFCOISAdditionalOptions"
FCOIS.invAdditionalButtonVars.craftBagInventoryButtonAdditionalOptions = "ZO_CraftBagButtonFCOISAdditionalOptions"
FCOIS.contextMenuVars.filterPanelIdToContextMenuButtonInvoker = {
	[LF_INVENTORY] 					= FCOIS.invAdditionalButtonVars.playerInventoryFCOAdditionalOptionsButton,
	[LF_BANK_WITHDRAW] 					= FCOIS.invAdditionalButtonVars.playerBankWithdrawButtonAdditionalOptions,
	[LF_BANK_DEPOSIT] 			= FCOIS.invAdditionalButtonVars.playerInventoryFCOAdditionalOptionsButton,
	[LF_GUILDBANK_WITHDRAW] 		    = FCOIS.invAdditionalButtonVars.guildBankFCOWithdrawButtonAdditionalOptions,
	[LF_GUILDBANK_DEPOSIT]		= FCOIS.invAdditionalButtonVars.playerInventoryFCOAdditionalOptionsButton,
	[LF_VENDOR_SELL] 				= FCOIS.invAdditionalButtonVars.playerInventoryFCOAdditionalOptionsButton,
	[LF_SMITHING_DECONSTRUCT]  		= FCOIS.invAdditionalButtonVars.smithingTopLevelDeconstructionPanelInventoryButtonAdditionalOptions,
	[LF_GUILDSTORE_SELL] 	 		= FCOIS.invAdditionalButtonVars.playerInventoryFCOAdditionalOptionsButton,
	[LF_MAIL_SEND] 					= FCOIS.invAdditionalButtonVars.playerInventoryFCOAdditionalOptionsButton,
	[LF_TRADE] 				= FCOIS.invAdditionalButtonVars.playerInventoryFCOAdditionalOptionsButton,
	[LF_ENCHANTING_CREATION]	= FCOIS.invAdditionalButtonVars.enchantingTopLevelInventoryButtonAdditionalOptions,
	[LF_ENCHANTING_EXTRACTION]	= FCOIS.invAdditionalButtonVars.enchantingTopLevelInventoryButtonAdditionalOptions,
	[LF_SMITHING_IMPROVEMENT]			= FCOIS.invAdditionalButtonVars.smithingTopLevelImprovementPanelInventoryButtonAdditionalOptions,
	[LF_FENCE_SELL] 				= FCOIS.invAdditionalButtonVars.playerInventoryFCOAdditionalOptionsButton,
	[LF_FENCE_LAUNDER] 				= FCOIS.invAdditionalButtonVars.playerInventoryFCOAdditionalOptionsButton,
    --Added with API 100015 for the crafting bags that you only got access too if you are an ESO+ subscriber
    [LF_CRAFTBAG]				= FCOIS.invAdditionalButtonVars.craftBagInventoryButtonAdditionalOptions,
}

--The mapping table for context menu buttons to icon id
FCOIS.contextMenuVars.buttonContextMenuToIconId = {
	["ButtonContextMenu1"] = {
	    iconId = 1,
        mark   = true,
        anchorButton = "ButtonContextMenu1",
    },
	["ButtonContextMenu2"] = {
	    iconId = 1,
        mark   = false,
        anchorButton = "ButtonContextMenu1",
    },
	["ButtonContextMenu3"] = {
	    iconId = 2,
        mark   = true,
        anchorButton = "ButtonContextMenu2",
    },
	["ButtonContextMenu4"] = {
	    iconId = 2,
        mark   = false,
        anchorButton = "ButtonContextMenu3",
    },
	["ButtonContextMenu5"] = {
	    iconId = 4,
        mark   = true,
        anchorButton = "ButtonContextMenu4",
    },
	["ButtonContextMenu6"] = {
	    iconId = 4,
        mark   = false,
        anchorButton = "ButtonContextMenu5",
    },
	["ButtonContextMenu7"] = {
	    iconId = 6,
        mark   = true,
        anchorButton = "ButtonContextMenu6",
    },
	["ButtonContextMenu8"] = {
	    iconId = 6,
        mark   = false,
        anchorButton = "ButtonContextMenu7",
    },
	["ButtonContextMenu9"] = {
	    iconId = 7,
        mark   = true,
        anchorButton = "ButtonContextMenu8",
    },
	["ButtonContextMenu10"] = {
	    iconId = 7,
        mark   = false,
        anchorButton = "ButtonContextMenu9",
    },
	["ButtonContextMenu11"] = {
	    iconId = 8,
        mark   = true,
        anchorButton = "ButtonContextMenu10",
    },
	["ButtonContextMenu12"] = {
	    iconId = 8,
        mark   = false,
        anchorButton = "ButtonContextMenu11",
    },
	["ButtonContextMenu13"] = {
	    iconId = 3,
        mark   = true,
        anchorButton = "ButtonContextMenu12",
    },
	["ButtonContextMenu14"] = {
	    iconId = 3,
        mark   = false,
        anchorButton = "ButtonContextMenu13",
    },
	["ButtonContextMenu15"] = {
	    iconId = 9,
        mark   = true,
        anchorButton = "ButtonContextMenu14",
    },
	["ButtonContextMenu16"] = {
	    iconId = 9,
        mark   = false,
        anchorButton = "ButtonContextMenu15",
    },
	["ButtonContextMenu17"] = {
	    iconId = 10,
        mark   = true,
        anchorButton = "ButtonContextMenu16",
    },
	["ButtonContextMenu18"] = {
	    iconId = 10,
        mark   = false,
        anchorButton = "ButtonContextMenu17",
    },
	["ButtonContextMenu19"] = {
	    iconId = 5,
        mark   = true,
        anchorButton = "ButtonContextMenu18",
    },
	["ButtonContextMenu20"] = {
	    iconId = 5,
        mark   = false,
        anchorButton = "ButtonContextMenu19",
    },
    ["ButtonContextMenu21"] = {
        iconId = 11,
        mark   = true,
        anchorButton = "ButtonContextMenu20",
    },
    ["ButtonContextMenu22"] = {
        iconId = 11,
        mark   = false,
        anchorButton = "ButtonContextMenu21",
    },
    ["ButtonContextMenu23"] = {
        iconId = 12,
        mark   = true,
        anchorButton = "ButtonContextMenu22",
    },
    ["ButtonContextMenu24"] = {
        iconId = 12,
        mark   = false,
        anchorButton = "ButtonContextMenu23",
    },
}

--The mapping table for gear set split filter context menu buttons to icon id
FCOIS.contextMenuVars.GearSetFilter.buttonContextMenuToIconId = {
	["ButtonContextMenuGearSetFilter1"] = {
	    iconId = -1,
        anchorButton = "ButtonContextMenuGearSetFilter1",
    },
	["ButtonContextMenuGearSetFilter2"] = {
	    iconId = 2,
        anchorButton = "ButtonContextMenuGearSetFilter1",
    },
	["ButtonContextMenuGearSetFilter3"] = {
	    iconId = 4,
        anchorButton = "ButtonContextMenuGearSetFilter2",
    },
	["ButtonContextMenuGearSetFilter4"] = {
	    iconId = 6,
        anchorButton = "ButtonContextMenuGearSetFilter3",
    },
	["ButtonContextMenuGearSetFilter5"] = {
	    iconId = 7,
        anchorButton = "ButtonContextMenuGearSetFilter4",
    },
	["ButtonContextMenuGearSetFilter6"] = {
	    iconId = 8,
        anchorButton = "ButtonContextMenuGearSetFilter5",
    },
}

--The mapping table for RESEARCH & DECONSTRUCTION & IMPORVEMENT split filter context menu buttons to icon id
FCOIS.contextMenuVars.ResDecImpFilter.buttonContextMenuToIconId = {
	["ButtonContextMenuResDecImpFilter1"] = {
	    iconId = -1,
        anchorButton = "ButtonContextMenuResDecImpFilter1",
    },
	["ButtonContextMenuResDecImpFilter2"] = {
	    iconId = 3,
        anchorButton = "ButtonContextMenuResDecImpFilter1",
    },
	["ButtonContextMenuResDecImpFilter3"] = {
	    iconId = 9,
        anchorButton = "ButtonContextMenuResDecImpFilter2",
    },
	["ButtonContextMenuResDecImpFilter4"] = {
	    iconId = 10,
        anchorButton = "ButtonContextMenuResDecImpFilter3",
    },
}
--The mapping table for SELL & SELL IN GUILD STORE & INTRICATE split filter context menu buttons to icon id
FCOIS.contextMenuVars.SellGuildIntFilter.buttonContextMenuToIconId = {
    ["ButtonContextMenuSellGuildIntFilter1"] = {
        iconId = -1,
        anchorButton = "ButtonContextMenuSellGuildIntFilter1",
    },
    ["ButtonContextMenuSellGuildIntFilter2"] = {
        iconId = 5,
        anchorButton = "ButtonContextMenuSellGuildIntFilter1",
    },
    ["ButtonContextMenuSellGuildIntFilter3"] = {
        iconId = 11,
        anchorButton = "ButtonContextMenuSellGuildIntFilter2",
    },
    ["ButtonContextMenuSellGuildIntFilter4"] = {
        iconId = 12,
        anchorButton = "ButtonContextMenuSellGuildIntFilter3",
    },
}

--===================== FUNCTIONS ==============================================

--Output debug message in chat
local function debugMessage(msg_text, deep)
	if (deep and not FCOIS.settingsVars.settings.deepDebug) then
    	return
    end
	if (FCOIS.settingsVars.settings.debug == true) then
    	if deep then
        	--Blue colored "FCOIS" at the start of the string
	        d(FCOIS.preChatVars.preChatTextBlue .. msg_text)
        else
        	--Green colored "FCOIS" at the start of the string
	        d(FCOIS.preChatVars.preChatTextGreen .. msg_text)
        end
	end
end

--==============================================================================
--			Pre-Hook Handler Methods
--==============================================================================
-- gets handler from the event handler list
local function GetEventHandler(eventName, objName)
	if( not FCOIS.eventHandlers or not FCOIS.eventHandlers[eventName] ) then return nil end
	return FCOIS.eventHandlers[eventName][objName]
end

-- adds handler to the event handler list
local function SetEventHandler(eventName, objName, handler)
	if ( not FCOIS.eventHandlers[eventName] ) then FCOIS.eventHandlers[eventName] = {} end
	FCOIS.eventHandlers[eventName][objName] = handler
end

-- puts given handler in front of the event handler of given object
local function PreHookHandler(eventName, control, handler)
	SetEventHandler(eventName, control:GetName(), control:GetHandler(eventName) )
	control:SetHandler(eventName, handler)
end

--Show an alert message
local function showAlert(alertMsg)
	ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, FCOIS.preChatVars.preChatTextRed .. alertMsg)
end


--Choose the filter type
local function getSettingsIsFilterOn(p_filterId, p_filterPanel)
    local result
    if (FCOIS.settingsVars.settings.splitFilters == true) then
		local p_filterPanelNew = p_filterPanel or FCOIS.gFilterWhere

		--New behaviour with filters
		result = FCOIS.settingsVars.settings.isFilterPanelOn[p_filterPanelNew][p_filterId]
        if result == nil then
			return false
        end
        --debugMessage("[GetSettingsIsFilterOn] Filter Panel: " .. tostring(p_filterPanelNew) .. ", FilterId: " .. tostring(p_filterId) .. ", Result: " .. tostring(result), true)
        return result
	else
		--Old behaviour with filters
		result = FCOIS.settingsVars.settings.isFilterOn[p_filterId]
        debugMessage("[GetSettingsIsFilterOn] FilterId: " .. tostring(p_filterId) .. ", Result: " .. tostring(result), true)
		return result
    end
end

--Set the value of a filter type, and return it
local function setSettingsIsFilterOn(p_filterId, p_value, p_filterPanel)
    local p_filterPanelNew = p_filterPanel or FCOIS.gFilterWhere
    if (FCOIS.settingsVars.settings.splitFilters == true) then
		--New behaviour with filters
        FCOIS.settingsVars.settings.isFilterPanelOn[p_filterPanelNew][p_filterId] = p_value
        debugMessage("[SetSettingsIsFilterOn] Filter Panel: " .. tostring(p_filterPanelNew) .. ", FilterId: " .. tostring(p_filterId) .. ", Value: " .. tostring(p_value), true)
	else
		--Old behaviour with filters
		FCOIS.settingsVars.settings.isFilterOn[p_filterId] = p_value
        debugMessage("[SetSettingsIsFilterOn] FilterId: " .. tostring(p_filterId) .. ", Value: " .. tostring(p_value), true)
    end
    --return the value
    return p_value
end

-- Check the FCOIS.settingsVars.settings for the panels and return if enabled or disabled
local function getFilterWhereBySettings(p_filterWhere, onlyAnti)
	p_filterWhere = p_filterWhere or FCOIS.gFilterWhere
	onlyAnti = onlyAnti or false

	if onlyAnti == false then
		--Set the resultVar and update the FCOIS.settingsVars.settings.atPanelEnabled array
	    if     (p_filterWhere == LF_INVENTORY or p_filterWhere == LF_BANK_DEPOSIT or p_filterWhere == LF_GUILDBANK_DEPOSIT) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowInventoryFilter
	    elseif (p_filterWhere == LF_CRAFTBAG) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowCraftBagFilter
	    elseif (p_filterWhere == LF_VENDOR_SELL) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowVendorFilter
	    elseif (p_filterWhere == LF_FENCE_SELL) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowFenceFilter
	    elseif (p_filterWhere == LF_FENCE_LAUNDER) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowLaunderFilter
	    elseif (p_filterWhere == LF_SMITHING_DECONSTRUCT ) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowDeconstructionFilter
	    elseif (p_filterWhere == LF_SMITHING_IMPROVEMENT ) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowImprovementFilter
	    elseif (p_filterWhere == LF_ENCHANTING_EXTRACTION ) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowEnchantingFilter
	    elseif (p_filterWhere == LF_ENCHANTING_CREATION ) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowEnchantingFilter
	    elseif (p_filterWhere == LF_BANK_WITHDRAW) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowBankFilter
	    elseif (p_filterWhere == LF_GUILDBANK_WITHDRAW) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowGuildBankFilter
	    elseif (p_filterWhere == LF_GUILDSTORE_SELL) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowTradinghouseFilter
	    elseif (p_filterWhere == LF_TRADE) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowTradeFilter
	    elseif (p_filterWhere == LF_MAIL_SEND) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowMailFilter
	    elseif (p_filterWhere == LF_ALCHEMY_CREATION) then
			FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"] = FCOIS.settingsVars.settings.allowAlchemyFilter
		end
	end

    debugMessage("[getFilterWhereBySettings] " .. tostring(p_filterWhere) .. " = " .. tostring(FCOIS.settingsVars.settings.atPanelEnabled[p_filterWhere]["filters"]), false)

    return p_filterWhere
end

--Refresh the backpack list
local function RefreshBackpack()
    if not FCOIS.ZOControlVars.BACKPACK:IsHidden() then
		debugMessage("[RefreshBackpack] Backpack refresh", true)
    	ZO_ScrollList_RefreshVisible(FCOIS.ZOControlVars.BACKPACK)
    end
	--Added with patch to API 100015 -> New craft bag
	if HasCraftBagAccess() and INVENTORY_CRAFT_BAG and not FCOIS.ZOControlVars.CRAFTBAG:IsHidden() then
		PLAYER_INVENTORY:UpdateList(INVENTORY_CRAFT_BAG)
	end
end

--Refresh the bank list
local function RefreshBank()
	if not FCOIS.ZOControlVars.BANK:IsHidden() then
		debugMessage("[RefreshBank] Bank refresh", true)
    	ZO_ScrollList_RefreshVisible(FCOIS.ZOControlVars.BANK)
    end
end

--Refresh the guild bank list
local function RefreshGuildBank()
	if not FCOIS.ZOControlVars.GUILD_BANK:IsHidden() then
		debugMessage("[RefreshGuildBank] Guild bank refresh", true)
	    ZO_ScrollList_RefreshVisible(FCOIS.ZOControlVars.GUILD_BANK)
    end
end

--Refresh the repair list
local function RefreshRepairList()
    if not FCOIS.ZOControlVars.REPAIR_LIST:IsHidden() then
		debugMessage("[RefreshRepairList] Repair list refresh", true)
    	ZO_ScrollList_RefreshVisible(FCOIS.ZOControlVars.REPAIR_LIST)
    end
end

--Refresh the quickslot list
local function RefreshQuickSlots()
    if not FCOIS.ZOControlVars.QUICKSLOT_LIST:IsHidden() then
		debugMessage("[RefreshQuickSlots] Quickslot refresh", true)
    	ZO_ScrollList_RefreshVisible(FCOIS.ZOControlVars.QUICKSLOT_LIST)
    end
end

--Refresh the list dialog 1 scroll list
local function RefreshListDialog()
	if not FCOIS.ZOControlVars.LIST_DIALOG:IsHidden() then
		debugMessage("[RefreshListDialog] List Dialog refresh", true)
    	ZO_ScrollList_RefreshVisible(FCOIS.ZOControlVars.LIST_DIALOG)
    end
end

--The function to update the repair list
local function UpdateRepairList()
	debugMessage("[UpdateRepairList]", true)

	--Update the scroll list controls for the repair list
	RefreshRepairList()
end

--The function to update the quickslots inventory
local function UpdateQuickSlots()
	debugMessage("[UpdateQuickSlots]", true)

	--Update the scroll list controls for the quick slots inventory
	RefreshQuickSlots()
end

--Refresh the scroll lists
local function UpdateInventories()
    --Check if we are at a crafting station
	local craftInteractiontype = GetCraftingInteractionType()
	debugMessage("[UpdateInventories] CraftingInteractionType: " .. tostring(craftInteractiontype), true)
    --Are we not inside a crafting station? Else abort as this function will only update the player inventory
    if craftInteractiontype ~= CRAFTING_TYPE_INVALID then return end

	--Update the current shown inventory
    if (libFilters ~= nil and FCOIS.gFilterWhere ~= nil and FCOIS.lastVars.gLastFilterId[FCOIS.gFilterWhere] ~= nil) then
		debugMessage("Filter Id: " ..tostring(FCOIS.lastVars.gLastFilterId[FCOIS.gFilterWhere]) .. ", Filter panel Id: " .. tostring(FCOIS.gFilterWhere), true)
		--Get the current set settings for the filter panels
        FCOIS.gFilterWhere = getFilterWhereBySettings(FCOIS.gFilterWhere, false)
		--Is the filter for this panel enabled in the FCOIS.settingsVars.settings?
        if FCOIS.settingsVars.settings.atPanelEnabled[FCOIS.gFilterWhere]["filters"] == true then
			--Is the filter we have added the icon for currently enabled(registered)?
           	--local isFilterEnabled = getSettingsIsFilterOn(FCOIS.lastVars.gLastFilterId[FCOIS.gFilterWhere], FCOIS.gFilterWhere)
			--if isFilterEnabled == true or isFilterEnabled == -99 then
		    	--debugMessage("UpdaterName: " .. libFilterVars.filterTypeToUpdaterName[FCOIS.gFilterWhere], true)
			    --libFilterVars.inventoryUpdaters[libFilterVars.filterTypeToUpdaterName[FCOIS.gFilterWhere]]()
                libFilters:RequestUpdate( FCOIS.gFilterWhere )
            --end
        end
    end

	--Update the scroll list controls for the player inventories
	RefreshBackpack()
	RefreshBank()
	RefreshGuildBank()
    RefreshListDialog()
end

local function UpdateCraftingInventory()
    --Check if we are at a crafting station
	local locCraftType = GetCraftingInteractionType()
	debugMessage("[UpdateCraftingInventory] CraftingInteractionType: " .. tostring(locCraftType), true)
	--Abort if we are not at a crafting station
	if locCraftType == CRAFTING_TYPE_INVALID then return end

	--Update the current shown inventory
    if (libFilters ~= nil and FCOIS.gFilterWhere ~= nil and FCOIS.lastVars.gLastFilterId[FCOIS.gFilterWhere] ~= nil) then
		--Get the current set settings for the filter panels
        FCOIS.gFilterWhere = getFilterWhereBySettings(FCOIS.gFilterWhere, false)
		--Is the filter for this panel enabled in the FCOIS.settingsVars.settings?
		if FCOIS.settingsVars.settings.atPanelEnabled[FCOIS.gFilterWhere]["filters"] == true then
			--Is the filter we have added the icon for currently enabled(registered)?
           	--local isFilterEnabled = getSettingsIsFilterOn(FCOIS.lastVars.gLastFilterId[FCOIS.gFilterWhere], FCOIS.gFilterWhere)
			--if isFilterEnabled == true or isFilterEnabled == -99 then
		    	--debugMessage("Filter Id: " ..tostring(FCOIS.lastVars.gLastFilterId[FCOIS.gFilterWhere]) .. ", Filter panel Id: " .. tostring(FCOIS.gFilterWhere) .. ", UpdaterName: " .. libFilterVars.filterTypeToUpdaterName[FCOIS.gFilterWhere], true)
		    	debugMessage("Filter Id: " ..tostring(FCOIS.lastVars.gLastFilterId[FCOIS.gFilterWhere]) .. ", Filter panel Id: " .. tostring(FCOIS.gFilterWhere), true)
			    --libFilterVars.inventoryUpdaters[libFilterVars.filterTypeToUpdaterName[FCOIS.gFilterWhere]]()
                libFilters:RequestUpdate( FCOIS.gFilterWhere )
            --end
        end
    end

	--Alchemy?
	if (locCraftType == CRAFTING_TYPE_ALCHEMY) then
       	debugMessage("[UpdateCraftingInventory] Alchemy refresh", true)
		--Only refresh the scroll list
		ZO_ScrollList_RefreshVisible(FCOIS.ZOControlVars.ALCHEMY_STATION)
	--Enchanting?
	elseif (locCraftType == CRAFTING_TYPE_ENCHANTING) then
       	debugMessage("[UpdateCraftingInventory] Enchanting refresh", true)
		--Only refresh the scroll list
		ZO_ScrollList_RefreshVisible(FCOIS.ZOControlVars.ENCHANTING_STATION)
	else
	--Other crafting stations

		--Refinement
		if (not FCOIS.ZOControlVars.REFINEMENT:IsHidden()) then
	       	debugMessage("[UpdateCraftingInventory] Refinement refresh", true)
			--Are we at a refinement panel?
			--Only refresh the scroll list
			ZO_ScrollList_RefreshVisible(FCOIS.ZOControlVars.REFINEMENT)
		--Improvement
        elseif     (FCOIS.gFilterWhere == LF_SMITHING_IMPROVEMENT) then
	       	debugMessage("[UpdateCraftingInventory] Improvement refresh", true)
			--Are we at an improvement panel?
			--Only refresh the scroll list
			ZO_ScrollList_RefreshVisible(FCOIS.ZOControlVars.IMPROVEMENT)
		--Deconstruction
		elseif (FCOIS.gFilterWhere == LF_SMITHING_DECONSTRUCT) then
	       	debugMessage("[UpdateCraftingInventory] Deconstruction refresh", true)
			--Are we at a deconstruction panel?
			--Only refresh the scroll list
			ZO_ScrollList_RefreshVisible(FCOIS.ZOControlVars.DECONSTRUCTION)
		end
	end
end

--converts unsigned itemId to signed
local function SignItemId(itemId)
	local SIGNED_INT_MAX = 2^32 / 2 - 1
	local INT_MAX 		 = 2^32
	if(itemId and itemId > SIGNED_INT_MAX) then
		itemId = itemId - INT_MAX
	end
	return itemId
end

local function MyGetItemNameNoControl(bagId, slotIndex)
	local name = "Not found"
    local itemData
    if bagId < 1 or bagId > 4 then bagId = 1 end
	--[[
		inventoryTypes for PLAYER_INVENTORY:
		INVENTORY_BACKPACK 	 = 1
		INVENTORY_QUEST_ITEM = 2
		INVENTORY_BANK		 = 3
		INVENTORY_GUILD_BANK = 4
	]]
	itemData = PLAYER_INVENTORY.inventories[bagId].slots[slotIndex]
	if(itemData ~= nil) then
		name = itemData.name
	end
	return name
end

local function MyGetItemName(rowControl)
	local name
    if (rowControl == nil) then return end
	local dataEntry = rowControl.dataEntry

	--case to handle equiped items
	if(not dataEntry) then
		name = rowControl.name
	else
		name = dataEntry.data.name
	end
	return name
end

local function MyGetItemDetails(rowControl)
	local bagId, slotIndex

	--gotta do this in case deconstruction, or player equipment
    local dataEntry = rowControl.dataEntry

	--case to handle equiped items
	if(not dataEntry) then
    	bagId = rowControl.bagId
        slotIndex = rowControl.slotIndex
	else
		bagId = dataEntry.data.bagId
		slotIndex = dataEntry.data.slotIndex
	end

	--case to handle list dialog, list dialog uses index instead of slotIndex and bag instead of bagId...?
	if(dataEntry and not bagId and not slotIndex) then
		bagId = rowControl.dataEntry.data.bag
		slotIndex = rowControl.dataEntry.data.index
	end

	return bagId, slotIndex
end

local function MyGetItemInstanceIdNoControl(bagId, slotIndex, signToo)
	signToo = signToo or false
	local itemId = GetItemInstanceId(bagId, slotIndex)
	if signToo then
		return SignItemId(itemId)
    else
    	return itemId
    end
end

local function MyGetItemInstanceId(rowControl, signToo)
	signToo = signToo or false
	local bagId, slotIndex = MyGetItemDetails(rowControl)

	local itemId = GetItemInstanceId(bagId, slotIndex)
	if signToo then
		return SignItemId(itemId)
    else
		return itemId
    end
end

local function isItemBindable(bagId, slotIndex)
	local itemLink = GetItemLink(bagId, slotIndex)
	if itemLink then
		--Bound
	    if(IsItemLinkBound(itemLink)) then
			--Item is already bound
	        return false
	    else
	        local bindType = GetItemLinkBindType(itemLink)
	        if(bindType ~= BIND_TYPE_NONE and bindType ~= BIND_TYPE_UNSET) then
            	--Item can still be bound
            	return true
	    	else
				--Item is already bound or got no bind type
            	return false
	        end
	    end
    else
    	return nil
    end
end

--Get the bagid and slotIndex from the item below the mouse cursor
local function FCOGetBagAndSlotFromControlUnderMouse()
	--Get the control below the mouse cursor
	local moc = WINDOW_MANAGER:GetMouseOverControl()
	if moc == nil then return end
--d("[FCOGetBagAndSlotFromControlUnderMouse] " .. moc:GetName())
	local bagId
    local slotIndex
	--if it's a backpack row or child of one -> PRE API 1000015
	if moc:GetName():find("^ZO_%a+Backpack%dRow%d%d*") then
		if moc:GetName():find("^ZO_%a+Backpack%dRow%d%d*$") then
			bagId, slotIndex = MyGetItemDetails(moc)
		else
			moc = moc:GetParent()
			if moc:GetName():find("^ZO_%a+Backpack%dRow%d%d*$") then
				bagId, slotIndex = MyGetItemDetails(moc)
			end
		end
	--if it's a backpack row or child of one -> Since API 1000015
	elseif moc:GetName():find("^ZO_%a+InventoryList%dRow%d%d*") then
		if moc:GetName():find("^ZO_%a+InventoryList%dRow%d%d*$") then
			bagId, slotIndex = MyGetItemDetails(moc)
		else
			moc = moc:GetParent()
			if moc:GetName():find("^ZO_%a+InventoryList%dRow%d%d*$") then
				bagId, slotIndex = MyGetItemDetails(moc)
			end
		end
	--CRAFTBAG: if it's a backpack row or child of one -> Since API 1000015
	elseif moc:GetName():find("^ZO_CraftBagList%dRow%d%d*") then
		if moc:GetName():find("^ZO_CraftBagList%dRow%d%d*$") then
			bagId, slotIndex = MyGetItemDetails(moc)
		else
			moc = moc:GetParent()
			if moc:GetName():find("^ZO_CraftBagList%dRow%d%d*$") then
				bagId, slotIndex = MyGetItemDetails(moc)
			end
		end
	--Character
	elseif moc:GetName():find("^ZO_CharacterEquipmentSlots.+$") then
		bagId, slotIndex = MyGetItemDetails(moc)
	--Quickslot
	elseif moc:GetName():find("^ZO_QuickSlotList%dRow%d%d*") then
		bagId, slotIndex = MyGetItemDetails(moc)
	--Vendor rebuy
	elseif moc:GetName():find("^ZO_RepairWindowList%dRow%d%d*") then
		bagId, slotIndex = MyGetItemDetails(moc)
	end
	if bagId ~= nil and slotIndex ~= nil then
    	return bagId, slotIndex
    else
    	return false
    end
end

--Is the item a container and is autoloot enabled in the ESO settings
local function isAutolootContainer(bag, slot)
    return (GetItemType(bag, slot) == ITEMTYPE_CONTAINER and GetSetting(SETTING_TYPE_LOOT, LOOT_SETTING_AUTO_LOOT)=="1")
end

--Is the item a recipe and is it known by one of your chars?
local function isRecipeKnown(bagId, slotIndex)
	--Get recipe info from Sous Chef addon
	if SousChef and SousChef.settings.showAltKnowledge then
		local itemLink = GetItemLink(bagId, slotIndex)
		if itemLink == "" then return nil end
		-- item is a recipe
		if GetItemLinkItemType(itemLink) == ITEMTYPE_RECIPE then
			local resultLink = GetItemLinkRecipeResultItemLink(itemLink)
			local knownByUsersTable = SousChef.settings.Cookbook[SousChef.Utility.CleanString(GetItemLinkName(resultLink))]
			local known = false
			if knownByUsersTable ~= nil then
                local currentCharacterName = ""
            	if FCOIS.settingsVars.settings.autoMarkRecipesOnlyThisChar then
					--Only check if recipe is known for the current character?
					currentCharacterName = GetUnitName("player")
               	else
                	--Check if recipe is known for your main provisioning character
					local recipeMainChar = SousChef.settings.mainChar
                    if recipeMainChar == "(current)" then
                    	recipeMainChar = GetUnitName("player")
                    end
                    currentCharacterName = recipeMainChar
                end
                if currentCharacterName and currentCharacterName ~= "(current)" and currentCharacterName ~= "" then
	                known = knownByUsersTable[currentCharacterName] or false
                end
            end
			return known
		end
	end
    return nil
end

local function isItemType(bag, slot, itemType)
	if not itemType then return false end
    return (GetItemType(bag, slot) == itemType)
end

--Function to check if the item is marked ("protected") with the icon number & settings
local function checkIfItemIsProtected(iconId, itemInstanceId)
	local isIn = false
	if iconId == nil then return false end
    if itemInstanceId == nil then return false end
--d("[checkIfItemIsProtected] iconId: " .. iconId .. ", itemId: " .. itemInstanceId)
	--Item is not enabled
	local itemIsMarked = false
	--Is the item enabled, or is the item disabled and the setting to check disabled icons too is enabled?
	if (   (FCOIS.settingsVars.settings.isIconEnabled[iconId])
    	or (not FCOIS.settingsVars.settings.isIconEnabled[iconId] and FCOIS.settingsVars.settings.checkDeactivatedIcons) ) then
		--Workaround to return a not-marked icon, if the icon is disabled, so the icon won't be removed
	    -- >> Set & unset in function "ClearOrRestoreAllMarkers"
    	if (not FCOIS.settingsVars.settings.isIconEnabled[iconId] and FCOIS.preventerVars.doFalseOverride) then
	    	return false
		end

		--Check if the item is marked with the icon
    	local isMarked = FCOIS.markedItems[iconId][SignItemId(itemInstanceId)]
        if isMarked == nil then isMarked = false end
		--Update the return variable
        itemIsMarked = isMarked
    end
	return itemIsMarked
end

--Function do check if an array of icons is marked and thus protected
local function checkIfItemArrayIsProtected(iconIdArray, itemInstanceId)
	if iconIdArray == nil or #iconIdArray == 0 or itemInstanceId == nil then return false end
	local retVal = false
	--Check each iconId in the arry now for a protection
    for i=1, #iconIdArray, 1 do
		retVal = checkIfItemIsProtected(iconIdArray[i], itemInstanceId)
        if retVal then break end
    end
    return retVal
end

--Build the icon's index list
local function buildIconsIndexList()
	local iconsIndexList = {}
	for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
		local locNameStr = FCOIS.localizationVars.fcois_loc["options_icon" .. tostring(i) .. "_" .. FCOIS.localizationVars.iconEndStrArray[i]]
		iconsIndexList[locNameStr] = i
	end
	return iconsIndexList
end
--Get the index of the icon, by help of the name
local function getIconIndex(iconName)
	if iconName == nil or iconName == "" then return false end
	if FCOIS.iconVars.iconsIndexList == nil or #FCOIS.iconVars.iconsIndexList <= 0 then return false end
	--Loop over the icon indexlist
	for listIconName, iconId in pairs(FCOIS.iconVars.iconsIndexList) do
		if listIconName == iconName then
			return iconId
		end
	end
	return false
end

--===== GLOBAL FUNCTION - BEGIN ================================================
--Global function to return boolean value, if an item is marked
-- + it will return an array with 8 boolean entries, one for each iconId. True if item is marked with this iconId, false if not
--
--The item's instance ID and the icon number are needed to check, if the item is marked.
--Instance: gives the current item's instance ID, determined by function GetItemInstanceId(bagId, slotIndex)
--iconIds: Specifies the icon the item is marked with. iconIds can be 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 or -1 for all icons.
--		   The parameter can be an array/table too.
--		   The array's/table's key can be any index/value that you like.
--         The value to the key must be the icon number, or -1 for all icons.
function FCOIsMarked(instance, iconIds)
	if (instance ~= nil and iconIds ~= nil) then
		local markedArray = {}
        for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
        	markedArray[i] = false
        end
		local isMarked = false

		--Is parameter iconIds an array/table?
		if type(iconIds)=="table" then
			--Counter to check if any icon was already checked inside the for... loop
			local iconsChecked = 0
            local iconIsSet = false
			--Get each iconId from array/table
			for _, iconId in pairs(iconIds) do
				--Check for all icons if the item is marked
				if iconId == -1 then
                	--Was any iconId checked before?
                    if iconsChecked > 0 then
						--Initialize the return value and array again
				        for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				        	markedArray[i] = false
				        end
                        isMarked = false
                    end
					--Check all iconIds now
					for icoId = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
						if (checkIfItemIsProtected(icoId, instance) == true) then
							--return true, if any icon is set
		                    isMarked = true
		                	markedArray[icoId] = true
		                end
					end

                	--Abort here as all icons were checked now and no further single iconId from parameter iconIds must be checked
                    return isMarked, markedArray

                --Check only a specific iconId
                else
					--Is the iconId a number?
                	if type(iconId)=="number" then
                        --Increase the icons checked so any entry with iconId == -1, after this entry here was checked, will
                        --initialize the return array with false again!
                    	iconsChecked = iconsChecked + 1
                        --is the item marked with that iconId?
			            if (FCOIS.markedItems[iconId] ~= nil) then
                            iconIsSet = checkIfItemIsProtected(iconId, instance)
							markedArray[iconId] = iconIsSet
                            if not isMarked then
	                            isMarked = iconIsSet
                            end
			            end
                    end
                end
			end
	       	return isMarked, markedArray

        else

	        --iconIds is no array/table
			--Check only 1 icon
			if (iconIds ~= -1) then
	            isMarked = FCOIS.markedItems[iconIds] ~= nil and checkIfItemIsProtected(iconIds, instance)
	            if isMarked then
					markedArray[iconIds] = true
	            end
	        	return isMarked, markedArray
	        else
				--Check for all icons if the item is marked. return true, if any icon is set
				for icoId = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
					if (checkIfItemIsProtected(icoId, instance) == true) then
	                    isMarked = true
	                	markedArray[icoId] = true
	                end
				end
	            return isMarked, markedArray
	        end

		end
    else
        --At least one of the needed function parameters is missing. Return nil, nil
    	return nil, nil
    end
end
--===== GLOBAL FUNCTION - END ==================================================

--is the gamepad mode enabled in the ESO settings?
local function FCOItemSaver_CheckGamePadMode()
	--Gamepad enabled?
	if IsInGamepadPreferredMode() then
		d(FCOIS.preChatVars.preChatTextRed .. FCOIS.localizationVars.fcois_loc["error_gamepad_mode_not_supported"])
    	return true

    else
	--Gamepad not enabled
    	return false
    end
end

--This function will change the actual ANTI-DETSROY etc. settings according to the active filter panel ID (inventory, vendor, mail, trade, bank, etc.)
local function changeAntiSettingsAccordingToFilterPanel()
	if FCOIS.gFilterWhere == nil then return false end
	--Get the current FCOIS.settingsVars.settings state and inverse them
	if   FCOIS.gFilterWhere == LF_INVENTORY or FCOIS.gFilterWhere == LF_BANK_WITHDRAW or FCOIS.gFilterWhere == LF_GUILDBANK_WITHDRAW
      or FCOIS.gFilterWhere == LF_BANK_DEPOSIT or FCOIS.gFilterWhere == LF_GUILDBANK_DEPOSIT or FCOIS.gFilterWhere == LF_CRAFTBAG then
		FCOIS.settingsVars.settings.blockDestroying = not FCOIS.settingsVars.settings.blockDestroying
    elseif FCOIS.gFilterWhere == LF_VENDOR_SELL then
    	FCOIS.settingsVars.settings.blockSelling = not FCOIS.settingsVars.settings.blockSelling
    elseif FCOIS.gFilterWhere == LF_FENCE_SELL then
    	FCOIS.settingsVars.settings.blockFence = not FCOIS.settingsVars.settings.blockFence
    elseif FCOIS.gFilterWhere == LF_FENCE_LAUNDER then
    	FCOIS.settingsVars.settings.blockLaunder = not FCOIS.settingsVars.settings.blockLaunder
	elseif FCOIS.gFilterWhere == LF_SMITHING_DECONSTRUCT then
    	FCOIS.settingsVars.settings.blockDeconstruction = not FCOIS.settingsVars.settings.blockDeconstruction
	elseif FCOIS.gFilterWhere == LF_GUILDSTORE_SELL then
    	FCOIS.settingsVars.settings.blockSellingGuildStore = not FCOIS.settingsVars.settings.blockSellingGuildStore
	elseif FCOIS.gFilterWhere == LF_MAIL_SEND then
    	FCOIS.settingsVars.settings.blockSendingByMail = not FCOIS.settingsVars.settings.blockSendingByMail
	elseif FCOIS.gFilterWhere == LF_TRADE then
    	FCOIS.settingsVars.settings.blockTrading = not FCOIS.settingsVars.settings.blockTrading
	elseif FCOIS.gFilterWhere == LF_ENCHANTING_CREATION then
    	FCOIS.settingsVars.settings.blockEnchantingCreation = not FCOIS.settingsVars.settings.blockEnchantingCreation
	elseif FCOIS.gFilterWhere == LF_ENCHANTING_EXTRACTION then
    	FCOIS.settingsVars.settings.blockEnchantingExtraction = not FCOIS.settingsVars.settings.blockEnchantingExtraction
	elseif FCOIS.gFilterWhere == LF_SMITHING_IMPROVEMENT then
    	FCOIS.settingsVars.settings.blockImprovement = not FCOIS.settingsVars.settings.blockImprovement
    else
    	FCOIS.settingsVars.settings.blockDestroying = not FCOIS.settingsVars.settings.blockDestroying
    end
end

--Function to build the text for the toggle buttons (Anti-Deconstruct, Anti-Destroy, Anti_Sell, etc. FCOIS.settingsVars.settings)
--The function will return as first parameter the text and as second parameter a boolean value: true if the setting for the current panel is enabled, and false if not
local function getContextMenuAntiSettingsTextAndState(p_filterWhere, buildText)
	if p_filterWhere == nil or p_filterWhere == 0 then p_filterWhere = FCOIS.gFilterWhere end
    if p_filterWhere == nil or p_filterWhere == 0 then return end
	buildText = buildText or false

	debugMessage("[getContextMenuAntiSettingsTextAndState] PanelId: " .. p_filterWhere .. ", BuildText: " .. tostring(buildText), true)

	--Build the settings value
	--Update the Anti-* FCOIS.settingsVars.settings for the panel
    getFilterWhereBySettings(p_filterWhere, true)

    local currentSettingsState
	--Get the current FCOIS.settingsVars.settings state
	if   p_filterWhere == LF_INVENTORY or p_filterWhere == LF_BANK_WITHDRAW or p_filterWhere == LF_GUILDBANK_WITHDRAW
      or p_filterWhere == LF_BANK_DEPOSIT or p_filterWhere == LF_GUILDBANK_DEPOSIT or FCOIS.gFilterWhere == LF_CRAFTBAG then
		currentSettingsState = FCOIS.settingsVars.settings.blockDestroying
    elseif p_filterWhere == LF_VENDOR_SELL then
    	currentSettingsState = FCOIS.settingsVars.settings.blockSelling
    elseif p_filterWhere == LF_FENCE_SELL then
    	currentSettingsState = FCOIS.settingsVars.settings.blockFence
    elseif p_filterWhere == LF_FENCE_LAUNDER then
    	currentSettingsState = FCOIS.settingsVars.settings.blockLaunder
	elseif p_filterWhere == LF_SMITHING_DECONSTRUCT then
    	currentSettingsState = FCOIS.settingsVars.settings.blockDeconstruction
	elseif p_filterWhere == LF_GUILDSTORE_SELL then
    	currentSettingsState = FCOIS.settingsVars.settings.blockSellingGuildStore
	elseif p_filterWhere == LF_MAIL_SEND then
    	currentSettingsState = FCOIS.settingsVars.settings.blockSendingByMail
	elseif p_filterWhere == LF_TRADE then
    	currentSettingsState = FCOIS.settingsVars.settings.blockTrading
	elseif p_filterWhere == LF_ENCHANTING_CREATION then
    	currentSettingsState = FCOIS.settingsVars.settings.blockEnchantingCreation
	elseif p_filterWhere == LF_ENCHANTING_EXTRACTION then
    	currentSettingsState = FCOIS.settingsVars.settings.blockEnchantingExtraction
	elseif p_filterWhere == LF_SMITHING_IMPROVEMENT then
    	currentSettingsState = FCOIS.settingsVars.settings.blockImprovement
    else
    	currentSettingsState = FCOIS.settingsVars.settings.blockDestroying
    end
    currentSettingsState = currentSettingsState or false

    --Build the text too?
	local retStrVal = ""
    if buildText then
		--Mapping array for the on/off text
	    local mappingButtonOnOffText = {
	    	["true"]  = "off",
	    	["false"] = "on",
	    }
	    local onOffText = mappingButtonOnOffText[tostring(currentSettingsState)]
	    if onOffText ~= "" then
	        --Mapping array for the localized button texts
	        local mappingButtonText = {
	            [LF_INVENTORY] 					= "button_context_menu_toggle_anti_destroy_",
	            [LF_BANK_WITHDRAW] 					= "button_context_menu_toggle_anti_destroy_",
	            [LF_BANK_DEPOSIT] 			= "button_context_menu_toggle_anti_destroy_",
	            [LF_GUILDBANK_WITHDRAW] 		    = "button_context_menu_toggle_anti_destroy_",
	            [LF_GUILDBANK_DEPOSIT]		= "button_context_menu_toggle_anti_destroy_",
	            [LF_VENDOR_SELL] 				= "button_context_menu_toggle_anti_sell_",
	            [LF_SMITHING_DECONSTRUCT]  		= "button_context_menu_toggle_anti_deconstruct_",
	            [LF_GUILDSTORE_SELL] 	 		= "button_context_menu_toggle_anti_sell_",
	            [LF_MAIL_SEND] 					= "button_context_menu_toggle_anti_mail_",
	            [LF_TRADE] 				= "button_context_menu_toggle_anti_trade_",
	            [LF_ENCHANTING_CREATION]	= "button_context_menu_toggle_anti_create_",
	            [LF_ENCHANTING_EXTRACTION]	= "button_context_menu_toggle_anti_extract_",
	            [LF_SMITHING_IMPROVEMENT]			= "button_context_menu_toggle_anti_improve_",
	            [LF_FENCE_SELL] 				= "button_context_menu_toggle_anti_fence_sell_",
	            [LF_FENCE_LAUNDER] 				= "button_context_menu_toggle_anti_launder_sell_",
	            [LF_CRAFTBAG]				= "button_context_menu_toggle_anti_destroy_",
	        }
	    	local btnText = mappingButtonText[p_filterWhere]
	        btnText = btnText .. onOffText
	        if btnText ~= "" then
	        	retStrVal = FCOIS.localizationVars.fcois_loc[btnText]
	        end
	    end
	    if retStrVal == "" then retStrVal = "N/A" end
	end
    return retStrVal, currentSettingsState
end

--Returns the color for the context menu button if the "anti" settings is enabled or disabled
local function getContextMenuAntiSettingsColor(settingIsEnabled)
	settingIsEnabled = settingIsEnabled or false
	local retCol = {}
	if not FCOIS.settingsVars.settings.colorizeFCOISAdditionalInventoriesButton then
			retCol = {
		    	["r"] = 1,
		        ["g"] = 1,
		        ["b"] = 1,
		        ["a"] = 1,
		    }
    else
	    if settingIsEnabled then
			retCol = {
		    	["r"] = 0,
		        ["g"] = 1,
		        ["b"] = 0,
		        ["a"] = 1,
		    }
		else
			retCol = {
		    	["r"] = 1,
		        ["g"] = 0,
		        ["b"] = 0,
		        ["a"] = 1,
		    }
	    end
	end
    return retCol["r"], retCol["g"], retCol["b"], retCol["a"]
end

--Get the context menu invoker button by help of the panel Id
local function getContextMenuInvokerButton(panelId)
	panelId = panelId or FCOIS.gFilterWhere
    if not panelId or panelId == 0 then return false end

	--Workaround: Craftbag stuff
    if HasCraftBagAccess() and INVENTORY_CRAFT_BAG and not FCOIS.ZOControlVars.CRAFTBAG:IsHidden() then
        panelId = LF_CRAFTBAG
    end

    local retVal = FCOIS.contextMenuVars.filterPanelIdToContextMenuButtonInvoker[panelId]
    if retVal ~= nil and retVal ~= "" then
		return retVal
    else
    	return false
    end
end

--Change the context menu invoker button's color by help of the button control and the anti-* settings state
local function changeContextMenuInvokerButtonColor(contextMenuInvokerButton, settingsEnabled)
	if not contextMenuInvokerButton then return false end
	settingsEnabled = settingsEnabled or false
--d("contextMenuInvokerButton: " .. contextMenuInvokerButton:GetName() .. ", settingsEnabled: " .. tostring(settingsEnabled))

	--Update the context menu "flag" button's color according to the current settings state
	local colR, colG, colB, colA = getContextMenuAntiSettingsColor(settingsEnabled)
	--local contInvButTextureName = contextMenuInvokerButton:GetName() .. "Texture"
    local contInvButTexture = WINDOW_MANAGER:GetControlByName(contextMenuInvokerButton:GetName(), "Texture")
    if contInvButTexture then
		contInvButTexture:SetColor(colR, colG, colB, colA)
    end
end

--Change the context menu invoker button's color by help of the current panel ID
local function changeContextMenuInvokerButtonColorByPanelId(panelId)
	panelId = panelId or FCOIS.gFilterWhere
	debugMessage("[changeContextMenuInvokerButtonColorByPanelId] PanelId: " .. panelId, true)
	if not panelId or panelId == 0 then return false end
	--Change the color of the context menu invoker button now
	local _, settingsEnabled = getContextMenuAntiSettingsTextAndState(panelId, false)
	local contextMenuInvokerButtonName = getContextMenuInvokerButton(panelId)
	if contextMenuInvokerButtonName ~= "" and contextMenuInvokerButtonName ~= false then
--d("[changeContextMenuInvokerButtonColorByPanelId]: " .. contextMenuInvokerButtonName .. ", settings: " .. tostring(settingsEnabled))
		local contextMenuInvokerButton = WINDOW_MANAGER:GetControlByName(contextMenuInvokerButtonName, "")
        if contextMenuInvokerButton then
			changeContextMenuInvokerButtonColor(contextMenuInvokerButton, settingsEnabled)
        end
	end
end

--Function to reenable the Anti-* settings again
local function autoReenableAntiSettingsCheck(checkWhere)
	if checkWhere == nil or checkWhere == "" then return false end
    --Should all checks be done now?
    if checkWhere == "-ALL-" then
		local checksToDo = {
        	[1] = "CRAFTING_STATION",
        	[2] = "STORE",
            [3]	= "GUILD_STORE",
        	[4] = "DESTROY",
        	[5] = "TRADE",
        	[6] = "MAIL",
        }
    	--Get the checks to do and run them all after each other
        for _, checkWhereNow in ipairs(checksToDo) do
        	if checkWhereNow ~= "-ALL-" then
	            autoReenableAntiSettingsCheck(checkWhereNow)
            end
        end
    	return true
    end

    if checkWhere == "CRAFTING_STATION" then
		--Reenable the Anti-Deconstruction methods if activated in the settings
	    if FCOIS.settingsVars.settings.autoReenable_blockDeconstruction then
	    	FCOIS.settingsVars.settings.blockDeconstruction = true
	    end
		--Reenable the Anti-Improvement methods if activated in the settings
	    if FCOIS.settingsVars.settings.autoReenable_blockImprovement then
	    	FCOIS.settingsVars.settings.blockImprovement = true
	    end
		--Reenable the Anti-Enchanting creation methods if activated in the settings
	    if FCOIS.settingsVars.settings.autoReenable_blockEnchantingCreation then
	    	FCOIS.settingsVars.settings.blockEnchantingCreation = true
	    end
		--Reenable the Anti-Enchanting extraction methods if activated in the settings
	    if FCOIS.settingsVars.settings.autoReenable_blockEnchantingExtraction then
	    	FCOIS.settingsVars.settings.blockEnchantingExtraction = true
	    end

	elseif checkWhere == "STORE" then
		--Reenable the Anti-Sell methods if activated in the settings
	    if FCOIS.settingsVars.settings.autoReenable_blockSelling then
	    	FCOIS.settingsVars.settings.blockSelling = true
	    end
		--Reenable the Fence Anti-Sell methods if activated in the settings
	    if FCOIS.settingsVars.settings.autoReenable_blockFenceSelling then
	    	FCOIS.settingsVars.settings.blockFence = true
	    end
		--Reenable the Fence Anti-Laundering methods if activated in the settings
	    if FCOIS.settingsVars.settings.autoReenable_blockLaunderSelling then
	    	FCOIS.settingsVars.settings.blockLaunder = true
	    end

	elseif checkWhere == "GUILD_STORE" then
		--Reenable the Anti-Sell methods if activated in the settings
	    if FCOIS.settingsVars.settings.autoReenable_blockSellingGuildStore then
	    	FCOIS.settingsVars.settings.blockSellingGuildStore = true
	    end

	elseif checkWhere == "MAIL" then
		--Reenable the Anti-Mail methods if activated in the settings
	    if FCOIS.settingsVars.settings.autoReenable_blockSendingByMail then
	    	FCOIS.settingsVars.settings.blockSendingByMail = true
	    end

	elseif checkWhere == "TRADE" then
		--Reenable the Anti-Trade methods if activated in the settings
		if FCOIS.settingsVars.settings.autoReenable_blockTrading then
	        FCOIS.settingsVars.settings.blockTrading = true
        end

	elseif checkWhere == "DESTROY" then
		--Reenable the Anti-Destroy methods if activated in the settings
	    if FCOIS.settingsVars.settings.autoReenable_blockDestroying then
	    	FCOIS.settingsVars.settings.blockDestroying = true
	    end
    end

    --Workaround to enable the correct additional inventory context menu invoker button color for the normal inventory again
	changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)
end

local function GetItemQuality(bagId, slotIndex)
	--get the item link
    local itemLink = GetItemLink(bagId, slotIndex)
    if itemLink == nil then return false end
    -- Gets the item quality
	local itemQuality = GetItemLinkQuality(itemLink)
    if not itemQuality then return false end
    return itemQuality
end

--Is the item a set part?
local function isItemSetPartNoControl(bagId, slotIndex)
    local retVal = false
	local itemLink = GetItemLink(bagId, slotIndex)
	if itemLink ~= "" then
		-- item is a recipe
		local itemType = GetItemLinkItemType(itemLink)
		if itemType ~= nil then
			local allowedItemTypes = FCOIS.checkVars.allowedSetItemTypes
			local allowed = allowedItemTypes[itemType] or false
            if allowed then
	            --Get the set item information
				local hasSet, _, _, _, _ = GetItemLinkSetInfo(itemLink)
	            retVal = hasSet
            end
		end
	end
    return retVal
end

-- Is the item researchable?
local function isItemResearchableNoControl(bagId, slotIndex, markId)
	local retVal = false
	--Check if the item is virtually researchable as the settings is enabled to allow marking of non researchable items as gear
	markId = markId or nil
    if markId ~= nil then
    	retVal = FCOIS.settingsVars.settings.disableResearchCheck[markId]
        if retVal == nil then retVal = false end
    end
    if retVal == false then
		if bagId == nil or slotIndex == nil then return false end
		local itemType = GetItemType(bagId, slotIndex)
	    if itemType == nil then return false end
		local allowedTab = {}
        allowedTab = FCOIS.checkVars.allowedResearchableItemTypes[itemType]
        if allowedTab == nil then return false end
        if markId == nil then
        	retVal = allowedTab.allowed and not allowedTab.isGlpyh
        else
            if allowedTab.allowedIcons == nil then
				--All icons are allowed for this item type
		        if allowedTab.allowed then
					retVal = true
				end
            else
				--Only some of the marker icons are allowed to use for this item type (e.g. glyphs are only allowed to be marked with the deconstruction icon)
		        if allowedTab.allowed and allowedTab.allowedIcons[markId] then
					retVal = true
				end
            end
		end
    end
    return retVal
end

-- Is the item researchable?
local function isItemResearchable(p_rowControl, markId)
	if p_rowControl == nil then return false end
	local retVal = false
	--Check if the item is virtually researchable as the settings is enabled to allow marking of non researchable items as gear
	markId = markId or nil
    if markId ~= nil then
    	retVal = FCOIS.settingsVars.settings.disableResearchCheck[markId]
        if retVal == nil then retVal = false end
    end
    if retVal == false then
	    local bagId, slotIndex = MyGetItemDetails(p_rowControl)
		if bagId == nil or slotIndex == nil then return false end
		local itemType = GetItemType(bagId, slotIndex)
	    if itemType == nil then return false end
		local allowedTab = {}
        allowedTab = FCOIS.checkVars.allowedResearchableItemTypes[itemType]
        if allowedTab == nil then return false end
        if markId == nil then
        	retVal = allowedTab.allowed and not allowedTab.isGlpyh
        else
            if allowedTab.allowedIcons == nil then
				--All icons are allowed for this item type
		        if allowedTab.allowed then
					retVal = true
				end
            else
				--Only some of the marker icons are allowed to use for this item type (e.g. glyphs are only allowed to be marked with the deconstruction icon)
		        if allowedTab.allowed and allowedTab.allowedIcons[markId] then
					retVal = true
				end
            end
		end
	end
   	return retVal
end

-- Is the item an ornate one?
local function isItemOrnate(bagId, slotIndex)
    local itemTrait = GetItemTrait(bagId, slotIndex)
	if FCOIS.checkVars.allowedOrnateItemTraits[itemTrait] then
    	return true
	end
   	return false
end

-- Is the item an intricate one?
local function isItemIntricate(bagId, slotIndex)
    local itemTrait = GetItemTrait(bagId, slotIndex)
    if FCOIS.checkVars.allowedIntricateItemTraits[itemTrait] then
        return true
    end
    return false
end

--Check if withdraw from guild bank is allowed, or block deposit of items
local function checkIfGuildBankWithdrawAllowed(currentGuildBank)
	--if DoesPlayerHaveGuildPermission(currentGuildBank, GUILD_PERMISSION_BANK_DEPOSIT) and DoesPlayerHaveGuildPermission(currentGuildBank, GUILD_PERMISSION_BANK_WITHDRAW) then
	local retVal = DoesPlayerHaveGuildPermission(currentGuildBank, GUILD_PERMISSION_BANK_WITHDRAW)
--d("[FCOIS] checkIfGuildBankWithdrawAllowed: " .. tostring(retVal))
	return retVal
end

--Get the FCOItemSaver control
local function GetItemSaverControl(parent, controlId, useParentFallback)
    --debugMessage("[GetItemSaverControl] Parent: " .. parent:GetName() .. ", ControlId: " .. tostring(controlId), true)
	local retControl = parent:GetNamedChild(FCOIS.addonVars.gAddonName .. tostring(controlId))

	--Use the parent control as a fallback?
	if useParentFallback == true then
	    --e.g. Inside enchanting the parent control is the correct one already
	    if (retControl == nil) then
	    	retControl = parent
	    end
    end
    return retControl
end

--The function to update the inventories and lists after an item was un/marked
local function FilterBasics(onlyPlayer)
	--Check if we are in the player inventory
	if not FCOIS.ZOControlVars.BACKPACK:IsHidden() then
		--we are in the player inventory (or in the banks at the deposit inventories, or at mail sending, or trading)
		onlyPlayer = true
	end

	debugMessage("[FilterBasics] onlyPlayer: " .. tostring(onlyPlayer), true)

	--Only update the lists if not currently already updating
	if (FCOIS.preventerVars.gFilteringBasics == false) then
		FCOIS.preventerVars.gFilteringBasics = true
        if (not FCOIS.ZOControlVars.QUICKSLOT_LIST:IsHidden() and onlyPlayer == false) then
        	UpdateQuickSlots()
        elseif (not FCOIS.ZOControlVars.REPAIR_LIST:IsHidden() and onlyPlayer == false) then
        	UpdateRepairList()
        elseif (onlyPlayer == true) then
			UpdateInventories()
	    else
			UpdateInventories()
			UpdateCraftingInventory()
	    end
		FCOIS.preventerVars.gFilteringBasics = false
    end
end

-- Check the weapon type
local function checkWeaponType(p_weaponType, p_check)
	if (p_check == nil or p_check == '') then
		p_check = '1hd'
	end
	if (p_weaponType ~= nil) then
		if FCOIS.checkVars.weaponTypeCheckTable[p_check] and FCOIS.checkVars.weaponTypeCheckTable[p_check][p_weaponType] then
			return true
		end
	end
    return false
end

--Check if the current equipment slot is the offhand or backup offhand slot
local function checkWeaponOffHand(controlName, weaponTypeName, doCheckOffHand, doCheckBackupOffHand, echo)
    if echo then
		debugMessage("[CheckWeaponOffHand] ControlName: " .. controlName ..", WeaponTypeName: " .. weaponTypeName, true)
    end
	local weaponControl
    local weaponType

	--Check the offhand?
    if doCheckOffHand == true then
		if (controlName == 'ZO_CharacterEquipmentSlotsOffHand') then
			--Check if the weapon in the main slot is a 2hd weapon/staff
			weaponControl = WINDOW_MANAGER:GetControlByName("ZO_CharacterEquipmentSlotsMainHand", "")
			if (weaponControl ~= nil) then
				weaponType = GetItemWeaponType(weaponControl.bagId, weaponControl.slotIndex)
				if (weaponType ~= nil) then
					return checkWeaponType(weaponType, weaponTypeName)
				end
			end
		end
	end

	--Check the backup offhand?
    if doCheckBackupOffHand == true then
	    if (controlName == 'ZO_CharacterEquipmentSlotsBackupOff') then
			--Check if the weapon in the backup slot is a 2hd weapon/staff
			weaponControl = WINDOW_MANAGER:GetControlByName("ZO_CharacterEquipmentSlotsBackupMain", "")
			if (weaponControl ~= nil) then
				weaponType = GetItemWeaponType(weaponControl.bagId, weaponControl.slotIndex)
				if (weaponType ~= nil) then
					return checkWeaponType(weaponType, weaponTypeName)
				end
			end
		end
    end

    return false
end

--Check if a two handed weapon is used and hide the marker for the offhand slot, or find empty slots and remove markers
local function checkForEmptyOrTwoHandedWeaponMarker(slot, onlyWeapons)

	--Only check if character window is shown
    if (FCOIS.ZOControlVars.CHARACTER:IsHidden()) then
    	return
    end

	local control
    local offHandControlName
    local twoHandedWeaponOffHand = false
    local offHandControl
    local offHandCheck = false
    local backupOffHandCheck = false

	--Only check a specific character item slot?
    if slot ~= nil and slot ~= -1 and onlyWeapons == false then
        local equipmentName = FCOIS.mappingVars.characterEquipmentSlotNameByIndex[slot]
		--Abort if equipment name was not found
        if equipmentName == nil or equipmentName == '' then return end

		debugMessage("[CheckForEmptyOrTwoHandedWeaponMarker] Slot: " ..tostring(slot) .. ", Name: " .. equipmentName, false)

		offHandControl = WINDOW_MANAGER:GetControlByName(equipmentName, "")
		if offHandControl ~= nil then
			if offHandControl.stackCount > 0 then
                if (slot==4 or slot==5 or slot==20 or slot==21) then
	            	if (slot==4 or slot==5) then offHandCheck = true end
					if (slot==20 or slot==21) then backupOffHandCheck = true end
					offHandControlName = offHandControl:GetName()
					twoHandedWeaponOffHand = checkWeaponOffHand(offHandControlName, "2hdall", offHandCheck, backupOffHandCheck, true)
                end -- if (slot==4 ...
            end -- if offHandControl.stack ...
		    --Do for each icon Id
		    for j=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
       			control = nil
				--OffHand is a 2hd weapon or no item is equipped, so remove the markers
		        if (offHandControl.stackCount <= 0 or twoHandedWeaponOffHand) then
					--control = offHandControl:GetNamedChild(FCOIS.addonVars.gAddonName .. tostring(j))
                    control = GetItemSaverControl(offHandControl, j, false)
		            if control ~= nil then
						control:SetHidden(true)
			        end
		        end
			end -- for j=1 ...
		end -- 	if offHandControl ~= nil

	else
		--Check all equipment items or weapons only

		--For each equipment item
	   	for index, _ in pairs(FCOIS.mappingVars.characterEquipmentSlotNameByIndex) do
			if ( onlyWeapons == false) then
				--Recursively check all the character equipment slots
               	checkForEmptyOrTwoHandedWeaponMarker(index, onlyWeapons)
 			else
				--Only check the weapons
				if (index==4 or index==5 or index==20 or index==21) then
					--Recursively check all the character equipment slots
					checkForEmptyOrTwoHandedWeaponMarker(index, false)
				end
            end -- if ( (onlyWeapons....
		end -- for ... in pairs ...
    end

end

--Function that close the context-menu (if for example the user closes the inventory without
--choosing a option on the context-menu)
--This function also closes the gear sets split filter button context-menu
local function hideContextMenu(whichContextMenu)
	if whichContextMenu == nil or whichContextMenu == -1 then
        hideContextMenu(FCOIS.gFilterWhere)
    else
		--Hide the inventory button context-menu
		if FCOIS.contextMenu[whichContextMenu] ~= nil then
			if not FCOIS.contextMenu[whichContextMenu]:IsHidden() then
				FCOIS.contextMenu[whichContextMenu]:SetHidden(true)
			end
		end
		--Hide the gear sets split filter button context-menu
		if FCOIS.contextMenu.GearSetFilter[whichContextMenu] ~= nil then
			if not FCOIS.contextMenu.GearSetFilter[whichContextMenu]:IsHidden() then
				FCOIS.contextMenu.GearSetFilter[whichContextMenu]:SetHidden(true)
			end
		end
		--Hide the RESEARCH & DECONSTRUCTION & IMPORVEMENT button context-menu
		if FCOIS.contextMenu.ResDecImpFilter[whichContextMenu] ~= nil then
			if not FCOIS.contextMenu.ResDecImpFilter[whichContextMenu]:IsHidden() then
				FCOIS.contextMenu.ResDecImpFilter[whichContextMenu]:SetHidden(true)
			end
		end
        --Hide the SELL & SELL IN GUILD STORE & INTRICATE button context-menu
        if FCOIS.contextMenu.SellGuildIntFilter[whichContextMenu] ~= nil then
            if not FCOIS.contextMenu.SellGuildIntFilter[whichContextMenu]:IsHidden() then
                FCOIS.contextMenu.SellGuildIntFilter[whichContextMenu]:SetHidden(true)
            end
        end
    end
end

--Create the tooltip for the marker
local function CreateToolTip(markerControl, markerId, doHide, pUpdateAllEquipmentTooltips, pIsEquipmentSlot)
	local doAbort = false
	doHide = doHide or false
    pUpdateAllEquipmentTooltips = pUpdateAllEquipmentTooltips or false
    pIsEquipmentSlot = pIsEquipmentSlot or false

	if markerControl == nil or markerId == nil then doAbort = true end

--[[
	if not doAbort then
		--debugMessage("[CreateToolTip] MarkerControl: " .. markerControl:GetName() .. ", markerId: " .. tostring(markerId) .. ", doHide: " .. tostring(doHide) .. "EquipmentSlot: " .. tostring(pIsEquipmentSlot), true)
    else
		debugMessage("[CreateToolTip] Aborting!", true)
    end
]]

    --If the character tooltips are disabled - Abort here
    if pIsEquipmentSlot and not FCOIS.settingsVars.settings.showIconTooltipAtCharacter then doAbort = true end

	--Set a tooltip?
	if not doAbort and FCOIS.settingsVars.settings.showMarkerTooltip[markerId] == true and markerControl ~= nil then
		local tooltipText = ""
        local tooltipGearText = ""
        local finalTooltipText = ""
		local markedCounter = 0
        local markedGear = 0
        local equipmentMarkerControlName

		--Are we adding a tooltip to an equipment slot?
        if FCOIS.settingsVars.settings.showIconTooltipAtCharacter and pUpdateAllEquipmentTooltips then
	        --Get current controls name
			equipmentMarkerControlName = markerControl:GetName()
            --to strip N characters from the end, you can use negative end index:
            --equipmentMarkerControlName = equipmentMarkerControlName:sub(1, -1-FCOIS.gFCOMaxDigitsForIcons)
			--New replacement method, faster by pattern
            equipmentMarkerControlName = equipmentMarkerControlName:gsub("%d+$", "")
		end

        --Check if the item is marked with several icons
        local markedIcons = {}
        local bagId, slotIndex = MyGetItemDetails(markerControl:GetParent())
		if bagId ~= nil or slotIndex ~= nil then
	        local _, markedIcons = FCOIsMarked(GetItemInstanceId(bagId, slotIndex), -1)
	        if markedIcons then

		        for iconId, iconIsMarked in pairs(markedIcons) do
		        	if iconIsMarked and FCOIS.settingsVars.settings.showMarkerTooltip[iconId] then
		            	markedCounter = markedCounter + 1
						--Tooltip for any gear set?
				        if (FCOIS.mappingVars.iconIsGear[iconId]) then
			                markedGear = markedGear + 1
							if tooltipGearText ~= "" then tooltipGearText = tooltipGearText .. "\n" end
                            local gearName = FCOIS.settingsVars.settings.icon[iconId].name
							--Mark the disabled gear sets gray in the tooltip
                            if not FCOIS.settingsVars.settings.isIconEnabled[iconId] then
                            	gearName = "|c404040" .. gearName .. "|r"
                            end
		                	tooltipGearText = tooltipGearText .. gearName
		                else
							--No gear
							if tooltipText ~= "" then tooltipText = tooltipText .. "\n" end
							local iconName = FCOIS.localizationVars.fcois_loc["options_icon" .. tostring(iconId) .. "_tooltip_text"]
							--Mark the disabled icons gray in the tooltip
                            if not FCOIS.settingsVars.settings.isIconEnabled[iconId] then
                            	iconName = "|c404040" .. iconName .. "|r"
                            end
		                	tooltipText = tooltipText .. iconName
		                end

		                --As only one marker was changed in the equipped items all markes for this item must be updated too (except the currently used marker ID)
						if FCOIS.settingsVars.settings.showIconTooltipAtCharacter and iconId ~= markerId and pUpdateAllEquipmentTooltips then
		                    --Replace current control name ending by the current iconId
		                    local equipmentMarkerControlNewName = equipmentMarkerControlName .. tostring(iconId)
							--get the control by it's name
		                    local equipmentMarkerControl = WINDOW_MANAGER:GetControlByName(equipmentMarkerControlNewName, "")
		                    if equipmentMarkerControl ~= nil then
					            CreateToolTip(equipmentMarkerControl, iconId, doHide, false)
		                    end
		                end
		            end
		        end

				--Build the final tooltip text
				if markedCounter == 1 then
			        --Build the tooltip for only 1 marked icon
					--Tooltip for any gear set?
			        if (FCOIS.mappingVars.iconIsGear[markerId]) then
				        finalTooltipText = FCOIS.preChatVars.preChatTextGreen .. " " .. FCOIS.localizationVars.fcois_loc["options_icon_gear_tooltip_text"] .. FCOIS.settingsVars.settings.icon[markerId].name
			        else
				        finalTooltipText = FCOIS.preChatVars.preChatTextGreen .. " " .. FCOIS.localizationVars.fcois_loc["options_icon" .. tostring(markerId) .. "_tooltip_text"]
			        end
		        else
		           	finalTooltipText = FCOIS.preChatVars.preChatTextGreen .. "\n"
					--Build the tooltip text for several marked icons
			        if     tooltipGearText ~= "" and tooltipText ~= "" then
		            	finalTooltipText = finalTooltipText .. tooltipText .. "\n\n"
						if markedGear > 1 then
			            	finalTooltipText = finalTooltipText .. FCOIS.localizationVars.fcois_loc["options_icon_gears_tooltip_text"] .. "\n"
		                else
			            	finalTooltipText = finalTooltipText .. FCOIS.localizationVars.fcois_loc["options_icon_gear_tooltip_text"]
		                end
		            	finalTooltipText = finalTooltipText .. tooltipGearText
			        elseif tooltipGearText ~= "" and tooltipText == "" then
						if markedGear > 1 then
			            	finalTooltipText = finalTooltipText .. FCOIS.localizationVars.fcois_loc["options_icon_gears_tooltip_text"] .. "\n"
		                else
			            	finalTooltipText = finalTooltipText .. FCOIS.localizationVars.fcois_loc["options_icon_gear_tooltip_text"]
		                end
		            	finalTooltipText = finalTooltipText .. tooltipGearText
			        elseif tooltipGearText == "" and tooltipText ~= "" then
		            	finalTooltipText = finalTooltipText .. tooltipText
			        end
		        end

				if finalTooltipText ~= nil then
		        	markerControl:SetMouseEnabled(true)
		   			markerControl:SetDrawTier(DT_HIGH)
                    markerControl.tooltipText = finalTooltipText
                    if markerControl:GetHandler("OnMouseEnter") == nil then
						markerControl:SetHandler("OnMouseEnter", function(self)
							if FCOIS.settingsVars.settings.showMarkerTooltip[markerId] == true then
								ZO_Tooltips_ShowTextTooltip(self, BOTTOM, self.tooltipText)
							else
								ZO_Tooltips_HideTextTooltip()
							end
						end)
					end
                    if markerControl:GetHandler("OnMouseExit") == nil then
						markerControl:SetHandler("OnMouseExit", function(self)
							ZO_Tooltips_HideTextTooltip()
						end)
                    end
				end

			else
            	doAbort = true
	        end
		else
			doAbort = true
        end
	else
    	doAbort = true
	end

    --Abort?
    if doAbort then
       	markerControl:SetMouseEnabled(false)
		markerControl:SetDrawTier(DT_MEDIUM)
		ZO_Tooltips_HideTextTooltip()
    end
end

--Create the marker controls, holding the icon textures
local function CreateMarkerControl(parent, controlId, pWidth, pHeight, pTexture, pIsEquipmentSlot, pCreateControlIfNotThere, pUpdateAllEquipmentTooltips, pArmorTypeIcon)
    --No parent? Abort here
	if parent == nil then return nil end
	pArmorTypeIcon = pArmorTypeIcon or false

    local InventoryGridViewActivated = false

	--Preset the variable for control creation with false, if it is not given
    pCreateControlIfNotThere	= pCreateControlIfNotThere or false
	pUpdateAllEquipmentTooltips	= pUpdateAllEquipmentTooltips or false

	if(parent:GetOwningWindow() ~= FCOIS.ZOControlVars.QUICKSLOT_CIRCLE) then
        if pIsEquipmentSlot == nil then pIsEquipmentSlot = false end

    	--Does the FCOItemSaver marker control exist already?
		local control = GetItemSaverControl(parent, controlId, false)
        local doHide

		--Marker control for a disabled icon? Hide the icon then
	    if not FCOIS.settingsVars.settings.isIconEnabled[controlId] then
        	doHide = true
		else
			doHide = not checkIfItemIsProtected(controlId, MyGetItemInstanceId(parent))
        end
        if doHide == nil then doHide = false end

		--Remove the sell icon and price if Inventory gridview is active
		if(parent:GetWidth() - parent:GetHeight() < 5) then
			if(parent:GetNamedChild("SellPrice")) then
				parent:GetNamedChild("SellPrice"):SetHidden(true)
			end
			InventoryGridViewActivated = true
		end

		--It does not exist yet, so create it now
		if(control == parent or not control) then
			--Abort here if control should be hiden and is not created yet
            if doHide == true and pCreateControlIfNotThere == false then
                ZO_Tooltips_HideTextTooltip()
            	return
            end
            --If not aborted: Create the marker control now
            control = WINDOW_MANAGER:CreateControl(parent:GetName() .. FCOIS.addonVars.gAddonName .. tostring(controlId), parent, CT_TEXTURE)
		end
		--Control did already exist or was created now
	    if control ~= nil then
			--Hide or show the control now
			control:SetHidden(doHide)

			if not doHide then
	        	control:SetDimensions(pWidth, pHeight)
				control:SetTexture(pTexture)
				control:SetColor(FCOIS.settingsVars.settings.icon[controlId].color.r, FCOIS.settingsVars.settings.icon[controlId].color.g, FCOIS.settingsVars.settings.icon[controlId].color.b, FCOIS.settingsVars.settings.icon[controlId].color.a)
				--Marker was created/updated for the character equipment slots?
				if pIsEquipmentSlot == true then
					control:ClearAnchors()
					--Move the marker controls of equipment slots according to FCOIS.settingsVars.settings
					--control:SetAnchor(BOTTOMLEFT, parent, BOTTOMLEFT, -6, 5)
                    control:SetAnchor(BOTTOMLEFT, parent, BOTTOMLEFT, FCOIS.settingsVars.settings.iconPositionCharacter.x, FCOIS.settingsVars.settings.iconPositionCharacter.y)
					control:SetDrawTier(DT_HIGH)
				else
					if InventoryGridViewActivated == true then
						control:SetDrawTier(DT_HIGH)
						control:ClearAnchors()
						control:SetAnchor(CENTER, parent, BOTTOMLEFT, 12, -12)
					else
                        control:SetDrawTier(DT_HIGH)
                    	control:ClearAnchors()
                        --Get the currently active filter panel ID and map the appropriate inventory for the icon X axis offset
						local iconOffset = FCOIS.mappingVars.filterPanelIdToIconOffset[FCOIS.gFilterWhere] or FCOIS.settingsVars.settings.iconPosition
                        control:SetAnchor(LEFT, parent, LEFT, iconOffset.x, iconOffset.y)
					end
				end

            end  -- if not doHide then
			--Set the tooltip if wished
            CreateToolTip(control, controlId, doHide, pUpdateAllEquipmentTooltips, pIsEquipmentSlot)

			return control
	    else
	    	return nil
	    end
    else
    	return nil
    end
end

--Function to get the armor type of an equipped item
local function GetArmorType(equipmentSlotControl)
	if equipmentSlotControl == nil then return false end
    local bagId
    local slotIndex
    bagId, slotIndex = MyGetItemDetails(equipmentSlotControl)
    local armorType = GetItemArmorType(bagId, slotIndex)
--d("[GetArmorType] bag: " .. bagId .. ", slot: " .. slotIndex .. " --- armorType: " .. tostring(armorType))
   	return armorType
end

--Function to add an icon for the equipped armor type (light, medium, heavy) to an equipment slot control
local function AddArmorTypeIconToEquipmentSlot(equipmentSlotControl, armorType)
	if equipmentSlotControl == nil then return false end
	--Check if the equipment slot control is an armor control
   	if not FCOIS.mappingVars.characterEquipmentArmorSlots[equipmentSlotControl:GetName()] then return false end
	if armorType == nil or armorType == ARMORTYPE_NONE then return false end
    debugMessage("[AddArmorTypeIconToEquipmentSlot] EquipmentSlot: " .. equipmentSlotControl:GetName() .. ", armorType: " .. tostring(armorType), true)

	local armorTypeLabel = WINDOW_MANAGER:GetControlByName("FCOIS_" .. equipmentSlotControl:GetName() .. "_ArmorTypeLabel")
    if FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter then
		if armorTypeLabel == nil then
			armorTypeLabel = WINDOW_MANAGER:CreateControl("FCOIS_" .. equipmentSlotControl:GetName() .. "_ArmorTypeLabel", equipmentSlotControl, CT_LABEL)
		end
		if armorTypeLabel ~= nil then
			armorTypeLabel:SetFont("ZoFontAlert")
			armorTypeLabel:SetScale(0.6)
			armorTypeLabel:SetDrawLayer(DT_HIGH)
			armorTypeLabel:SetDrawTier(DT_HIGH)
			armorTypeLabel:SetAnchor(TOPRIGHT, equipmentSlotControl, TOPRIGHT, FCOIS.settingsVars.settings.armorTypeIconAtCharacterX, FCOIS.settingsVars.settings.armorTypeIconAtCharacterY)
			armorTypeLabel:SetDimensions(20,20)
        	armorTypeLabel:SetHidden(false)
		end
	else
    	--Hide the label if it exists
		if armorTypeLabel ~= nil then
        	armorTypeLabel:SetHidden(true)
		end
	end
	--Count the different equipped armor parts and show the icon at the player doll, if enabled in the settings
    if 		armorType == ARMORTYPE_LIGHT then
        FCOIS.countVars.countLightArmor = FCOIS.countVars.countLightArmor + 1
        if FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter and armorTypeLabel ~= nil then
			armorTypeLabel:SetText(FCOIS.localizationVars.fcois_loc["options_armor_type_icon_light_short"])
			armorTypeLabel:SetColor(FCOIS.settingsVars.settings.armorTypeIconAtCharacterLightColor.r, FCOIS.settingsVars.settings.armorTypeIconAtCharacterLightColor.g, FCOIS.settingsVars.settings.armorTypeIconAtCharacterLightColor.b, FCOIS.settingsVars.settings.armorTypeIconAtCharacterLightColor.a)
        end
	elseif	armorType == ARMORTYPE_MEDIUM then
        FCOIS.countVars.countMediumArmor = FCOIS.countVars.countMediumArmor + 1
        if FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter and armorTypeLabel ~= nil then
			armorTypeLabel:SetText(FCOIS.localizationVars.fcois_loc["options_armor_type_icon_medium_short"])
			armorTypeLabel:SetColor(FCOIS.settingsVars.settings.armorTypeIconAtCharacterMediumColor.r, FCOIS.settingsVars.settings.armorTypeIconAtCharacterMediumColor.g, FCOIS.settingsVars.settings.armorTypeIconAtCharacterMediumColor.b, FCOIS.settingsVars.settings.armorTypeIconAtCharacterMediumColor.a)
        end
	elseif	armorType == ARMORTYPE_HEAVY then
        FCOIS.countVars.countHeavyArmor = FCOIS.countVars.countHeavyArmor + 1
        if FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter and armorTypeLabel ~= nil then
			armorTypeLabel:SetText(FCOIS.localizationVars.fcois_loc["options_armor_type_icon_heavy_short"])
			armorTypeLabel:SetColor(FCOIS.settingsVars.settings.armorTypeIconAtCharacterHeavyColor.r, FCOIS.settingsVars.settings.armorTypeIconAtCharacterHeavyColor.g, FCOIS.settingsVars.settings.armorTypeIconAtCharacterHeavyColor.b, FCOIS.settingsVars.settings.armorTypeIconAtCharacterHeavyColor.a)
        end
    end
end

--Update the equipment header text with the information about the amount of equipped armor types
local function updateEquipmentHeaderCountText()
	if not FCOIS.settingsVars.settings.showArmorTypeHeaderTextAtCharacter then
		ZO_CharacterApparelSectionText:SetText(GetString(SI_CHARACTER_EQUIP_SECTION_APPAREL))
    	return
    end
    if ZO_CharacterApparelSectionText ~= nil and
       FCOIS.countVars.countLightArmor ~= nil and FCOIS.countVars.countMediumArmor ~= nil and FCOIS.countVars.countHeavyArmor ~= nil then
		ZO_CharacterApparelSectionText:SetText(GetString(SI_CHARACTER_EQUIP_SECTION_APPAREL) .. " (" .. FCOIS.localizationVars.fcois_loc["options_armor_type_icon_light_short"] .. ": " .. FCOIS.countVars.countLightArmor .. ", " .. FCOIS.localizationVars.fcois_loc["options_armor_type_icon_medium_short"] .. ": " .. FCOIS.countVars.countMediumArmor .. ", " .. FCOIS.localizationVars.fcois_loc["options_armor_type_icon_heavy_short"] .. ": " .. FCOIS.countVars.countHeavyArmor .. ")")
    end
end

--function to count and update the equipped aromor parts
local function countAndUpdateEquippedArmorTypes()
	--Reset the armor type counters
	FCOIS.countVars.countLightArmor		= 0
	FCOIS.countVars.countMediumArmor	= 0
	FCOIS.countVars.countHeavyArmor		= 0

	--Check all equipment controls
	local equipmentSlotControl
	for _, equipmentSlotName in pairs(FCOIS.mappingVars.characterEquipmentSlotNameByIndex) do
		--Get the control of the equipment slot
		equipmentSlotControl = WINDOW_MANAGER:GetControlByName(equipmentSlotName, "")
		if equipmentSlotControl ~= nil then
			--Show the armor type icons at the player doll?
	        AddArmorTypeIconToEquipmentSlot(equipmentSlotControl, GetArmorType(equipmentSlotControl))
		end
	end

	--Update the equipment header text and show the amount of armor types equipped
	updateEquipmentHeaderCountText()
end

--Remove the armor type marker from character doll
local function removeArmorTypeMarker(bagId, slotId)
	if not FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter then return false end
	if bagId == nil or slotId == nil then return false end
    local equipmentSlotControlName = FCOIS.mappingVars.characterEquipmentSlotNameByIndex[slotId]
    if equipmentSlotControlName == nil then return false end
    local equipmentSlotControl = WINDOW_MANAGER:GetControlByName(equipmentSlotControlName, "")
    if equipmentSlotControl == nil then return false end
	--Item is unequipped?
    if equipmentSlotControl.stackCount == 0 then
    	--Hide the text control showing the armor type for this equipment slot
		local armorTypeLabel = WINDOW_MANAGER:GetControlByName("FCOIS_" .. equipmentSlotControl:GetName() .. "_ArmorTypeLabel")
        if armorTypeLabel == nil then return true end
        armorTypeLabel:SetHidden(true)
    end
end

--The function to refresh the equipped items
local function RefreshEquipmentControl(equipmentControl, doCreateMarkerControl, p_markId)
	--Preset the value for "Create control if not existing yet" with false
    doCreateMarkerControl = doCreateMarkerControl or false

	--is the equipment control already given?
    if equipmentControl ~= nil then
		--Check all marker ids?
        if p_markId == nil or p_markId == 0 then
			debugMessage("[RefreshEquipmentControl] Control: " .. equipmentControl:GetName() .. ", Create: " .. tostring(doCreateMarkerControl) .. ", MarkId: ALL", true)
	    	--Add/Update the markers for the filter icons at the equipment slot
			for j=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				CreateMarkerControl(equipmentControl, j, FCOIS.equipmentVars.gEquipmentIconWidth, FCOIS.equipmentVars.gEquipmentIconHeight, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[j].texture], true, doCreateMarkerControl, false)
			end
		--Only check a specific marker id
        else
			debugMessage("[RefreshEquipmentControl] Control: " .. equipmentControl:GetName() .. ", Create: " .. tostring(doCreateMarkerControl) .. ", MarkId: " .. tostring(p_markId), true)
	    	--Add/Update the marker p_markId for the filter icons at the equipment slot
			CreateMarkerControl(equipmentControl, p_markId, FCOIS.equipmentVars.gEquipmentIconWidth, FCOIS.equipmentVars.gEquipmentIconHeight, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[p_markId].texture], true, doCreateMarkerControl, true)
        end

    else
		debugMessage("[RefreshEquipmentControl] ALL CONTROLS! Create: " .. tostring(doCreateMarkerControl) .. ", MarkId: " .. tostring(p_markId), true)
		--Reset the armor type counters
        FCOIS.countVars.countLightArmor		= 0
        FCOIS.countVars.countMediumArmor	= 0
        FCOIS.countVars.countHeavyArmor		= 0

		--Check all equipment controls
        local equipmentSlotControl
	    for _, equipmentSlotName in pairs(FCOIS.mappingVars.characterEquipmentSlotNameByIndex) do
			--Get the control of the equipment slot
	    	equipmentSlotControl = WINDOW_MANAGER:GetControlByName(equipmentSlotName, "")
	        if equipmentSlotControl ~= nil then
				--Refresh each equipped item's marker icons
            	RefreshEquipmentControl(equipmentSlotControl, doCreateMarkerControl, p_markId)
				--Show the armor type icons at the player doll?
		        AddArmorTypeIconToEquipmentSlot(equipmentSlotControl, GetArmorType(equipmentSlotControl))
	        end
		end

		--Update the equipment header text and show the amount of armor types equipped
        updateEquipmentHeaderCountText()
    end
end

--Update one specific equipment slot by help of the slotIndex
local function updateEquipmentSlotMarker(slotIndex, delay)
	--Only execute if character window is shown
	if (not FCOIS.ZOControlVars.CHARACTER:IsHidden() and slotIndex ~= nil) then
		delay = delay or 0
        if delay > 0 then
			zo_callLater(function()
				updateEquipmentSlotMarker(slotIndex, 0)
	        end, delay)
        else
			--Get the equipment control by help of the slotIndex
			local equipSlotControlName = FCOIS.mappingVars.characterEquipmentSlotNameByIndex[slotIndex]
			local equipSlotControl
			if equipSlotControlName ~= nil and equipSlotControlName ~= "" then
	debugMessage("[updateEquipmentSlotMarker] control name="..equipSlotControlName..", index: " .. slotIndex, true)
				equipSlotControl = WINDOW_MANAGER:GetControlByName(equipSlotControlName, "")
			end
			if equipSlotControl ~= nil then
				--Add or refresh the equipped items, create marker control if not already there
				RefreshEquipmentControl(equipSlotControl, true)
				--Check if the equipment slot control is an armor control
			   	if FCOIS.mappingVars.characterEquipmentArmorSlots[equipSlotControl:GetName()] then
		            --Count the now equipped armor types and update the text at the character window armor header text
					countAndUpdateEquippedArmorTypes()
				end
				--Are we chaning equipped weapons?
				--MainHand and OffHand
				if   slotIndex == 4  or slotIndex == 5
				--BackupHand and BackupOffHand
				  or slotIndex == 20 or slotIndex == 21 then
					--Check if the offhand weapons are 2hd weapons and remove the markers then
					zo_callLater(function() checkForEmptyOrTwoHandedWeaponMarker(Index, true) end, 100)
				end
			end
    	end
    end
end

--The callback function for the right click/context menus to mark an item -> Called from function "AddMark"
local function MarkMe(rowControl, markId, updateNow, doUnmark, refreshPopupDialog)
    if FCOIS.gFilterWhere == nil then return end
	--Set the last used filter Id
    FCOIS.lastVars.gLastFilterId[FCOIS.gFilterWhere] = FCOIS.mappingVars.iconToFilter[markId]

	local itemId = MyGetItemInstanceId(rowControl, true)
    if markId ~= nil and itemId ~= nil then
		if doUnmark == true then
			--Remove the marker now
			FCOIS.markedItems[markId][itemId] = nil
	    else
			--Unmark all other markers before?
			--Sell icon
            if ( (markId == 5 and FCOIS.settingsVars.settings.autoDeMarkSell)
			--Sell in guild store icon
            or (markId == 11 and FCOIS.settingsVars.settings.autoDeMarkSellInGuildStore) ) then
				--Remove all markers now
            	FCOMarkItem(nil, nil, -1, false, false, itemId)
            end
			--Set the marker now
			FCOIS.markedItems[markId][itemId] = true
	    end

		GetItemSaverControl(rowControl, markId, true):SetHidden(doUnmark)
	    if (updateNow == true) then
			--Inventories or character equipment?
			if rowControl:GetParent() == FCOIS.ZOControlVars.CHARACTER then
				RefreshEquipmentControl(rowControl, not doUnmark, markId)
			elseif rowControl:GetParent():GetParent() == FCOIS.ZOControlVars.QUICKSLOT_LIST then
	        	FilterBasics(false)
	        else
	        	FilterBasics(false)
	        end
		else
			--Refresh the ZO_ListDialog1 popup now after right click/context menu was used?
			if refreshPopupDialog then
				--To remove the active selected row again (was activated during mouse right click/context menu
				RefreshListDialog()
	            ClearCursor()
--Button 1 must not ne disabled if we are not inside the research popup (but inside the wayshrine port popup, or weapon enchant popup, or ...)
--This got probably to do with the "SetKeyboardEnabled(true) function!
                --is the research dialog shown?
				if not ZO_ListDialog1:IsHidden() then
					local isRepairItemDialog = ZO_Dialogs_IsShowing(FCOIS.ZOControlVars.RepairItemDialogName)

                	local disableResearchNow = false
                    --get the marked icons of the item
			        local _, markedIcons = FCOIsMarked(GetItemInstanceId(rowControl.dataEntry.data.bag, rowControl.dataEntry.data.index), -1)
			        if markedIcons then
				        for iconId, iconIsMarked in pairs(markedIcons) do
                            --Is the current item marked?
                        	if iconIsMarked then
								--Research (or at least NO repair item dialog!)
								if not isRepairItemDialog then
									--Was the marked icon the research icon?
	                            	if iconId == 3 then
						                --if researching of marked items is not allowed
										if not FCOIS.settingsVars.settings.allowResearch then
	                                    	disableResearchNow = true
	                                   		break
	                                    end
	                                else
		                                disableResearchNow = true
	                                    break
	                                end
								else
								--Repair item dialog
									--if usage of marked repair kits is not allowed
									if FCOIS.settingsVars.settings.blockMarkedRepairKits then
										disableResearchNow = true
										break
	                                end
								end
                            end
						end
                    end
					--Is the research not allowed?
					if disableResearchNow then
                    	ClearCursor()
                        FCOIS.ZOControlVars.LIST_DIALOG.selectedControl = nil
                        rowControl:GetNamedChild("Selected"):SetHidden(true)
						--Disable the "Research" button in the popup
						WINDOW_MANAGER:GetControlByName("ZO_ListDialog1Button1", ""):SetEnabled(false)
                    end
                end
			end
	    end
    end
end

--The callback function for the right click/context menus to mark all equipment items with one click
local function MarkAllEquipment(rowControl, markId, updateNow, doHide)
    if FCOIS.gFilterWhere == nil then return end
	--Set the last used filter Id
    FCOIS.lastVars.gLastFilterId[FCOIS.gFilterWhere] = FCOIS.mappingVars.iconToFilter[markId]

	local equipmentControl
    local itemId
    local equipmentSlotName
    local dontChange
	local fcoMarkerControl

	--Get each equipped item
	for i=1, FCOIS.ZOControlVars.CHARACTER:GetNumChildren() do
        dontChange = false
    	equipmentSlotName = FCOIS.ZOControlVars.CHARACTER:GetChild(i):GetName()

   		debugMessage("[MarkAllEquipment] MarkId: " .. tostring(markId) .. ", EquipmentSlot: "..equipmentSlotName..", no_auto_mark: " .. tostring(FCOIS.checkVars.equipmentSlotsNames["no_auto_mark"][equipmentSlotName]))
		--Is the current equipment slot a changeable one?
		if(string.find(equipmentSlotName, "ZO_CharacterEquipmentSlots")) then
			--Only mark weapons too if enabled in settings
			if ( (FCOIS.settingsVars.settings.autoMarkAllWeapon == false and FCOIS.checkVars.allowedCharacterEquipmentWeaponControlNames[equipmentSlotName] == true)
			--Only mark jewelry too if enabled in settings
			  or (FCOIS.settingsVars.settings.autoMarkAllJewelry == false and FCOIS.checkVars.allowedCharacterEquipmentJewelryControlNames[equipmentSlotName] == true) ) then
	            dontChange = true
            end
			--Still need to change the texture and un-/mark item?
            if dontChange == false then
				dontChange = FCOIS.checkVars.equipmentSlotsNames["no_auto_mark"][equipmentSlotName] or false
            end
			--Are we allowed to change the marker?
        	if (dontChange == false) then
				--get the control of the current equipment slot
	            equipmentControl = FCOIS.ZOControlVars.CHARACTER:GetChild(i)
				if (equipmentControl ~= nil) then
					--get the itemid
		            itemId = MyGetItemInstanceId(equipmentControl, true)
                    if markId ~= nil and itemId ~= nil then
						if doHide == true then
							FCOIS.markedItems[markId][itemId] = nil
						else
							FCOIS.markedItems[markId][itemId] = true
						end
						--Update the texture, create it if not there yet
						RefreshEquipmentControl(equipmentControl, true, markId)
                    end
	            end
			end
        end
    end

    --if (updateNow == true) then
	    --FilterBasics(false)
    --end
end

--Clear all current markers of the selected row
local function ClearOrRestoreAllMarkers(rowControl)
--d("FCOIS [ClearOrRestoreAllMarkers]")
	if rowControl == nil then return end
    local bagId, slotIndex = MyGetItemDetails(rowControl)
    if bagId == nil or slotIndex == nil then return false end
    FCOIS.preventerVars.gOverrideInvUpdateAfterMarkItem = false
	if FCOIS.lastMarkedIcons ~= nil and #FCOIS.lastMarkedIcons > 0 then
--d("restore")
		--Restore saved markers?
		local loc_counter = 1
		for iconId, iconIsMarked in pairs(FCOIS.lastMarkedIcons) do
			--Reset all markers
            FCOIS.preventerVars.gOverrideInvUpdateAfterMarkItem = (loc_counter == #FCOIS.lastMarkedIcons)
			FCOMarkItem(bagId, slotIndex, iconId, iconIsMarked, (loc_counter == #FCOIS.lastMarkedIcons), nil)
			loc_counter = loc_counter + 1
		end
		--Reset the last saved marker array
		FCOIS.lastMarkedIcons = nil

    else
--d("clear")

		--Clear all markers of current item
	    local currentMarkedIcons = {}
        local currentMarkedIconsUnchanged = {}
	    FCOIS.lastMarkedIcons = nil
		local itemInstanceId = GetItemInstanceId(bagId, slotIndex)
		--Return false for marked icons, where the icon id is disabled in the settings
		FCOIS.preventerVars.doFalseOverride = true
		local _, currentMarkedIcons = FCOIsMarked(itemInstanceId, -1)
		--Reset to normal return values for marked & en-/disabled icons now
        FCOIS.preventerVars.doFalseOverride = false
		--Build the backup array with normal marked icons now
		local _, currentMarkedIconsUnchanged = FCOIsMarked(itemInstanceId, -1)

		--For each marked icon of the currently improved item:
	    --Set the icons/markers of the previous item again
		if currentMarkedIcons and #currentMarkedIcons > 0 then
	        local loc_counter = 1
            local loc_marked_counter = 0
			--Loop over each of the marker icons and remove them
            for iconId, iconIsMarked in pairs(currentMarkedIcons) do
                --Only go on if item is marked or is the last marker (to update the inventory afterwards)
                if iconIsMarked or (loc_counter == #currentMarkedIcons) then
					if iconIsMarked then
	                	loc_marked_counter = loc_marked_counter + 1
                    end
                    FCOIS.preventerVars.gOverrideInvUpdateAfterMarkItem = (loc_counter == #currentMarkedIcons)
                    --Remove all markers
					FCOMarkItem(bagId, slotIndex, iconId, false, ((loc_counter == #currentMarkedIcons) and loc_marked_counter > 0), nil)
                end
                loc_counter = loc_counter + 1
	        end
			--Only save the last saved markers if any marker was set
			if loc_marked_counter > 0 then
            	FCOIS.lastMarkedIcons = currentMarkedIconsUnchanged
            end
	    end
    end
end

--Show the FCO ItemSaver FCOIS.settingsVars.settings panel
function ShowFCOItemSaverSettings()
	LAM:OpenToPanel(FCOIS.FCOSettingsPanel)
end

--Add the marker icon to the item and add the right click menu entry to the context menu, if allowed
local function AddMark(rowControl, markId, isEquipmentSlot, refreshPopupDialog, useSubMenu)
	useSubMenu = useSubMenu or false
	local parentName = rowControl:GetParent():GetName()
    local controlName = rowControl:GetName()

    if parentName == nil or controlName == nil then return end

	local firstAdd = false
	--Are we adding the first new entry in the context menu?
    if (FCOIS.customMenuVars.customMenuCurrentCounter == 1) then
    	firstAdd = true
    end

	--To prevent spamming only output the debug message once for the first added context menu item
    if firstAdd then
	    debugMessage("[AddMark]: Parent: " .. parentName .. ", Control: " .. controlName .. ", IsEquipmentSlot: " ..tostring(isEquipmentSlot) .. ", useSubMenu: " .. tostring(useSubMenu), true)
    end
    local isEquipmentSlotContextmenu = false

    --For equipment slots: Only go on if markId == 2, 4, 6, 7 or 8 (gear sets 1 to 5)
    --and an item is equipped (rowControl.stackCount ~= 0)
    if (isEquipmentSlot == true) then
    	if (not FCOIS.mappingVars.iconIsGear[markId] and rowControl.stackCount == 0) then
	   		return
        end

	    --For 2handed weapons/staffs: Only go on if the current equipment slot is not
        --the 1st weapon set's backup or the 2nd weapon set's backup
		if checkWeaponOffHand(controlName, "2hdall", true, true, firstAdd) == true then return end
    end

	--===========================================================================================================
	--Check if the right click menu should be updated. Only allowed panels and menus apply
    -- NEW
    -- Check two tables for parent and control names. If current control and parent are not in the relating table
    -- the contextmenu will be enhanced with FCOItemSaver entries
    if (FCOIS.checkVars.notAllowedContextMenuParentControls[parentName] or FCOIS.checkVars.notAllowedContextMenuControls[controlName]) then
		--Not allowed parent or control is given -> Abort here
        if firstAdd then
	        debugMessage("[AddMark]: Not allowed (parent) control -> Aborted!", true)
        end
        return
    end

	-- Equipment gear 1, 2, 3, 4, 5, Research, Improve, Deconstruct or Intricate
	if (FCOIS.mappingVars.iconIsResearchable[markId]) then
		-- Check if item is researchable (as only researchable items can work as equipment too)
        if not isItemResearchable(rowControl, markId) then
        	return false
        end
    end

	--Is the setting for "mark all equipped items at once" activated and we are handling an equipment slot
       --and are we trying to add options for gear 1, 2, 3, 4, 5?
	if (FCOIS.settingsVars.settings.autoMarkAllEquipment == true) then
		if ( isEquipmentSlot == true and FCOIS.mappingVars.iconIsGear[markId] ) then
			isEquipmentSlotContextmenu = true
			if (FCOIS.settingsVars.settings.autoMarkAllWeapon == false) then
				if ( FCOIS.checkVars.allowedCharacterEquipmentWeaponControlNames[controlName] ) then
					isEquipmentSlotContextmenu = false
				end
			end
			if (FCOIS.settingsVars.settings.autoMarkAllJewelry == false and isEquipmentSlotContextmenu == true) then
				if ( FCOIS.checkVars.allowedCharacterEquipmentJewelryControlNames[controlName] ) then
					isEquipmentSlotContextmenu = false
				end
			end
		end
	else
		isEquipmentSlotContextmenu = false
	end

	--Update the list / popup dialog list?
    local refreshList
    if refreshPopupDialog == true then
    	refreshList = false
    else
    	refreshList = true
    end

	--Add a divider as first item, between standard ESO and FCOIS context menu entries?
	--Use sub menu entries for the item's context menu?
    FCOIS.contextMenuVars.contextMenuIndex = -1
    if useSubMenu then
		local newSubEntry = {}
	else
		--If the first icon/option is present: Add an information line to the context menu to split the FCOIS options from the rest/standard
    	if firstAdd and FCOIS.settingsVars.settings.showContextMenuDivider then
            local callbackFnc
            local menuItemType
        	if FCOIS.settingsVars.settings.contextMenuDividerShowsSettings then
				callbackFnc = function() ShowFCOItemSaverSettings() end
                menuItemType = MENU_ADD_OPTION_LABEL

			elseif FCOIS.settingsVars.settings.contextMenuDividerClearsMarkers
                   and not isEquipmentSlot and FCOIS.ZOControlVars.LIST_DIALOG:IsHidden() then
	            	callbackFnc = function() ClearOrRestoreAllMarkers(rowControl) end
	                menuItemType = MENU_ADD_OPTION_LABEL

            else
            	callbackFnc = function() end
                menuItemType = MENU_ADD_OPTION_LABEL
            end
        	FCOIS.contextMenuVars.contextMenuIndex = AddCustomMenuItem("     - |c22DD22FCO|r ItemSaver -", function() callbackFnc() end, menuItemType)
        end
    end

	--Add the equipment right click / context menu entries
	if (isEquipmentSlotContextmenu == true) then

		-- Add/Update the right click menu item for character slot now
		if(not checkIfItemIsProtected(markId, MyGetItemInstanceId(rowControl))) then
			if useSubMenu then
				newSubEntry = {
					label = FCOIS.localizationVars.lTextEquipmentMark[markId],
					callback = function() MarkAllEquipment(rowControl, markId, refreshList, false) end,
				}
    		else
        		--AddMenuItem(FCOIS.localizationVars.lTextEquipmentMark[markId], function() MarkAllEquipment(rowControl, markId, refreshList, false) end, MENU_ADD_OPTION_LABEL)
                AddCustomMenuItem(FCOIS.localizationVars.lTextEquipmentMark[markId], function() MarkAllEquipment(rowControl, markId, refreshList, false) end, MENU_ADD_OPTION_LABEL)
            end
		else
			if useSubMenu then
                if FCOIS.settingsVars.settings.useContextMenuCustomMarkedNormalColor then
					newSubEntry = {
						label = FCOIS.localizationVars.lTextEquipmentDemark[markId],
						callback = function() MarkAllEquipment(rowControl, markId, refreshList, true) end,
						myfont = "ZoFontGame",
						normalColor = ZO_ColorDef:New(FCOIS.settingsVars.settings.contextMenuCustomMarkedNormalColor),
					}
                else
					newSubEntry = {
						label = FCOIS.localizationVars.lTextEquipmentDemark[markId],
						callback = function() MarkAllEquipment(rowControl, markId, refreshList, true) end,
					}
                end
    		else
				--AddMenuItem(FCOIS.localizationVars.lTextEquipmentDemark[markId], function() MarkAllEquipment(rowControl, markId, refreshList, true) end, MENU_ADD_OPTION_LABEL)
                if FCOIS.settingsVars.settings.useContextMenuCustomMarkedNormalColor then
					AddCustomMenuItem(FCOIS.localizationVars.lTextEquipmentDemark[markId], function() MarkAllEquipment(rowControl, markId, refreshList, true) end, MENU_ADD_OPTION_LABEL, "ZoFontGame", ZO_ColorDef:New(FCOIS.settingsVars.settings.contextMenuCustomMarkedNormalColor))
				else
					AddCustomMenuItem(FCOIS.localizationVars.lTextEquipmentDemark[markId], function() MarkAllEquipment(rowControl, markId, refreshList, true) end, MENU_ADD_OPTION_LABEL)
                end
			end
		end

	--Add the normal (e.g. inventory) right click / context menu entries
	else

		--AddCustomMenuItem(mytext, myfunction, itemType, myfont, normalColor, highlightColor, itemYPad)

		-- Add/Update the right click menu item now
		if(not checkIfItemIsProtected(markId, MyGetItemInstanceId(rowControl))) then
			if useSubMenu then
				newSubEntry = {
					label = FCOIS.localizationVars.lTextMark[markId],
					callback = function() MarkMe(rowControl, markId, refreshList, false, refreshPopupDialog) end,
				}
    		else
				--AddMenuItem(FCOIS.localizationVars.lTextMark[markId], function() MarkMe(rowControl, markId, refreshList, false, refreshPopupDialog) end, MENU_ADD_OPTION_LABEL)
	            AddCustomMenuItem(FCOIS.localizationVars.lTextMark[markId], function() MarkMe(rowControl, markId, refreshList, false, refreshPopupDialog) end, MENU_ADD_OPTION_LABEL)
			end
		else
			if useSubMenu then
                if FCOIS.settingsVars.settings.useContextMenuCustomMarkedNormalColor then
					newSubEntry = {
						label = FCOIS.localizationVars.lTextDemark[markId],
						callback = function() MarkMe(rowControl, markId, refreshList, true, refreshPopupDialog) end,
						myfont = "ZoFontGame",
						normalColor = ZO_ColorDef:New(FCOIS.settingsVars.settings.contextMenuCustomMarkedNormalColor),
					}
				else
					newSubEntry = {
						label = FCOIS.localizationVars.lTextDemark[markId],
						callback = function() MarkMe(rowControl, markId, refreshList, true, refreshPopupDialog) end,
					}
                end
    		else
				--AddMenuItem(FCOIS.localizationVars.lTextDemark[markId], function() MarkMe(rowControl, markId, refreshList, true, refreshPopupDialog) end, MENU_ADD_OPTION_LABEL)
                if FCOIS.settingsVars.settings.useContextMenuCustomMarkedNormalColor then
					AddCustomMenuItem(FCOIS.localizationVars.lTextDemark[markId], function() MarkMe(rowControl, markId, refreshList, true, refreshPopupDialog) end, MENU_ADD_OPTION_LABEL, "ZoFontGame", ZO_ColorDef:New(FCOIS.settingsVars.settings.contextMenuCustomMarkedNormalColor))
                else
					AddCustomMenuItem(FCOIS.localizationVars.lTextDemark[markId], function() MarkMe(rowControl, markId, refreshList, true, refreshPopupDialog) end, MENU_ADD_OPTION_LABEL)
                end
			end
		end

	end

    if useSubMenu then
        --Show the new added menu entries inside the context menu borders
	    table.insert(FCOIS.customMenuVars.customMenuSubEntries, newSubEntry)
	else
	    ShowMenu(rowControl)

		--Modify the spacer context menu entry so it isn't enabled for the mouse
    	if firstAdd and FCOIS.settingsVars.settings.showContextMenuDivider then
            if FCOIS.contextMenuVars.contextMenuIndex ~= -1 and ZO_Menu.items[FCOIS.contextMenuVars.contextMenuIndex] ~= nil then
				--Overwrite onmouseenter events
                local contextMenuItemControl = ZO_Menu.items[FCOIS.contextMenuVars.contextMenuIndex].item
                if contextMenuItemControl~= nil then
	               	if ( (isEquipmentSlot) or  (not FCOIS.ZOControlVars.LIST_DIALOG:IsHidden())
                      or (not FCOIS.settingsVars.settings.contextMenuDividerShowsSettings and not FCOIS.settingsVars.settings.contextMenuDividerClearsMarkers) ) then
	                    contextMenuItemControl:SetMouseEnabled(false)
	                    --Reenable the mouse for this menu item if the menu closes
	                    SetMenuHiddenCallback(function()
		                    contextMenuItemControl:SetMouseEnabled(true)
	                    end)
					else
						contextMenuItemControl:SetMouseEnabled(true)
                    end
                end
            end
		end
    end
end

--The function to filter an item in your inventories
--return value "true" = Show item
--return value "false" = Hide (filter) item
local function filterItemNow(slotItemInstanceId)
	--Check for each filter the marked items
	local result = true
    local isFilterActivated
	for filterId=1, FCOIS.numVars.gFCONumFilters, 1 do
		--Check if filter is activated for current slot
        isFilterActivated = getSettingsIsFilterOn(filterId)

        --Special treatment for filter type 2 as it handels "gear sets" marked items arrays 2, 4, 6, 7 and 8
		if (filterId == 2) then

			--Gear filter split disabled
            if FCOIS.settingsVars.settings.lastGearFilterIconId[FCOIS.gFilterWhere] == nil or FCOIS.settingsVars.settings.lastGearFilterIconId[FCOIS.gFilterWhere] == -1 or not FCOIS.settingsVars.settings.splitGearSetsFilter then

				--Filter 2 on
		    	if(isFilterActivated == true
	               and (
	                   	       checkIfItemIsProtected(2, slotItemInstanceId)
		               	    or checkIfItemIsProtected(4, slotItemInstanceId)
		               	    or checkIfItemIsProtected(6, slotItemInstanceId)
		               	    or checkIfItemIsProtected(7, slotItemInstanceId)
		               	    or checkIfItemIsProtected(8, slotItemInstanceId)
	                    )
	            ) then
	            	return false
				--Filter 2 "show only marked"
			    elseif(isFilterActivated == -99) then
			    	if (
	                    		checkIfItemIsProtected(2,slotItemInstanceId)
		                	 or checkIfItemIsProtected(4,slotItemInstanceId)
		               	     or checkIfItemIsProtected(6,slotItemInstanceId)
		               	     or checkIfItemIsProtected(7,slotItemInstanceId)
		               	     or checkIfItemIsProtected(8,slotItemInstanceId)
	                ) then
				    	return true
			        else
						result = false
			        end
				--Filter 2 off
	            else
					if (result ~= false) then
						result = true
	                end
				end

            else

				--Gear filter split enabled
                --Last used icon ID at the gear filter split context menu is stored in variable FCOIS.settingsVars.settings.lastGearFilterIconId[panelId]
				--Filter 2 on
		    	if( isFilterActivated == true and checkIfItemIsProtected(FCOIS.settingsVars.settings.lastGearFilterIconId[FCOIS.gFilterWhere], slotItemInstanceId) ) then
	            	return false
				--Filter 2 "show only marked"
			    elseif( isFilterActivated == -99 ) then
					return checkIfItemIsProtected(FCOIS.settingsVars.settings.lastGearFilterIconId[FCOIS.gFilterWhere], slotItemInstanceId)
				--Filter 2 off
	            else
					if (result ~= false) then
						result = true
	                end
				end

            end

        --Special treatment for filter type 3, as the marked items are 3, 9 and 10
		elseif (filterId == 3) then

			--Research, Deconstruction, Improvement filter split disabled
            if FCOIS.settingsVars.settings.lastResDecImpFilterIconId[FCOIS.gFilterWhere] == nil or FCOIS.settingsVars.settings.lastResDecImpFilterIconId[FCOIS.gFilterWhere] == -1 or not FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then

				--Filter 3 on
		    	if(isFilterActivated == true
	               and (
                   	       checkIfItemIsProtected(3, slotItemInstanceId)
	               	    or checkIfItemIsProtected(9, slotItemInstanceId)
	               	    or checkIfItemIsProtected(10, slotItemInstanceId)
	               )
	            ) then
	            	return false
				--Filter 3 "show only marked"
			    elseif(isFilterActivated == -99) then
			    	if (
                           checkIfItemIsProtected(3, slotItemInstanceId)
	               	    or checkIfItemIsProtected(9, slotItemInstanceId)
	               	    or checkIfItemIsProtected(10, slotItemInstanceId)
	                ) then
				    	return true
			        else
						result = false
			        end
				--Filter 3 off
	            else
					if (result ~= false) then
						result = true
	                end
				end

            else

				--Research, Deconstruction, Improvement filter split ensabled
                --Last used icon ID at the research/deconstruction/improvement filter split context menu is stored in variable FCOIS.settingsVars.settings.lastResDecImpFilterIconId[panelId]
				--Filter 3 on
		    	if( isFilterActivated == true
                	and (checkIfItemIsProtected(FCOIS.settingsVars.settings.lastResDecImpFilterIconId[FCOIS.gFilterWhere], slotItemInstanceId) and FCOIS.settingsVars.settings.isIconEnabled[FCOIS.settingsVars.settings.lastResDecImpFilterIconId[FCOIS.gFilterWhere]]) ) then
	            	return false
				--Filter 3 "show only marked"
			    elseif( isFilterActivated == -99 ) then
					return checkIfItemIsProtected(FCOIS.settingsVars.settings.lastResDecImpFilterIconId[FCOIS.gFilterWhere], slotItemInstanceId)
				--Filter 3 off
	            else
					if (result ~= false) then
						result = true
	                end
				end

            end

        --Special treatment for filter type 4, as the marked items are 5, 11 and 12
        elseif (filterId == 4) then
            -- Split Sell, Sell in guild store & Intricate not activated in settings
            if FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[FCOIS.gFilterWhere] == nil or FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[FCOIS.gFilterWhere] == -1 or not FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then

                --Filter 4 on
                if(isFilterActivated == true
                        and (
	                        	checkIfItemIsProtected(5, slotItemInstanceId)
	                        or  checkIfItemIsProtected(11, slotItemInstanceId)
	                        or  checkIfItemIsProtected(12, slotItemInstanceId)
                		)
                ) then
                    return false
                    --Filter 4 "show only marked"
                elseif(isFilterActivated == -99) then
                    if (
					       checkIfItemIsProtected(5, slotItemInstanceId)
						or checkIfItemIsProtected(11, slotItemInstanceId)
						or checkIfItemIsProtected(12, slotItemInstanceId)
                    ) then
                        return true
                    else
                        result = false
                    end
                    --Filter 4 off
                else
                    if (result ~= false) then
                        result = true
                    end
                end

            else

                --Sell, Sell in guild store & Intricate filter split disabled
                --Last used icon ID at the Sell, Sell in guild store & Intricate filter split context menu is stored in variable FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[panelId]
                --Filter 4 on
                if( isFilterActivated == true and checkIfItemIsProtected(FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[FCOIS.gFilterWhere], slotItemInstanceId) and FCOIS.settingsVars.settings.isIconEnabled[FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[FCOIS.gFilterWhere]]) then
                    return false
                    --Filter 4 "show only marked"
                elseif( isFilterActivated == -99 ) then
                    return checkIfItemIsProtected(FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[FCOIS.gFilterWhere], slotItemInstanceId)
                    --Filter 4 off
                else
                    if (result ~= false) then
                        result = true
                    end
                end

            end

		-- Normal treatment here
        else
			--Filter 1 (or others) on
	    	if(isFilterActivated == true and checkIfItemIsProtected(filterId, slotItemInstanceId)) then
            	return false
			--Filter 1 (or others) "show only marked"
		    elseif(isFilterActivated == -99) then
		    	return checkIfItemIsProtected(filterId, slotItemInstanceId)
			--Filter 1 (or others) off
            else
				if (result ~= false) then
					result = true
                end
			end
        end
	end
    return result
end

--Filter callBack function for deconstruction & enchanting panels
local function FilterSavedItems(bagId, slotIndex, ...)
	--This function will be executed once for EACH ITEM SLOT in your inventory/bank,
    --if you open a crafting station
	--The returned value of this filter function must be either
    --true  - Show the slot
    --false - Hide the slot

	-- Return value variable initalization: Show the slot
    local slotItemInstanceId = MyGetItemInstanceIdNoControl(bagId, slotIndex)

	--Get the filter result variable for the current item
	local itemIsShown = filterItemNow(slotItemInstanceId)

	-- Return the result if all filters were cross-checked and last filter is reached
	return itemIsShown
end

--filter callBack function for bags, bank, mail, trade, guild bank, guild store, etc.
local function FilterSavedItemsForShop(slot)
	--This function will be executed once for EACH ITEM SLOT in your inventory
	--The returned value of this filter function must be either
    --true  - Show the slot
    --false - Hide the slot

    local slotItemInstanceId = MyGetItemInstanceId(slot)
    local itemIsShown = filterItemNow(slotItemInstanceId)

	-- Return the result if all filters were cross-checked and last filter is reached
	return itemIsShown
end

--Update the button colors and textures depending on the filters
local function UpdateButtonColorsAndTextures(p_buttonId, p_button, p_status, p_filterPanelId)
	local p_statusText = p_status or "Not changed!"

    local updateTextureSizeIndex
	local btnName = "No button"
    if p_button ~= nil then
    	btnName = p_button:GetName()
    end
	--If the filterPanelId parameter is empty use the global filter panel Id from events and button clicks
    p_filterPanelId = p_filterPanelId or FCOIS.gFilterWhere

    debugMessage("[UpdateButtonColorsAndTextures] Button: " .. btnName .. ", ButtonId: " .. tostring(p_buttonId) .. ", Status: " .. tostring(p_status) .. ", FilterPanelId: " .. tostring(p_filterPanelId), true)
   	local texture
	local offset

--------------------------------------------------------------------------------
-- New filter method
--------------------------------------------------------------------------------
	if(FCOIS.settingsVars.settings.splitFilters == true and p_filterPanelId ~= nil) then
		-- New filter method
		if (p_buttonId == -1 or p_status == -1) then
	    -- for Schleife für 4 Buttons -> Initialisierung beim Laden des Addons
	    -- Läd für die Ansicht p_filterPanelId die Farben der 4 Filter Buttons
		    for i=1, FCOIS.numVars.gFCONumFilters, 1 do
		    	UpdateButtonColorsAndTextures(i, nil, getSettingsIsFilterOn(i, p_filterPanelId), p_filterPanelId)
		    end
	    else
	        if (p_button == nil) then
	        	-- Die 4 Filter Buttons einer Ansicht updaten!
	    		-- Ansichten = (Inventar, Guild shop, Verkäufer, Handwerksstation, Guild bank , Player bank, Verzauberungen erstellen, Verzauberungen extrahieren, Mail, Trade, Improvement)
--			    for j=1, FCOIS.numVars.gFCONumFilterPanels, 1 do
			        texture  = WINDOW_MANAGER:GetControlByName(string.format(FCOIS.mappingVars.gFilterPanelIdToTextureName[p_filterPanelId], p_buttonId), "")
                    --Does texture exist now?
                    if(texture ~= nil) then
						--Is the gear sets split filter button context-menu active and are we trying to change the texture of the gear sets button?
		            	if p_buttonId == 2 and FCOIS.settingsVars.settings.splitGearSetsFilter then
                            --Are all gear sets selected?
                        	if FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId] == -1 then
								texture:SetTexture(FCOIS.textureVars.allGearSets)
                                updateTextureSizeIndex = "gear"
                            --Only one of the gear sets is selected
                            else
						        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId]].texture])
                                updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId]].texture
                            end
		            	elseif p_buttonId == 3 and FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then
							--Are all entries seleted (Research, Deconstruction, Improvement)?
                        	if FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId] == -1 then
								texture:SetTexture(FCOIS.textureVars.allResDecImp)
                                updateTextureSizeIndex = "ResDecImp"
                            --Only one of the options is selected
                            else
						        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId]].texture])
                                updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId]].texture
                            end
                        elseif p_buttonId == 4 and FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then
                            --Are all entries seleted (Sell, Sell in guild store & Intricate)?
                            if FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId] == -1 then
                                texture:SetTexture(FCOIS.textureVars.allSellGuildInt)
                                updateTextureSizeIndex = "SellGuildInt"
                                --Only one of the options is selected
                            else
                                texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId]].texture])
                                updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId]].texture
                            end
						else
					        --Workaround to show at least a default texure, if none is found
					  		local iconId = FCOIS.mappingVars.filterToIcon[p_buttonId] or 1
                            if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[iconId].texture] ~= nil) then
						        --Set the texture now
						        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[iconId].texture])
                                updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[iconId].texture
							else
						        --Set fallback texture now
						        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[iconId])
	                            updateTextureSizeIndex = iconId
								--Set the fallback texture to the settings menu
					            FCOIS.settingsVars.settings.icon[iconId].texture = iconId
					        end
                        end
						--Get current status and reset the current texture color
                        if ((p_buttonId == 2 or p_buttonId == 3 or p_buttonId == 4) and p_statusText == "Not changed!") then
                        	p_status = getSettingsIsFilterOn(p_buttonId, p_filterPanelId)
                        end
						--Only update colors if wished
                        if (p_status ~= -999) then
							-- Special state between true and false (only shows marked items)
		                    if(p_status == -99) then
								-- Yellow button
					    		texture:SetColor(1, 1, 0, 1)
		                    else
			                	if(not p_status) then
		                        	-- red button
						    		texture:SetColor(1, 0, 0, 1)
						        else
		                        	-- Green button
						            texture:SetColor(0, 1, 0, 1)
						        end
		                    end
	                    end
					end

--	            end -- for j ...

	        else -- if (p_button == nil) then

				-- Einen ganz speziellen Button nur updaten
                texture = WINDOW_MANAGER:GetControlByName(string.format(FCOIS.mappingVars.gFilterPanelIdToTextureName[p_filterPanelId], p_buttonId), "")
				if(texture ~= nil) then
					--Is the gear sets split filter button context-menu active and are we trying to change the texture of the gear sets button?
					if p_buttonId == 2 and FCOIS.settingsVars.settings.splitGearSetsFilter then
						--Are all gear sets selected?
						if FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId] == -1 then
							texture:SetTexture(FCOIS.textureVars.allGearSets)
							updateTextureSizeIndex = "gear"
						--Only one of the gear sets is selected
						else
							texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId]].texture])
							updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId]].texture
						end
					elseif p_buttonId == 3 and FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then
						--Are all entries seleted (Research, Deconstruction, Improvement)?
						if FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId] == -1 then
							texture:SetTexture(FCOIS.textureVars.allResDecImp)
							updateTextureSizeIndex = "ResDecImp"
						--Only one of the options is selected
						else
							texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId]].texture])
                            updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId]].texture
						end
                    elseif p_buttonId == 4 and FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then
                        --Are all entries seleted (Sell, Sell in guild store & Intricate)?
                        if FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId] == -1 then
                            texture:SetTexture(FCOIS.textureVars.allSellGuildInt)
							updateTextureSizeIndex = "SellGuildInt"
                            --Only one of the options is selected
                        else
                            texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId]].texture])
                            updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId]].texture
                        end
					else
                        --Workaround to show at least a default texure, if none is found
                        local iconId = FCOIS.mappingVars.filterToIcon[p_buttonId] or 1
                        if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[iconId].texture] ~= nil) then
                            --Set the texture now
                            texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[iconId].texture])
                            updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[iconId].texture
                        else
                            --Set fallback texture now
                            texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[iconId])
                            updateTextureSizeIndex = iconId
                            --Set the fallback texture to the settings menu
                            FCOIS.settingsVars.settings.icon[iconId].texture = iconId
                        end
					end

					--Only update colors if wished
					if ((p_buttonId == 2 or p_buttonId == 3 or p_buttonId == 4) and p_statusText == "Not changed!") then
                    	p_status = getSettingsIsFilterOn(p_buttonId, p_filterPanelId)
					end
					if (p_status ~= -999) then
						-- Special state between true and false (only shows marked items)
						if(p_status == -99) then
						-- Yellow button
							texture:SetColor(1, 1, 0, 1)
						else
							if(not p_status) then
								-- red button
								texture:SetColor(1, 0, 0, 1)
							else
								-- Green button
								texture:SetColor(0, 1, 0, 1)
							end
						end
					end
				end
		    end

	    end

--------------------------------------------------------------------------------
-- Old filter method
--------------------------------------------------------------------------------
	else

        if FCOIS.settingsVars.settings.splitFilters == false then
	    -- Old filter method

			if (p_buttonId == -1 or p_status == -1) then

            -- for Schleife für 4 Buttons -> Initialisierung beim Laden des Addons
		    -- Läd für alle Ansichten die Farben der 4 Filter Buttons
			    for i=1, FCOIS.numVars.gFCONumFilters, 1 do
			    	UpdateButtonColorsAndTextures(i, nil, getSettingsIsFilterOn(i))
			    end
		    else

		        if (p_button == nil) then

		        	-- Die gleichen Filter Buttons aller Ansichten updaten!
		    		-- Ansichten = (Inventar, Guild shop, Verkäufer, Handwerksstation, Guild bank , Player bank, Verzauberungen erstellen, Verzauberungen extrahieren, Mail, Trade, Improvement)
				    --for j=1, FCOIS.numVars.gFCONumFilterPanels, 1 do

				        texture  = WINDOW_MANAGER:GetControlByName(string.format(FCOIS.mappingVars.gFilterPanelIdToTextureName[p_filterPanelId], p_buttonId), "")
						if(texture ~= nil) then

							--Is the gear sets split filter button context-menu active and are we trying to change the texture of the gear sets button?
			            	if p_buttonId == 2 and FCOIS.settingsVars.settings.splitGearSetsFilter then
	                            --Are all gear sets selected?
	                        	if FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId] == -1 then
									texture:SetTexture(FCOIS.textureVars.allGearSets)
                                    updateTextureSizeIndex = "gear"
	                            --Only one of the gear sets is selected
	                            else
							        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId]].texture])
                                    updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId]].texture
	                            end
			            	elseif p_buttonId == 3 and FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then
	                            --Are all entries seleted (Research, Deconstruction, Improvement)?
	                        	if FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId] == -1 then
									texture:SetTexture(FCOIS.textureVars.allResDecImp)
                                    updateTextureSizeIndex = "ResDecImp"
	                            --Only one of the options is selected
	                            else
							        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId]].texture])
                                    updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId]].texture
	                            end
                            elseif p_buttonId == 4 and FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then
                                --Are all entries seleted (Sell, Sell in guild store & Intricate)?
                                if FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId] == -1 then
                                    texture:SetTexture(FCOIS.textureVars.allSellGuildInt)
                                    updateTextureSizeIndex = "SellGuildInt"
                                    --Only one of the options is selected
                                else
                                    texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId]].texture])
                                    updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId]].texture
                                end
							else
                                --Workaround to show at least a default texure, if none is found
                                local iconId = FCOIS.mappingVars.filterToIcon[p_buttonId] or 1
                                if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[iconId].texture] ~= nil) then
                                    --Set the texture now
                                    texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[iconId].texture])
                                    updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[iconId].texture
                                else
                                    --Set fallback texture now
                                    texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[iconId])
                                    updateTextureSizeIndex = iconId
                                    --Set the fallback texture to the settings menu
                                    FCOIS.settingsVars.settings.icon[iconId].texture = iconId
                                end
	                         end

							--Get current status and reset the current texture color
	                        if ((p_buttonId == 2 or p_buttonId == 3 or p_buttonId == 4) and p_statusText == "Not changed!") then
	                        	p_status = getSettingsIsFilterOn(p_buttonId, p_filterPanelId)
	                        end
							--Only update colors if wished
		                    if (p_status ~= -999) then
								-- Special state between true and false (only shows marked items)
			                    if(p_status == -99) then
									-- Yellow button
						    		texture:SetColor(1, 1, 0, 1)
			                    else
				                	if(not p_status) then
			                        	-- red button
							    		texture:SetColor(1, 0, 0, 1)
							        else
			                        	-- Green button
							            texture:SetColor(0, 1, 0, 1)
							        end
			                    end
		                    end
						end

		            --end -- for j=1, FCOIS.numVars.gFCONumFilterPanels, 1 do

		        else -- if (p_button == nil) then

					-- Einen ganz speziellen Button nur updaten
				    texture = nil
	                texture = WINDOW_MANAGER:GetControlByName(string.format(FCOIS.mappingVars.gFilterPanelIdToTextureName[p_filterPanelId], p_buttonId), "")
					if(texture ~= nil) then

						--Is the gear sets split filter button context-menu active and are we trying to change the texture of the gear sets button?
		            	if p_buttonId == 2 and FCOIS.settingsVars.settings.splitGearSetsFilter then
                            --Are all gear sets selected?
                        	if FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId] == -1 then
								texture:SetTexture(FCOIS.textureVars.allGearSets)
                                updateTextureSizeIndex = "gear"
                            --Only one of the gear sets is selected
                            else
						        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId]].texture])
                                updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastGearFilterIconId[p_filterPanelId]].texture
                            end
						elseif p_buttonId == 3 and FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then
							--Are all entries seleted (Research, Deconstruction, Improvement)?
							if FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId] == -1 then
								texture:SetTexture(FCOIS.textureVars.allResDecImp)
                                updateTextureSizeIndex = "ResDecImp"
							--Only one of the options is selected
							else
								texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId]].texture])
                                updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastResDecImpFilterIconId[p_filterPanelId]].texture
							end
                        elseif p_buttonId == 4 and FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then
                            --Are all entries seleted (Sell, Sell in guild store & Intricate)?
                            if FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId] == -1 then
                                texture:SetTexture(FCOIS.textureVars.allSellGuildInt)
                                updateTextureSizeIndex = "SellGuildInt"
                                --Only one of the options is selected
                            else
                                texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId]].texture])
                                updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[p_filterPanelId]].texture
                            end
						else
                            --Workaround to show at least a default texure, if none is found
                            local iconId = FCOIS.mappingVars.filterToIcon[p_buttonId] or 1
                            if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[iconId].texture] ~= nil) then
                                --Set the texture now
                                texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[iconId].texture])
                                updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[iconId].texture
                            else
                                --Set fallback texture now
                                texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[iconId])
                                updateTextureSizeIndex = iconId
                                --Set the fallback texture to the settings menu
                                FCOIS.settingsVars.settings.icon[iconId].texture = iconId
                            end
                        end

						--Get current status and reset the current texture color
                        if ((p_buttonId == 2 or p_buttonId == 3 or p_buttonId == 4) and p_statusText == "Not changed!") then
                        	p_status = getSettingsIsFilterOn(p_buttonId, p_filterPanelId)
                        end
						--Only update colors if wished
						if (p_status ~= -999) then
							-- Special state between true and false (only shows marked items)
							if(p_status == -99) then
							-- Yellow button
								texture:SetColor(1, 1, 0, 1)
							else
								if(not p_status) then
									-- red button
									texture:SetColor(1, 0, 0, 1)
								else
									-- Green button
									texture:SetColor(0, 1, 0, 1)
								end
							end
						end
					end
			    end

		    end
        end -- if FCOIS.settingsVars.settings.splitFilters == false then

    end -- if filterPanelId ~= nil

    --Update the texture's size now
    if updateTextureSizeIndex ~= nil and texture ~= nil then
        local parentWindow = texture:GetParent()
        local pLeft = texture:GetLeft()
        local pTop  = texture:GetTop()

		--Get the new texture size by help of the index/index string
        if updateTextureSizeIndex == "gear" then
            texture:ClearAnchors()
           	texture:SetDimensions(FCOIS.textureVars.allGearSetsWidth, FCOIS.textureVars.allGearSetsHeight)
            texture:SetAnchorFill()
            --texture:SetAnchor(TOP, parentWindow, BOTTOM, pLeft, pTop)

        elseif updateTextureSizeIndex == "ResDecImp" then
            texture:ClearAnchors()
          	texture:SetDimensions(FCOIS.textureVars.allResDecImpWidth, FCOIS.textureVars.allResDecImpHeight)
            texture:SetAnchorFill()
            --texture:SetAnchor(TOP, parentWindow, BOTTOM, pLeft, pTop)

        elseif updateTextureSizeIndex == "SellGuildInt" then
            texture:ClearAnchors()
           	texture:SetDimensions(FCOIS.textureVars.allSellGuildIntWidth, FCOIS.textureVars.allSellGuildIntHeight)
            texture:SetAnchorFill()
            --texture:SetAnchor(TOP, parentWindow, BOTTOM, pLeft, pTop)

        elseif type(updateTextureSizeIndex) == "number" then
			if FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex] ~= nil and
               FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].width ~= nil and FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].height ~= nil then
                texture:ClearAnchors()
				texture:SetDimensions(FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].width, FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].height)
                --texture:SetAnchorFill()
                texture:SetAnchor(TOP, parentWindow, BOTTOM, FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].offsetLeft, FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].offsetTop)
            else
                texture:ClearAnchors()
				texture:SetDimensions(FCOIS.filterButtonVars.gFilterButtonWidth, FCOIS.filterButtonVars.gFilterButtonHeight)
                texture:SetAnchorFill()
	            --texture:SetAnchor(TOP, parentWindow, BOTTOM, pLeft, pTop)
			end
        end
    end
end

--Filter the player inventory
local function FilterPlayerInventory(filterId, panelId)
    local newFilterMethod = FCOIS.settingsVars.settings.splitFilters
    panelId = panelId or FCOIS.gFilterWhere

    --Filtering inside inventory is enabled in the FCOIS.settingsVars.settings?
	if (FCOIS.settingsVars.settings.allowInventoryFilter == true) then
        --Register only 1 filter in the player inventory
    	if (filterId ~= -1) then
            if (newFilterMethod == true) then
	        	if(not libFilters:IsFilterRegistered("FCOItemSaver_PlayerInventoryFilterNew" .. tostring(panelId) .. "_" .. tostring(filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_PlayerInventoryFilterNew" .. tostring(panelId) .. "_" .. tostring(filterId), LF_INVENTORY, FilterSavedItemsForShop)
				end
            else
	        	if(not libFilters:IsFilterRegistered("FCOItemSaver_PlayerInventoryFilter" .. tostring(filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_PlayerInventoryFilter" .. tostring(filterId), LF_INVENTORY, FilterSavedItemsForShop)
				end
            end
        else
        	--Register all the filters in the player inventory
            if (newFilterMethod) then
	        	for i=1, FCOIS.numVars.gFCONumFilters, 1 do
					if(not libFilters:IsFilterRegistered("FCOItemSaver_PlayerInventoryFilterNew" .. tostring(panelId) .. "_" .. tostring(i))) then
						libFilters:RegisterFilter("FCOItemSaver_PlayerInventoryFilterNew" .. tostring(panelId) .. "_" .. tostring(i), LF_INVENTORY, FilterSavedItemsForShop)
					end
	            end
            else
	        	for i=1, FCOIS.numVars.gFCONumFilters, 1 do
					if(not libFilters:IsFilterRegistered("FCOItemSaver_PlayerInventoryFilter" .. tostring(i))) then
						libFilters:RegisterFilter("FCOItemSaver_PlayerInventoryFilter" .. tostring(i), LF_INVENTORY, FilterSavedItemsForShop)
					end
	            end
            end
        end

    else
    --Filtering inside inventory is NOT enabled in the FCOIS.settingsVars.settings
		if (newFilterMethod == true) then
			if (filterId ~= -1) then
				libFilters:UnregisterFilter("FCOItemSaver_PlayerInventoryFilterNew" .. tostring(panelId) .. "_" .. tostring(filterId), LF_INVENTORY)
	        else
				for i=1, FCOIS.numVars.gFCONumFilters, 1 do
					libFilters:UnregisterFilter("FCOItemSaver_PlayerInventoryFilterNew" .. tostring(panelId) .. "_" .. tostring(i), LF_INVENTORY)
	            end
	        end
        else
			if (filterId ~= -1) then
				libFilters:UnregisterFilter("FCOItemSaver_PlayerInventoryFilter" .. tostring(filterId))
	        else
				for i=1, FCOIS.numVars.gFCONumFilters, 1 do
					libFilters:UnregisterFilter("FCOItemSaver_PlayerInventoryFilter" .. tostring(i))
	            end
	        end
        end
    end
end

--Unregister all filters
local function unregisterFilters(filterId, onlyPlayerInvFilter, filterPanelId)
	--Only remove filters for player inventory?
    if (onlyPlayerInvFilter == nil) then
    	onlyPlayerInvFilter = false
    end
	--Only update a special panel, or all?
	local forVar, maxVar
	if (filterPanelId ~= nil) then
    	forVar  = filterPanelId
        maxVar  = filterPanelId
		debugMessage("[UnregisterFilters] FilterPanelId: " .. tostring(filterPanelId) .. ", filterId: " .. tostring(filterId) .. ", OnlyPlayerInvFilter: " .. tostring(onlyPlayerInvFilter), false)
    else
    	forVar  = 1
        maxVar  = FCOIS.numVars.gFCONumFilterInventoryTypes
		debugMessage("[UnregisterFilters] From panel Id: " .. tostring(forVar) .. ", To panel Id: " .. tostring(maxVar) .. ", filterId: " .. tostring(filterId) .. ", OnlyPlayerInvFilter: " .. tostring(onlyPlayerInvFilter), false)
    end

    local unregisterArrayNew = {}
    --Unregister only 1 filter ID?
	if (filterId ~= nil and filterId ~= -1) then
    	if (FCOIS.settingsVars.settings.splitFilters == true) then
			--New filter method
			for lFilterWhere=forVar, maxVar , 1 do
               	if FCOIS.mappingVars.activeFilterPanelIds[lFilterWhere] == true then
					if (onlyPlayerInvFilter == true) then
						libFilters:UnregisterFilter("FCOItemSaver_PlayerInventoryFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId))
		            else
	                	unregisterArrayNew = {
	                       	[LF_INVENTORY] = "FCOItemSaver_PlayerInventoryFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
		                    [LF_BANK_WITHDRAW] = "FCOItemSaver_PlayerBankFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
	                       	[LF_BANK_DEPOSIT] = "FCOItemSaver_PlayerBankInventoryFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_GUILDBANK_WITHDRAW] = "FCOItemSaver_GuildBankFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
	                       	[LF_GUILDBANK_DEPOSIT] = "FCOItemSaver_GuildBankInventoryFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_VENDOR_SELL] = "FCOItemSaver_VendorFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_SMITHING_DECONSTRUCT] = "FCOItemSaver_DeconstructionFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_SMITHING_IMPROVEMENT] = "FCOItemSaver_ImprovementFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_GUILDSTORE_SELL] = "FCOItemSaver_GuildStoreFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_MAIL_SEND] = "FCOItemSaver_MailFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_TRADE] = "FCOItemSaver_Player2PlayerFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_ENCHANTING_EXTRACTION] = "FCOItemSaver_EnchantingExtractionFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_ENCHANTING_CREATION] = "FCOItemSaver_EnchantingCreationFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_FENCE_SELL] = "FCOItemSaver_FenceFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_FENCE_LAUNDER] = "FCOItemSaver_LaunderFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							[LF_ALCHEMY_CREATION] = "FCOItemSaver_AlchemyFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
	                       	[LF_CRAFTBAG] = "FCOItemSaver_CraftBagFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
						}
                        if libFilters:IsFilterRegistered(unregisterArrayNew[lFilterWhere], lFilterWhere) then
	                        --Unregister the registered filters for each panel
							libFilters:UnregisterFilter(unregisterArrayNew[lFilterWhere], lFilterWhere)
                        end
					end
                end
            end

		else -- if (FCOIS.settingsVars.settings.splitFilters == true) then
			--Old filter method
			if (onlyPlayerInvFilter == true) then
				libFilters:UnregisterFilter("FCOItemSaver_PlayerInventoryFilter" .. tostring(filterId))
            else
				--Unregister the registered filters
				libFilters:UnregisterFilter("FCOItemSaver_VendorFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_DeconstructionFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_ImprovementFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_GuildStoreFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_GuildBankFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_PlayerBankFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_PlayerInventoryFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_Player2PlayerFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_MailFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_EnchantingCreationFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_EnchantingExtractionFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_FenceFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_LaunderFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_AlchemyFilter" .. tostring(filterId))
				libFilters:UnregisterFilter("FCOItemSaver_CraftBagFilter" .. tostring(filterId))
			end
        end

    else

	-- Unregister all filter IDs
		if (FCOIS.settingsVars.settings.splitFilters == true or FCOIS.overrideVars.gSplitFilterOverride == true) then
			--New filter method
			for filterId=1, FCOIS.numVars.gFCONumFilters, 1 do
				for lFilterWhere=forVar, maxVar , 1 do
					if FCOIS.mappingVars.activeFilterPanelIds[lFilterWhere] == true then
						if (onlyPlayerInvFilter == true) then
							libFilters:UnregisterFilter("FCOItemSaver_PlayerInventoryFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId))
			            else
		                	unregisterArrayNew = {
	                        	[LF_INVENTORY] = "FCOItemSaver_PlayerInventoryFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
			                    [LF_BANK_WITHDRAW] = "FCOItemSaver_PlayerBankFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
	                        	[LF_BANK_DEPOSIT] = "FCOItemSaver_PlayerBankInventoryFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_GUILDBANK_WITHDRAW] = "FCOItemSaver_GuildBankFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
	                        	[LF_GUILDBANK_DEPOSIT] = "FCOItemSaver_GuildBankInventoryFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_VENDOR_SELL] = "FCOItemSaver_VendorFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_SMITHING_DECONSTRUCT] = "FCOItemSaver_DeconstructionFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_SMITHING_IMPROVEMENT] = "FCOItemSaver_ImprovementFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_GUILDSTORE_SELL] = "FCOItemSaver_GuildStoreFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_MAIL_SEND] = "FCOItemSaver_MailFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_TRADE] = "FCOItemSaver_Player2PlayerFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_ENCHANTING_EXTRACTION] = "FCOItemSaver_EnchantingExtractionFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_ENCHANTING_CREATION] = "FCOItemSaver_EnchantingCreationFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_FENCE_SELL] = "FCOItemSaver_FenceFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_FENCE_LAUNDER] = "FCOItemSaver_LaunderFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
								[LF_ALCHEMY_CREATION] = "FCOItemSaver_AlchemyFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
		                       	[LF_CRAFTBAG] = "FCOItemSaver_CraftBagFilterNew" .. tostring(lFilterWhere) .. "_" .. tostring(filterId),
							}
		                	--Unregister the registered filters for each panel
							libFilters:UnregisterFilter(unregisterArrayNew[lFilterWhere], lFilterWhere)
						end
					end
		        end
	        end

        elseif (FCOIS.settingsVars.settings.splitFilters == false or FCOIS.overrideVars.gSplitFilterOverride == true) then
  			--Old filter method
        	for filterId=1, FCOIS.numVars.gFCONumFilters, 1 do
				if (onlyPlayerInvFilter == true) then
					libFilters:UnregisterFilter("FCOItemSaver_PlayerInventoryFilter" .. tostring(filterId))
	            else
					--Unregister the registered filters
					libFilters:UnregisterFilter("FCOItemSaver_VendorFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_DeconstructionFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_ImprovementFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_GuildStoreFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_GuildBankFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_PlayerBankFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_PlayerInventoryFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_Player2PlayerFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_MailFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_EnchantingCreationFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_EnchantingExtractionFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_FenceFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_LaunderFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_AlchemyFilter" .. tostring(filterId))
					libFilters:UnregisterFilter("FCOItemSaver_CraftBagFilter" .. tostring(filterId))
				end
	        end
		end
	end
end

--Helper function for method registerFilters when FCOIS.settingsVars.settings.splitFilters == true
local function registerFilterId(p_onlyPlayerInvFilter, p_filterId, p_panelId)
	--Get the current LAF (filter type, e.g. LF_INVENTORY, LF_BANK_WITHDRAW, etc.)
	--local lf = libFilters:GetCurrentLAF()
    --if lf == nil then lf = LF_INVENTORY end

	if (p_onlyPlayerInvFilter == true) then
		--Player inventory -> Only if activated in settings
		FilterPlayerInventory(p_filterId, p_panelId)
	else

		--(Re)register the filters for each panel
		if     (p_panelId == LF_INVENTORY) then
			FilterPlayerInventory(p_filterId, p_panelId)

		elseif (p_panelId == LF_CRAFTBAG) then
        	-- Filter craft bag panels
			if (FCOIS.settingsVars.settings.allowCraftBagFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_CraftBagFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_CraftBagFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_CRAFTBAG, FilterSavedItemsForShop)
				end
			end

		elseif (p_panelId == LF_BANK_WITHDRAW) then
        	-- Filter player bank panels
			if (FCOIS.settingsVars.settings.allowBankFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_PlayerBankFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_PlayerBankFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_BANK_WITHDRAW, FilterSavedItemsForShop)
				end
			end

		elseif (p_panelId == LF_BANK_DEPOSIT) then
        	-- Filter player bank inventory panel
			if (FCOIS.settingsVars.settings.allowInventoryFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_PlayerBankInventoryFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_PlayerBankInventoryFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_BANK_DEPOSIT, FilterSavedItemsForShop)
				end
			end

		elseif (p_panelId == LF_GUILDBANK_WITHDRAW) then
		-- Filter guildbank panels
			if (FCOIS.settingsVars.settings.allowGuildBankFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_GuildBankFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_GuildBankFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_GUILDBANK_WITHDRAW, FilterSavedItemsForShop)
				end
			end

		elseif (p_panelId == LF_GUILDBANK_DEPOSIT) then
		-- Filter guildbank inventory panel
			if (FCOIS.settingsVars.settings.allowInventoryFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_GuildBankInventoryFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_GuildBankInventoryFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_GUILDBANK_DEPOSIT, FilterSavedItemsForShop)
				end
			end

		elseif (p_panelId == LF_VENDOR_SELL) then
		-- Filter stores
			if (FCOIS.settingsVars.settings.allowVendorFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_VendorFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					-- Register the before chosen filter type and function
					libFilters:RegisterFilter("FCOItemSaver_VendorFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_VENDOR_SELL, FilterSavedItemsForShop)
				end
			end
		elseif (p_panelId == LF_FENCE_SELL) then
		-- Filter fence
			if (FCOIS.settingsVars.settings.allowFenceFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_FenceFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					-- Register the before chosen filter type and function
					libFilters:RegisterFilter("FCOItemSaver_FenceFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_FENCE_SELL, FilterSavedItemsForShop)
				end
			end
		elseif (p_panelId == LF_FENCE_LAUNDER) then
		-- Filter fence
			if (FCOIS.settingsVars.settings.allowLaunderFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_LaunderFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					-- Register the before chosen filter type and function
					libFilters:RegisterFilter("FCOItemSaver_LaunderFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_FENCE_LAUNDER, FilterSavedItemsForShop)
				end
			end
		elseif (p_panelId == LF_SMITHING_DECONSTRUCT) then
		-- Filter deconstruction panels
			if (FCOIS.settingsVars.settings.allowDeconstructionFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_DeconstructionFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_DeconstructionFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_SMITHING_DECONSTRUCT, FilterSavedItems)
				end
			end
		elseif (p_panelId == LF_SMITHING_IMPROVEMENT) then
		-- Filter improvement panels
			if (FCOIS.settingsVars.settings.allowImprovementFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_ImprovementFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_ImprovementFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_SMITHING_IMPROVEMENT, FilterSavedItems)
				end
            end

		elseif (p_panelId == LF_GUILDSTORE_SELL) then
		-- Filter guildstore panels
			if (FCOIS.settingsVars.settings.allowTradinghouseFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_GuildStoreFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_GuildStoreFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_GUILDSTORE_SELL, FilterSavedItemsForShop)
				end
			end

		elseif (p_panelId == LF_MAIL_SEND) then
		--Mail
			if (FCOIS.settingsVars.settings.allowMailFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_MailFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_MailFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_MAIL_SEND, FilterSavedItemsForShop)
				end
			end

		elseif (p_panelId == LF_TRADE) then
			--Player2player trading
			if (FCOIS.settingsVars.settings.allowTradeFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_Player2PlayerFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_Player2PlayerFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_TRADE, FilterSavedItemsForShop)
				end
			end

		elseif (p_panelId == LF_ENCHANTING_EXTRACTION) then
		-- Filter enchanting panels
			if (FCOIS.settingsVars.settings.allowEnchantingFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_EnchantingExtractionFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_EnchantingExtractionFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_ENCHANTING_EXTRACTION, FilterSavedItems)
				end
			end

		elseif (p_panelId == LF_ENCHANTING_CREATION) then
		-- Filter enchanting panels
			if (FCOIS.settingsVars.settings.allowEnchantingFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_EnchantingCreationFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_EnchantingCreationFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_ENCHANTING_CREATION, FilterSavedItems)
				end
			end

		elseif (p_panelId == LF_ALCHEMY_CREATION) then
		-- Filter alchemy panels
			if (FCOIS.settingsVars.settings.allowAlchemyFilter == true) then
				if(not libFilters:IsFilterRegistered("FCOItemSaver_AlchemyFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId))) then
					libFilters:RegisterFilter("FCOItemSaver_AlchemyFilterNew" .. tostring(p_panelId) .. "_" .. tostring(p_filterId), LF_ALCHEMY_CREATION, FilterSavedItems)
				end
            end

		end

	end
end

--Register the filters by help of library libFilters
local function registerFilters(filterId, onlyPlayerInvFilter, p_FilterPanelId)
	--Only register filters for player inventory?
    if (onlyPlayerInvFilter == nil) then
    	onlyPlayerInvFilter = false
    end

	--Register only 1 filter ID?
	if (filterId ~= nil and filterId ~= -1) then
		--Register only one filter ID

		--New or old behaviour of filtering?
		if (FCOIS.settingsVars.settings.splitFilters == true) then
			--New filtering using panels
			debugMessage("[RegisterFilters] Panel: " .. tostring(p_FilterPanelId) .. ", OnlyPlayerInv: " .. tostring(onlyPlayerInvFilter) .. ", filterId: " .. tostring(filterId), false)

			--Register the filter for the given panel ID and filter ID
			registerFilterId(onlyPlayerInvFilter, filterId, p_FilterPanelId)

		else
	        -- Old filter behaviour without panels
			debugMessage("[RegisterFilters] filterId: " .. tostring(filterId) .. ", OnlyPlayerInv: " .. tostring(onlyPlayerInvFilter), false)

	        if (onlyPlayerInvFilter == true) then
				--Player inventory -> Only if activated in settings
				FilterPlayerInventory(filterId)
	        else

	        	-- Filter craft bag
				if (FCOIS.settingsVars.settings.allowCraftBagFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_CraftBagFilter" .. tostring(p_filterId))) then
						libFilters:RegisterFilter("FCOItemSaver_CraftBagFilter" .. tostring(p_filterId), LF_CRAFTBAG, FilterSavedItemsForShop)
					end
                end

	        	-- Filter stores
	            if (FCOIS.settingsVars.settings.allowVendorFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_VendorFilter" .. tostring(filterId))) then
			   			-- Register the before chosen filter type and function
		                libFilters:RegisterFilter("FCOItemSaver_VendorFilter" .. tostring(filterId), LF_VENDOR_SELL, FilterSavedItemsForShop)
		            end
				end

				-- Filter deconstruction panels
	            if (FCOIS.settingsVars.settings.allowDeconstructionFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_DeconstructionFilter" .. tostring(filterId))) then
		                libFilters:RegisterFilter("FCOItemSaver_DeconstructionFilter" .. tostring(filterId), LF_SMITHING_DECONSTRUCT, FilterSavedItems)
		            end
				end

				-- Filter improvement panels
	            if (FCOIS.settingsVars.settings.allowImprovementFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_ImprovementFilter" .. tostring(filterId))) then
		                libFilters:RegisterFilter("FCOItemSaver_ImprovementFilter" .. tostring(filterId), LF_SMITHING_IMPROVEMENT, FilterSavedItems)
		            end
				end

				-- Filter enchanting panels
	            if (FCOIS.settingsVars.settings.allowEnchantingFilter == true) then
                    if      ENCHANTING.enchantingMode == 1 then
	                	if(not libFilters:IsFilterRegistered("FCOItemSaver_EnchantingCreationFilter" .. tostring(filterId))) then
			                libFilters:RegisterFilter("FCOItemSaver_EnchantingCreationFilter" .. tostring(filterId), LF_ENCHANTING_CREATION, FilterSavedItems)
			            end
                    elseif ENCHANTING.enchantingMode == 2 then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_EnchantingExtractionFilter" .. tostring(filterId))) then
			                libFilters:RegisterFilter("FCOItemSaver_EnchantingExtractionFilter" .. tostring(filterId), LF_ENCHANTING_EXTRACTION, FilterSavedItems)
			            end
                   	end
				end

				-- Filter alchemy panels
				if (FCOIS.settingsVars.settings.allowAlchemyFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_AlchemyFilter" .. tostring(filterId))) then
						libFilters:RegisterFilter("FCOItemSaver_AlchemyFilter" .. tostring(filterId), LF_ALCHEMY_CREATION, FilterSavedItems)
					end
	            end

	            -- Filter player bank panels
	            if (FCOIS.settingsVars.settings.allowBankFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_PlayerBankFilter" .. tostring(filterId))) then
		                libFilters:RegisterFilter("FCOItemSaver_PlayerBankFilter" .. tostring(filterId), LF_BANK_WITHDRAW, FilterSavedItemsForShop)
	                end
				end

	 			-- Filter guildbank panels
	            if (FCOIS.settingsVars.settings.allowGuildBankFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_GuildBankFilter" .. tostring(filterId))) then
		                libFilters:RegisterFilter("FCOItemSaver_GuildBankFilter" .. tostring(filterId), LF_GUILDBANK_WITHDRAW, FilterSavedItemsForShop)
	                end
				end

				-- Filter guildstore panels
	            if (FCOIS.settingsVars.settings.allowTradinghouseFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_GuildStoreFilter" .. tostring(filterId))) then
		                libFilters:RegisterFilter("FCOItemSaver_GuildStoreFilter" .. tostring(filterId), LF_GUILDSTORE_SELL, FilterSavedItemsForShop)
					end
	            end

				-- Filter fence
				if (FCOIS.settingsVars.settings.allowFenceFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_FenceFilter" .. tostring(filterId))) then
		                libFilters:RegisterFilter("FCOItemSaver_FenceFilter" .. tostring(filterId), LF_FENCE_SELL, FilterSavedItemsForShop)
					end
                end

				-- Filter launder
				if (FCOIS.settingsVars.settings.allowLaunderFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_LaunderFilter" .. tostring(filterId))) then
		                libFilters:RegisterFilter("FCOItemSaver_LaunderFilter" .. tostring(filterId), LF_FENCE_LAUNDER, FilterSavedItemsForShop)
					end
                end

			    -- !!!!!												   !!!!!
			    -- !!!!! In addition always activate the following filters !!!!!
			    -- !!!!!												   !!!!!
				--Player inventory -> Only if activated in settings
				FilterPlayerInventory(filterId)

				--Player2player trading
		        if (FCOIS.settingsVars.settings.allowTradeFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_Player2PlayerFilter" .. tostring(filterId))) then
						libFilters:RegisterFilter("FCOItemSaver_Player2PlayerFilter" .. tostring(filterId), LF_TRADE, FilterSavedItemsForShop)
					end
				end
			    --Mail
		        if (FCOIS.settingsVars.settings.allowMailFilter == true) then
					if(not libFilters:IsFilterRegistered("FCOItemSaver_MailFilter" .. tostring(filterId))) then
						libFilters:RegisterFilter("FCOItemSaver_MailFilter" .. tostring(filterId), LF_MAIL_SEND, FilterSavedItemsForShop)
					end
				end
            end
		end  --new filter method?

	else
		--Register all filter IDs

		if (FCOIS.settingsVars.settings.splitFilters == true) then
			--Using panels

			--Only update a special panel, or all?
			local forVar, maxVar
			if (p_FilterPanelId == nil) then
		    	forVar  = 1
		        maxVar  = FCOIS.numVars.gFCONumFilterInventoryTypes
		    end

			for filterId=1, FCOIS.numVars.gFCONumFilters, 1 do
				for lFilterWhere=forVar, maxVar , 1 do
					if FCOIS.mappingVars.activeFilterPanelIds[lFilterWhere] == true then
						debugMessage("[registerFilters] Panel: " .. tostring(forVar) .. ", filterId: " .. tostring(filterId) .. ", OnlyPlayerInv: " .. tostring(onlyPlayerInvFilter), false)
	                    --register the filters for the given panels
	                	registerFilterId(onlyPlayerInvFilter, filterId, lFilterWhere)
                    end
		        end
	        end

		else
			--NOT using panels (old method)
			debugMessage("[RegisterFilters] All filters!, OnlyPlayerInv: " .. tostring(onlyPlayerInvFilter), false)

			for filterId=1, FCOIS.numVars.gFCONumFilters, 1 do
		        if (onlyPlayerInvFilter == true) then
					--Player inventory -> Only if activated in settings
					FilterPlayerInventory(filterId)
		        else

		        	-- Filter craft bag
					if (FCOIS.settingsVars.settings.allowCraftBagFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_CraftBagFilter" .. tostring(p_filterId))) then
							libFilters:RegisterFilter("FCOItemSaver_CraftBagFilter" .. tostring(p_filterId), LF_CRAFTBAG, FilterSavedItemsForShop)
						end
	                end

		        	-- Filter stores
		            if (FCOIS.settingsVars.settings.allowVendorFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_VendorFilter" .. tostring(filterId))) then
				   			-- Register the before chosen filter type and function
			                libFilters:RegisterFilter("FCOItemSaver_VendorFilter" .. tostring(filterId), LF_VENDOR_SELL, FilterSavedItemsForShop)
			            end
                    end

					-- Filter deconstruction panels
		            if (FCOIS.settingsVars.settings.allowDeconstructionFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_DeconstructionFilter" .. tostring(filterId))) then
			                libFilters:RegisterFilter("FCOItemSaver_DeconstructionFilter" .. tostring(filterId), LF_SMITHING_DECONSTRUCT, FilterSavedItems)
			            end
                    end

					-- Filter improvement panels
		            if (FCOIS.settingsVars.settings.allowImprovementFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_ImprovementFilter" .. tostring(filterId))) then
			                libFilters:RegisterFilter("FCOItemSaver_ImprovementFilter" .. tostring(filterId), LF_SMITHING_IMPROVEMENT, FilterSavedItems)
			            end
                    end

					-- Filter enchanting panels
		            if (FCOIS.settingsVars.settings.allowEnchantingFilter == true) then
	                    if      ENCHANTING.enchantingMode == 1 then
		                	if(not libFilters:IsFilterRegistered("FCOItemSaver_EnchantingCreationFilter" .. tostring(filterId))) then
				                libFilters:RegisterFilter("FCOItemSaver_EnchantingCreationFilter" .. tostring(filterId), LF_ENCHANTING_CREATION, FilterSavedItems)
				            end
	                    elseif ENCHANTING.enchantingMode == 2 then
							if(not libFilters:IsFilterRegistered("FCOItemSaver_EnchantingExtractionFilter" .. tostring(filterId))) then
				                libFilters:RegisterFilter("FCOItemSaver_EnchantingExtractionFilter" .. tostring(filterId), LF_ENCHANTING_EXTRACTION, FilterSavedItems)
				            end
	                   	end
					end

					-- Filter alchemy panels
		            if (FCOIS.settingsVars.settings.allowAlchemyFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_AlchemyFilter" .. tostring(filterId))) then
			                libFilters:RegisterFilter("FCOItemSaver_AlchemyFilter" .. tostring(filterId), LF_ALCHEMY_CREATION, FilterSavedItems)
			            end
                    end

		            -- Filter player bank panels
		            if (FCOIS.settingsVars.settings.allowBankFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_PlayerBankFilter" .. tostring(filterId))) then
			                libFilters:RegisterFilter("FCOItemSaver_PlayerBankFilter" .. tostring(filterId), LF_BANK_WITHDRAW, FilterSavedItemsForShop)
		                end
					end

		 			-- Filter guildbank panels
		            if (FCOIS.settingsVars.settings.allowGuildBankFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_GuildBankFilter" .. tostring(filterId))) then
			                libFilters:RegisterFilter("FCOItemSaver_GuildBankFilter" .. tostring(filterId), LF_GUILDBANK_WITHDRAW, FilterSavedItemsForShop)
		                end
					end

					-- Filter guildstore panels
		            if (FCOIS.settingsVars.settings.allowTradinghouseFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_GuildStoreFilter" .. tostring(filterId))) then
			                libFilters:RegisterFilter("FCOItemSaver_GuildStoreFilter" .. tostring(filterId), LF_GUILDSTORE_SELL, FilterSavedItemsForShop)
						end
		            end

					-- Filter fence
					if (FCOIS.settingsVars.settings.allowFenceFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_FenceFilter" .. tostring(filterId))) then
			                libFilters:RegisterFilter("FCOItemSaver_FenceFilter" .. tostring(filterId), LF_FENCE_SELL, FilterSavedItemsForShop)
						end
	                end

					-- Filter launder
					if (FCOIS.settingsVars.settings.allowLaunderFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_LaunderFilter" .. tostring(filterId))) then
			                libFilters:RegisterFilter("FCOItemSaver_LaunderFilter" .. tostring(filterId), LF_FENCE_LAUNDER, FilterSavedItemsForShop)
						end
	                end


				    -- !!!!!												   !!!!!
				    -- !!!!! In addition always activate the following filters !!!!!
				    -- !!!!!												   !!!!!
					--Player inventory -> Only if activated in settings
					FilterPlayerInventory(filterId)

					--Player2player trading
			        if (FCOIS.settingsVars.settings.allowTradeFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_Player2PlayerFilter" .. tostring(filterId))) then
							libFilters:RegisterFilter("FCOItemSaver_Player2PlayerFilter" .. tostring(filterId), LF_TRADE, FilterSavedItemsForShop)
						end
					end
				    --Mail
			        if (FCOIS.settingsVars.settings.allowMailFilter == true) then
						if(not libFilters:IsFilterRegistered("FCOItemSaver_MailFilter" .. tostring(filterId))) then
							libFilters:RegisterFilter("FCOItemSaver_MailFilter" .. tostring(filterId), LF_MAIL_SEND, FilterSavedItemsForShop)
						end
					end
	            end
	        end
		end

    end -- only 1 filter ID or all filter IDs?

end

--Write the info about the current filter state into the chat
--or build the tooltip text
local function outputFilterState(p_outputToChat, p_newFilterMethod, p_panelId, p_filterId, p_stateText)
	local filterText
    local preChatText
    local outputText

    --Abort given because filter is/will be unregistered?
    if p_stateText == 'ABORT' then
    	filterText = "_"
        preChatText = FCOIS.preChatVars.preChatTextRed
    else
		--Not aborted
        filterText = tostring(p_filterId) .. "_"
		--Is the split research/deconstruction/improvement filter activated?
        if p_filterId == 3 and FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then
			filterText = filterText .. "split_"
        --Is the split sell/sell in guild store/intricate filter activated?
        elseif p_filterId == 4 and FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then
            filterText = filterText .. "split_"
        end
        preChatText = FCOIS.preChatVars.preChatTextGreen
    end

    p_newFilterMethod = p_newFilterMethod or FCOIS.settingsVars.settings.splitFilters

	debugMessage("[OutputFilterState] OutputToChat: " .. tostring(p_outputToChat) ..", splitFilters: " ..tostring(p_newFilterMethod)..", filterPanelId: " ..tostring(p_panelId)..", filterId: "..tostring(p_filterId)..", stateText: " .. p_stateText, true)

	--New filter method activated?
	if (p_newFilterMethod == true) then
    	if     (p_panelId == LF_INVENTORY or p_panelId == LF_BANK_DEPOSIT or p_panelId == LF_GUILDBANK_DEPOSIT) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_inventory"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_CRAFTBAG) then
		   outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_craftbag"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_GUILDBANK_WITHDRAW) then
		   outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_guildbank"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_GUILDSTORE_SELL) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_guildstore"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_BANK_WITHDRAW) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_bank"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_SMITHING_DECONSTRUCT) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_deconstruction"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_SMITHING_IMPROVEMENT) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_improvement"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_ENCHANTING_EXTRACTION) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_enchantingstation_extraction"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_ENCHANTING_CREATION) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_enchantingstation_creation"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_VENDOR_SELL) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_store"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_FENCE_SELL) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_fence"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_FENCE_LAUNDER) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_launder"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_MAIL_SEND) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_mail"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_TRADE) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_trade"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		elseif (p_panelId == LF_ALCHEMY_CREATION) then
			outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter_alchemy"] .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
		end

	--Old filter method activated?
	else
		outputText = preChatText .. FCOIS.localizationVars.fcois_loc["filter" .. filterText .. p_stateText]
	end

	--Add another tooltip line with the currently selected gear set filter?
    if p_filterId == 2 and FCOIS.settingsVars.settings.splitGearSetsFilter then
        outputText = outputText .. "\n"
    	if      FCOIS.settingsVars.settings.lastGearFilterIconId[FCOIS.gFilterWhere] == -1 then
        	outputText = outputText .. FCOIS.localizationVars.fcois_loc["filter_gears_all"]
        elseif  FCOIS.settingsVars.settings.lastGearFilterIconId[FCOIS.gFilterWhere] == FCOIS.mappingVars.gearToIcon[1] then
        	outputText = outputText .. FCOIS.settingsVars.settings.icon[FCOIS.mappingVars.gearToIcon[1]].name
        elseif  FCOIS.settingsVars.settings.lastGearFilterIconId[FCOIS.gFilterWhere] == FCOIS.mappingVars.gearToIcon[2] then
        	outputText = outputText .. FCOIS.settingsVars.settings.icon[FCOIS.mappingVars.gearToIcon[2]].name
        elseif  FCOIS.settingsVars.settings.lastGearFilterIconId[FCOIS.gFilterWhere] == FCOIS.mappingVars.gearToIcon[3] then
        	outputText = outputText .. FCOIS.settingsVars.settings.icon[FCOIS.mappingVars.gearToIcon[3]].name
        elseif  FCOIS.settingsVars.settings.lastGearFilterIconId[FCOIS.gFilterWhere] == FCOIS.mappingVars.gearToIcon[4] then
        	outputText = outputText .. FCOIS.settingsVars.settings.icon[FCOIS.mappingVars.gearToIcon[4]].name
        elseif  FCOIS.settingsVars.settings.lastGearFilterIconId[FCOIS.gFilterWhere] == FCOIS.mappingVars.gearToIcon[5] then
        	outputText = outputText .. FCOIS.settingsVars.settings.icon[FCOIS.mappingVars.gearToIcon[5]].name
		end

    elseif p_filterId == 3 and FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then
        outputText = outputText .. "\n"
    	if      FCOIS.settingsVars.settings.lastResDecImpFilterIconId[FCOIS.gFilterWhere] == -1 then
        	outputText = outputText .. FCOIS.localizationVars.fcois_loc["filter_resdecimp_all"]
        else
        	outputText = outputText .. FCOIS.localizationVars.fcois_loc["filter_resdecimp_" .. tostring(FCOIS.mappingVars.iconToResDecImp[FCOIS.settingsVars.settings.lastResDecImpFilterIconId[FCOIS.gFilterWhere]])]
		end

    elseif p_filterId == 4 and FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then
        outputText = outputText .. "\n"
        if      FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[FCOIS.gFilterWhere] == -1 then
            outputText = outputText .. FCOIS.localizationVars.fcois_loc["filter_sellguildint_all"]
        else
            outputText = outputText .. FCOIS.localizationVars.fcois_loc["filter_sellguildint_" .. tostring(FCOIS.mappingVars.iconToSellGuildInt[FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[FCOIS.gFilterWhere]])]
        end
    end

    --Output to chat or return text only?
    if p_outputToChat == true then
    	d(outputText)
    else
    	return outputText
    end
end

--Get the "real" active panel.
--If you are at the bank e.g. panelId is 2 (FCOIS.gFilterWhere,set in event BANK_OPEN), but you could also be at the deposit tab which uses
--the normal inventory filters of panelId = 1. The same applies for mail, trade, and others
local function checkActivePanel(comingFrom)
	local inventoryName
	local oldFilterWhere
	if comingFrom == 0 or comingFrom == nil then
	    --Get the current filter panel id
	    oldFilterWhere = FCOIS.gFilterWhere
    else
	    oldFilterWhere = comingFrom
    end

	--Player bank
    if     not FCOIS.ZOControlVars.BANK:IsHidden() or comingFrom == LF_BANK_WITHDRAW then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_BANK_WITHDRAW)
        inventoryName = FCOIS.ZOControlVars.BANK_INV
	--Player inventory at bank
    elseif comingFrom == LF_BANK_DEPOSIT then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_BANK_DEPOSIT)
        inventoryName = FCOIS.ZOControlVars.INV
	--Guild bank
    elseif not FCOIS.ZOControlVars.GUILD_BANK:IsHidden() or comingFrom == LF_GUILDBANK_WITHDRAW then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_GUILDBANK_WITHDRAW)
        inventoryName = FCOIS.ZOControlVars.GUILD_BANK_INV
	--Player inventory at guild bank
    elseif comingFrom == LF_GUILDBANK_DEPOSIT then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_GUILDBANK_DEPOSIT)
        inventoryName = FCOIS.ZOControlVars.INV
	--Guild store
    elseif not FCOIS.ZOControlVars.GUILD_STORE:IsHidden() or comingFrom == LF_GUILDSTORE_SELL then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_GUILDSTORE_SELL)
        inventoryName = FCOIS.ZOControlVars.INV
	--Vendor
    elseif not FCOIS.ZOControlVars.VENDOR_SELL:IsHidden() or comingFrom == LF_VENDOR_SELL then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_VENDOR_SELL)
        --inventoryName = FCOIS.ZOControlVars.VENDOR_SELL
        inventoryName = FCOIS.ZOControlVars.INV
	--Fence
    elseif comingFrom == LF_FENCE_SELL then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_FENCE_SELL)
        --inventoryName = FCOIS.ZOControlVars.FENCE
        inventoryName = FCOIS.ZOControlVars.INV
	--Launder
    elseif comingFrom == LF_FENCE_LAUNDER then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_FENCE_LAUNDER)
        --inventoryName = FCOIS.ZOControlVars.LAUNDER
        inventoryName = FCOIS.ZOControlVars.INV
	--Mail
    elseif not FCOIS.ZOControlVars.MAIL_SEND:IsHidden() or comingFrom == LF_MAIL_SEND then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_MAIL_SEND)
        --inventoryName = FCOIS.ZOControlVars.MAIL_SEND
        inventoryName = FCOIS.ZOControlVars.INV
	--Trade
    elseif not FCOIS.ZOControlVars.PLAYER_TRADE:IsHidden() or comingFrom == LF_TRADE then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_TRADE)
        --inventoryName = FCOIS.ZOControlVars.PLAYER_TRADE
        inventoryName = FCOIS.ZOControlVars.INV
    --Enchanting creation mode
    elseif comingFrom == LF_ENCHANTING_CREATION then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_ENCHANTING_CREATION)
        inventoryName = FCOIS.ZOControlVars.ENCHANTING_STATION
	--Enchanting extraction mode
    elseif comingFrom == LF_ENCHANTING_EXTRACTION then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_ENCHANTING_EXTRACTION)
        inventoryName = FCOIS.ZOControlVars.ENCHANTING_STATION
    elseif comingFrom == "ENCHANTING" then
    --Determine which enchanting mode is used
	    --Enchanting creation mode
	    if FCOIS.ZOControlVars.ENCHANTING.enchantingMode == 1 then
			--Update the filterPanelId
            comingFrom = LF_ENCHANTING_CREATION
            FCOIS.gFilterWhere = getFilterWhereBySettings(LF_ENCHANTING_CREATION)
	        inventoryName = FCOIS.ZOControlVars.ENCHANTING_STATION
		--Enchanting extraction mode
	    elseif FCOIS.ZOControlVars.ENCHANTING.enchantingMode == 2 then
			--Update the filterPanelId
            comingFrom = LF_ENCHANTING_EXTRACTION
			FCOIS.gFilterWhere = getFilterWhereBySettings(LF_ENCHANTING_EXTRACTION)
	        inventoryName = FCOIS.ZOControlVars.ENCHANTING_STATION
		end
	--Alchemy
    elseif not FCOIS.ZOControlVars.ALCHEMY_STATION:IsHidden() or comingFrom == LF_ALCHEMY_CREATION then
    	--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_ALCHEMY_CREATION)
        inventoryName = FCOIS.ZOControlVars.ALCHEMY_INV
   	--Deconstruction
    elseif comingFrom == LF_SMITHING_DECONSTRUCT then
    	--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_SMITHING_DECONSTRUCT)
        inventoryName = FCOIS.ZOControlVars.DECONSTRUCTION_INV
    --Improvement
    elseif comingFrom == LF_SMITHING_IMPROVEMENT then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_SMITHING_IMPROVEMENT)
        inventoryName = FCOIS.ZOControlVars.IMPROVEMENT_INV
	--Deconstruction
    elseif not FCOIS.ZOControlVars.DECONSTRUCTION_INV:IsHidden() then
    	--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_SMITHING_DECONSTRUCT)
        inventoryName = FCOIS.ZOControlVars.DECONSTRUCTION_INV
    --Improvement
    elseif not FCOIS.ZOControlVars.IMPROVEMENT_INV:IsHidden() then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_SMITHING_IMPROVEMENT)
        inventoryName = FCOIS.ZOControlVars.IMPROVEMENT_INV
	--Craft bag
    elseif HasCraftBagAccess() and not FCOIS.ZOControlVars.CRAFTBAG:IsHidden() then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_CRAFTBAG)
        inventoryName = FCOIS.ZOControlVars.CRAFTBAG
	--Player inventory
    elseif not FCOIS.ZOControlVars.INV:IsHidden() then
		--Update the filterPanelId
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_INVENTORY)
        inventoryName = FCOIS.ZOControlVars.INV
    else
		--Update the filterPanelId with a standard value
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_INVENTORY)
		inventoryName = BAGS_INV
    end

    debugMessage("[CheckActivePanel] Before: " .. tostring(oldFilterWhere) .. ", after: " .. tostring(FCOIS.gFilterWhere), true)

	--Return the found inventory type (e.g. LF_BANK_WITHDRAW)
 	return inventoryName
end

--The GearSetFilter button's context menu OnMouseExit event callback function
local function ContextFilterOnMouseExit(button, contextMenuType)
	if contextMenuType == "gear" then
	    if FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere] ~= nil then
			FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(true)
	    end
        if FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere] ~= nil then
            FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(true)
        end
        --Hide tooltip
        if FCOIS.settingsVars.settings.showFilterButtonContextTooltip == true then
	        ZO_Tooltips_HideTextTooltip()
        end

    elseif contextMenuType == "ResDecImp" then
	    if FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere] ~= nil then
			FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(true)
	    end
        if FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere] ~= nil then
            FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(true)
        end
        --Hide tooltip
        if FCOIS.settingsVars.settings.showFilterButtonContextTooltip == true then
	        ZO_Tooltips_HideTextTooltip()
        end

    elseif contextMenuType == "SellGuildInt" then
        if FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere] ~= nil then
            FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(true)
        end
        if FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere] ~= nil then
            FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(true)
        end
        --Hide tooltip
        if FCOIS.settingsVars.settings.showFilterButtonContextTooltip == true then
	        ZO_Tooltips_HideTextTooltip()
        end

	else
    	return
	end
	local labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_DISABLED_TEXT:UnpackRGBA())
    end

end

--The GearSetFilter button's context menu OnMouseEnter event callback function
local function ContextFilterOnMouseEnter(button, contextMenuType, iconId)
    local tooltipText
	if contextMenuType == "gear" then
		--Create the background for the selected line
	    if FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere] == nil then
			FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere] = WINDOW_MANAGER:CreateControl("bdBackDropGearSetFilter"..tostring(FCOIS.gFilterWhere), button, CT_BACKDROP)
			FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(true)
			--FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetEdgeColor(0.4,0.4,0.4)
			--FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetCenterColor(0.1,0.1,0.1)
			FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetDimensions(250,20)
			FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetAlpha(1)
			FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetDrawLayer(0)
	        local menuInfo = FCOIS.contextMenuVars.menuInfo[MENU_TYPE_DEFAULT]
			FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetCenterTexture(menuInfo.backdropCenter)
	        FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetEdgeTexture(menuInfo.backdropEdge, menuInfo.backdropEdgeWidth, menuInfo.backdropEdgeHeight)
	        FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetInsets(unpack(menuInfo.backdropInsets))
	    end
		FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:ClearAnchors()
		FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetAnchor(TOPLEFT, button, TOPLEFT, -4, -2)
	    FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetAnchor(BOTTOMRIGHT, button, BOTTOMRIGHT, 0, 2)
		FCOIS.contextMenu.GearSetFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(false)

		--Set a tooltip?
		if FCOIS.settingsVars.settings.showFilterButtonContextTooltip == true then
            if iconId == -1 then
            	tooltipText = FCOIS.localizationVars.fcois_loc["button_context_menu_gear_sets_all_tooltip"]
            else
	        	tooltipText = FCOIS.settingsVars.settings.icon[iconId].name
            end
			if tooltipText ~= "" then
				local showToolTip = true
				--Only show a tooltip if the context menu for gear sets is shown at the filter button
				if FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere] ~= nil then
					showToolTip = not FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:IsHidden()
				end
				--Don't show a tooltip if the context menu for research, deconstruction & improvement is shown at the filter button
				if showToolTip and FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere] ~= nil then
					showToolTip = FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:IsHidden()
				end
				if showToolTip and FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere] ~= nil then
					showToolTip = FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:IsHidden()
				end
				if showToolTip then
					ZO_Tooltips_ShowTextTooltip(button, LEFT, tooltipText)
				end
			end
		else
			ZO_Tooltips_HideTextTooltip()
		end

    elseif contextMenuType == "ResDecImp" then
		--Create the background for the selected line
	    if FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere] == nil then
			FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere] = WINDOW_MANAGER:CreateControl("bdBackDropResDecImpFilter"..tostring(FCOIS.gFilterWhere), button, CT_BACKDROP)
			FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(true)
			--FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetEdgeColor(0.4,0.4,0.4)
			--FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetCenterColor(0.1,0.1,0.1)
			FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetDimensions(250,20)
			FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetAlpha(1)
			FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetDrawLayer(0)
	        local menuInfo = FCOIS.contextMenuVars.menuInfo[MENU_TYPE_DEFAULT]
			FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetCenterTexture(menuInfo.backdropCenter)
	        FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetEdgeTexture(menuInfo.backdropEdge, menuInfo.backdropEdgeWidth, menuInfo.backdropEdgeHeight)
	        FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetInsets(unpack(menuInfo.backdropInsets))
	    end
		FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:ClearAnchors()
		FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetAnchor(TOPLEFT, button, TOPLEFT, -4, -2)
	    FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetAnchor(BOTTOMRIGHT, button, BOTTOMRIGHT, 0, 2)
		FCOIS.contextMenu.ResDecImpFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(false)

		--Set a tooltip?
		if FCOIS.settingsVars.settings.showFilterButtonContextTooltip == true then
            if iconId == -1 then
            	tooltipText = FCOIS.localizationVars.fcois_loc["button_context_menu_all_markers_tooltip"]
            else
	        	tooltipText = FCOIS.localizationVars.fcois_loc["options_icon" .. iconId .. "_color"]
            end
			if tooltipText ~= "" then
				local showToolTip = true
				--Only show a tooltip if the context menu for gear sets is shown at the filter button
				if FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere] ~= nil then
					showToolTip = FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:IsHidden()
				end
				--Don't show a tooltip if the context menu for research, deconstruction & improvement is shown at the filter button
				if showToolTip and FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere] ~= nil then
					showToolTip = not FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:IsHidden()
				end
				if showToolTip and FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere] ~= nil then
					showToolTip = FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:IsHidden()
				end
				if showToolTip then
					ZO_Tooltips_ShowTextTooltip(button, LEFT, tooltipText)
				end
			end
		else
			ZO_Tooltips_HideTextTooltip()
		end

    elseif contextMenuType == "SellGuildInt" then
        --Create the background for the selected line
        if FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere] == nil then
            FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere] = WINDOW_MANAGER:CreateControl("bdBackDropSellGuildIntFilter"..tostring(FCOIS.gFilterWhere), button, CT_BACKDROP)
            FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(true)
            --FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetEdgeColor(0.4,0.4,0.4)
            --FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetCenterColor(0.1,0.1,0.1)
            FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetDimensions(250,20)
            FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetAlpha(1)
            FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetDrawLayer(0)
            local menuInfo = FCOIS.contextMenuVars.menuInfo[MENU_TYPE_DEFAULT]
            FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetCenterTexture(menuInfo.backdropCenter)
            FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetEdgeTexture(menuInfo.backdropEdge, menuInfo.backdropEdgeWidth, menuInfo.backdropEdgeHeight)
            FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetInsets(unpack(menuInfo.backdropInsets))
        end
        FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:ClearAnchors()
        FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetAnchor(TOPLEFT, button, TOPLEFT, -4, -2)
        FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetAnchor(BOTTOMRIGHT, button, BOTTOMRIGHT, 0, 2)
        FCOIS.contextMenu.SellGuildIntFilter.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(false)

		--Set a tooltip?
		if FCOIS.settingsVars.settings.showFilterButtonContextTooltip == true then
            if iconId == -1 then
            	tooltipText = FCOIS.localizationVars.fcois_loc["button_context_menu_all_markers_tooltip"]
            else
	        	tooltipText = FCOIS.localizationVars.fcois_loc["options_icon" .. iconId .. "_color"]
            end
			if tooltipText ~= "" then
				local showToolTip = true
				--Only show a tooltip if the context menu for gear sets is shown at the filter button
				if FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere] ~= nil then
					showToolTip = FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:IsHidden()
				end
				--Don't show a tooltip if the context menu for research, deconstruction & improvement is shown at the filter button
				if showToolTip and FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere] ~= nil then
					showToolTip = FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:IsHidden()
				end
				if showToolTip and FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere] ~= nil then
					showToolTip = not FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:IsHidden()
				end
				if showToolTip then
					ZO_Tooltips_ShowTextTooltip(button, LEFT, tooltipText)
				end
			end
		else
			ZO_Tooltips_HideTextTooltip()
		end
	else
    	return
    end


	local labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_HIGHLIGHT_TEXT:UnpackRGBA())
    end
end

--The GearSetfilter button's context menu Onclicked callback function
local function ContextFilterOnClicked(button, contextMenuType)
	if contextMenuType == "gear" then
		--Hide the gears contextmenu after entry was clicked
		FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:SetHidden(true)
    elseif contextMenuType == "ResDecImp" then
		--Hide the RESEARCH & DECONSTRUCTION & IMPORVEMENT contextmenu after entry was clicked
		FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:SetHidden(true)
    elseif contextMenuType == "SellGuildInt" then
        --Hide the SELL & SELL AT GUILD STORE & INTRICATE contextmenu after entry was clicked
        FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:SetHidden(true)
	else
    	return
    end

	--Get the button name and strip the numbers
	local buttonName = button:GetName()
--d("Clicked button: " .. buttonName)
	local buttonNameLen = string.len(FCOIS.addonVars.gAddonName) + 1
	local sbegin
    local send
    sbegin, send = string.find(buttonName, "_", buttonNameLen)
	local buttonNr = string.sub(buttonName, buttonNameLen, sbegin - 1)

	--local buttonNameTotalLen = string.len(buttonName) - 1
    --local buttonEnd = string.sub(buttonName, buttonNameTotalLen)

	local iconId
	--Get icon ID from button number
	if contextMenuType == "gear" then
	    iconId = FCOIS.contextMenuVars.GearSetFilter.buttonContextMenuToIconId[buttonNr].iconId
		FCOIS.settingsVars.settings.lastGearFilterIconId[FCOIS.gFilterWhere] = iconId
    elseif contextMenuType == "ResDecImp" then
	    iconId = FCOIS.contextMenuVars.ResDecImpFilter.buttonContextMenuToIconId[buttonNr].iconId
		FCOIS.settingsVars.settings.lastResDecImpFilterIconId[FCOIS.gFilterWhere] = iconId
    elseif contextMenuType == "SellGuildInt" then
        iconId = FCOIS.contextMenuVars.SellGuildIntFilter.buttonContextMenuToIconId[buttonNr].iconId
        FCOIS.settingsVars.settings.lastSellGuildIntFilterIconId[FCOIS.gFilterWhere] = iconId
	end

    debugMessage("[ContextFilterOnClicked] ContextType: " .. contextMenuType .. ", clicked buttonNr: " .. buttonNr .. ", IconId: " .. tostring(iconId), true)
	--Get button number by help of the context menu type
	local buttonNr = FCOIS.mappingVars.contextMenuButtonClickedMenuToButton[contextMenuType]
	if iconId ~= nil then
		--Update the last filter ID (determined by the used icon) for the correct inventory refresh
		if iconId ~= -1 then
			FCOIS.lastVars.gLastFilterId[FCOIS.gFilterWhere] = FCOIS.mappingVars.iconToFilter[iconId]
		else
			FCOIS.lastVars.gLastFilterId[FCOIS.gFilterWhere] = buttonNr
		end
		--Update the inventory
        FilterBasics()
        --Change the gear sets filter context-menu button's texture
        UpdateButtonColorsAndTextures(buttonNr, button, nil, FCOIS.gFilterWhere)
    end
end

--Show/Hide the gear set split filter context menu buttons according to the enabled/disabled "gear settings"
--or show/hide the research/deconstruction/improvement split filter context menu buttons
--or show/hide the sell/sell at guild store/intricate split filter context menu buttons
local function updateContextMenuFilterButtons(p_optionId, creatingContextMenu, p_filterPanelId, contextMenuType)
--============= GEAR SETS CONTEXT MENU =========================================
	if contextMenuType == "gear" then

		--Settings for gear sets split filter context-menu button activated? Else: Abort here
		if not FCOIS.settingsVars.settings.splitGearSetsFilter then return end

	    --No gear Id given? Abort here
		local p_gearId = p_optionId
	    if p_gearId == nil then return end

		creatingContextMenu = creatingContextMenu or false

	    --Gget the filter panel Id
	    p_filterPanelId = p_filterPanelId or FCOIS.gFilterWhere

		--Should be check all filter panel Ids?
	    if p_filterPanelId == -1 then
			for i_filterPanelId = 1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
				if FCOIS.mappingVars.activeFilterPanelIds[i_filterPanelId] == true then
					updateContextMenuFilterButtons(p_gearId, creatingContextMenu, i_filterPanelId, contextMenuType)
	            end
	        end
		else
			--No filter panel Id? -> Abort here
			if p_filterPanelId == nil then return end

			--Build the 2digits character filter panel Id for the button name
			local filterPanelId = p_filterPanelId
		    if filterPanelId < 10 then filterPanelId = "0" .. tostring(filterPanelId) end

			local gearButton
		    local gearId
		    local changedGearButton = false
			local offsetY = 0
			local texture

	        --Check each of the context menu buttons
			for contextMenuButton, v in pairs(FCOIS.contextMenuVars.GearSetFilter.buttonContextMenuToIconId) do
                gearId = nil
            	if v.iconId ~= -1 then
	            	gearId = FCOIS.mappingVars.iconToGear[v.iconId]
                end
				--Current button is a gear set relating one?
				if gearId ~= nil and gearId == p_gearId then
					gearButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. tostring(contextMenuButton) .. "_" .. tostring(filterPanelId), "")
					if gearButton ~= nil then
						local anchorButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. tostring(v.anchorButton) .. "_" .. tostring(filterPanelId), "")
		                if FCOIS.settingsVars.settings.isIconEnabled[v.iconId] then
		                    if not creatingContextMenu then
			                    gearButton:SetHeight(FCOIS.contextMenuVars.GearSetFilter.entryHeight)
			                    if anchorButton ~= nil then
									gearButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,FCOIS.contextMenuVars.GearSetFilter.entryHeight)
			                    end
			                	offsetY = offsetY + gearButton:GetHeight()

						        --Update the texture for the button that holds the gear set image
							    texture = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. tostring(contextMenuButton) .. "_" .. tostring(filterPanelId) .. "Texture", "")
	                            if texture ~= nil then
		                            texture:SetAnchorFill()
							        --Workaround to show at least a default texure, if none is found
							  		if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[v.iconId].texture] ~= nil) then
								        --Set the texture now
								        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[v.iconId].texture])
	                                    texture:SetColor(FCOIS.settingsVars.settings.icon[v.iconId].color.r, FCOIS.settingsVars.settings.icon[v.iconId].color.g, FCOIS.settingsVars.settings.icon[v.iconId].color.b, FCOIS.settingsVars.settings.icon[v.iconId].color.a)
									else
								        --Set fallback texture now
								        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[2])
										--Set the fallback texture to the settings menu
							            FCOIS.settingsVars.settings.icon[2].texture = 2
							        end
	                            end
		                    end
						    gearButton:SetMouseEnabled(true)
						    gearButton:SetHidden(false)
		                else
			                offsetY = offsetY - gearButton:GetHeight()
			                gearButton:ClearAnchors()
			                gearButton:SetHeight(0)
						    gearButton:SetMouseEnabled(false)
						    gearButton:SetHidden(true)
		                    if anchorButton ~= nil then
								gearButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,0)
		                    end
		                end
		                changedGearButton = true
				    end
		        end -- if FCOIS.mappingVars.iconToGear[k.iconId] ~= nil then
		        gearButton = nil
		        gearId	   = nil
		        --Abort here now if gearId was
		    end -- 	for k, v in pairs(FCOIS.contextMenuVars.GearSetFilter.buttonContextMenuToIconId) do

		    --Did any of the gear buttons change?
			if changedGearButton == true then
			    --Get the context menu control, move the non-gear buttons and resize the context menu
				local contextMenuName = FCOIS.addonVars.gAddonName .. "ContextMenuGearSetFilter" .. "_" .. tostring(filterPanelId)
			    local contextMenu = WINDOW_MANAGER:GetControlByName(contextMenuName, "")
			    if contextMenu ~= nil then
		        	--Change the height of the context menu
					local newHeight = contextMenu:GetHeight() + offsetY
		            if newHeight > FCOIS.contextMenuVars.GearSetFilter.maxHeight then
		            	newHeight = FCOIS.contextMenuVars.GearSetFilter.maxHeight
		            end
		        	contextMenu:SetHeight(newHeight)
			    end
		    end
	    end -- If p_filterPanelId == -1

--============= RESEARCH & DECONSTRUCTION & IMPORVEMENT CONTEXT MENU =========================
    elseif contextMenuType == "ResDecImp" then

		--Settings for research/deconstruction/improvement split filter context-menu button activated? Else: Abort here
		if not FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then return end

	    --No gear Id given? Abort here
	    if p_optionId == nil then return end

		creatingContextMenu = creatingContextMenu or false

	    --Gget the filter panel Id
	    p_filterPanelId = p_filterPanelId or FCOIS.gFilterWhere

--d("[FCOIS]updateContextMenuFilterButtons: ResDecImp, p_optionId: " .. p_optionId .. ", p_filterPanelId: " .. p_filterPanelId .. ", creating: " .. tostring(creatingContextMenu))

		--Should be check all filter panel Ids?
	    if p_filterPanelId == -1 then
			for i_filterPanelId = 1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
				if FCOIS.mappingVars.activeFilterPanelIds[i_filterPanelId] == true then
					updateContextMenuFilterButtons(p_optionId, creatingContextMenu, i_filterPanelId, contextMenuType)
	            end
	        end
		else
			--No filter panel Id? -> Abort here
			if p_filterPanelId == nil then return end

			--Build the 2digits character filter panel Id for the button name
			local filterPanelId = p_filterPanelId
		    if filterPanelId < 10 then filterPanelId = "0" .. tostring(filterPanelId) end

			local resDecImpButton
		    local changedResDecImpButton = false
			local offsetY = 0
			local texture
            local foundButtonToUpdate = false

	        --Check each of the context menu buttons
			for contextMenuButton, v in pairs(FCOIS.contextMenuVars.ResDecImpFilter.buttonContextMenuToIconId) do
--d("v.iconId: " .. v.iconId)
				if not foundButtonToUpdate and v.iconId ~= -1 and p_optionId ~= nil and p_optionId == v.iconId then
--d("OptionId: " .. p_optionId .. "=".. v.iconId)
	                resDecImpButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. tostring(contextMenuButton) .. "_" .. tostring(filterPanelId), "")
					if resDecImpButton ~= nil then
                        local anchorButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. tostring(v.anchorButton) .. "_" .. tostring(filterPanelId), "")
--d("Settings["..v.iconId.."]: " .. tostring(FCOIS.settingsVars.settings.isIconEnabled[v.iconId]))
	                    if FCOIS.settingsVars.settings.isIconEnabled[v.iconId] then
		                    if not creatingContextMenu then
			                    resDecImpButton:SetHeight(FCOIS.contextMenuVars.ResDecImpFilter.entryHeight)
			                    if anchorButton ~= nil then
									resDecImpButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,FCOIS.contextMenuVars.ResDecImpFilter.entryHeight)
			                    end
			                	offsetY = offsetY + resDecImpButton:GetHeight()

						        --Update the texture for the button that holds the ResDecImp image
							    texture = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. tostring(contextMenuButton) .. "_" .. tostring(filterPanelId) .. "Texture", "")
	                            if texture ~= nil then
		                            texture:SetAnchorFill()
							        --Workaround to show at least a default texure, if none is found
							  		if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[v.iconId].texture] ~= nil) then
								        --Set the texture now
								        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[v.iconId].texture])
	                                    texture:SetColor(FCOIS.settingsVars.settings.icon[v.iconId].color.r, FCOIS.settingsVars.settings.icon[v.iconId].color.g, FCOIS.settingsVars.settings.icon[v.iconId].color.b, FCOIS.settingsVars.settings.icon[v.iconId].color.a)
									else
								        --Set fallback texture now
								        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[3])
										--Set the fallback texture to the settings menu
							            FCOIS.settingsVars.settings.icon[3].texture = 3
							        end
	                            end
		                    end
						    resDecImpButton:SetMouseEnabled(true)
						    resDecImpButton:SetHidden(false)
		                else
			                offsetY = offsetY - resDecImpButton:GetHeight()
			                resDecImpButton:ClearAnchors()
			                resDecImpButton:SetHeight(0)
						    resDecImpButton:SetMouseEnabled(false)
						    resDecImpButton:SetHidden(true)
		                    if anchorButton ~= nil then
								resDecImpButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,0)
		                    end
		                end
		                changedResDecImpButton = true
                        foundButtonToUpdate = true
				    end
				end
		        resDecImpButton = nil
		    end -- 	for k, v in pairs(FCOIS.contextMenuVars.ResDecImpFilter.buttonContextMenuToIconId) do

		    --Did any of the gear buttons change?
			if changedResDecImpButton == true then
			    --Get the context menu control, move the non-gear buttons and resize the context menu
				local contextMenuName = FCOIS.addonVars.gAddonName .. "ContextMenuResDecImpFilter" .. "_" .. tostring(filterPanelId)
			    local contextMenu = WINDOW_MANAGER:GetControlByName(contextMenuName, "")
			    if contextMenu ~= nil then
		        	--Change the height of the context menu
					local newHeight = contextMenu:GetHeight() + offsetY
		            if newHeight > FCOIS.contextMenuVars.ResDecImpFilter.maxHeight then
		            	newHeight = FCOIS.contextMenuVars.ResDecImpFilter.maxHeight
		            end
		        	contextMenu:SetHeight(newHeight)
			    end
		    end
	    end -- If p_filterPanelId == -1

        --============= SELL & SELL IN GUILD STORE & INTRICATE CONTEXT MENU =========================
    elseif contextMenuType == "SellGuildInt" then
        --Settings for research/deconstruction/improvement split filter context-menu button activated? Else: Abort here
        if not FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then return end

        --No gear Id given? Abort here
        if p_optionId == nil then return end

        creatingContextMenu = creatingContextMenu or false

        --Gget the filter panel Id
        p_filterPanelId = p_filterPanelId or FCOIS.gFilterWhere

--d("[FCOIS]updateContextMenuFilterButtons: SellGuildInt, p_optionId: " .. p_optionId .. ", p_filterPanelId: " .. p_filterPanelId .. ", creating: " .. tostring(creatingContextMenu))

        --Should be check all filter panel Ids?
        if p_filterPanelId == -1 then
            for i_filterPanelId = 1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
                if FCOIS.mappingVars.activeFilterPanelIds[i_filterPanelId] == true then
                    updateContextMenuFilterButtons(p_optionId, creatingContextMenu, i_filterPanelId, contextMenuType)
                end
            end
        else
            --No filter panel Id? -> Abort here
            if p_filterPanelId == nil then return end

            --Build the 2digits character filter panel Id for the button name
            local filterPanelId = p_filterPanelId
            if filterPanelId < 10 then filterPanelId = "0" .. tostring(filterPanelId) end

            local sellGuildIntButton
            local changedSellGuildIntButton = false
            local offsetY = 0
            local texture
            local foundButtonToUpdate = false

            --Check each of the context menu buttons
            for contextMenuButton, v in pairs(FCOIS.contextMenuVars.SellGuildIntFilter.buttonContextMenuToIconId) do
--d("v.iconId: " .. v.iconId)
                if not foundButtonToUpdate and v.iconId ~= -1 and p_optionId ~= nil and p_optionId == v.iconId then
--d("OptionId: " .. p_optionId .. "=".. v.iconId)
                    sellGuildIntButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. tostring(contextMenuButton) .. "_" .. tostring(filterPanelId), "")
                    if sellGuildIntButton ~= nil then
--d("sellGuildIntButton: " .. sellGuildIntButton:GetName())
                        local anchorButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. tostring(v.anchorButton) .. "_" .. tostring(filterPanelId), "")
--d("anchorButton: " .. anchorButton:GetName())
--d("Settings["..FCOIS.mappingVars.iconToSellGuildInt[v.iconId].."]: " .. tostring(FCOIS.settingsVars.settings.isIconEnabled[v.iconId]))
                        if FCOIS.settingsVars.settings.isIconEnabled[v.iconId] then
                            if not creatingContextMenu then
                                sellGuildIntButton:SetHeight(FCOIS.contextMenuVars.SellGuildIntFilter.entryHeight)
                                if anchorButton ~= nil then
                                    sellGuildIntButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,FCOIS.contextMenuVars.SellGuildIntFilter.entryHeight)
                                end
                                offsetY = offsetY + sellGuildIntButton:GetHeight()

                                --Update the texture for the button that holds the SellGuildInt image
                                texture = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. tostring(contextMenuButton) .. "_" .. tostring(filterPanelId) .. "Texture", "")
                                if texture ~= nil then
									--Does the texture needs adjustments because of it's size and offset?
					                local updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[v.iconId].texture
					                if FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex] ~= nil and
					                        FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].width ~= nil and FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].height ~= nil then
					                    texture:ClearAnchors()
					                    texture:SetDimensions(FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].width, FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].height)
					                    --texture:SetAnchorFill()
					                    local parentWindow = texture:GetParent()
					                    texture:SetAnchor(TOP, parentWindow, BOTTOM, FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].contextMenuOffsetLeft, FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].contextMenuOffsetTop)
					                else
	                                    texture:SetAnchorFill()
	                                    --Workaround to show at least a default texure, if none is found
	                                    if (FCOIS.textureVars.MARKER_TEXTURES[updateTextureSizeIndex] ~= nil) then
	                                        --Set the texture now
	                                        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[updateTextureSizeIndex])
	                                        texture:SetColor(FCOIS.settingsVars.settings.icon[v.iconId].color.r, FCOIS.settingsVars.settings.icon[v.iconId].color.g, FCOIS.settingsVars.settings.icon[v.iconId].color.b, FCOIS.settingsVars.settings.icon[v.iconId].color.a)
	                                    else
	                                        --Set fallback texture now
	                                        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[5])
	                                        --Set the fallback texture to the settings menu
	                                        FCOIS.settingsVars.settings.icon[5].texture = 5
	                                    end
					                end
                                end
                            end
                            sellGuildIntButton:SetMouseEnabled(true)
                            sellGuildIntButton:SetHidden(false)
                        else
                            offsetY = offsetY - sellGuildIntButton:GetHeight()
                            sellGuildIntButton:ClearAnchors()
                            sellGuildIntButton:SetHeight(0)
                            sellGuildIntButton:SetMouseEnabled(false)
                            sellGuildIntButton:SetHidden(true)
                            if anchorButton ~= nil then
                                sellGuildIntButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,0)
                            end
                        end
                        changedSellGuildIntButton = true
                        foundButtonToUpdate = true
                    end
                end
                sellGuildIntButton = nil
            end -- 	for k, v in pairs(FCOIS.contextMenuVars.SellGuildIntFilter.buttonContextMenuToIconId) do

            --Did any of the gear buttons change?
            if changedSellGuildIntButton == true then
                --Get the context menu control, move the non-gear buttons and resize the context menu
                local contextMenuName = FCOIS.addonVars.gAddonName .. "ContextMenuSellGuildIntFilter" .. "_" .. tostring(filterPanelId)
                local contextMenu = WINDOW_MANAGER:GetControlByName(contextMenuName, "")
                if contextMenu ~= nil then
                    --Change the height of the context menu
                    local newHeight = contextMenu:GetHeight() + offsetY
                    if newHeight > FCOIS.contextMenuVars.SellGuildIntFilter.maxHeight then
                        newHeight = FCOIS.contextMenuVars.SellGuildIntFilter.maxHeight
                    end
                    contextMenu:SetHeight(newHeight)
                end
            end
        end -- If p_filterPanelId == -1

    else
	    return
    end

end

--Function that creates the GearSetFilter context menu. Adding to it buttons and the handlers for them
--or create the context menu for the RESEARCH & DECONSTRUCTION & IMPORVEMENT filter button + adding handlers
local function createContextMenuFilter(parentWindow, panelId, contextMenuType)

--========= GEAR SETS CONTEXT MENU =============================================
	if contextMenuType == "gear" then
		if FCOIS.settingsVars.settings.splitGearSetsFilter then
			local button
		    local lastButton
			local labelControl
			--Abort if parent does not exist
		    if parentWindow == nil then return end
			panelId = panelId or FCOIS.gFilterWhere
			local filterPanelId = panelId
		    if filterPanelId < 10 then filterPanelId = "0" .. tostring(filterPanelId) end

		    local parentName = parentWindow:GetName()
			local onClickedFunction = "normal"
		    if 		string.find(parentName, "ZO_SmithingTopLevelDeconstruction") then
		    	onClickedFunction = "deconstruction"
		    elseif  string.find(parentName, "ZO_SmithingTopLevelImprovement") then
		    	onClickedFunction = "improvement"
		    elseif  string.find(parentName, "ZO_Enchanting") then
		    	onClickedFunction = "enchanting"
			else
		    	onClickedFunction = "normal"
		    end
		    --Array with callback functions OnClicked
		    local onClickedFunctions = {
			    ["normal"] 			= ContextFilterOnClicked,
			    ["deconstruction"]  = ContextFilterOnClicked,
			    ["improvement"]  	= ContextFilterOnClicked,
			    ["enchanting"]  	= ContextFilterOnClicked,
		    }

		    FCOIS.contextMenu.GearSetFilter[panelId] = WINDOW_MANAGER:CreateTopLevelWindow(FCOIS.addonVars.gAddonName .. "ContextMenuGearSetFilter" .. "_" .. tostring(filterPanelId))
			local menuWindow = FCOIS.contextMenu.GearSetFilter[panelId]
			menuWindow:SetDimensions(FCOIS.contextMenuVars.GearSetFilter.maxWidth, FCOIS.contextMenuVars.GearSetFilter.maxHeight)
		    menuWindow:SetAnchor(TOP,parentWindow,BOTTOM, 12, 6)
		    menuWindow:SetMovable(false)
		    menuWindow:SetMouseEnabled(true)

			menuWindow.bg = WINDOW_MANAGER:CreateControlFromVirtual(FCOIS.addonVars.gAddonName .. "ContextMenuGearSetFilterBg" .. "_" .. tostring(filterPanelId), menuWindow, "ZO_DefaultBackdrop")

			menuWindow:SetDrawLevel(0)
			menuWindow:SetDrawLayer(1)
			menuWindow:SetDrawTier(1)

			--Button1: All gear set icons
			button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuGearSetFilter1".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
			button:SetDimensions( FCOIS.contextMenuVars.GearSetFilter.maxWidth,FCOIS.contextMenuVars.GearSetFilter.entryHeight )
			button:SetFont("ZoFontGame")
			button:SetState( BSTATE_NORMAL )
			button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "gear") end)
			button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "gear", -1) end)
			button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "gear") end)
			button:SetText(FCOIS.localizationVars.fcois_loc["button_context_menu_gear_sets_all"])
			button:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
			button:SetAnchor(TOPLEFT,menuWindow,TOPLEFT,0,0)
			labelControl = button:GetLabelControl()
		    if labelControl ~= nil then
		    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
		    end
			button:SetMouseEnabled(true)
			button:SetHidden(false)
		    lastButton = button
		    button = nil
		    labelControl = nil

			--Button2: Gear set 1
			button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuGearSetFilter2".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
			button:SetDimensions( FCOIS.contextMenuVars.GearSetFilter.maxWidth,FCOIS.contextMenuVars.GearSetFilter.entryHeight )
			button:SetFont("ZoFontGame")
			button:SetState( BSTATE_NORMAL )
			button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "gear") end)
			button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "gear", 2) end)
			button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "gear") end)
			--button:SetText("1")
			button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
			button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
			--labelControl = button:GetLabelControl()
		    --if labelControl ~= nil then
		    --	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
		    --end
	        local buttonName = button:GetName()
	        --Create the texture for the button to hold the image
		    local texture = WINDOW_MANAGER:CreateControl(buttonName .. "Texture", button, CT_TEXTURE)
		    texture:SetAnchorFill()
	        --Workaround to show at least a default texure, if none is found
	  		if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[2].texture] ~= nil) then
		        --Set the texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[2].texture])
	            texture:SetColor(FCOIS.settingsVars.settings.icon[2].color.r, FCOIS.settingsVars.settings.icon[2].color.g, FCOIS.settingsVars.settings.icon[2].color.b, FCOIS.settingsVars.settings.icon[2].color.a)
			else
		        --Set fallback texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[2])
				--Set the fallback texture to the settings menu
	            FCOIS.settingsVars.settings.icon[2].texture = 2
	        end
			button:SetMouseEnabled(true)
			button:SetHidden(false)
		    lastButton = button
		    button = nil
		    labelControl = nil

			--Button3: Gear set 2
			button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuGearSetFilter3".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
			button:SetDimensions( FCOIS.contextMenuVars.GearSetFilter.maxWidth,FCOIS.contextMenuVars.GearSetFilter.entryHeight )
			button:SetFont("ZoFontGame")
			button:SetState( BSTATE_NORMAL )
			button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "gear") end)
			button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "gear", 4) end)
			button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "gear") end)
			--button:SetText("2")
			button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
			button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
			--labelControl = button:GetLabelControl()
		    --if labelControl ~= nil then
		    --	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
		    --end
	        buttonName = button:GetName()
	        --Create the texture for the button to hold the image
		    texture = WINDOW_MANAGER:CreateControl(buttonName .. "Texture", button, CT_TEXTURE)
		    texture:SetAnchorFill()
	        --Workaround to show at least a default texure, if none is found
	  		if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[4].texture] ~= nil) then
		        --Set the texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[4].texture])
	            texture:SetColor(FCOIS.settingsVars.settings.icon[4].color.r, FCOIS.settingsVars.settings.icon[4].color.g, FCOIS.settingsVars.settings.icon[4].color.b, FCOIS.settingsVars.settings.icon[4].color.a)
			else
		        --Set fallback texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[2])
				--Set the fallback texture to the settings menu
	            FCOIS.settingsVars.settings.icon[4].texture = 2
	        end
			button:SetMouseEnabled(true)
			button:SetHidden(false)
		    lastButton = button
		    button = nil
		    labelControl = nil

			--Button4: Gear set 3
			button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuGearSetFilter4".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
			button:SetDimensions( FCOIS.contextMenuVars.GearSetFilter.maxWidth,FCOIS.contextMenuVars.GearSetFilter.entryHeight )
			button:SetFont("ZoFontGame")
			button:SetState( BSTATE_NORMAL )
			button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "gear") end)
			button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "gear", 6) end)
			button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "gear") end)
			--button:SetText("3")
			button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
			button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
			--labelControl = button:GetLabelControl()
		    --if labelControl ~= nil then
		    --	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
		    --end
	        buttonName = button:GetName()
	        --Create the texture for the button to hold the image
		    texture = WINDOW_MANAGER:CreateControl(buttonName .. "Texture", button, CT_TEXTURE)
		    texture:SetAnchorFill()
	        --Workaround to show at least a default texure, if none is found
	  		if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[6].texture] ~= nil) then
		        --Set the texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[6].texture])
	            texture:SetColor(FCOIS.settingsVars.settings.icon[6].color.r, FCOIS.settingsVars.settings.icon[6].color.g, FCOIS.settingsVars.settings.icon[6].color.b, FCOIS.settingsVars.settings.icon[6].color.a)
			else
		        --Set fallback texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[2])
				--Set the fallback texture to the settings menu
	            FCOIS.settingsVars.settings.icon[6].texture = 2
	        end
			button:SetMouseEnabled(true)
			button:SetHidden(false)
		    lastButton = button
		    button = nil
		    labelControl = nil

			--Button5: Gear set 4
			button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuGearSetFilter5".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
			button:SetDimensions( FCOIS.contextMenuVars.GearSetFilter.maxWidth,FCOIS.contextMenuVars.GearSetFilter.entryHeight )
			button:SetFont("ZoFontGame")
			button:SetState( BSTATE_NORMAL )
			button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "gear") end)
			button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "gear", 7) end)
			button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "gear") end)
			--button:SetText("4")
			button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
			button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
			--labelControl = button:GetLabelControl()
		    --if labelControl ~= nil then
		    --	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
		    --end
	        buttonName = button:GetName()
	        --Create the texture for the button to hold the image
		    texture = WINDOW_MANAGER:CreateControl(buttonName .. "Texture", button, CT_TEXTURE)
		    texture:SetAnchorFill()
	        --Workaround to show at least a default texure, if none is found
	  		if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[7].texture] ~= nil) then
		        --Set the texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[7].texture])
	            texture:SetColor(FCOIS.settingsVars.settings.icon[7].color.r, FCOIS.settingsVars.settings.icon[7].color.g, FCOIS.settingsVars.settings.icon[7].color.b, FCOIS.settingsVars.settings.icon[7].color.a)
			else
		        --Set fallback texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[2])
				--Set the fallback texture to the settings menu
	            FCOIS.settingsVars.settings.icon[7].texture = 2
	        end
			button:SetMouseEnabled(true)
			button:SetHidden(false)
		    lastButton = button
		    button = nil
		    labelControl = nil

			--Button6: Gear set 5
			button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuGearSetFilter6".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
			button:SetDimensions( FCOIS.contextMenuVars.GearSetFilter.maxWidth,FCOIS.contextMenuVars.GearSetFilter.entryHeight )
			button:SetFont("ZoFontGame")
			button:SetState( BSTATE_NORMAL )
			button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "gear") end)
			button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "gear", 8) end)
			button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "gear") end)
			--button:SetText("5")
			button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
			button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
			--labelControl = button:GetLabelControl()
		    --if labelControl ~= nil then
		    --	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
		    --end
	        buttonName = button:GetName()
	        --Create the texture for the button to hold the image
		    texture = WINDOW_MANAGER:CreateControl(buttonName .. "Texture", button, CT_TEXTURE)
		    texture:SetAnchorFill()
	        --Workaround to show at least a default texure, if none is found
	  		if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[8].texture] ~= nil) then
		        --Set the texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[8].texture])
	            texture:SetColor(FCOIS.settingsVars.settings.icon[8].color.r, FCOIS.settingsVars.settings.icon[8].color.g, FCOIS.settingsVars.settings.icon[8].color.b, FCOIS.settingsVars.settings.icon[8].color.a)
			else
		        --Set fallback texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[2])
				--Set the fallback texture to the settings menu
	            FCOIS.settingsVars.settings.icon[8].texture = 2
	        end
			button:SetMouseEnabled(true)
			button:SetHidden(false)
		    lastButton = button
		    button = nil
		    labelControl = nil

			--Hide the menu now
		 	menuWindow:SetHidden(true)

			--Update each gear set buttons (enable & show / disable & hide)
		    for gearId = 5, 1, -1 do
			    updateContextMenuFilterButtons(gearId, true, panelId, "gear")
		    end

		    return menuWindow
	    else
	    	return nil
	    end

--========= RESEARCH & DECONSTRUCTION & IMPORVEMENT CONTEXT MENU =============================
	elseif contextMenuType == "ResDecImp" then

		if FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then
			local button
		    local lastButton
			local labelControl
			--Abort if parent does not exist
		    if parentWindow == nil then return end
			panelId = panelId or FCOIS.gFilterWhere
			local filterPanelId = panelId
		    if filterPanelId < 10 then filterPanelId = "0" .. tostring(filterPanelId) end

		    local parentName = parentWindow:GetName()
			local onClickedFunction = "normal"
		    if 		string.find(parentName, "ZO_SmithingTopLevelDeconstruction") then
		    	onClickedFunction = "deconstruction"
		    elseif  string.find(parentName, "ZO_SmithingTopLevelImprovement") then
		    	onClickedFunction = "improvement"
		    elseif  string.find(parentName, "ZO_Enchanting") then
		    	onClickedFunction = "enchanting"
			else
		    	onClickedFunction = "normal"
		    end
		    --Array with callback functions OnClicked
		    local onClickedFunctions = {
			    ["normal"] 			= ContextFilterOnClicked,
			    ["deconstruction"]  = ContextFilterOnClicked,
			    ["improvement"]  	= ContextFilterOnClicked,
			    ["enchanting"]  	= ContextFilterOnClicked,
		    }

		    FCOIS.contextMenu.ResDecImpFilter[panelId] = WINDOW_MANAGER:CreateTopLevelWindow(FCOIS.addonVars.gAddonName .. "ContextMenuResDecImpFilter" .. "_" .. tostring(filterPanelId))
			local menuWindow = FCOIS.contextMenu.ResDecImpFilter[panelId]
			menuWindow:SetDimensions(FCOIS.contextMenuVars.ResDecImpFilter.maxWidth, FCOIS.contextMenuVars.ResDecImpFilter.maxHeight)
		    menuWindow:SetAnchor(TOP,parentWindow,BOTTOM, 12, 6)
		    menuWindow:SetMovable(false)
		    menuWindow:SetMouseEnabled(true)

			menuWindow.bg = WINDOW_MANAGER:CreateControlFromVirtual(FCOIS.addonVars.gAddonName .. "ContextMenuResDecImpFilterBg" .. "_" .. tostring(filterPanelId), menuWindow, "ZO_DefaultBackdrop")

			menuWindow:SetDrawLevel(0)
			menuWindow:SetDrawLayer(1)
			menuWindow:SetDrawTier(1)

			--Button1: All (Research, Deconstruction & Improvement)
			button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuResDecImpFilter1".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
			button:SetDimensions( FCOIS.contextMenuVars.ResDecImpFilter.maxWidth,FCOIS.contextMenuVars.ResDecImpFilter.entryHeight )
			button:SetFont("ZoFontGame")
			button:SetState( BSTATE_NORMAL )
			button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "ResDecImp") end)
			button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "ResDecImp", -1) end)
			button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "ResDecImp") end)
			button:SetText(FCOIS.localizationVars.fcois_loc["button_context_menu_gear_sets_all"])
			button:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
			button:SetAnchor(TOPLEFT,menuWindow,TOPLEFT,0,0)
			labelControl = button:GetLabelControl()
		    if labelControl ~= nil then
		    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
		    end
			button:SetMouseEnabled(true)
			button:SetHidden(false)
		    lastButton = button
		    button = nil
		    labelControl = nil

			--Button2: Research
			button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuResDecImpFilter2".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
			button:SetDimensions( FCOIS.contextMenuVars.ResDecImpFilter.maxWidth,FCOIS.contextMenuVars.ResDecImpFilter.entryHeight )
			button:SetFont("ZoFontGame")
			button:SetState( BSTATE_NORMAL )
			button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "ResDecImp") end)
			button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "ResDecImp", 3) end)
			button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "ResDecImp") end)
			--button:SetText("1")
			button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
			button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
			--labelControl = button:GetLabelControl()
		    --if labelControl ~= nil then
		    --	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
		    --end
	        local buttonName = button:GetName()
	        --Create the texture for the button to hold the image
		    local texture = WINDOW_MANAGER:CreateControl(buttonName .. "Texture", button, CT_TEXTURE)
		    texture:SetAnchorFill()
	        --Workaround to show at least a default texure, if none is found
	  		if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[3].texture] ~= nil) then
		        --Set the texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[3].texture])
	            texture:SetColor(FCOIS.settingsVars.settings.icon[3].color.r, FCOIS.settingsVars.settings.icon[3].color.g, FCOIS.settingsVars.settings.icon[3].color.b, FCOIS.settingsVars.settings.icon[3].color.a)
			else
		        --Set fallback texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[3])
				--Set the fallback texture to the settings menu
	            FCOIS.settingsVars.settings.icon[3].texture = 3
	        end
			button:SetMouseEnabled(true)
			button:SetHidden(false)
		    lastButton = button
		    button = nil
		    labelControl = nil

			--Button3: Deconstruction
			button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuResDecImpFilter3".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
			button:SetDimensions( FCOIS.contextMenuVars.ResDecImpFilter.maxWidth,FCOIS.contextMenuVars.ResDecImpFilter.entryHeight )
			button:SetFont("ZoFontGame")
			button:SetState( BSTATE_NORMAL )
			button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "ResDecImp") end)
			button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "ResDecImp", 9) end)
			button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "ResDecImp") end)
			--button:SetText("2")
			button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
			button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
			--labelControl = button:GetLabelControl()
		    --if labelControl ~= nil then
		    --	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
		    --end
	        buttonName = button:GetName()
	        --Create the texture for the button to hold the image
		    texture = WINDOW_MANAGER:CreateControl(buttonName .. "Texture", button, CT_TEXTURE)
		    texture:SetAnchorFill()
	        --Workaround to show at least a default texure, if none is found
	  		if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[9].texture] ~= nil) then
		        --Set the texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[9].texture])
	            texture:SetColor(FCOIS.settingsVars.settings.icon[9].color.r, FCOIS.settingsVars.settings.icon[9].color.g, FCOIS.settingsVars.settings.icon[9].color.b, FCOIS.settingsVars.settings.icon[9].color.a)
			else
		        --Set fallback texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[55])
				--Set the fallback texture to the settings menu
	            FCOIS.settingsVars.settings.icon[9].texture = 55
	        end
			button:SetMouseEnabled(true)
			button:SetHidden(false)
		    lastButton = button
		    button = nil
		    labelControl = nil

			--Button4: Improvement
			button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuResDecImpFilter4".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
			button:SetDimensions( FCOIS.contextMenuVars.ResDecImpFilter.maxWidth,FCOIS.contextMenuVars.ResDecImpFilter.entryHeight )
			button:SetFont("ZoFontGame")
			button:SetState( BSTATE_NORMAL )
			button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "ResDecImp") end)
			button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "ResDecImp", 10) end)
			button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "ResDecImp") end)
			--button:SetText("3")
			button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
			button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
			--labelControl = button:GetLabelControl()
		    --if labelControl ~= nil then
		    --	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
		    --end
	        buttonName = button:GetName()
	        --Create the texture for the button to hold the image
		    texture = WINDOW_MANAGER:CreateControl(buttonName .. "Texture", button, CT_TEXTURE)
		    texture:SetAnchorFill()
	        --Workaround to show at least a default texure, if none is found
	  		if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[10].texture] ~= nil) then
		        --Set the texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[10].texture])
	            texture:SetColor(FCOIS.settingsVars.settings.icon[10].color.r, FCOIS.settingsVars.settings.icon[10].color.g, FCOIS.settingsVars.settings.icon[10].color.b, FCOIS.settingsVars.settings.icon[10].color.a)
			else
		        --Set fallback texture now
		        texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[56])
				--Set the fallback texture to the settings menu
	            FCOIS.settingsVars.settings.icon[10].texture = 56
	        end
			button:SetMouseEnabled(true)
			button:SetHidden(false)
		    lastButton = button
		    button = nil
		    labelControl = nil

			--Hide the menu now
		 	menuWindow:SetHidden(true)

			--Update the context menu buttons (enable & show / disable & hide)
		    for optionId = 3, 1, -1 do
			    updateContextMenuFilterButtons(FCOIS.mappingVars.resDecImpToIcon[optionId], true, panelId, "ResDecImp")
		    end

		    return menuWindow
        else
            return nil
        end

    --========= SELL & SELL AT GUILD STORE & INTRICATE CONTEXT MENU =============================
    elseif contextMenuType == "SellGuildInt" then

        if FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then
            local button
            local lastButton
            local labelControl
            --Abort if parent does not exist
            if parentWindow == nil then return end
            panelId = panelId or FCOIS.gFilterWhere
            local filterPanelId = panelId
            if filterPanelId < 10 then filterPanelId = "0" .. tostring(filterPanelId) end

            local parentName = parentWindow:GetName()
            local onClickedFunction = "normal"
            if 		string.find(parentName, "ZO_SmithingTopLevelDeconstruction") then
                onClickedFunction = "deconstruction"
            elseif  string.find(parentName, "ZO_SmithingTopLevelImprovement") then
                onClickedFunction = "improvement"
            elseif  string.find(parentName, "ZO_Enchanting") then
                onClickedFunction = "enchanting"
            else
                onClickedFunction = "normal"
            end
            --Array with callback functions OnClicked
            local onClickedFunctions = {
                ["normal"] 			= ContextFilterOnClicked,
                ["deconstruction"]  = ContextFilterOnClicked,
                ["improvement"]  	= ContextFilterOnClicked,
                ["enchanting"]  	= ContextFilterOnClicked,
            }

            FCOIS.contextMenu.SellGuildIntFilter[panelId] = WINDOW_MANAGER:CreateTopLevelWindow(FCOIS.addonVars.gAddonName .. "ContextMenuSellGuildIntFilter" .. "_" .. tostring(filterPanelId))
            local menuWindow = FCOIS.contextMenu.SellGuildIntFilter[panelId]
            menuWindow:SetDimensions(FCOIS.contextMenuVars.SellGuildIntFilter.maxWidth, FCOIS.contextMenuVars.SellGuildIntFilter.maxHeight)
            menuWindow:SetAnchor(TOP,parentWindow,BOTTOM, 12, 6)
            menuWindow:SetMovable(false)
            menuWindow:SetMouseEnabled(true)

            menuWindow.bg = WINDOW_MANAGER:CreateControlFromVirtual(FCOIS.addonVars.gAddonName .. "ContextMenuSellGuildIntFilterBg" .. "_" .. tostring(filterPanelId), menuWindow, "ZO_DefaultBackdrop")

            menuWindow:SetDrawLevel(0)
            menuWindow:SetDrawLayer(1)
            menuWindow:SetDrawTier(1)

            --Button1: All (Sell & Sell at guild store & Intricate)
            button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuSellGuildIntFilter1".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
            button:SetDimensions( FCOIS.contextMenuVars.SellGuildIntFilter.maxWidth,FCOIS.contextMenuVars.SellGuildIntFilter.entryHeight )
            button:SetFont("ZoFontGame")
            button:SetState( BSTATE_NORMAL )
            button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "SellGuildInt") end)
            button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "SellGuildInt", -1) end)
            button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "SellGuildInt") end)
            button:SetText(FCOIS.localizationVars.fcois_loc["button_context_menu_gear_sets_all"])
            button:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
            button:SetAnchor(TOPLEFT,menuWindow,TOPLEFT,0,0)
            labelControl = button:GetLabelControl()
            if labelControl ~= nil then
                labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
            end
            button:SetMouseEnabled(true)
            button:SetHidden(false)
            lastButton = button
            button = nil
            labelControl = nil

            --Button2: Sell
            button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuSellGuildIntFilter2".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
            button:SetDimensions( FCOIS.contextMenuVars.SellGuildIntFilter.maxWidth,FCOIS.contextMenuVars.SellGuildIntFilter.entryHeight )
            button:SetFont("ZoFontGame")
            button:SetState( BSTATE_NORMAL )
            button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "SellGuildInt") end)
            button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "SellGuildInt", 5) end)
            button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "SellGuildInt") end)
            --button:SetText("1")
            button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
            button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
            --labelControl = button:GetLabelControl()
            --if labelControl ~= nil then
            --	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
            --end
            local buttonName = button:GetName()
            --Create the texture for the button to hold the image
            local texture = WINDOW_MANAGER:CreateControl(buttonName .. "Texture", button, CT_TEXTURE)
            texture:SetAnchorFill()
            --Workaround to show at least a default texure, if none is found
            if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[5].texture] ~= nil) then
                --Set the texture now
                texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[5].texture])
                texture:SetColor(FCOIS.settingsVars.settings.icon[5].color.r, FCOIS.settingsVars.settings.icon[5].color.g, FCOIS.settingsVars.settings.icon[5].color.b, FCOIS.settingsVars.settings.icon[5].color.a)
            else
                --Set fallback texture now
                texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[3])
                --Set the fallback texture to the settings menu
                FCOIS.settingsVars.settings.icon[5].texture = 5
            end
            button:SetMouseEnabled(true)
            button:SetHidden(false)
            lastButton = button
            button = nil
            labelControl = nil

            --Button3: Sell at guild store
            button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuSellGuildIntFilter3".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
            button:SetDimensions( FCOIS.contextMenuVars.SellGuildIntFilter.maxWidth,FCOIS.contextMenuVars.SellGuildIntFilter.entryHeight )
            button:SetFont("ZoFontGame")
            button:SetState( BSTATE_NORMAL )
            button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "SellGuildInt") end)
            button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "SellGuildInt", 11) end)
            button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "SellGuildInt") end)
            --button:SetText("2")
            button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
            button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
            --labelControl = button:GetLabelControl()
            --if labelControl ~= nil then
            --	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
            --end
            buttonName = button:GetName()
            --Create the texture for the button to hold the image
            texture = WINDOW_MANAGER:CreateControl(buttonName .. "Texture", button, CT_TEXTURE)
            texture:SetAnchorFill()
            --Workaround to show at least a default texure, if none is found
            if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[11].texture] ~= nil) then
                --Set the texture now
                texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[11].texture])
                texture:SetColor(FCOIS.settingsVars.settings.icon[11].color.r, FCOIS.settingsVars.settings.icon[11].color.g, FCOIS.settingsVars.settings.icon[11].color.b, FCOIS.settingsVars.settings.icon[11].color.a)
                local updateTextureSizeIndex = FCOIS.settingsVars.settings.icon[11].texture
                if FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex] ~= nil and
                        FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].width ~= nil and FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].height ~= nil then
                    texture:ClearAnchors()
                    texture:SetDimensions(FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].width, FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].height)
                    --texture:SetAnchorFill()
                    local parentWindow = texture:GetParent()
                    texture:SetAnchor(TOP, parentWindow, BOTTOM, FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].contextMenuOffsetLeft, FCOIS.textureVars.MARKER_TEXTURES_SIZE[updateTextureSizeIndex].contextMenuOffsetTop)
                else
                    texture:ClearAnchors()
                    texture:SetDimensions(FCOIS.filterButtonVars.gFilterButtonWidth, FCOIS.filterButtonVars.gFilterButtonHeight)
                    texture:SetAnchorFill()
                end

            else
                --Set fallback texture now
                texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[58])
                --Set the fallback texture to the settings menu
                FCOIS.settingsVars.settings.icon[11].texture = 58
            end
            button:SetMouseEnabled(true)
            button:SetHidden(false)
            lastButton = button
            button = nil
            labelControl = nil

            --Button4: Improvement
            button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuSellGuildIntFilter4".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
            button:SetDimensions( FCOIS.contextMenuVars.SellGuildIntFilter.maxWidth,FCOIS.contextMenuVars.SellGuildIntFilter.entryHeight )
            button:SetFont("ZoFontGame")
            button:SetState( BSTATE_NORMAL )
            button:SetHandler( "OnClicked", function(self) onClickedFunctions[onClickedFunction](self, "SellGuildInt") end)
            button:SetHandler( "OnMouseEnter", function (self) ContextFilterOnMouseEnter(self, "SellGuildInt", 12) end)
            button:SetHandler( "OnMouseExit", function (self) ContextFilterOnMouseExit(self, "SellGuildInt") end)
            --button:SetText("3")
            button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
            button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
            --labelControl = button:GetLabelControl()
            --if labelControl ~= nil then
            --	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
            --end
            buttonName = button:GetName()
            --Create the texture for the button to hold the image
            texture = WINDOW_MANAGER:CreateControl(buttonName .. "Texture", button, CT_TEXTURE)
            texture:SetAnchorFill()
            --Workaround to show at least a default texure, if none is found
            if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[12].texture] ~= nil) then
                --Set the texture now
                texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[12].texture])
                texture:SetColor(FCOIS.settingsVars.settings.icon[12].color.r, FCOIS.settingsVars.settings.icon[12].color.g, FCOIS.settingsVars.settings.icon[12].color.b, FCOIS.settingsVars.settings.icon[12].color.a)
            else
                --Set fallback texture now
                texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[60])
                --Set the fallback texture to the settings menu
                FCOIS.settingsVars.settings.icon[12].texture = 60
            end
            button:SetMouseEnabled(true)
            button:SetHidden(false)
            lastButton = button
            button = nil
            labelControl = nil

            --Hide the menu now
            menuWindow:SetHidden(true)

            --Update the context menu buttons (enable & show / disable & hide)
            for optionId = 3, 1, -1 do
                updateContextMenuFilterButtons(FCOIS.mappingVars.sellGuildIntToIcon[optionId], true, panelId, "SellGuildInt")
            end

            return menuWindow

        else
            return nil
        end

    else
    	return
    end
end

--Function that display the GearSetFilter context menu after the player clicks on the filter button on the inventory
--or shows the context menu for the GEARs, RESEARCH & DECONSTRUCTION & IMPORVEMENT or SELL & SELL AT GUILD STORE & INTRICATE filter button
local function showContextMenuFilterButton(parentWindow, contextMenuType)
    debugMessage("[showContextMenuFilterButton] Parent name: " .. parentWindow:GetName() .. ", Type: " .. contextMenuType .. ", PanelId: " .. FCOIS.gFilterWhere, true)
    if contextMenuType == "gear" then
        if FCOIS.settingsVars.settings.splitGearSetsFilter then
            if FCOIS.preventerVars.gGearSetFilterContextCreated[FCOIS.gFilterWhere] == false then
                FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere] = createContextMenuFilter(parentWindow, FCOIS.gFilterWhere, "gear")
            end
            if FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere] ~= nil then
                FCOIS.preventerVars.gGearSetFilterContextCreated[FCOIS.gFilterWhere] = true
                if FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:IsHidden() then
                    FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:ClearAnchors()
                    FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:SetAnchor(TOP,parentWindow,BOTTOM, 0, 8)
                    FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:SetHidden(false)
                else
                    FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:SetHidden(true)
                end
            end
        else
            hideContextMenu(FCOIS.gFilterWhere)
        end
    elseif contextMenuType == "ResDecImp" then
        if FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then
            if FCOIS.preventerVars.gResDecImpFilterContextCreated[FCOIS.gFilterWhere] == false then
                FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere] = createContextMenuFilter(parentWindow, FCOIS.gFilterWhere , "ResDecImp")
            end
            if FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere] ~= nil then
                FCOIS.preventerVars.gResDecImpFilterContextCreated[FCOIS.gFilterWhere] = true
                if FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:IsHidden() then
                    FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:ClearAnchors()
                    FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:SetAnchor(TOP,parentWindow,BOTTOM, 0, 8)
                    FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:SetHidden(false)
                else
                    FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:SetHidden(true)
                end
            end
        else
            hideContextMenu(FCOIS.gFilterWhere)
        end
    elseif contextMenuType == "SellGuildInt" then
        if FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then
            if FCOIS.preventerVars.gSellGuildIntFilterContextCreated[FCOIS.gFilterWhere] == false then
                FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere] = createContextMenuFilter(parentWindow, FCOIS.gFilterWhere , "SellGuildInt")
            end
            if FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere] ~= nil then
                FCOIS.preventerVars.gSellGuildIntFilterContextCreated[FCOIS.gFilterWhere] = true
                if FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:IsHidden() then
                    FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:ClearAnchors()
                    FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:SetAnchor(TOP,parentWindow,BOTTOM, 0, 8)
                    FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:SetHidden(false)
                else
                    FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:SetHidden(true)
                end
            end
        else
            hideContextMenu(FCOIS.gFilterWhere)
        end
    else
        return
    end
end

--Function to build the gear set split filter context menu
local function buildFilterButtonContextMenu(parentButton, p_FilterPanelId, contextMenuType)
	--Hide open context menus
  	hideContextMenu(p_FilterPanelId)

    p_FilterPanelId = p_FilterPanelId or FCOIS.gFilterWhere
    if parentButton == nil or p_FilterPanelId == nil or p_FilterPanelId == 0 then return end

	--Add the context menu to the gear set filter button, if enabled
    if contextMenuType == "gear" then
		if FCOIS.settingsVars.settings.splitGearSetsFilter then
			showContextMenuFilterButton(parentButton, contextMenuType)
		end

	--Add the context menu to the research filter button, if enabled
    elseif contextMenuType == "ResDecImp" then
		if FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then
            showContextMenuFilterButton(parentButton, contextMenuType)
		end

       --Add the context menu to the sell filter button, if enabled
    elseif contextMenuType == "SellGuildInt" then
        if FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then
            showContextMenuFilterButton(parentButton, contextMenuType)
        end

    else
    	return
    end
end

--Enable/Disable a filter/all filters
local function doFilter(onoff, p_button, filterId, beQuiet, doFilterBasicsPlayer, doUpdateButtonColorsAndTextures, onlyPlayerInvFilter, p_FilterPanelId)
	--Check if the current filter panel Id is given
    if (p_FilterPanelId == nil) then
		--For initialization, called from function EnableFilters()
		if (onoff == -100) then
        	p_FilterPanelId = LF_INVENTORY
	    else
	    	p_FilterPanelId = FCOIS.gFilterWhere
        end
    end
    --Check if the settings are enabled for the current panel
    p_FilterPanelId = getFilterWhereBySettings(p_FilterPanelId)

	--Set the last used filter Id
	FCOIS.lastVars.gLastFilterId[p_FilterPanelId] = filterId

	--Hide the button FCOIS.contextMenu if shown and button was clicked
    if p_button ~= nil then
	    hideContextMenu(p_FilterPanelId)
    end

    --Only perform the check if we are not initializing the addon, called from function EnableFilters()
    if (onoff ~= -100) then
        --Check if we are in the player inventory
        if ( p_FilterPanelId == LF_INVENTORY ) then
        	--we are in the player inventory (or in the banks at the deposit inventories, or at mail sending, or trading)
            onlyPlayerInvFilter 	= true
            doFilterBasicsPlayer	= true
        end
    end

	--============================================================================
	-- ABORT HERE IF the settings for the current filter panel Id is not enabled
	--============================================================================
	if FCOIS.settingsVars.settings.atPanelEnabled[p_FilterPanelId]["filters"] == nil or FCOIS.settingsVars.settings.atPanelEnabled[p_FilterPanelId]["filters"] == false then
        --Unregister the filter
    	unregisterFilters(filterId, onlyPlayerInvFilter, p_FilterPanelId)

		debugMessage("doFilter: !!! ABORT !!! " .. onoff .. ", filterId: " ..filterId .. ", FilterPanelId: " .. tostring(p_FilterPanelId) .. ", beQuiet: " .. tostring(beQuiet), false)

		--Give chat output if beQuiet is false
        if beQuiet == false then
        	outputFilterState(true, newFilterMethod, p_FilterPanelId, filterId, 'ABORT')
        end

		--Abort function here now
    	return
    end
	--============================================================================

	debugMessage("doFilter: " .. onoff .. ", filterId: " ..filterId .. ", beQuiet: " .. tostring(beQuiet) .. ", doFilterBasiscsPlayer: " .. tostring(doFilterBasicsPlayer) .. ", doUpdateButtonColorsAndTextures: " .. tostring(doUpdateButtonColorsAndTextures) .. ", onlyPlayerInvFilter: " .. tostring(onlyPlayerInvFilter) .. ", FilterPanelId: " .. tostring(p_FilterPanelId), false)

    --Fallback solution if filterPanelId is still empty
    if ( FCOIS.settingsVars.settings.splitFilters == true and (p_FilterPanelId == nil or p_FilterPanelId == 0) ) then
        d("[FCOIS] doFilter("..tostring(filterId).."): No filter panel ID given!")
    	return
    end

    local isFilterActive
	-- change the filter value in the FCOIS.settingsVars.settings
    if (onoff == 1) then
		isFilterActive = setSettingsIsFilterOn(filterId, true, p_FilterPanelId)
	elseif (onoff == 2) then
		isFilterActive = setSettingsIsFilterOn(filterId, false, p_FilterPanelId)
	elseif (onoff == -99) then
		isFilterActive = setSettingsIsFilterOn(filterId, -99, p_FilterPanelId)
    else
		-- Should the filter be changed to next state?
    	if (onoff == -1) then
            isFilterActive = getSettingsIsFilterOn(filterId, p_FilterPanelId)

        	--Filter is on? Turn it off
			if (isFilterActive == true) then
				isFilterActive = setSettingsIsFilterOn(filterId, false, p_FilterPanelId)
			--Filter is off? Only show filtered
			elseif (isFilterActive == false) then
				isFilterActive = setSettingsIsFilterOn(filterId, -99, p_FilterPanelId)
			--Filter only shows filtered? Turn it on
			else
				isFilterActive = setSettingsIsFilterOn(filterId, true, p_FilterPanelId)
			end
		--elseif (onoff == -100) then
            --For initialization (onoff = -100) the filter will be kept as read from the FCOIS.settingsVars.settings
        end
    end

	--Register / Unregister the libFilter filters
    local registerFiltersNow = false

   	--Are we using the old (only 1 panel) or new (different panels) filtering?
   	local newFilterMethod = FCOIS.settingsVars.settings.splitFilters

--------------------------------------------------------------------------------
-- Initializing from function EnableFilters() at e.g. addon loading
--------------------------------------------------------------------------------
	--Are we initializing from function Enablefilters() ?
	if (onoff == -100) then

		--Unregister all old filters if the addon is already loaded and filters have been registered before.
        --This happens only by function Enablefilters() called after addon has been fully loaded (e.g. the settings menu "Split filters")
        if FCOIS.addonVars.gAddonLoaded == true then
			--Did we change the FCOIS.settingsVars.settings "Split filters" and have to unregister all old "split filter" panel filters?
            if FCOIS.overrideVars.gSplitFilterOverride == true then
            	unregisterFilters(-1)
			else
            	unregisterFilters(filterId)
			end
        end

		--We are using filters with splitted panels
        if (newFilterMethod == true) then
			--Only update panel LF_INVENTORY (player inventory)
            local panels = LF_INVENTORY

			--Check for each panel if filters are enabled
			isFilterActive = nil
			isFilterActive = getSettingsIsFilterOn(filterId, panels)

			if    (isFilterActive == true) then
				--Filter is ON
				-- Set the filters to be registered
				registerFiltersNow = true
			elseif(isFilterActive == false) then
				--Filter is OFF
				-- Set the filters to stay unregistered
				registerFiltersNow = false
			else
				--Filter is ONLY SHOWING MARKED ITEMS (-99)
				-- Set the filters to be registered again
				registerFiltersNow = true
			end

			--Register the filters now
			if (registerFiltersNow == true) then
				registerFilters(filterId, onlyPlayerInvFilter, panels)
			end

			-- Update the colors of the 4 "player inventory" filter buttons. All others will be updated upon opening (on event)
			if (filterId == 1 and panels == LF_INVENTORY and doUpdateButtonColorsAndTextures == true) then
				UpdateButtonColorsAndTextures(-1, nil, -1, LF_INVENTORY)
			end

		else -- if (newFilterMethod == true) then

			--We are using filters for only 1 panel
			isFilterActive = nil
			isFilterActive = getSettingsIsFilterOn(filterId)

			if    (isFilterActive == true) then
				--Filter is ON
				-- Set the filters to be registered
				registerFiltersNow = true
			elseif(isFilterActive == false) then
				--Filter is OFF
				-- Set the filters to stay unregistered
				registerFiltersNow = false
			else
				--Filter is ONLY SHOWING MARKED ITEMS (-99)
				-- Set the filters to be registered again
				registerFiltersNow = true
			end

			--Register the filters now
			if (registerFiltersNow == true) then
				registerFilters(filterId, onlyPlayerInvFilter)
			end

			-- Update the colors of the 4 "player inventory" filter buttons. All others will be updated upon opening (on event)
			if (filterId == 1 and doUpdateButtonColorsAndTextures == true) then
				UpdateButtonColorsAndTextures(-1, nil, -1)
			end

        end


--------------------------------------------------------------------------------
-- Responding to a filter button OnClicked() event or a chat command
--------------------------------------------------------------------------------
	else -- if (onoff == -100) then


	--We are not initializing -> Check filter state
        --The NEW filter status was set by function setSettingsIsFilterOn() at the beginning of this function
        --be determined here once again
        if isFilterActive == nil then
			isFilterActive = getSettingsIsFilterOn(filterId, p_FilterPanelId)
			--Is the new filter status still not set initialize it with "false"
            if isFilterActive == nil then
				isFilterActive = setSettingsIsFilterOn(filterId, false, p_FilterPanelId)
            end
        end

		-- Filter is "OFF"
	    if(isFilterActive == false) then

	    	-- Output "filter off" text
			if (FCOIS.settingsVars.settings.deepDebug or (beQuiet == false and FCOIS.settingsVars.settings.showFilterStatusInChat == true)) then
				outputFilterState(true, newFilterMethod, p_FilterPanelId, filterId, 'off')
	        end

			-- Set the filters to stay unregistered
	        registerFiltersNow = false

            -- Unregister all old filters for the given filterId and panel
       		unregisterFilters(filterId, onlyPlayerInvFilter, p_FilterPanelId)

		-- Filter is "ON"
		elseif (isFilterActive == true) then

			-- Output "filter on" text
	        if (FCOIS.settingsVars.settings.deepDebug or (beQuiet == false and FCOIS.settingsVars.settings.showFilterStatusInChat == true)) then
				outputFilterState(true, newFilterMethod, p_FilterPanelId, filterId, 'on')
	        end

            -- Unregister all old filters for the given filterId and panel
       		unregisterFilters(filterId, onlyPlayerInvFilter, p_FilterPanelId)

			-- Set the filters to be registered
	        registerFiltersNow = true

		--Filter got value "-99"
		--Special treatment for filter to show only marked items
	    else

			-- Output "show only marked" text
	        if (FCOIS.settingsVars.settings.deepDebug or (beQuiet == false and FCOIS.settingsVars.settings.showFilterStatusInChat == true)) then
				outputFilterState(true, newFilterMethod, p_FilterPanelId, filterId, 'onlyfiltered')
	        end

            -- Unregister all old filters for the given filterId and panel
       		unregisterFilters(filterId, onlyPlayerInvFilter, p_FilterPanelId)

			-- Set the filters to be registered again
	        registerFiltersNow = true

	    end

--=====================================================================================================
		--Register filters now?
		if(registerFiltersNow == true) then
			--(Re)register the filter (for the given panel again)
			registerFilters(filterId, onlyPlayerInvFilter, p_FilterPanelId)
		end

    end -- if (onoff == -100) then

    --Refresh the inventories scroll lists (the inventories itsself are updated within libFilters until version r14!
    --> As version r15 was implemented the inventory refresh must be done within the addons!)
    --Only update if button was clicked manually or this is the last call to this function dofilter()
    --from function enableFilters() (at initialization of this addon e.g.)
    if ( onoff ~= -100 or (onoff == -100 and filterId == FCOIS.numVars.gFCONumFilters) ) then
		--Update all inventories (false) / only the player inventory (true)
		FilterBasics(doFilterBasicsPlayer)
    end

	--Update the colors of the changed buttons inside the inventory panels, but only
    --if we are not coming from addon initialization
    if (onoff ~= - 100 and doUpdateButtonColorsAndTextures == true) then
    	UpdateButtonColorsAndTextures(filterId, nil, isFilterActive, p_FilterPanelId)
    end

end

-- Scan inventory for quality of items and mark them as "saved" if quality is equal or higher
local function scanInventoryForQuality()
	if (FCOIS.settingsVars.settings.autoMarkQuality ~= 1 and FCOIS.preventerVars.gScanningInv == false) then
    	FCOIS.preventerVars.gScanningInv = true

		local atLeastOneQualityFound = false
	    --local bagId = BAG_BACKPACK
		local itemData
        local itemId
        --for each item in your player inventory
--        for slotIndex = 0, GetBagSize(bagId) - 1 do
		local bagCache = SHARED_INVENTORY:GenerateFullSlotData(nil, BAG_BACKPACK)
		for slotId, data in pairs(bagCache) do
        	local bagId 	= data.bagId
        	local slotIndex = data.slotIndex
			--Initialize variables again
			itemId = nil
			itemData = nil
			itemId = GetItemInstanceId(bagId, slotIndex)
            if (itemId ~= nil and (checkIfItemIsProtected(1, itemId) == false)) then
				--Check if item's quality is a selected, or higher one?
				local qualityCheck = false
               	local itemQuality = GetItemQuality(bagId, slotIndex)
                if FCOIS.settingsVars.settings.autoMarkHigherQuality and FCOIS.settingsVars.settings.autoMarkQuality < ITEM_QUALITY_LEGENDARY then
					if (itemQuality and itemQuality ~= false and itemQuality >= FCOIS.settingsVars.settings.autoMarkQuality) then
                		qualityCheck = true
                    end
                else
					if (itemQuality and itemQuality ~= false and itemQuality == FCOIS.settingsVars.settings.autoMarkQuality) then
                		qualityCheck = true
                    end
                end
                if (qualityCheck == true) then
	        		itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[slotIndex]
			    	if (itemData ~= nil) then
					   atLeastOneQualityFound = true
		            	FCOIS.markedItems[1][SignItemId(itemId)] = true
						if (FCOIS.settingsVars.settings.showQualityItemsInChat == true) then
							d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["quality_item_found"])
		                else
		    				debugMessage("[ScanInventoryForQuality]: " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["quality_item_found"], false)
						end
					end
				end
            end
		end
        FCOIS.preventerVars.gScanningInv = false

        --Was at least one ornate item found?
        if atLeastOneQualityFound == true then
		    FilterBasics(true)
        end
	end
end

-- Scan inventory for ornate items and mark them as "to sell"
local function scanInventoryForOrnate()
	if (FCOIS.settingsVars.settings.autoMarkOrnate == true and FCOIS.settingsVars.settings.isIconEnabled[5] and FCOIS.preventerVars.gScanningInv == false) then
    	FCOIS.preventerVars.gScanningInv = true

		local atLeastOneOrnateFound = false
	    --local bagId = BAG_BACKPACK
		local itemData
        local itemId
        --for each item in your player inventory
--        for slotIndex = 0, GetBagSize(bagId) - 1 do
		local bagCache = SHARED_INVENTORY:GenerateFullSlotData(nil, BAG_BACKPACK)
		for slotId, data in pairs(bagCache) do
        	local bagId 	= data.bagId
        	local slotIndex = data.slotIndex
			--Initialize variables again
			itemId = nil
			itemData = nil
			itemId = GetItemInstanceId(bagId, slotIndex)
            if (itemId ~= nil and (checkIfItemIsProtected(5, itemId) == false)) then
				--Check if item is ornate
			    if (isItemOrnate(bagId, slotIndex)) then
	        		itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[slotIndex]
			    	if (itemData ~= nil) then
					   atLeastOneOrnateFound = true
		            	FCOIS.markedItems[5][SignItemId(itemId)] = true
						if (FCOIS.settingsVars.settings.showOrnateItemsInChat == true) then
							d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["ornate_item_found"])
		                else
		    				debugMessage("[ScanInventoryForOrnate]: " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["ornate_item_found"], false)
						end
					end
				end
            end
		end
        FCOIS.preventerVars.gScanningInv = false

        --Was at least one ornate item found?
        if atLeastOneOrnateFound == true then
		    FilterBasics(true)
        end
	end
end

-- Scan inventory for intricate items and mark them as "intricate"
local function scanInventoryForIntricate()
    if (FCOIS.settingsVars.settings.autoMarkIntricate == true and FCOIS.settingsVars.settings.isIconEnabled[12] and FCOIS.preventerVars.gScanningInv == false) then
        FCOIS.preventerVars.gScanningInv = true

        local atLeastOneIntricateFound = false
        --local bagId = BAG_BACKPACK
        local itemData
        local itemId
        --for each item in your player inventory
--        for slotIndex = 0, GetBagSize(bagId) - 1 do
		local bagCache = SHARED_INVENTORY:GenerateFullSlotData(nil, BAG_BACKPACK)
		for slotId, data in pairs(bagCache) do
        	local bagId 	= data.bagId
        	local slotIndex = data.slotIndex
            --Initialize variables again
            itemId = nil
            itemData = nil
            itemId = GetItemInstanceId(bagId, slotIndex)
            if (itemId ~= nil and (checkIfItemIsProtected(12, itemId) == false)) then
                --Check if item is ornate
                if (isItemIntricate(bagId, slotIndex)) then
                    itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[slotIndex]
                    if (itemData ~= nil) then
                        atLeastOneIntricateFound = true
                        FCOIS.markedItems[12][SignItemId(itemId)] = true
                        if (FCOIS.settingsVars.settings.showIntricateItemsInChat == true) then
                            d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["intricate_item_found"])
                        else
                            debugMessage("[ScanInventoryForIntricate]: " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["intricate_item_found"], false)
                        end
                    end
                end
            end
        end
        FCOIS.preventerVars.gScanningInv = false

        --Was at least one ornate item found?
        if atLeastOneIntricateFound == true then
            FilterBasics(true)
        end
    end
end

--Scan inventory for researchable items, marked by addon ResearchAssistant
local function scanInventoryForResearch()
    --If the addon ResearchAssistant is active
    if (FCOIS.otherAddons.researchAssistantActive == true and FCOIS.settingsVars.settings.autoMarkResearch == true and FCOIS.settingsVars.settings.isIconEnabled[3] and FCOIS.preventerVars.gScanningInv == false) then
        FCOIS.preventerVars.gScanningInv = true

        local atLeastOneResearchableFound = false
        --local bagId = BAG_BACKPACK
        local itemData
        local itemId
        --for each item in your player inventory
--        for slotIndex = 0, GetBagSize(bagId) - 1 do
		local bagCache = SHARED_INVENTORY:GenerateFullSlotData(nil, BAG_BACKPACK)
		for slotId, data in pairs(bagCache) do
        	local bagId 	= data.bagId
        	local slotIndex = data.slotIndex
            --Check if item is researchable
            if isItemResearchableNoControl(bagId, slotIndex, nil) then
                --Initialize variables again
                itemId = nil
                itemData = nil
                itemId = GetItemInstanceId(bagId, slotIndex)
                if (itemId ~= nil and (checkIfItemIsProtected(3, itemId) == false)) then
                    itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[slotIndex]
                    if (itemData ~= nil) then
                        if (itemData.researchAssistant ~= nil and itemData.researchAssistant == 'researchable') then
                            atLeastOneResearchableFound = true
                            FCOIS.markedItems[3][SignItemId(itemId)] = true
                            if (FCOIS.settingsVars.settings.showResearchItemsInChat == true) then
                                d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["research_item_found"])
                            else
                                debugMessage("[ScanInventoryForResearch]: " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["research_item_found"], false)
                            end
                        end
                    end
                end
            end
        end

        FCOIS.preventerVars.gScanningInv = false

        --Was at least one researchable item found?
        if atLeastOneResearchableFound == true then
            FilterBasics(true)
        end

    end
end

--Scan inventory for set parts
local function scanInventoryForSets()
    --If the automatic marking of set parts is enabled
    if (FCOIS.settingsVars.settings.autoMarkSets == true and FCOIS.settingsVars.settings.isIconEnabled[FCOIS.settingsVars.settings.autoMarkSetsGearIconNr] and FCOIS.preventerVars.gScanningInv == false) then
        FCOIS.preventerVars.gScanningInv = true

        local atLeastOneSetPartFound = false
        --local bagId = BAG_BACKPACK
        local itemData
        local itemId
        --for each item in your player inventory
--        for slotIndex = 0, GetBagSize(bagId) - 1 do
		local bagCache = SHARED_INVENTORY:GenerateFullSlotData(nil, BAG_BACKPACK)
		for slotId, data in pairs(bagCache) do
        	local bagId 	= data.bagId
        	local slotIndex = data.slotIndex
            --Check if item is a set part
            if isItemSetPartNoControl(bagId, slotIndex) then
                --Initialize variables again
                itemId = nil
                itemData = nil
                itemId = GetItemInstanceId(bagId, slotIndex)

				local isProtected = true
                --If the option is enabled to check for all gear set icons: If the set part is alreay marked with any of the gear set icons it shouldn't be marked with another marker icon again
                if FCOIS.settingsVars.settings.autoMarkSetsCheckAllGearIcons then
					--Build the array with the gear set icon ids
					local iconIdArray = {}
					local count = 1
 					for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
	                	--Check if icon is a gear set icon and if it's enabled
                		if FCOIS.mappingVars.iconIsGear[i] then
                        	iconIdArray[count] = i
                            count = count + 1
                        end
                    end
                    if iconIdArray ~= nil and #iconIdArray > 0 then
		                isProtected = checkIfItemArrayIsProtected(iconIdArray, itemId)
					else
                    	isProtected = checkIfItemIsProtected(FCOIS.settingsVars.settings.autoMarkSetsGearIconNr, itemId)
                    end
                else
					isProtected = checkIfItemIsProtected(FCOIS.settingsVars.settings.autoMarkSetsGearIconNr, itemId)
                end
                if (itemId ~= nil and isProtected == false) then
                    itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[slotIndex]
                    if (itemData ~= nil) then
	                    atLeastOneSetPartFound = true
	                    FCOIS.markedItems[FCOIS.settingsVars.settings.autoMarkSetsGearIconNr][SignItemId(itemId)] = true
	                    if (FCOIS.settingsVars.settings.showSetsInChat == true) then
	                        d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["set_part_found"])
	                    else
	                        debugMessage("[ScanInventoryForResearch]: " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["set_part_found"], false)
	                    end
                    end
                end
            end
        end

        FCOIS.preventerVars.gScanningInv = false

        --Was at least one researchable item found?
        if atLeastOneSetPartFound == true then
            FilterBasics(true)
        end

    end
end

--Scan inventory for recipes and mrk unknown (compared by the help of addon SousChef)
local function scanInventoryForRecipes()
    --If the addon SousChef is active
    if (FCOIS.otherAddons.sousChefActive == true and SousChef.settings.showAltKnowledge and FCOIS.settingsVars.settings.autoMarkRecipes == true and FCOIS.settingsVars.settings.isIconEnabled[1] and FCOIS.preventerVars.gScanningInv == false) then
        FCOIS.preventerVars.gScanningInv = true
        local atLeastOneRecipeFound = false
        --local bagId = BAG_BACKPACK
        local itemData
        local itemId
        --for each item in your player inventory
--        for slotIndex = 0, GetBagSize(bagId) - 1 do
		local bagCache = SHARED_INVENTORY:GenerateFullSlotData(nil, BAG_BACKPACK)
		for slotId, data in pairs(bagCache) do
        	local bagId 	= data.bagId
        	local slotIndex = data.slotIndex
            --Check if item is an unknown recipe
            if isRecipeKnown(bagId, slotIndex) == false then
                --Initialize variables again
                itemId = nil
                itemData = nil
                itemId = GetItemInstanceId(bagId, slotIndex)
                if (itemId ~= nil and (checkIfItemIsProtected(1, itemId) == false)) then
                    itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[slotIndex]
                    if (itemData ~= nil) then
                        atLeastOneRecipeFound = true
                        FCOIS.markedItems[1][SignItemId(itemId)] = true
                        if (FCOIS.settingsVars.settings.showRecipesInChat == true) then
                            d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["unknown_recipe_found"])
                        else
                            debugMessage("[ScanInventoryForResearch]: " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["unknown_recipe_found"], false)
                        end
                    end
                end
            end
        end

        FCOIS.preventerVars.gScanningInv = false

        --Was at least one researchable item found?
        if atLeastOneRecipeFound == true then
            FilterBasics(true)
        end

    end
end

--Scan the inventory for ornate and/or researchable items
local function scanInventory(p_bagId, p_slotIndex)
    local updateInv = false

	-- Scan the whole inventory because no bagId and slotIndex are given
	if p_bagId == nil and p_slotIndex == nil then
		debugMessage("[ScanInventory] Start ALL", false)
		local bagCache = SHARED_INVENTORY:GenerateFullSlotData(nil, BAG_BACKPACK)
		for slotId, data in pairs(bagCache) do
            scanInventory(data.bagId, data.slotIndex)
		end
        return
    end

	--Automatic marking of ornate and/or researchable items (researchAssistant addon needed!) is activated?
	if ( (
    	       (FCOIS.settingsVars.settings.autoMarkOrnate == true and FCOIS.settingsVars.settings.isIconEnabled[5])
    		or (FCOIS.settingsVars.settings.autoMarkIntricate == true and FCOIS.settingsVars.settings.isIconEnabled[12])
        	or (FCOIS.otherAddons.researchAssistantActive == true and FCOIS.settingsVars.settings.autoMarkResearch == true and FCOIS.settingsVars.settings.isIconEnabled[3])
        	or (FCOIS.settingsVars.settings.autoMarkQuality ~= 1)
            or (FCOIS.otherAddons.sousChefActive == true and SousChef.settings.showAltKnowledge and FCOIS.settingsVars.settings.autoMarkRecipes == true and FCOIS.settingsVars.settings.isIconEnabled[1])
            or (FCOIS.settingsVars.settings.autoMarkSets == true and FCOIS.settingsVars.settings.isIconEnabled[FCOIS.settingsVars.settings.autoMarkSetsGearIconNr])
          )
		  and FCOIS.preventerVars.gScanningInv == false
    ) then

		debugMessage("[ScanInventory] Start", false)

	       	-- Update only one item in inventory
			-- bagId AND slotIndex are given?
	        if (p_bagId ~= nil and p_slotIndex ~= nil) then
				--Check if item is researchable
                local itemData
            	local itemId = MyGetItemInstanceIdNoControl(p_bagId, p_slotIndex)

                if (itemId ~= nil) then
					--Update ornate items
					if (FCOIS.settingsVars.settings.autoMarkOrnate == true and FCOIS.settingsVars.settings.isIconEnabled[5]) then
	                	if (checkIfItemIsProtected(5, itemId) == false) then
				        	if (p_bagId == BAG_BACKPACK) then
								--Check ornate
								if (isItemOrnate(p_bagId, p_slotIndex)) then
		                 		    itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[p_slotIndex]
                                    if (itemData ~= nil) then
		                                FCOIS.markedItems[5][SignItemId(itemId)] = true
										if (FCOIS.settingsVars.settings.showOrnateItemsInChat == true) then
											d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["ornate_item_found"])
										else
											debugMessage("[ScanInventory]: Ornate - " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["ornate_item_found"], false)
										end
										--Update the inventory
			                            updateInv = true
	                                end
								end
							end
	                    end
					end

                    --Update intricate items
                    if (FCOIS.settingsVars.settings.autoMarkIntricate == true and FCOIS.settingsVars.settings.isIconEnabled[12]) then
                        if (checkIfItemIsProtected(12, itemId) == false) then
                            if (p_bagId == BAG_BACKPACK) then
                                --Check ornate
                                if (isItemIntricate(p_bagId, p_slotIndex)) then
		                            if (itemData == nil) then
	                                    itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[p_slotIndex]
		                            end
                                    if (itemData ~= nil) then
                                        FCOIS.markedItems[12][SignItemId(itemId)] = true
                                        if (FCOIS.settingsVars.settings.showIntricateItemsInChat == true) then
                                            d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["intricate_item_found"])
                                        else
                                            debugMessage("[ScanInventory]: Intricate - " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["intricate_item_found"], false)
                                        end
                                        --Update the inventory
                                        updateInv = true
                                    end
                                end
                            end
                        end
                    end

					--Update researchable items
					if (FCOIS.otherAddons.researchAssistantActive == true and FCOIS.settingsVars.settings.autoMarkResearch == true and FCOIS.settingsVars.settings.isIconEnabled[3]) then
			            if isItemResearchableNoControl(p_bagId, p_slotIndex, nil) then
		                	if (checkIfItemIsProtected(3, itemId) == false) then
								--Check researchable
								if (p_bagId == BAG_BACKPACK) then
		                            if (itemData == nil) then
										itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[p_slotIndex]
		                            end
		                            if (itemData ~= nil) then
			                        	if (itemData.researchAssistant ~= nil and itemData.researchAssistant == 'researchable') then
											FCOIS.markedItems[3][SignItemId(itemId)] = true
											if (FCOIS.settingsVars.settings.showResearchItemsInChat == true) then
												d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["research_item_found"])
											else
												debugMessage("[ScanInventory]: Research - " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["research_item_found"], false)
											end
											--Update the inventory
				                            updateInv = true
										end
		                            end
								end
							end
						end  -- if isItemResearchableNoControl(...
			        end

                    --Check for item quality
                    if (FCOIS.settingsVars.settings.autoMarkQuality ~= 1) then
	                	if (checkIfItemIsProtected(1, itemId) == false) then
							--Check if item's quality is a selected, or higher one?
							local qualityCheck = false
		                    local itemQuality = GetItemQuality(p_bagId, p_slotIndex)
			                if FCOIS.settingsVars.settings.autoMarkHigherQuality and FCOIS.settingsVars.settings.autoMarkQuality < ITEM_QUALITY_LEGENDARY then
			                    if itemQuality and itemQuality ~= false and itemQuality >= FCOIS.settingsVars.settings.autoMarkQuality then
			                		qualityCheck = true
	                            end
			                else
	                        	if(itemQuality and itemQuality ~= false and itemQuality == FCOIS.settingsVars.settings.autoMarkQuality) then
		                			qualityCheck = true
				                end
							end
			                if (qualityCheck == true) then
	                        	if (itemData == nil) then
		                        	itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[p_slotIndex]
	                            end
		                        if (itemData ~= nil) then
		                            FCOIS.markedItems[1][SignItemId(itemId)] = true
		                            if (FCOIS.settingsVars.settings.showQualityItemsInChat == true) then
		                                d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["quality_item_found"])
		                            else
		                                debugMessage("[ScanInventory]: Quality - " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["quality_item_found"], false)
		                            end
		                            --Update the inventory
		                            updateInv = true
		                        end
		                    end
						end
                    end

                    --Update recipes
                    if (FCOIS.otherAddons.sousChefActive == true and SousChef.settings.showAltKnowledge and FCOIS.settingsVars.settings.autoMarkRecipes == true and FCOIS.settingsVars.settings.isIconEnabled[1]) then
                        if (checkIfItemIsProtected(1, itemId) == false) then
                            if (p_bagId == BAG_BACKPACK) then
                                --Check ornate
                                if (isRecipeKnown(p_bagId, p_slotIndex) == false) then
		                            if (itemData == nil) then
	                                    itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[p_slotIndex]
		                            end
                                    if (itemData ~= nil) then
                                        FCOIS.markedItems[1][SignItemId(itemId)] = true
                                        if (FCOIS.settingsVars.settings.showRecipesInChat == true) then
                                            d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["unknown_recipe_found"])
                                        else
                                            debugMessage("[ScanInventory]: Recipes - " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["unknown_recipe_found"], false)
                                        end
                                        --Update the inventory
                                        updateInv = true
                                    end
                                end
                            end
                        end
                    end

                    --Mark set parts
                    if (FCOIS.settingsVars.settings.autoMarkSets == true and FCOIS.settingsVars.settings.isIconEnabled[FCOIS.settingsVars.settings.autoMarkSetsGearIconNr]) then
						local isProtected = true
		                --If the option is enabled to check for all gear set icons: If the set part is alreay marked with any of the gear set icons it shouldn't be marked with another marker icon again
                        if FCOIS.settingsVars.settings.autoMarkSetsCheckAllGearIcons then
							--Build the array with the gear set icon ids
							local iconIdArray = {}
							local count = 1
		 					for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
			                	--Check if icon is a gear set icon and if it's enabled
		                		if FCOIS.mappingVars.iconIsGear[i] then
		                        	iconIdArray[count] = i
		                            count = count + 1
		                        end
		                    end
		                    if iconIdArray ~= nil and #iconIdArray > 0 then
				                isProtected = checkIfItemArrayIsProtected(iconIdArray, itemId)
							else
		                    	isProtected = checkIfItemIsProtected(FCOIS.settingsVars.settings.autoMarkSetsGearIconNr, itemId)
		                    end
		                else
							isProtected = checkIfItemIsProtected(FCOIS.settingsVars.settings.autoMarkSetsGearIconNr, itemId)
		                end
                        if (isProtected == false) then
                            if (p_bagId == BAG_BACKPACK) then
                                --Check ornate
                                if (isItemSetPartNoControl(p_bagId, p_slotIndex)) then
		                            if (itemData == nil) then
	                                    itemData = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots[p_slotIndex]
		                            end
                                    if (itemData ~= nil) then
                                        FCOIS.markedItems[FCOIS.settingsVars.settings.autoMarkSetsGearIconNr][SignItemId(itemId)] = true
                                        if (FCOIS.settingsVars.settings.showSetsInChat == true) then
                                            d(FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["set_part_found"])
                                        else
                                            debugMessage("[ScanInventory]: Set parts - " .. FCOIS.localizationVars.fcois_loc["marked"] .. itemData.name .. FCOIS.localizationVars.fcois_loc["set_part_found"], false)
                                        end
                                        --Update the inventory
                                        updateInv = true
                                    end
                                end
                            end
                        end
                    end

				end -- if (itemId ~= nil) then
			end --if (p_bagId ~= nil and p_slotIndex ~= nil) then

		--Update the inventories
        if updateInv == true then
        	FilterBasics(true)
        end

    end --if ( (FCOIS.settingsVars.settings.autoMarkOrnate == true and FCOIS.preventerVars.gScanningInv == false) or...

	debugMessage("[ScanInventory] End", false)

	--Inventory scan is latest finished here
	FCOIS.preventerVars.gScanningInv = false

end

--Enable the filters
local function EnableFilters(p_onoff)
	-- Enable the filters for the different panels (inventory, deconstruction, guild store, etc.)
    for filters = 1, FCOIS.numVars.gFCONumFilters, 1 do
		--function doFilter(onoff, p_button, filterId, beQuiet, doFilterBasicsPlayer, doUpdateButtonColorsAndTextures, onlyPlayerInvFilter, p_FilterPanelId)
    	doFilter(p_onoff, nil, filters, true, true, true, true)
    end
	--Reset the override flag for unregistering all filters if FCOIS.settingsVars.settings "Split filters" has changed
	FCOIS.overrideVars.gSplitFilterOverride = false
end

--Show a help inside the chat
local function help()
	d(FCOIS.localizationVars.fcois_loc["chatcommands_info"])
	d("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
	d(FCOIS.localizationVars.fcois_loc["chatcommands_help"])
	d(FCOIS.localizationVars.fcois_loc["chatcommands_status"])
	d(FCOIS.localizationVars.fcois_loc["chatcommands_split_filter"])
	if (FCOIS.settingsVars.settings.splitFilters == true) then
        d(FCOIS.localizationVars.fcois_loc["chatcommands_filterpanels"])
        d(FCOIS.localizationVars.fcois_loc["chatcommands_filterpanels2"])
        d(FCOIS.localizationVars.fcois_loc["chatcommands_filtervalues"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filter1_new"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filter2_new"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filter3_new"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filter4_new"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filter_new"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filteron_new"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filteroff_new"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filtershow_new"])
    else
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filter1"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filter2"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filter3"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filter4"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filter"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filteron"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filteroff"])
		d(FCOIS.localizationVars.fcois_loc["chatcommands_filtershow"])
    end
    d(FCOIS.localizationVars.fcois_loc["chatcommands_debug"])
end

--Helper function for function filterStatus
local function filterStatusLoop(filterId, silent, givenArray, p_atLeastOneFilterActive)
    local returnArray = {}
    local atLeastOneFilterActive = p_atLeastOneFilterActive

	--Is an array already prepared?
	if #givenArray == 0 then
		--Create 2-dimensional arrays for the filters
	    for help_inv = 1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
			if FCOIS.mappingVars.activeFilterPanelIds[help_inv] == true then
        	    returnArray[help_inv] = {false, false, false, false}
            end
	    end
    else
	    --Array was already given, so use it
		returnArray = givenArray
    end

   	-- Check only one filter
	if (FCOIS.settingsVars.settings.splitFilters == true) then
		for j=1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
        	if FCOIS.mappingVars.activeFilterPanelIds[j] == true then
				if(getSettingsIsFilterOn(filterId, j)) then
					returnArray[j][filterId] = true
	                atLeastOneFilterActive = true
					if     (j == LF_INVENTORY or j == LF_BANK_DEPOSIT or j == LF_GUILDBANK_DEPOSIT) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowInventoryFilter
						if (not silent) then
							d(FCOIS.localizationVars.fcois_loc["filter_inventory"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end

					elseif (j == LF_CRAFTBAG) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowCraftBagFilter
						if (not silent) then
							d(FCOIS.localizationVars.fcois_loc["filter_craftbag"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end

					elseif     (j == LF_VENDOR_SELL) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowVendorFilter
						if (not silent) then
						d(FCOIS.localizationVars.fcois_loc["filter_store"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					elseif     (j == LF_FENCE_SELL) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowFenceFilter
						if (not silent) then
						d(FCOIS.localizationVars.fcois_loc["filter_fence"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					elseif     (j == LF_FENCE_LAUNDER) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowLaunderFilter
						if (not silent) then
						d(FCOIS.localizationVars.fcois_loc["filter_launder"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					elseif (j == LF_GUILDBANK_WITHDRAW) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowGuildBankFilter
						if (not silent) then
							d(FCOIS.localizationVars.fcois_loc["filter_guildbank"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					elseif (j == LF_GUILDSTORE_SELL) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowTradinghouseFilter
						if (not silent) then
							d(FCOIS.localizationVars.fcois_loc["filter_guildstore"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					elseif (j == LF_BANK_WITHDRAW) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowBankFilter
						if (not silent) then
							d(FCOIS.localizationVars.fcois_loc["filter_bank"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					elseif (j == LF_SMITHING_DECONSTRUCT) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowDeconstructionFilter
						if (not silent) then
							d(FCOIS.localizationVars.fcois_loc["filter_deconstruction"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					elseif (j == LF_SMITHING_IMPROVEMENT) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowImprovementFilter
						if (not silent) then
							d(FCOIS.localizationVars.fcois_loc["filter_improvement"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					elseif (j == LF_MAIL_SEND) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowMailFilter
						if (not silent) then
							d(FCOIS.localizationVars.fcois_loc["filter_mail"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					elseif (j == LF_TRADE) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowTradeFilter
						if (not silent) then
							d(FCOIS.localizationVars.fcois_loc["filter_trade"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					elseif (j == LF_ENCHANTING_EXTRACTION) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowEnchantingFilter
						if (not silent) then
							d(FCOIS.localizationVars.fcois_loc["filter_enchantingstation_extraction"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					elseif (j==LF_ENCHANTING_CREATION) then
						returnArray[j][filterId] = FCOIS.settingsVars.settings.allowEnchantingFilter
						if (not silent) then
							d(FCOIS.localizationVars.fcois_loc["filter_enchantingstation_creation"] .. FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
						end
					end
				end -- if(getSettingsIsFilterOn(filterId, j)) then
			end
		end -- for j=1, ...

	else --if (FCOIS.settingsVars.settings.splitFilters == true)

		if(getSettingsIsFilterOn(filterId)) then
               returnArray[1][filterId] = true
               atLeastOneFilterActive = true
			if (not silent) then
				d(FCOIS.localizationVars.fcois_loc["chatcommands_status_filter" .. tostring(filterId)])
			end
		end

    end --if (FCOIS.settingsVars.settings.splitFilters == true)


    return returnArray, atLeastOneFilterActive
end

--Check a filter status
local function filterStatus(filterId, silent, doReturnFilterStatus)
	--Prepare the return array
    local retArray = {}
    local atLeastOneFilterActive = false

	if (filterId ~= -1) then
		retArray, atLeastOneFilterActive = filterStatusLoop(filterId, silent)
    else
    	-- Check all filters
		for i=1, FCOIS.numVars.gFCONumFilters, 1 do
        	retArray, atLeastOneFilterActive = filterStatusLoop(i, silent, retArray, atLeastOneFilterActive)
		end
    end

	--Was at least one active filter found and chat output is enabled?
    if atLeastOneFilterActive == false and silent == false then
    	d(FCOIS.localizationVars.fcois_loc["chatcommands_status_nofilters"])
    end

	--Return the array with filter states?
    if (doReturnFilterStatus == true) then
	    return retArray
    end
end

--Show a status inside the chat
local function status()
	d(FCOIS.localizationVars.fcois_loc["chatcommands_status_info"])

    --Check all filters, not silently (chat output: enabled)
    filterStatus(-1, false, false)

	if (FCOIS.settingsVars.settings.debug == true) then
		d(FCOIS.localizationVars.fcois_loc["chatcommands_debug_on"])
	else
		d(FCOIS.localizationVars.fcois_loc["chatcommands_debug_off"])
	end
end

--Check the commands ppl type to the chat
local function command_handler(args)
    --Parse the arguments string
	local options = {}
    local searchResult = { string.match(args, "^(%S*)%s*(.-)$") }
    for i,v in pairs(searchResult) do
        if (v ~= nil and v ~= "") then
            options[i] = string.lower(v)
        end
    end

	local opt3 = false
	if (options[3] ~= nil and (options[3] == false or options[3] == true or options[3] == "show")) then
		opt3 = true
		if 	   (options[3]=="show" or options[3]=="montre" or options[3]=="zeigen") then
			options[3] = -99
		elseif (options[3]==true or options[3]=="true" or options[3]=="vrai" or options[3]=="an") then
			options[3] = 1
		elseif (options[3]==false or options[3]=="false" or options[3]=="faux" or options[3]=="aus") then
			options[3] = 2
		end
	end

	if(options[1] == "help" or options[1] == "hilfe" or options[1] == "list" or options[1] == "aide") then
       	help()
	elseif(#options == 0 or options[1] == "status" or options[1] == "") then
       	status()
    elseif(options[1] == "filter1" or options[1] == "filtre1") then
        if (opt3) then
	    	doFilter(tonumber(options[3]), nil, 1, false, false, true, false, tonumber(options[2]))
        else
	        if (options[2] ~= nil) then
		    	doFilter(-1, nil, 1, false, false, true, false, tonumber(options[2]))
	        else
		    	doFilter(-1, nil, 1, false, false, true, false)
	        end
        end
	elseif(options[1] == "filter2" or options[1] == "filtre2") then
        if (opt3) then
	    	doFilter(tonumber(options[3]), nil, 2, false, false, true, false, tonumber(options[2]))
        else
	        if (options[2] ~= nil) then
		        doFilter(-1, nil, 2, false, false, true, false, tonumber(options[2]))
	        else
		        doFilter(-1, nil, 2, false, false, true, false)
	        end
        end
	elseif(options[1] == "filter3" or options[1] == "filtre3") then
        if (opt3) then
	    	doFilter(tonumber(options[3]), nil, 3, false, false, true, false, tonumber(options[2]))
        else
	        if (options[2] ~= nil) then
		    	doFilter(-1, nil, 3, false, false, true, false, tonumber(options[2]))
	        else
		    	doFilter(-1, nil, 3, false, false, true, false)
	        end
		end
	elseif(options[1] == "filter4" or options[1] == "filtre4") then
        if (opt3) then
	    	doFilter(tonumber(options[3]), nil, 4, false, false, true, false, tonumber(options[2]))
        else
	        if (options[2] ~= nil) then
		    	doFilter(-1, nil, 4, false, false, true, false, tonumber(options[2]))
	        else
		    	doFilter(-1, nil, 4, false, false, true, false)
	        end
		end
	elseif(options[1] == "filtern" or options[1] == "filter" or options[1] == "filtres") then
        if (opt3) then
	    	for i=1, FCOIS.numVars.gFCONumFilters, 1 do
		    	doFilter(tonumber(options[3]), nil, i, false, false, true, false, tonumber(options[2]))
            end
        else
	    	for i=1, FCOIS.numVars.gFCONumFilters, 1 do
		        if (options[2] ~= nil) then
			    	doFilter(-1, nil, i, false, false, true, false, tonumber(options[2]))
		        else
			    	doFilter(-1, nil, i, false, false, true, false)
		        end
	        end
		end
	elseif(options[1] == "allean" or options[1] == "allon" or options[1] == "touson") then
		if (FCOIS.settingsVars.settings.splitFilters == true) then
	    	for i=1, FCOIS.numVars.gFCONumFilters, 1 do
				for j=1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
					if FCOIS.mappingVars.activeFilterPanelIds[j] == true then
						if (options[2] ~= nil) then
							doFilter(1, nil, i, false, false, true, false, j, tonumber(options[2]))
						else
							doFilter(1, nil, i, false, false, true, false, j)
						end
                    end
                end
	        end
        else
	    	for i=1, FCOIS.numVars.gFCONumFilters, 1 do
		    	doFilter(1, nil, i, false, false, true, false)
	        end
        end
	elseif(options[1] == "alleaus" or options[1] == "alloff" or options[1] == "tousoff") then
		if (FCOIS.settingsVars.settings.splitFilters == true) then
	    	for i=1, FCOIS.numVars.gFCONumFilters, 1 do
				for j=1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
					if FCOIS.mappingVars.activeFilterPanelIds[j] == true then
						if (options[2] ~= nil) then
							doFilter(2, nil, i, false, false, true, false, j, tonumber(options[2]))
						else
							doFilter(2, nil, i, false, false, true, false, j)
						end
                    end
                end
	        end
        else
	    	for i=1, FCOIS.numVars.gFCONumFilters, 1 do
		    	doFilter(2, nil, i, false, false, true, false)
	        end
        end
	elseif(options[1] == "allezeigen" or options[1] == "allshow" or options[1] == "tousmontre") then
		if (FCOIS.settingsVars.settings.splitFilters == true) then
	    	for i=1, FCOIS.numVars.gFCONumFilters, 1 do
				for j=1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
					if FCOIS.mappingVars.activeFilterPanelIds[j] == true then
						if (options[2] ~= nil) then
							doFilter(-99, nil, i, false, false, true, false, j, tonumber(options[2]))
						else
							doFilter(-99, nil, i, false, false, true, false, j)
						end
					end
                end
	        end
        else
	    	for i=1, FCOIS.numVars.gFCONumFilters, 1 do
		    	doFilter(-99, nil, i, false, false, true, false)
	        end
        end
    elseif(options[1] == "split") then
    	FCOIS.settingsVars.settings.splitFilters = not FCOIS.settingsVars.settings.splitFilters
		--Set the global flag to override the function's unregisterFilters() unregister method
		--because it will check for FCOIS.settingsVars.settings.SplitFilters and is called from function dofilter() at
		--function EnableFilters(-100) below
		FCOIS.overrideVars.gSplitFilterOverride = true
		EnableFilters(-100)
    elseif(options[1] == "debug") then
    	FCOIS.settingsVars.settings.debug = not FCOIS.settingsVars.settings.debug
        if (FCOIS.settingsVars.settings.debug == true) then
        	d(FCOIS.localizationVars.fcois_loc["chatcommands_debug_on"])
        else
        	FCOIS.settingsVars.settings.deepDebug = false
        	d(FCOIS.localizationVars.fcois_loc["chatcommands_debug_off"])
        end
    elseif(options[1] == "deepdebug" or options[1] == "dd") then
        FCOIS.settingsVars.settings.deepDebug = not FCOIS.settingsVars.settings.deepDebug
        if (FCOIS.settingsVars.settings.deepDebug == true) then
			FCOIS.settingsVars.settings.debug = true
        	d(FCOIS.localizationVars.fcois_loc["chatcommands_deepdebug_on"])
        else
			FCOIS.settingsVars.settings.debug = false
        	d(FCOIS.localizationVars.fcois_loc["chatcommands_deepdebug_off"])
        end
    end
end


--Add the filter buttons to the inventory/bank/mail/trade/guild bank/guild store/crafting stations panels
--and change them upon opening a new filter panel (update the button id, button filter panel, tooltips, callback functions
local function AddOrChangeFilterButton(parentWindow, buttonId, pWidth, pHeight, pLeft, pTop, hide, p_FilterPanelId)
	--Get the current filter panel Id
    p_FilterPanelId = p_FilterPanelId or FCOIS.gFilterWhere

    debugMessage("[AddOrChangeFilterButton] " .. parentWindow:GetName() .. ", buttonId: " .. buttonId .. ", filterPanelId: " .. p_FilterPanelId .. ", hide: " .. tostring(hide), true)

    local tooltipText
	local button = parentWindow:GetNamedChild("_FilterButton" .. tostring(buttonId))

	--Hide the button?
	if hide then
	    if button then
	        button:SetHidden(true)
	    end
		-- If we reach here hide is enabled. Return nil then
	    return nil
	end

    local buttonExists = true
	--Create the button?
	if not button then
    	buttonExists = false
	    -- create it
		button = WINDOW_MANAGER:CreateControl(parentWindow:GetName() .. "_FilterButton" .. tostring(buttonId), parentWindow, CT_BUTTON)
        if not button then return nil end

        debugMessage("+++ AddOrChangeFilterButton: ADD ButtonName=" .. button:GetName() .. ", Width/Height: " .. pWidth .. "/" .. pHeight .. ", Left/Top: " .. pLeft .. "/" .. pTop, true)

        --Create the texture for the button to hold the image
	    local texture = WINDOW_MANAGER:CreateControl(button:GetName() .. "Texture", button, CT_TEXTURE)
	    texture:SetAnchorFill()
        --Are the inventory filter buttons split into several filter ids + context menu?
		if FCOIS.settingsVars.settings.splitGearSetsFilter and buttonId == 2 then
			texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.textureVars.allGearSets])
		elseif FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter and buttonId == 3 then
			texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.textureVars.allResDecImp])
		elseif FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter and buttonId == 4 then
			texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.textureVars.allSellGuildInt])
		else
			--Workaround to show at least a default texure, if none is found
			if (FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[buttonId].texture] ~= nil) then
				--Set the texture now
				texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[buttonId].texture])
			else
				--Set fallback texture now
				texture:SetTexture(FCOIS.textureVars.MARKER_TEXTURES[buttonId])
				--Set the fallback texture to the settings menu
				FCOIS.settingsVars.settings.icon[buttonId].texture = buttonId
			end
		end
	end -- if not button then

	--Update the button's personal variables
    button.FCOfilterPanelId = p_FilterPanelId
    button.FCObuttonId		= buttonId

	--Button already existed?
	if buttonExists then
        debugMessage("-> AddOrChangeFilterButton: CHANGE ButtonName=" .. button:GetName() .. ", Width/Height: " .. pWidth .. "/" .. pHeight .. ", Left/Top: " .. pLeft .. "/" .. pTop, true)
    end

	--Set/Update handlers
	--Set/Update a tooltip?
    button:SetHandler("OnMouseEnter", function(self)
       	if FCOIS.settingsVars.settings.showFilterButtonTooltip == true then
       		tooltipText = outputFilterState(false, FCOIS.settingsVars.settings.splitFilters, self.FCOfilterPanelId, self.FCObuttonId, FCOIS.mappingVars.settingsFilterStateToText[tostring(getSettingsIsFilterOn(self.FCObuttonId, self.FCOfilterPanelId))])
			if tooltipText ~= "" then
				local showToolTip = true
				--Don't show a tooltip if the context menu for gear sets is shown at the filter button
				if FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere] ~= nil then
					showToolTip = FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:IsHidden()
				end
				--Don't show a tooltip if the context menu for research, deconstruction & improvement is shown at the filter button
				if showToolTip and FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere] ~= nil then
					showToolTip = FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:IsHidden()
				end
                      if showToolTip and FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere] ~= nil then
                          showToolTip = FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:IsHidden()
                      end
				if showToolTip then
					ZO_Tooltips_ShowTextTooltip(self, BOTTOM, tooltipText)
				end
			end
		else
			ZO_Tooltips_HideTextTooltip()
        end
	end)
	button:SetHandler("OnMouseExit", function(self)
		ZO_Tooltips_HideTextTooltip()
	end)

	--Overwrite the callback function of the button so it is channging the correct panel filter
    --as mail, trade, bank, guild bank, and others all use the ZO_PlayerInventory buttons to filter!
    button:SetHandler("OnClicked", function(self)
		debugMessage("=========>", true)
		debugMessage("[FilterButtonOnClick]: ButtonName: " .. self:GetName() .. ", ButtonId: " .. self.FCObuttonId .. ", PanelId (global): " .. FCOIS.gFilterWhere .. ", PanelId (button): " .. self.FCOfilterPanelId, true)

		doFilter(-1, self, self.FCObuttonId, false, false, true, false, self.FCOfilterPanelId)
		if FCOIS.settingsVars.settings.showFilterButtonTooltip then
			tooltipText = outputFilterState(false, FCOIS.settingsVars.settings.splitFilters, self.FCOfilterPanelId, self.FCObuttonId, FCOIS.mappingVars.settingsFilterStateToText[tostring(getSettingsIsFilterOn(self.FCObuttonId, self.FCOfilterPanelId))])
			if tooltipText ~= "" then
				ZO_Tooltips_ShowTextTooltip(self, BOTTOM, tooltipText)
			end
		else
			--Hide the tooltip
			ZO_Tooltips_HideTextTooltip()
		end
       end)

       --Set on mouse up handler for the button
	button:SetHandler("OnMouseUp", function(self, mouseButton, upInside)
		--button 1= left mouse button / 2= right mouse button
		local doBuildContextMenu = false
		--Right click/mouse button 2 context menu hook part:
		if mouseButton == 2 and upInside then
			--Hide the tooltip
			ZO_Tooltips_HideTextTooltip()

			--Build the context menu for the gear sets filter button, if activated
		    --Only do this if the right mouse button was pressed and filter button ID is 2 (gear sets)
		    if buttonId == 2 and FCOIS.settingsVars.settings.splitGearSetsFilter then
				--Hide the gear sets split filter button context-menu
				if FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere] ~= nil then
					if not FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:IsHidden() then
						FCOIS.contextMenu.GearSetFilter[FCOIS.gFilterWhere]:SetHidden(true)
                       else
                          	doBuildContextMenu = true
                       end
                   else
                      	doBuildContextMenu = true
                   end
				if doBuildContextMenu then
                       --Build and show the context menu for the gear sets
                       buildFilterButtonContextMenu(self, self.FCOfilterPanelId, "gear")
                   end

			--Build the context menu for the research filter button, if activated
		    --Only do this if the right mouse button was pressed and filter button ID is 3 (research)
		    elseif buttonId == 3 and FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter then
				--Hide the RESEARCH & DECONSTRUCTION & IMPORVEMENT button context-menu
				if FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere] ~= nil then
					if not FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:IsHidden() then
						FCOIS.contextMenu.ResDecImpFilter[FCOIS.gFilterWhere]:SetHidden(true)
                       else
						doBuildContextMenu = true
					end
				else
					doBuildContextMenu = true
				end

				if doBuildContextMenu then
                       --Build and show the context menu for the research, deconstruction & improvement icons
                       buildFilterButtonContextMenu(self, self.FCOfilterPanelId, "ResDecImp")
                   end

			--Build the context menu for the sell filter button, if activated
			--Only do this if the right mouse button was pressed and filter button ID is 4 (sell)
			elseif buttonId == 4 and FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter then
				--Hide the SELL & SELL AT GUILD STORE & INTRICATE button context-menu
				if FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere] ~= nil then
					if not FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:IsHidden() then
						FCOIS.contextMenu.SellGuildIntFilter[FCOIS.gFilterWhere]:SetHidden(true)
					else
						doBuildContextMenu = true
					end
				else
					doBuildContextMenu = true
				end

				if doBuildContextMenu then
					--Build and show the context menu for the sell, sell at guild store & intricate icons
					buildFilterButtonContextMenu(self, self.FCOfilterPanelId, "SellGuildInt")
				end
			end
		end
	end)

	-- setup & modify button's size, position etc.
    button:SetDimensions(pWidth, pHeight)
	--Move the button more to the right, if standard left values are used and GridView Addon is activated
	if (FCOIS.otherAddons.inventoryGridViewActive == true
		and ( FCOIS.settingsVars.settings.filterButtonLeft[1] == FCOIS.filterButtonVars.gFilterButtonLeft[1] and FCOIS.settingsVars.settings.filterButtonLeft[2] == FCOIS.filterButtonVars.gFilterButtonLeft[2]
		and FCOIS.settingsVars.settings.filterButtonLeft[3] == FCOIS.filterButtonVars.gFilterButtonLeft[3] and FCOIS.settingsVars.settings.filterButtonLeft[4] == FCOIS.filterButtonVars.gFilterButtonLeft[4]) ) then
		pLeft = pLeft + FCOIS.otherAddons.gGriedViewOffsetX
	end
	--Place the buttons at the bottom of the inventory.
	--Special treatment for improvement panel here, because the "Booster container" is located at the bottom and the buttons
	--will be shown above him, not below (as he gives the BOTTOM anchor) of the inventory
	if (p_FilterPanelId ~= LF_SMITHING_IMPROVEMENT) then
		button:SetAnchor(TOP, parentWindow, BOTTOM, pLeft, pTop)
	else
		pTop = pTop + FCOIS.filterButtonVars.buttonOffsetYImprovement
		button:SetAnchor(TOP, FCOIS.ZOControlVars.IMPROVEMENT_BOOSTER_CONTAINER, BOTTOM, pLeft, pTop)
	end
	button:SetFont("ZoFontGameSmall")

	--Show the button and make it react on mouse input
	button:SetHidden(false)
	button:SetMouseEnabled(true)

	--Return the new created/changed button control
	return button
end

--Create the textures inside inventories etc.
local function CreateTextures(whichTextures)

	local doCreateMarkerControl = false
	if whichTextures == -1 then
    	doCreateMarkerControl = true
    end

	if (whichTextures == 1 or whichTextures == -1) then
	    --Create textures in inventories
	    for _,v in pairs(PLAYER_INVENTORY.inventories) do
			local listView = v.listView
			--Do not hook quest items
			if (listView and listView.dataTypes and listView.dataTypes[1] and (listView:GetName() ~= "ZO_PlayerInventoryQuest")) then
				local hookedFunctions = listView.dataTypes[1].setupCallback

				listView.dataTypes[1].setupCallback =
					function(rowControl, slot)
						hookedFunctions(rowControl, slot)
                        --Do not execute if horse is changed
						if SCENE_MANAGER:GetCurrentScene() ~= STABLES_SCENE then
		                    -- for all filters: Create/Update the icons
							for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
								CreateMarkerControl(rowControl, i, FCOIS.settingsVars.settings.icon[i].size, FCOIS.settingsVars.settings.icon[i].size, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[i].texture], false, doCreateMarkerControl)
		                    end
	                        --Add additional FCO point to the dataEntry.data slot
	                        --FCOItemSaver_AddInfoToData(rowControl)
                        end
					end
			end
		end
	end

	if (whichTextures == 2 or whichTextures == -1) then
	    --Create textures in repair window
        local listView = FCOIS.ZOControlVars.REPAIR_LIST
		if listView and listView.dataTypes and listView.dataTypes[1] then
			local hookedFunctions = listView.dataTypes[1].setupCallback

			listView.dataTypes[1].setupCallback =
				function(rowControl, slot)
					hookedFunctions(rowControl, slot)

					--Do not execute if horse is changed
					if SCENE_MANAGER:GetCurrentScene() ~= STABLES_SCENE then
	                    -- for all filters: Create/Update the icons
						for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
							CreateMarkerControl(rowControl, i, FCOIS.settingsVars.settings.icon[i].size, FCOIS.settingsVars.settings.icon[i].size, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[i].texture], false, doCreateMarkerControl)
						end
	                    --Add additional FCO point to the dataEntry.data slot
	                    --FCOItemSaver_AddInfoToData(rowControl)
                    end
				end
		end
	end

	if (whichTextures == 3 or whichTextures == -1) then
	    -- Marker function for character equipment if character window is shown
        if (not FCOIS.ZOControlVars.CHARACTER:IsHidden() or FCOIS.addonVars.gAddonLoaded == false) then
	        RefreshEquipmentControl()
        end
	end

	if (whichTextures == 4 or whichTextures == -1) then
	    -- Marker function for quickslots inventory
        local listView = FCOIS.ZOControlVars.QUICKSLOT_LIST
		if listView and listView.dataTypes and listView.dataTypes[1] then
			local hookedFunctions = listView.dataTypes[1].setupCallback

			listView.dataTypes[1].setupCallback =
				function(rowControl, slot)
					hookedFunctions(rowControl, slot)

					--Do not execute if horse is changed
					if SCENE_MANAGER:GetCurrentScene() ~= STABLES_SCENE then
						-- for all filters: Create/Update the icons
						for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
							CreateMarkerControl(rowControl, i, FCOIS.settingsVars.settings.icon[i].size, FCOIS.settingsVars.settings.icon[i].size, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[i].texture], false, doCreateMarkerControl)
						end
						--Add additional FCO point to the dataEntry.data slot
						--FCOItemSaver_AddInfoToData(rowControl)
                    end
				end
		end
	end

end

--Check if markers should be updated
local function checkMarker(markerId)
	markerId = markerId or -1
   	debugMessage("[CheckMarker] MarkerId: " .. tostring(markerId) .. ", CheckNow: " .. tostring(FCOIS.preventerVars.gUpdateMarkersNow) .. ", Gears changed: " .. tostring(FCOIS.preventerVars.gChangedGears), true)

	--Should we update the marker textures, size and color?
	if FCOIS.preventerVars.gUpdateMarkersNow == true or FCOIS.preventerVars.gChangedGears == true then
		--Update the textures now
		RefreshBackpack()
		RefreshBank()
		RefreshGuildBank()

		if not FCOIS.preventerVars.gChangedGears then
			zo_callLater(function()
				--d("character hidden: " .. tostring(FCOIS.ZOControlVars.CHARACTER:IsHidden()))
				if not FCOIS.ZOControlVars.CHARACTER:IsHidden() then
			        RefreshEquipmentControl()
	            end
	        end, 100)
		end

		--Set the global preventer variable back to false
		FCOIS.preventerVars.gUpdateMarkersNow = false
        FCOIS.preventerVars.gChangedGears		= false
	end
end

-- Change the gear right click menu texts to the FCOIS.settingsVars.settings value
local function changeEquipmentGearMenuTexts(gearId)
    if FCOIS.localizationVars.contextEntries == nil then return end
	if (gearId == -1) then
	    FCOIS.localizationVars.contextEntries.menu_add_gear_text = {}
	    FCOIS.localizationVars.contextEntries.menu_remove_gear_text = {}
	    FCOIS.localizationVars.contextEntries.menu_add_all_gear_text = {}
	    FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text = {}
		--Loop over FCOIS.mappingVars.iconToGear mapping table
		for iconId, l_gearId in pairs(FCOIS.mappingVars.iconToGear) do
			if(FCOIS.settingsVars.settings.icon[iconId].name ~= nil and FCOIS.settingsVars.settings.icon[iconId].name ~= '' and FCOIS.settingsVars.settings.icon[iconId].name ~= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_gear" .. l_gearId]) then
				FCOIS.localizationVars.contextEntries.menu_add_gear_text[l_gearId] 	   = FCOIS.localizationVars.fcois_loc["rightclick_menu_add_start_gear"] .. FCOIS.settingsVars.settings.icon[iconId].name
	            FCOIS.localizationVars.contextEntries.menu_remove_gear_text[l_gearId] 	   = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_start_gear"] .. FCOIS.settingsVars.settings.icon[iconId].name
				FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[l_gearId]    = FCOIS.localizationVars.fcois_loc["rightclick_menu_add_all_start_gear"] .. FCOIS.settingsVars.settings.icon[iconId].name
	            FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[l_gearId] = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_all_start_gear"] .. FCOIS.settingsVars.settings.icon[iconId].name
	        else
				FCOIS.localizationVars.contextEntries.menu_add_gear_text[l_gearId] 	   = FCOIS.localizationVars.fcois_loc["rightclick_menu_add_gear" .. l_gearId]
	            FCOIS.localizationVars.contextEntries.menu_remove_gear_text[l_gearId] 	   = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_gear" .. l_gearId]
				FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[l_gearId]    = FCOIS.localizationVars.fcois_loc["rightclick_menu_add_all_gear" .. l_gearId]
	            FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[l_gearId] = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_all_gear" .. l_gearId]
	        end
        end
	else
		--Overwrite the localized texts for the equipment gear geadId, if changed in the FCOIS.settingsVars.settings
        local iconId = FCOIS.mappingVars.gearToIcon[gearId]
        if(iconId ~= nil and FCOIS.settingsVars.settings.icon[iconId].name ~= nil and FCOIS.settingsVars.settings.icon[iconId].name ~= '' and FCOIS.settingsVars.settings.icon[iconId].name ~= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_gear" .. gearId]) then
			FCOIS.localizationVars.contextEntries.menu_add_gear_text[gearId] 		 = FCOIS.localizationVars.fcois_loc["rightclick_menu_add_start_gear"] .. FCOIS.settingsVars.settings.icon[iconId].name
            FCOIS.localizationVars.contextEntries.menu_remove_gear_text[gearId] 	 = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_start_gear"] .. FCOIS.settingsVars.settings.icon[iconId].name
			FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[gearId]    = FCOIS.localizationVars.fcois_loc["rightclick_menu_add_all_start_gear"] .. FCOIS.settingsVars.settings.icon[iconId].name
            FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[gearId] = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_all_start_gear"] .. FCOIS.settingsVars.settings.icon[iconId].name
        else
			FCOIS.localizationVars.contextEntries.menu_add_gear_text[gearId] 		 = FCOIS.localizationVars.fcois_loc["rightclick_menu_add_gear" .. gearId]
            FCOIS.localizationVars.contextEntries.menu_remove_gear_text[gearId] 	 = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_gear" .. gearId]
			FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[gearId]    = FCOIS.localizationVars.fcois_loc["rightclick_menu_add_all_gear" .. gearId]
            FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[gearId] = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_all_gear" .. gearId]
        end
	end

	--Set texts for the right-click item menus
	FCOIS.localizationVars.lTextMark = {
		        FCOIS.localizationVars.contextEntries.menu_add_lock_text,
                FCOIS.localizationVars.contextEntries.menu_add_gear_text[1],
                FCOIS.localizationVars.contextEntries.menu_add_research_text,
                FCOIS.localizationVars.contextEntries.menu_add_gear_text[2],
                FCOIS.localizationVars.contextEntries.menu_add_sell_text,
                FCOIS.localizationVars.contextEntries.menu_add_gear_text[3],
                FCOIS.localizationVars.contextEntries.menu_add_gear_text[4],
                FCOIS.localizationVars.contextEntries.menu_add_gear_text[5],
			    FCOIS.localizationVars.contextEntries.menu_add_deconstruction_text,
                FCOIS.localizationVars.contextEntries.menu_add_improvement_text,
			    FCOIS.localizationVars.contextEntries.menu_add_sell_to_guild_text,
                FCOIS.localizationVars.contextEntries.menu_add_intricate_text,
	}
	FCOIS.localizationVars.lTextDemark = {
		        FCOIS.localizationVars.contextEntries.menu_remove_lock_text,
                FCOIS.localizationVars.contextEntries.menu_remove_gear_text[1],
                FCOIS.localizationVars.contextEntries.menu_remove_research_text,
                FCOIS.localizationVars.contextEntries.menu_remove_gear_text[2],
                FCOIS.localizationVars.contextEntries.menu_remove_sell_text,
                FCOIS.localizationVars.contextEntries.menu_remove_gear_text[3],
                FCOIS.localizationVars.contextEntries.menu_remove_gear_text[4],
                FCOIS.localizationVars.contextEntries.menu_remove_gear_text[5],
			    FCOIS.localizationVars.contextEntries.menu_remove_deconstruction_text,
			    FCOIS.localizationVars.contextEntries.menu_remove_improvement_text,
                FCOIS.localizationVars.contextEntries.menu_remove_sell_to_guild_text,
                FCOIS.localizationVars.contextEntries.menu_remove_intricate_text,
    }
	--Set texts for the right-click equipment item menus
	FCOIS.localizationVars.lTextEquipmentMark = {
		        FCOIS.localizationVars.contextEntries.menu_add_lock_text,
                FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[1],
                FCOIS.localizationVars.contextEntries.menu_add_research_text,
                FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[2],
                FCOIS.localizationVars.contextEntries.menu_add_sell_text,
                FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[3],
                FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[4],
                FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[5]
	}
    FCOIS.localizationVars.lTextEquipmentDemark = {
		        FCOIS.localizationVars.contextEntries.menu_remove_lock_text,
                FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[1],
                FCOIS.localizationVars.contextEntries.menu_remove_research_text,
                FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[2],
                FCOIS.localizationVars.contextEntries.menu_remove_sell_text,
                FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[3],
                FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[4],
                FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[5]
    }
end

--Add/Change the filter buttons in inventory
local function updateFilterButtonsInInv(buttonId)
	-- This function will only add the inventory buttons!
	-- All other buttons will be added as the relating panel (bank, store, deconstruction, etc.) will be shown the first time.
    debugMessage("[UpdateFilterButtonsInInv] buttonId: " .. tostring(buttonId) .. ", numFilters: ".. tostring(FCOIS.numVars.gFCONumFilters) .. ", InvFiltering: " .. tostring(FCOIS.settingsVars.settings.allowInventoryFilter), true)

    -- Update the filter enable/disable buttons to the inventory, bank, crafting stations, enchantment station, guild store, guild bank, vendor, trade, alchemy and mail panels
	if (buttonId == nil or buttonId == -1) then
		for buttonNr=1, FCOIS.numVars.gFCONumFilters, 1 do
            debugMessage("[UpdateFilterButtonsInInv] Next buttonId:" .. tostring(buttonNr), true)
			FCOIS.gFilterWhere = getFilterWhereBySettings(LF_INVENTORY)
			AddOrChangeFilterButton(FCOIS.ZOControlVars.INV, buttonNr, FCOIS.filterButtonVars.gFilterButtonWidth, FCOIS.filterButtonVars.gFilterButtonHeight, FCOIS.settingsVars.settings.filterButtonLeft[buttonNr], FCOIS.settingsVars.settings.filterButtonTop[buttonNr], not FCOIS.settingsVars.settings.allowInventoryFilter, FCOIS.gFilterWhere)
		end
    else
		FCOIS.gFilterWhere = getFilterWhereBySettings(LF_INVENTORY)
		AddOrChangeFilterButton(FCOIS.ZOControlVars.INV, buttonId, FCOIS.filterButtonVars.gFilterButtonWidth, FCOIS.filterButtonVars.gFilterButtonHeight, FCOIS.settingsVars.settings.filterButtonLeft[buttonId], FCOIS.settingsVars.settings.filterButtonTop[buttonId], not FCOIS.settingsVars.settings.allowInventoryFilter, FCOIS.gFilterWhere)
    end
end

--The context menu OnClicked callback function
local function contextMenuOnClicked(button)
	--Hide the contextmenu after entry was clicked
	FCOIS.contextMenu[FCOIS.gFilterWhere]:SetHidden(true)

	--Get the button name and strip the numbers
	local buttonName = button:GetName()
	local buttonNameLen = string.len(FCOIS.addonVars.gAddonName) + 1
	local sbegin
    local sende
    sbegin, sende = string.find(buttonName, "_", buttonNameLen)
	local buttonNr = string.sub(buttonName, buttonNameLen, sbegin - 1)
	--local buttonNameTotalLen = string.len(buttonName) - 1
    --local buttonEnd = string.sub(buttonName, buttonNameTotalLen)

	sbegin, sende = string.find(buttonNr, "UNDO")
	local isUNDOButton = sbegin ~= nil or false
	sbegin, sende = string.find(buttonNr, "REMOVEALLGEARS")
	local isREMOVEALLGEARSButton = sbegin ~= nil or false
	sbegin, sende = string.find(buttonNr, "REMOVEALL")
	local isREMOVEALLButton = sbegin ~= nil or false
	sbegin, sende = string.find(buttonNr, "TOGGLEANTISETTINGS")
    local isTOGGLEANTISETTINGSButton = sbegin ~= nil or false

    local atLeastOneMarkerChanged = false

	local INVENTORY_TO_SEARCH
    local contextmenuType
	--Deconstruction panel?
	if     (FCOIS.gFilterWhere == LF_SMITHING_DECONSTRUCT) then
		INVENTORY_TO_SEARCH = FCOIS.ZOControlVars.DECONSTRUCTION
        contextmenuType = "DECONSTRUCTION"
	elseif (FCOIS.gFilterWhere == LF_SMITHING_IMPROVEMENT) then
	--Improvement panel?
		INVENTORY_TO_SEARCH = FCOIS.ZOControlVars.IMPROVEMENT
        contextmenuType = "IMPROVEMENT"
	elseif (FCOIS.gFilterWhere == LF_ENCHANTING_CREATION) then
	--Enchanting creation
		INVENTORY_TO_SEARCH = FCOIS.ZOControlVars.ENCHANTING_STATION
        contextmenuType = "ENCHANTING CREATION"
	elseif (FCOIS.gFilterWhere == LF_ENCHANTING_EXTRACTION) then
	--Enchanting extraction
        contextmenuType = "ENCHANTING EXTRACTION"
		INVENTORY_TO_SEARCH = FCOIS.ZOControlVars.ENCHANTING_STATION
    else
    --Inventory or bank
	    --Get the current inventorytype
	    local inventoryType = FCOIS.mappingVars.InvToInventoryType[FCOIS.gFilterWhere]
	    --All non-filtered items will be in this list here:
		--PLAYER_INVENTORY.inventories[inventoryType].data[1-28].data   .bagId & ... .slotIndex
		INVENTORY_TO_SEARCH = PLAYER_INVENTORY.inventories[inventoryType].listView
        contextmenuType = "INVENTORY"
	end

	--No inventory to search in given? Abort here!
	if INVENTORY_TO_SEARCH == nil then return end

    --Are we marking/unmarking items or are we undoing the last change at this current panel?
    if not isUNDOButton and not isREMOVEALLGEARSButton and not isREMOVEALLButton and not isTOGGLEANTISETTINGSButton then

		--Mark/Unmark items
	    local iconId = FCOIS.contextMenuVars.buttonContextMenuToIconId[buttonNr].iconId
        --Check if this icon is enabled in the settings, or abort here
        if not FCOIS.settingsVars.settings.isIconEnabled[iconId] then return end

	    local doMark = FCOIS.contextMenuVars.buttonContextMenuToIconId[buttonNr].mark
	    debugMessage("Clicked "..contextmenuType.." context menu button " .. buttonNr .. ", IconId: " .. tostring(iconId) .. ", Mark: " .. tostring(doMark), true)

		if iconId ~= nil and doMark ~= nil then
		    --Get the current inventorytype
		    local inventoryType = FCOIS.mappingVars.InvToInventoryType[FCOIS.gFilterWhere]
		    --All non-filtered items will be in this list here:
			--ZO_SmithingTopLevelDeconstructionPanelInventoryBackpack.data[]   .bagId & ... .slotIndex
			--ZO_SmithingTopLevelImprovementPanelInventoryBackpack.data[]   .bagId & ... .slotIndex
			local data
			local bagId
			local slotIndex
			local myItemInstanceId
	        local allowedToMark = false
            local undoTableCleared = false
            local undoEntry

	        --Loop over each not-filtered item data in the current inventory
	        for _,v in pairs(INVENTORY_TO_SEARCH.data) do
                --Get the data from current unfiltered inventory item
            	data = v.data
		        if v ~= nil and data ~= nil then
					--get the bag and slot from current unfiltered inventory item
		        	local bagId     = data.bagId
		            local slotIndex = data.slotIndex
					if bagId ~= nil and slotIndex ~= nil then
		                --Initialize the "is item markable/researchable" variable
				        allowedToMark = false

	                    -- Check if equipment gear 1, 2, 3, 4, 5 or research is possible
                        if FCOIS.mappingVars.iconIsResearchable[iconId] then
							-- Check if item is researchable (as only researchable items can work as equipment too)
							allowedToMark = isItemResearchableNoControl(bagId, slotIndex, iconId)
	                    else
	                    	allowedToMark = true
						end

						--Is the item allowed to be marked with this iconId?
	                    if allowedToMark == true then
							myItemInstanceId = MyGetItemInstanceIdNoControl(bagId, slotIndex)
			                if myItemInstanceId ~= nil then
								--Clear the undo table once at the current panelId (keep all other panelIds !)
	    						if not undoTableCleared then
		    						FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere] = {}
	                                undoTableCleared = true
	                            end

								--Set the old marker value in the undo table
								undoEntry = {}
								undoEntry.bagId = bagId
								undoEntry.slotIndex = slotIndex
								undoEntry.iconId = iconId

                                --Mark: True
                                if doMark == true then
									--Check if the item is not marked already
				                    if not checkIfItemIsProtected(iconId, myItemInstanceId) then

				                    	FCOIS.markedItems[iconId][SignItemId(myItemInstanceId)] = true
		                                atLeastOneMarkerChanged = true

                                        --Old value: False
										undoEntry.marked = false
		                            end
                                --Mark: False
			                    else
									--Check if the item is marked already
				                    if checkIfItemIsProtected(iconId, myItemInstanceId) then

				                    	FCOIS.markedItems[iconId][SignItemId(myItemInstanceId)] = nil
		                                atLeastOneMarkerChanged = true

                                        --Old value: True
										undoEntry.marked = true
		                            end
			                    end

								--Set the old marker value in the undo table
						        table.insert(FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere], undoEntry)

			                end

	                    end -- if allowedToMark == true ...
		            end
		        end
		    end --for _,v in pairs(PLAYER_INV...

		end

    else -- if not isUNDOButton then ...

		--UNDO
		if isUNDOButton then
	    	--Undo the last change at this panel Id
		    debugMessage("Clicked "..contextmenuType.." context menu button " .. buttonNr .. ". Will undo last change at panel " .. tostring(FCOIS.gFilterWhere) .. " now!", true)

	        atLeastOneMarkerChanged = false
			local undoEntry

			--Is there a backup set for the current panelId ?
	        if #FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere] > 0 then
	            local myItemInstanceId

				for i=1, #FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere], 1 do
                    if   FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].bagId ~= nil
					 and FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].slotIndex ~= nil
                     and FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].iconId ~= nil
                     and FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].marked ~= nil then

						--d("[UNDO] slotIndex: " .. tostring(FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].slotIndex) .. ", bag: " .. tostring(FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].bagId) .. ", iconId: " .. tostring(FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].iconId) .. ", marked: " .. tostring(FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].marked))
						myItemInstanceId = MyGetItemInstanceIdNoControl(FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].bagId, FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].slotIndex, true)
						if myItemInstanceId ~= nil then
							--Undo the last changes now
							FCOIS.markedItems[FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].iconId][myItemInstanceId] = FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].marked
							atLeastOneMarkerChanged = true

							--Update the undo table with the previous "mark" value so we are able to redo the undo again
							FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].marked = not FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere][i].marked
						end

                	end
                end

	        end --if FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere] and #FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere] > 0 then

		--REMOVE ALL GEARS
        elseif isREMOVEALLGEARSButton then

		    debugMessage("Clicked "..contextmenuType.." context menu button " .. buttonNr .. ", Remove ALL GEARS", true)

		    --Get the current inventorytype
		    local inventoryType = FCOIS.mappingVars.InvToInventoryType[FCOIS.gFilterWhere]
		    --All non-filtered items will be in this list here:
			--PLAYER_INVENTORY.inventories[inventoryType].data[1-28].data   .bagId & ... .slotIndex
			local data
			local bagId
			local slotIndex
			local myItemInstanceId
	        local allowedToMark = false
            local undoTableCleared = false
            local undoEntry

	        --Loop over each not-filtered item data in the current inventory
			for _,v in pairs(INVENTORY_TO_SEARCH.data) do
                --Initialize the "is item markable/researchable" variable
                allowedToMark = false

                --Get the data from current unfiltered inventory item
            	data = v.data
		        if v ~= nil and data ~= nil then
					--get the bag and slot from current unfiltered inventory item
		        	local bagId     = data.bagId
		            local slotIndex = data.slotIndex
					if bagId ~= nil and slotIndex ~= nil then
						myItemInstanceId = MyGetItemInstanceIdNoControl(bagId, slotIndex)
		                if myItemInstanceId ~= nil then

                               --Check all equipment gear icon IDs: 2, 4, 6, 7 and 8
							--Map the iconIds of the 5 gear sets to the actual counter
							for iconId, _ in pairs(FCOIS.mappingVars.iconToGear) do
								--[[ -v- NEW after implementing FCOIS.settingsVars.settings.disableResearchCheck ]]
                               	allowedToMark = isItemResearchableNoControl(bagId, slotIndex, iconId)
								if allowedToMark then
								--[[ -^- NEW after implementing FCOIS.settingsVars.settings.disableResearchCheck ]]

									--Check if the item is marked already AND if the icon is enabled in the settings
				                    if checkIfItemIsProtected(iconId, myItemInstanceId) then

										--Clear the undo table once at the current panelId (keep all other panelIds !)
										if not undoTableCleared then
											FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere] = {}
											undoTableCleared = true
										end

                       					--d("[REMOVE ALL GEARS] ADD slotIndex: " .. tostring(slotIndex) .. ", bag: " .. tostring(bagId) .. ", iconId: " .. tostring(iconId) .. ", marked: false")

                                        --Remove the marker for the current gear set item
				                    	FCOIS.markedItems[iconId][SignItemId(myItemInstanceId)] = nil

										--Set the old marker value in the undo table
										undoEntry = {}
								        undoEntry.bagId = bagId
								        undoEntry.slotIndex = slotIndex
								        undoEntry.iconId = iconId
	                                    undoEntry.marked = true
										--Set the old marker value in the undo table
								        table.insert(FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere], undoEntry)

                                        atLeastOneMarkerChanged = true
		                            end
								--[[ -v- NEW after implementing FCOIS.settingsVars.settings.disableResearchCheck ]]
								end
								--[[ -^- NEW after implementing FCOIS.settingsVars.settings.disableResearchCheck ]]

                               end -- for iconId, gearId ...
		                end
		            end
		        end
		    end --for _,v in pairs(PLAYER_INV...


		--REMOVE ALL
        elseif isREMOVEALLButton then

		    debugMessage("Clicked "..contextmenuType.." context menu button " .. buttonNr .. ", Remove ALL", true)

		    --Get the current inventorytype
		    local inventoryType = FCOIS.mappingVars.InvToInventoryType[FCOIS.gFilterWhere]
		    --All non-filtered items will be in this list here:
			--PLAYER_INVENTORY.inventories[inventoryType].data[1-28].data   .bagId & ... .slotIndex
			local data
			local bagId
			local slotIndex
			local myItemInstanceId
            local undoTableCleared = false
            local undoEntry

	        --Loop over each not-filtered item data in the current inventory
			for _,v in pairs(INVENTORY_TO_SEARCH.data) do
                --Get the data from current unfiltered inventory item
            	data = v.data
		        if v ~= nil and data ~= nil then
					--get the bag and slot from current unfiltered inventory item
		        	local bagId     = data.bagId
		            local slotIndex = data.slotIndex
					if bagId ~= nil and slotIndex ~= nil then

						myItemInstanceId = MyGetItemInstanceIdNoControl(bagId, slotIndex)
						if myItemInstanceId ~= nil then
							--Check all icon Ids
							for iconId = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
								--Check if the item is marked already AND if the settings for this marker icon is activated
								if checkIfItemIsProtected(iconId, myItemInstanceId) then
									--Clear the undo table once at the current panelId (keep all other panelIds !)
									if not undoTableCleared then
										FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere] = {}
										undoTableCleared = true
									end

									--d("[REMOVE ALL] ADD slotIndex: " .. tostring(slotIndex) .. ", bag: " .. tostring(bagId) .. ", iconId: " .. tostring(iconId) .. ", marked: false")

									--Remove the marker for the current gear set item
									FCOIS.markedItems[iconId][SignItemId(myItemInstanceId)] = nil

									--Set the old marker value in the undo table
									undoEntry = {}
							        undoEntry.bagId = bagId
							        undoEntry.slotIndex = slotIndex
							        undoEntry.iconId = iconId
                                    undoEntry.marked = true
							        table.insert(FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere], undoEntry)

									atLeastOneMarkerChanged = true
								end
							end -- for iconId = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
                        end

		            end
		        end
		    end --for _,v in pairs(PLAYER_INV...

        -- TOGGLEANTISETTINGS
        elseif isTOGGLEANTISETTINGSButton then

		    debugMessage("Clicked "..contextmenuType.." context menu button " .. buttonNr .. ", TOGGLE ANTI SETTINGS", true)

            --Change the ANTI settings now
			changeAntiSettingsAccordingToFilterPanel()

			--Update the buttons text and get the settings state
			local buttonText, settingsEnabled = getContextMenuAntiSettingsTextAndState(FCOIS.gFilterWhere, true)
            button:SetText(buttonText)
			--Change the color of the context menu invoker button now
            if button.belongsToContextMenuButton then
	            changeContextMenuInvokerButtonColor(button.belongsToContextMenuButton, settingsEnabled)
            end
        end
    end -- if buttonNr ~= "ButtonContextMenuUNDO" then

	--Did at least one marker change?
	if atLeastOneMarkerChanged == true then
		--Update the inventory & markers
		FilterBasics(false)
	end

end

--The button's context menu OnMouseExit event callback function
local function contextMenuOnMouseExit(button)
    if FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere] ~= nil then
		FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(true)
		local labelControl = button:GetLabelControl()
	    if labelControl ~= nil then
	    	labelControl:SetColor(ZO_DISABLED_TEXT:UnpackRGBA())
	    end
    end
end

--The button's context menu OnMouseEnter event callback function
local function contextMenuOnMouseEnter(button)
	--Create the background for the selected line
    if FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere] == nil then
		FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere] = WINDOW_MANAGER:CreateControl("bdBackDrop"..tostring(FCOIS.gFilterWhere), button, CT_BACKDROP)
		FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(true)
		--FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetEdgeColor(0.4,0.4,0.4)
		--FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetCenterColor(0.1,0.1,0.1)
		FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetDimensions(250,20)
		FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetAlpha(1)
		FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetDrawLayer(0)
        local menuInfo = FCOIS.contextMenuVars.menuInfo[MENU_TYPE_DEFAULT]
		FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetCenterTexture(menuInfo.backdropCenter)
        FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetEdgeTexture(menuInfo.backdropEdge, menuInfo.backdropEdgeWidth, menuInfo.backdropEdgeHeight)
        FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetInsets(unpack(menuInfo.backdropInsets))
    end
	FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:ClearAnchors()
	FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetAnchor(TOPLEFT, button, TOPLEFT, -4, -2)
    FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetAnchor(BOTTOMRIGHT, button, BOTTOMRIGHT, 0, 2)
	FCOIS.contextMenu.bdSelectedLine[FCOIS.gFilterWhere]:SetHidden(false)

	local labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_HIGHLIGHT_TEXT:UnpackRGBA())
    end
end

--Show/Hide the inventory "flag" context menu buttons according to the enabled/disabled "icon enabled" settings
local function updateContextMenuInventoryButtons(p_iconId, creatingContextMenu, p_filterPanelId, p_onlyUpdateText)
    --No icon Id given? Abort here
    if p_iconId == nil then return end

	creatingContextMenu = creatingContextMenu or false
	p_onlyUpdateText	= p_onlyUpdateText or false

    --Gget the filter panel Id
    p_filterPanelId = p_filterPanelId or FCOIS.gFilterWhere

	--Should be check all filter panel Ids?
    if p_filterPanelId == -1 then
		for i_filterPanelId = 1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
			if FCOIS.mappingVars.activeFilterPanelIds[i_filterPanelId] == true then
				updateContextMenuInventoryButtons(p_iconId, creatingContextMenu, i_filterPanelId, p_onlyUpdateText)
            end
        end
	else
		--No filter panel Id? -> Abort here
		if p_filterPanelId == nil then return end

		--Build the 2digits character filter panel Id for the button name
		local filterPanelId = p_filterPanelId
	    if filterPanelId < 10 then filterPanelId = "0" .. tostring(filterPanelId) end

		local iconButton
	    local gearId
		local reAnchor = true

		--Check each of the context menu buttons
		for contextMenuButton, v in pairs(FCOIS.contextMenuVars.buttonContextMenuToIconId) do
            --Are we reanchoring etc. the current wanted (by the given parameter p_iconId) buttons?
        	if p_iconId == v.iconId then
				--Get the button name and control
				iconButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. tostring(contextMenuButton) .. "_" .. tostring(filterPanelId), "")
				if iconButton ~= nil then
	               	if not p_onlyUpdateText then
	                    --Reanchor buttons
	                	local anchorButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. tostring(v.anchorButton) .. "_" .. tostring(filterPanelId), "")
						--Security check to prevent reanchoring to isself
	                    if anchorButton == iconButton then
                            anchorButton = iconButton:GetParent()
                            reAnchor = false
	                    end
						--Is the current icon enabled in the settings?
		                if FCOIS.settingsVars.settings.isIconEnabled[v.iconId] then
		                    if not creatingContextMenu then
			                    iconButton:SetHeight(FCOIS.contextMenuVars.entryHeight)
			                    if anchorButton ~= nil and reAnchor then
									iconButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,FCOIS.contextMenuVars.entryHeight)
                                elseif anchorButton ~= nil and not reAnchor then
	                               	iconButton:SetAnchor(TOPLEFT,anchorButton,TOPLEFT,0,0)
	                                anchorButton = nil
			                    end
		                    end
						    iconButton:SetMouseEnabled(true)
						    iconButton:SetHidden(false)
		                else
							--Current icon is not enabled in the settings, so resize the button and hide it
			                iconButton:ClearAnchors()
			                iconButton:SetHeight(0)
						    iconButton:SetMouseEnabled(false)
						    iconButton:SetHidden(true)
		                    if anchorButton ~= nil and reAnchor then
								iconButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,0)
							elseif anchorButton ~= nil and not reAnchor then
								iconButton:SetAnchor(TOPLEFT,anchorButton,TOPLEFT,0,0)
								anchorButton = nil
		                    end
		                end
					end
					--Get the gear to the current icon, if available
		        	gearId = FCOIS.mappingVars.iconToGear[v.iconId]
					--Update the button's text now
					if v.mark then
						--Update the add to gear button text
						if gearId ~= nil then
							iconButton:SetText(FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[gearId])
						else
							iconButton:SetText(FCOIS.localizationVars.contextEntries.menu_add_all_text[p_iconId])
						end
					else
						--Update the remove from gear button text
						if gearId ~= nil then
							iconButton:SetText(FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[gearId])
						else
							iconButton:SetText(FCOIS.localizationVars.contextEntries.menu_remove_all_text[p_iconId])
						end
					end
			    end
		        iconButton	= nil
		        gearId	  	= nil
                reAnchor	= true
			end -- if p_iconId == v.iconId then
	    end -- 	for contextMenuButton, v in pairs(FCOIS.contextMenuVars.buttonContextMenuToIconId) do

		--If we are not creating a context menu
	    -- or we are creating a context menu and the gear id is the last (first) one (counting down from highest to lowest gear #)
		local allGearSettingsDisabled = false
	    if (not p_onlyUpdateText and (creatingContextMenu == false or (creatingContextMenu == true and p_iconId == 1))) then
        	allGearSettingsDisabled = true
		    --Are all gear sets disabled? Then remove the "Remove all gear sets button" too
		    for iconId = 1, #FCOIS.settingsVars.settings.isIconEnabled, 1 do
				--Check if the current icon is a gear icon and if the gear icon is enabled?
			    if FCOIS.mappingVars.iconToGear[iconId] ~= nil and FCOIS.settingsVars.settings.isIconEnabled[iconId] == true then
					--At least one gear icon is activated -> break here!
			        allGearSettingsDisabled = false
		       		break
		        end
		    end
			--The "Remove all gear sets button"
			iconButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. "ButtonContextMenuREMOVEALLGEARS" .. "_" .. tostring(filterPanelId), "")
			--The anchor button to the "Remove all gear sets button"
			local anchorButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. "ButtonContextMenu24" .. "_" .. tostring(filterPanelId), "")
			if iconButton ~= nil then
				if allGearSettingsDisabled == false then
					if not creatingContextMenu and not iconButton:IsMouseEnabled() then
						iconButton:SetHeight(FCOIS.contextMenuVars.entryHeight)
						if anchorButton ~= nil then
							iconButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,FCOIS.contextMenuVars.entryHeight)
						end
					end
					iconButton:SetMouseEnabled(true)
					iconButton:SetHidden(false)
				else
					iconButton:ClearAnchors()
					iconButton:SetHeight(0)
					iconButton:SetMouseEnabled(false)
					iconButton:SetHidden(true)
					if anchorButton ~= nil then
						iconButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,0)
					end
			    end
			end

			--If all gear sets are disabled in the settings and we are not creating a context menu
		    -- or we are creating a context menu and the icon Id is the last (first) one (counting down from highest to lowest icon #)
			local allIconSettingsDisabled = false
	    	if ( allGearSettingsDisabled) then
	            allIconSettingsDisabled = true
			    --Are all icons disabled? Then remove the "Remove all marker button" too
			    for iconId = 1, #FCOIS.settingsVars.settings.isIconEnabled, 1 do
				    if FCOIS.mappingVars.iconToGear[iconId] == nil and FCOIS.settingsVars.settings.isIconEnabled[iconId] == true then
						--At least one icon is activated -> break here!
				        allIconSettingsDisabled = false
			       		break
			        end
			    end
				--The "Remove all markers button"
				iconButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. "ButtonContextMenuREMOVEALL" .. "_" .. tostring(filterPanelId), "")
				--The anchor button to the "Remove all button"
				local anchorButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. "ButtonContextMenuREMOVEALLGEARS" .. "_" .. tostring(filterPanelId), "")
				if iconButton ~= nil then
					if allIconSettingsDisabled == false then
						if not creatingContextMenu and not iconButton:IsMouseEnabled() then
							iconButton:SetHeight(FCOIS.contextMenuVars.entryHeight)
							if anchorButton ~= nil then
								iconButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,FCOIS.contextMenuVars.entryHeight)
							end
						end
						iconButton:SetMouseEnabled(true)
						iconButton:SetHidden(false)
					else
						iconButton:ClearAnchors()
						iconButton:SetHeight(0)
						iconButton:SetMouseEnabled(false)
						iconButton:SetHidden(true)
						if anchorButton ~= nil then
							iconButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,0)
						end
				    end
	            	iconButton		= nil
	                anchorButton	= nil
	            end
            end -- if ( allGearSettingsDisabled) then

			--The "Undo button"
	        local undoButtonDisabled = false
			local iconButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. "ButtonContextMenuUNDO" .. "_" .. tostring(filterPanelId), "")
			--The anchor button to the "Remove all button"
			local anchorButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. "ButtonContextMenuTOGGLEANTISETTINGS" .. "_" .. tostring(filterPanelId), "")
			--Are all gear set buttons disabled and are all other buttons disabled too?
	        if allGearSettingsDisabled and allIconSettingsDisabled then
				--Yes, all buttons are disabled
		        undoButtonDisabled = true
				if iconButton ~= nil then
					iconButton:ClearAnchors()
					iconButton:SetHeight(0)
					iconButton:SetMouseEnabled(false)
					iconButton:SetHidden(true)
					if anchorButton ~= nil then
						iconButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,0)
					end
				end
	        else
				--No, at least one button is still enabled
	        	undoButtonDisabled = false
				if iconButton ~= nil then
					if not creatingContextMenu and not iconButton:IsMouseEnabled() then
						iconButton:SetHeight(FCOIS.contextMenuVars.entryHeight)
						if anchorButton ~= nil then
							iconButton:SetAnchor(BOTTOMLEFT,anchorButton,BOTTOMLEFT,0,FCOIS.contextMenuVars.entryHeight)
						end
					end
					iconButton:SetMouseEnabled(true)
					iconButton:SetHidden(false)
				end
	        end --if allGearSettingsDisabled and allIconSettingsDisabled then

			--Change the size of the context menu
            --Delay the check a few milliseconds to give the controls the time to SetHidden(true)
		    --Get the context menu control, move the buttons and resize the context menu
			local contextMenuName = FCOIS.addonVars.gAddonName .. "ContextMenu" .. "_" .. tostring(filterPanelId)
		    local contextMenu = WINDOW_MANAGER:GetControlByName(contextMenuName, "")
		    if contextMenu ~= nil then
				--Check the context menu children for allowed entries and see if they are not hidden -> multiply with FCOIS.contextMenuVars.entryHeight
				--Preset the contextmenu height with 1 entry (the de/Activate Anti-... setting button)
				local contextMenuTotalHeight = FCOIS.contextMenuVars.entryHeight
				--Is the remove all gear sets button visible?
				if not allGearSettingsDisabled then
					contextMenuTotalHeight = contextMenuTotalHeight + FCOIS.contextMenuVars.entryHeight
				end
				--Is the remove all marker button visible?
				if not allIconSettingsDisabled then
					contextMenuTotalHeight = contextMenuTotalHeight + FCOIS.contextMenuVars.entryHeight
				end
				--Is the undo changes button visible?
				if not undoButtonDisabled then
					contextMenuTotalHeight = contextMenuTotalHeight + FCOIS.contextMenuVars.entryHeight
				end
                --Check each icon if it is enabled
				for iconId = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
					if FCOIS.settingsVars.settings.isIconEnabled[iconId] then
						contextMenuTotalHeight = contextMenuTotalHeight + (FCOIS.contextMenuVars.entryHeight * 2)
					end
				end --for iconId = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				--Is the new height > 0 and < max height?
				if contextMenuTotalHeight > 0 then
					if contextMenuTotalHeight > FCOIS.contextMenuVars.maxHeight then
						contextMenuTotalHeight = FCOIS.contextMenuVars.maxHeight
					end
					contextMenu:SetHeight(contextMenuTotalHeight)
				end
		    end -- if FCOIS.contextMenu ~= nil then
	    end -- if (not p_onlyUpdateText and (creatingContextMenu == false or (creatingContextMenu == true and p_iconId == 1)) ) then

    end -- If p_filterPanelId == -1
end

--Function that creates the context menu for the additional inventory "flag" buttons -> Adding buttons and the handlers for them.
local function createContextMenuForAdditionalInvButtons(parentWindow, panelId)
	local button
    local lastButton
	local labelControl
    local buttonText
	--Abort if parent does not exist
    if parentWindow == nil then return end
	panelId = panelId or FCOIS.gFilterWhere
	local filterPanelId = panelId
    if filterPanelId < 10 then filterPanelId = "0" .. tostring(filterPanelId) end
	--Change the color of the context menu invoker button now
	local antiButtonText, settingsEnabled = getContextMenuAntiSettingsTextAndState(panelId, true)
	changeContextMenuInvokerButtonColor(parentWindow, settingsEnabled)
    local parentName = parentWindow:GetName()
	local onClickedFunction = "normal"
    if 		string.find(parentName, "ZO_SmithingTopLevelDeconstruction") then
    	onClickedFunction = "deconstruction"
    elseif  string.find(parentName, "ZO_SmithingTopLevelImprovement") then
    	onClickedFunction = "improvement"
    elseif  string.find(parentName, "ZO_Enchanting") then
    	onClickedFunction = "enchanting"
	else
    	onClickedFunction = "normal"
    end
    --Array with callback functions OnClicked
    local onClickedFunctions = {
	    ["normal"] 			= contextMenuOnClicked,
	    ["deconstruction"]  = contextMenuOnClicked,
	    ["improvement"]  	= contextMenuOnClicked,
	    ["enchanting"]  	= contextMenuOnClicked,
    }

--Context menu container
    FCOIS.contextMenu[panelId] = WINDOW_MANAGER:CreateTopLevelWindow(FCOIS.addonVars.gAddonName .. "ContextMenu" .. "_" .. tostring(filterPanelId))
	local menuWindow = FCOIS.contextMenu[panelId]
	menuWindow:SetDimensions(FCOIS.contextMenuVars.maxWidth, FCOIS.contextMenuVars.maxHeight)
    menuWindow:SetAnchor(TOP,parentWindow,BOTTOM, 12, 6)
    menuWindow:SetMovable(false)
    menuWindow:SetMouseEnabled(true)

--Context menu background
	menuWindow.bg = WINDOW_MANAGER:CreateControlFromVirtual(FCOIS.addonVars.gAddonName .. "ContextMenuBg" .. "_" .. tostring(filterPanelId), menuWindow, "ZO_DefaultBackdrop")

	menuWindow:SetDrawLevel(0)
	menuWindow:SetDrawLayer(1)
	menuWindow:SetDrawTier(1)

--Lock item
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu1".. "_" .. tostring(filterPanelId), menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.contextEntries.menu_add_all_text[1])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(TOPLEFT,menuWindow,TOPLEFT,0,0)
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--UnLock item
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu2".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.contextEntries.menu_remove_all_text[1])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--Add gear 1
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu3".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	buttonText = FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[1]
    if string.len(buttonText) > FCOIS.contextMenuVars.maxCharactersInLine then
    	buttonText = string.sub(buttonText, 1, FCOIS.contextMenuVars.maxCharactersInLine) .. " ..."
    end
	button:SetText(buttonText)
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
 	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button:SetMouseEnabled(FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[1]])
    button:SetHidden(not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[1]])
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil
    buttonText = nil

--Remove gear 1
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu4".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	buttonText = FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[1]
    if string.len(buttonText) > FCOIS.contextMenuVars.maxCharactersInLine then
    	buttonText = string.sub(buttonText, 1, FCOIS.contextMenuVars.maxCharactersInLine) .. " ..."
    end
	button:SetText(buttonText)
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
 	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil
    buttonText = nil

--Add gear 2
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu5".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	buttonText = FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[2]
    if string.len(buttonText) > FCOIS.contextMenuVars.maxCharactersInLine then
    	buttonText = string.sub(buttonText, 1, FCOIS.contextMenuVars.maxCharactersInLine) .. " ..."
    end
	button:SetText(buttonText)
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
 	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil
    buttonText = nil

--Remove gear 2
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu6".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	buttonText = FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[2]
    if string.len(buttonText) > FCOIS.contextMenuVars.maxCharactersInLine then
    	buttonText = string.sub(buttonText, 1, FCOIS.contextMenuVars.maxCharactersInLine) .. " ..."
    end
	button:SetText(buttonText)
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
 	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil
    buttonText = nil

--Add gear 3
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu7".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	buttonText = FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[3]
    if string.len(buttonText) > FCOIS.contextMenuVars.maxCharactersInLine then
    	buttonText = string.sub(buttonText, 1, FCOIS.contextMenuVars.maxCharactersInLine) .. " ..."
    end
	button:SetText(buttonText)
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil
    buttonText = nil

--Remove gear 3
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu8".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	buttonText = FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[3]
    if string.len(buttonText) > FCOIS.contextMenuVars.maxCharactersInLine then
    	buttonText = string.sub(buttonText, 1, FCOIS.contextMenuVars.maxCharactersInLine) .. " ..."
    end
	button:SetText(buttonText)
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil
    buttonText = nil

--Add gear 4
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu9".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	buttonText = FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[4]
    if string.len(buttonText) > FCOIS.contextMenuVars.maxCharactersInLine then
    	buttonText = string.sub(buttonText, 1, FCOIS.contextMenuVars.maxCharactersInLine) .. " ..."
    end
	button:SetText(buttonText)
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil
    buttonText = nil

--Remove gear 4
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu10".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	buttonText = FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[4]
    if string.len(buttonText) > FCOIS.contextMenuVars.maxCharactersInLine then
    	buttonText = string.sub(buttonText, 1, FCOIS.contextMenuVars.maxCharactersInLine) .. " ..."
    end
	button:SetText(buttonText)
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil
    buttonText = nil

--Add gear 5
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu11".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	buttonText = FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[5]
    if string.len(buttonText) > FCOIS.contextMenuVars.maxCharactersInLine then
    	buttonText = string.sub(buttonText, 1, FCOIS.contextMenuVars.maxCharactersInLine) .. " ..."
    end
	button:SetText(buttonText)
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil
    buttonText = nil

 --Remove gear 5
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu12".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	buttonText = FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[5]
    if string.len(buttonText) > FCOIS.contextMenuVars.maxCharactersInLine then
    	buttonText = string.sub(buttonText, 1, FCOIS.contextMenuVars.maxCharactersInLine) .. " ..."
    end
	button:SetText(buttonText)
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil
    buttonText = nil

--Research all
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu13".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.contextEntries.menu_add_all_text[3])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--UnResearch all
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu14".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.contextEntries.menu_remove_all_text[3])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--Deconstruct all
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu15".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.contextEntries.menu_add_all_text[9])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--UnDeconstruct all
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu16".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.contextEntries.menu_remove_all_text[9])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--Improve all
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu17".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.contextEntries.menu_add_all_text[10])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--UnImprove all
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu18".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.contextEntries.menu_remove_all_text[10])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--Sell all
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu19".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.contextEntries.menu_add_all_text[5])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--UnSell all
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu20" .. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.contextEntries.menu_remove_all_text[5])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

    --Sell all at guild store
    button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu21".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
    button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
    button:SetFont("ZoFontGame")
    button:SetState( BSTATE_NORMAL )
    button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
    button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
    button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
    button:SetText(FCOIS.localizationVars.contextEntries.menu_add_all_text[11])
    button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
    button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
    labelControl = button:GetLabelControl()
    if labelControl ~= nil then
        labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

    --UnSell all at guild store
    button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu22" .. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
    button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
    button:SetFont("ZoFontGame")
    button:SetState( BSTATE_NORMAL )
    button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
    button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
    button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
    button:SetText(FCOIS.localizationVars.contextEntries.menu_remove_all_text[11])
    button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
    button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
    labelControl = button:GetLabelControl()
    if labelControl ~= nil then
        labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

    --Mark all as intricate
    button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu23".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
    button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
    button:SetFont("ZoFontGame")
    button:SetState( BSTATE_NORMAL )
    button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
    button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
    button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
    button:SetText(FCOIS.localizationVars.contextEntries.menu_add_all_text[12])
    button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
    button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
    labelControl = button:GetLabelControl()
    if labelControl ~= nil then
        labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

    --Unmark all as intricate
    button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenu24" .. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
    button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
    button:SetFont("ZoFontGame")
    button:SetState( BSTATE_NORMAL )
    button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
    button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
    button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
    button:SetText(FCOIS.localizationVars.contextEntries.menu_remove_all_text[12])
    button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
    button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
    labelControl = button:GetLabelControl()
    if labelControl ~= nil then
        labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--Context menu button REMOVE ALL GEARS
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuREMOVEALLGEARS".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.fcois_loc["button_context_menu_remove_all_gears"])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--Context menu button REMOVE ALL
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuREMOVEALL".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.fcois_loc["button_context_menu_demark_all"])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--Context menu buttons for "Anti-*" FCOIS.settingsVars.settings
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuTOGGLEANTISETTINGS".. "_" .. tostring(filterPanelId) , menuWindow , CT_BUTTON)
	button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(antiButtonText)
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

--Context menu UNDO button
	button = WINDOW_MANAGER:CreateControl(FCOIS.addonVars.gAddonName .. "ButtonContextMenuUNDO" .. "_" .. tostring(filterPanelId), menuWindow , CT_BUTTON)
    button:SetDimensions( FCOIS.contextMenuVars.maxWidth,FCOIS.contextMenuVars.entryHeight )
	button:SetFont("ZoFontGame")
	button:SetState( BSTATE_NORMAL )
	button:SetHandler( "OnClicked", function(...) onClickedFunctions[onClickedFunction](...) end)
	button:SetHandler( "OnMouseEnter", function (self) contextMenuOnMouseEnter(self) end)
	button:SetHandler( "OnMouseExit", function (self) contextMenuOnMouseExit(self) end)
	button:SetText(FCOIS.localizationVars.fcois_loc["button_context_menu_undo"])
	button:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	button:SetAnchor(BOTTOMLEFT,lastButton,BOTTOMLEFT,0,lastButton:GetHeight())
    button:SetMouseEnabled(false)
	labelControl = button:GetLabelControl()
    if labelControl ~= nil then
    	labelControl:SetColor(ZO_DISABLED_TEXT:UnpackRGBA())
    end
    button.belongsToContextMenuButton = parentWindow
    lastButton = button
    button = nil
    labelControl = nil

	--Hide the menu now
 	menuWindow:SetHidden(true)

	--Update each icon button (enable & show / disable & hide) in the inventory context menu
    for iconId = FCOIS.numVars.gFCONumFilterIcons, 1, -1 do
	    updateContextMenuInventoryButtons(iconId, true, panelId)
    end

    --Update the color of the context menu invoker button, according to the  current "anti" settings
	changeContextMenuInvokerButtonColor(parentWindow, settingsEnabled)

    return menuWindow
end

--Function that is called upon onMouseUp event on the additional inventory context menu buttons
local function onContextMenuButtonMouseUp(inventoryAdditionalContextMenuInvokerButton, mouseButton, upInside)
	--Only go on if the context menu isn ot shown currently
	if (FCOIS.contextMenu[FCOIS.gFilterWhere] == nil) or (FCOIS.contextMenu[FCOIS.gFilterWhere] ~= nil and FCOIS.contextMenu[FCOIS.gFilterWhere]:IsHidden()) then
	    --Check if the ANTI-settings are enabled at the current panel
	    local _, settingsEnabled = getContextMenuAntiSettingsTextAndState(FCOIS.gFilterWhere, false)
		if settingsEnabled == nil then return false end
		--Invert the active setting
		changeAntiSettingsAccordingToFilterPanel()
		--Change the additional inventory context menu button color
		changeContextMenuInvokerButtonColor(inventoryAdditionalContextMenuInvokerButton, not settingsEnabled)
	end
end

--Function that display the context menu after the player clicks on the additional inventory "flag" button on the top left corner of the inventories (left to the "name" sort header)
local function showContextMenuForAdditionalInvButtons(inventoryAdditionalContextMenuInvokerButton)
	local newCreated = false
	if FCOIS.preventerVars.gContextCreated[FCOIS.gFilterWhere] == false then
		FCOIS.contextMenu[FCOIS.gFilterWhere] = createContextMenuForAdditionalInvButtons(inventoryAdditionalContextMenuInvokerButton, FCOIS.gFilterWhere)
        newCreated = true
	end
	if FCOIS.contextMenu[FCOIS.gFilterWhere] ~= nil then
		FCOIS.preventerVars.gContextCreated[FCOIS.gFilterWhere] = true
		if FCOIS.contextMenu[FCOIS.gFilterWhere]:IsHidden() then
       		FCOIS.contextMenu[FCOIS.gFilterWhere]:ClearAnchors()
			FCOIS.contextMenu[FCOIS.gFilterWhere]:SetAnchor(TOP,inventoryAdditionalContextMenuInvokerButton,BOTTOM, -145, 4)

			local filterPanelId = FCOIS.gFilterWhere
		    if filterPanelId < 10 then filterPanelId = "0" .. tostring(filterPanelId) end
            --Update the ANTI-* button text
			local antiButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. "ButtonContextMenuTOGGLEANTISETTINGS" .. "_" .. tostring(filterPanelId), "")
            if antiButton ~= nil then
   				--Change the color of the context menu invoker button now
				local antiButtonText, settingsEnabled = getContextMenuAntiSettingsTextAndState(FCOIS.gFilterWhere, true)
	         	antiButton:SetText(antiButtonText)
            end

            --Update the UNDO button: If nothing is there to undo it should be black text color and no click handler active (mouse disabled)
			local undoButton = WINDOW_MANAGER:GetControlByName(FCOIS.addonVars.gAddonName .. "ButtonContextMenuUNDO" .. "_" .. tostring(filterPanelId), "")
            if undoButton ~= nil then
	            local labelControl
                --Is anything there to undo?
            	if FCOIS.contextMenuVars.undoMarkedItems[FCOIS.gFilterWhere] then
					undoButton:SetMouseEnabled(true)
					labelControl = undoButton:GetLabelControl()
				    if labelControl ~= nil then
				    	labelControl:SetColor(ZO_NORMAL_TEXT:UnpackRGBA())
				    end
                else
					undoButton:SetMouseEnabled(false)
					labelControl = undoButton:GetLabelControl()
				    if labelControl ~= nil then
				    	labelControl:SetColor(ZO_DISABLED_TEXT:UnpackRGBA())
				    end
                end
            end

			--Was the context menu created now? If not we don't need to update the texts
            if not newCreated then
				--Update each icon button's text
			    for iconId = FCOIS.numVars.gFCONumFilterIcons, 1, -1 do
	                updateContextMenuInventoryButtons(iconId, false, FCOIS.gFilterWhere, true)
			    end
			end

   			FCOIS.contextMenu[FCOIS.gFilterWhere]:SetHidden(false)
		else
			FCOIS.contextMenu[FCOIS.gFilterWhere]:SetHidden(true)
	    end
    end
end

--Add a button to an existing parent control
local function AddButton(parent, name, callbackFunction, onMouseUpCallbackFunction, onMouseUpCallbackFunctionMouseButton, text, font, tooltipText, tooltipAlign, textureNormal, textureMouseOver, textureClicked, width, height, left, top, alignMain, alignBackup, alignControl, hideButton)
	--Abort needed?
    if  (parent == nil or name == nil or callbackFunction == nil
     or width <= 0 or height <= 0 or alignMain == nil or alignBackup == nil)
     and (textureNormal == nil or text == nil) then
    	return nil
    end
   	onMouseUpCallbackFunctionMouseButton = onMouseUpCallbackFunctionMouseButton or 1

    local button
    --Does the button already exist?
    button = WINDOW_MANAGER:GetControlByName(name, "")
    if button == nil then
    	--Button does not exist yet and it should be hidden? Abort here!
        if hideButton == true then return nil end
		--Create the button control at the parent
		button = WINDOW_MANAGER:CreateControl(name, parent, CT_BUTTON)
    end
	--Button was created?
    if button ~= nil then
		--Button should be hidden?
		if hideButton == false then
	        --Set the button's size
		    button:SetDimensions(width, height)

			--Align the button
	        if alignControl == nil then
	        	alignControl = parent
	        end

			--SetAnchor(point, relativeTo, relativePoint, offsetX, offsetY)
			button:SetAnchor(alignMain, alignControl, alignBackup, left, top)

	        --Texture or text?
			if (text ~= nil) then
				--Text
				--Set the button's font
			    if font == nil then
				    button:SetFont("ZoFontGameSmall")
			    else
				    button:SetFont(font)
			    end

	            --Set the button's text
	            button:SetText(text)

	        else
	        	--Texture
				local texture

                --Check if texture exists
                texture = WINDOW_MANAGER:GetControlByName(name .. "Texture", "")
				if texture == nil then
				    --Create the texture for the button to hold the image
				    texture = WINDOW_MANAGER:CreateControl(name .. "Texture", button, CT_TEXTURE)
                end
			    texture:SetAnchorFill()

		        --Set the texture for normale state now
		        texture:SetTexture(textureNormal)

				--Do we have seperate textures for the button states?
	 			button.upTexture 	  = textureNormal
				button.downTexture 	  = textureMouseOver or textureNormal
				button.clickedTexture = textureClicked or textureNormal
			end

            if tooltipAlign == nil then tooltipAlign = TOP end

			--Set a tooltip?
			if tooltipText ~= nil then
            	tooltipText = FCOIS.preChatVars.preChatTextGreen .. tooltipText
                button.tooltipText	= tooltipText
                button.tooltipAlign = tooltipAlign
                button:SetHandler("OnMouseEnter", function(self)
	                self:GetChild(1):SetTexture(self.downTexture)
					ZO_Tooltips_ShowTextTooltip(self, self.tooltipAlign, self.tooltipText)
				end)
				button:SetHandler("OnMouseExit", function(self)
	                self:GetChild(1):SetTexture(self.upTexture)
					ZO_Tooltips_HideTextTooltip()
				end)
	        else
				button:SetHandler("OnMouseEnter", function(self)
	                self:GetChild(1):SetTexture(self.downTexture)
				end)
				button:SetHandler("OnMouseExit", function(self)
	                self:GetChild(1):SetTexture(self.upTexture)
				end)
	        end
			--Set the callback function of the button
		    button:SetHandler("OnClicked", function(...)
	        	callbackFunction(...)
	        end)
			--Set the OnMouseUp callback function of the button
			if onMouseUpCallbackFunction ~= nil then
				button:SetHandler("OnMouseUp", function(butn, mouseButton, upInside)
                    if upInside then
	                	if mouseButton == onMouseUpCallbackFunctionMouseButton then
                        	onMouseUpCallbackFunction(butn, mouseButton, upInside)
	                    end
					end
				end)
            end
			button:SetHandler("OnMouseDown", function(butn, ctrl, alt, shift, command)
				butn:GetChild(1):SetTexture(butn.clickedTexture)
			end)

			--Show the button and make it react on mouse input
		    button:SetHidden(false)
		   	button:SetMouseEnabled(true)

	        --Return the button control
			return button
    	else
			--Hide the button and make it not reacting on mouse input
		    button:SetHidden(true)
		   	button:SetMouseEnabled(false)
        end
	else
    	return nil
    end
end

--Add additonal buttons, controlled by the FCOIS settings
local function AddAdditionalButtons(buttonName)
	--Add all additional buttons
	if (buttonName == -1) then
		AddAdditionalButtons("FCOSettings")
		AddAdditionalButtons("FCOInventoriesContextMenuButtons")
    else
	--Add only a specific button
    	if buttonName == "FCOSettings" then
			--Add or hide a button to the main menu category bar, right to the help button, if enabled in settings
			--AddButton(ZO_MainMenuCategoryBar, "ZO_MainMenuCategoryBarButtonFCOSettings", ShowFCOItemSaverSettings, nil, nil, nil, nil, FCOIS.localizationVars.fcois_loc["button_FCOIS_settings_tooltip"], RIGHT, "/esoui/art/charactercreate/rotate_right_up.dds", "/esoui/art/charactercreate/rotate_right_over.dds", "/esoui/art/charactercreate/rotate_right_down.dds", 32, 32, ZO_MainMenuCategoryBarButton15:GetWidth() + 30, 35, BOTTOM, TOP, ZO_MainMenuCategoryBarButton15, not FCOIS.settingsVars.settings.showFCOISMenuBarButton)

			local descriptor = FCOIS.addonVars.gAddonName
			local callbackFnc
			if not LAM then
				callbackFnc = function(buttonData) SCENE_MANAGER:Show("gameMenuInGame") end
			else
				callbackFnc = function(buttonData) ShowFCOItemSaverSettings() end
			end

			-- Add to main menu
			local categoryLayoutInfo =
			{
				binding = "FCOIS_SETTINGS_MENU",
				categoryName = SI_BINDING_NAME_FCOIS_SETTINGS_MENU,
				callback = callbackFnc,
				visible = function(buttonData)
                	if FCOIS.settingsVars.settings.showFCOISMenuBarButton then
                    	if VOTANS_MENU_SETTINGS and VOTANS_MENU_SETTINGS:IsMenuButtonEnabled() then
                        	return false
                        else
                        	return true
                        end
                    else
                    	return false
                    end
                end,
				normal    = "esoui/art/charactercreate/rotate_right_up.dds",
				pressed   = "esoui/art/charactercreate/rotate_right_down.dds",
				highlight = "esoui/art/charactercreate/rotate_right_over.dds",
				disabled  = "esoui/art/charactercreate/rotate_right_disabled.dds",
			}
			LMM2:AddMenuItem(descriptor, categoryLayoutInfo)

		elseif buttonName == "FCOInventoriesContextMenuButtons" then
			AddAdditionalButtons("FCOPlayerInventoryContextMenu")
			AddAdditionalButtons("FCOPlayerBankWithdrawContextMenu")
			AddAdditionalButtons("FCOGuildBankWithdrawContextMenu")
			AddAdditionalButtons("FCODeconstructionContextMenu")
			AddAdditionalButtons("FCOImprovementContextMenu")
			AddAdditionalButtons("FCOEnchantingContextMenu")

			--Check if the craft bag is enabled
            if HasCraftBagAccess() then
	            AddAdditionalButtons("FCOCraftBagContextMenu")
            end
    	elseif buttonName == "FCOPlayerInventoryContextMenu" then
			--Add or hide a button to the player bank withdraw sort bar
			AddButton(ZO_PlayerInventorySortBy, FCOIS.invAdditionalButtonVars.playerInventoryFCOAdditionalOptionsButton, showContextMenuForAdditionalInvButtons, onContextMenuButtonMouseUp, 2, nil, nil, FCOIS.localizationVars.fcois_loc["button_context_menu_tooltip"], RIGHT, "/esoui/art/ava/tabicon_bg_score_inactive.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", 32, 32, -16, 32, BOTTOM, TOP, ZO_PlayerInventorySortByNew, not FCOIS.settingsVars.settings.showFCOISAdditionalInventoriesButton)
    	elseif buttonName == "FCOPlayerBankWithdrawContextMenu" then
			--Add or hide a button to the player bank withdraw sort bar
			AddButton(ZO_PlayerBankSortBy, FCOIS.invAdditionalButtonVars.playerBankWithdrawButtonAdditionalOptions, showContextMenuForAdditionalInvButtons, onContextMenuButtonMouseUp, 2, nil, nil, FCOIS.localizationVars.fcois_loc["button_context_menu_tooltip"], RIGHT, "/esoui/art/ava/tabicon_bg_score_inactive.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", 32, 32, -213, 26, BOTTOM, TOP, ZO_PlayerBankSortByName, not FCOIS.settingsVars.settings.showFCOISAdditionalInventoriesButton)
    	elseif buttonName == "FCOGuildBankWithdrawContextMenu" then
			--Add or hide a button to the player bank withdraw sort bar
			AddButton(ZO_GuildBankSortBy, FCOIS.invAdditionalButtonVars.guildBankFCOWithdrawButtonAdditionalOptions, showContextMenuForAdditionalInvButtons, onContextMenuButtonMouseUp, 2, nil, nil, FCOIS.localizationVars.fcois_loc["button_context_menu_tooltip"], RIGHT, "/esoui/art/ava/tabicon_bg_score_inactive.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", 32, 32, -213, 26, BOTTOM, TOP, ZO_GuildBankSortByName, not FCOIS.settingsVars.settings.showFCOISAdditionalInventoriesButton)
    	elseif buttonName == "FCODeconstructionContextMenu" then
			--Add or hide a button to the crafting station deconstruction sort bar
			AddButton(ZO_SmithingTopLevelDeconstructionPanelInventorySortBy, FCOIS.invAdditionalButtonVars.smithingTopLevelDeconstructionPanelInventoryButtonAdditionalOptions, showContextMenuForAdditionalInvButtons, onContextMenuButtonMouseUp, 2, nil, nil, FCOIS.localizationVars.fcois_loc["button_context_menu_tooltip"], RIGHT, "/esoui/art/ava/tabicon_bg_score_inactive.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", 32, 32, -208, 26, BOTTOM, TOP, ZO_SmithingTopLevelDeconstructionPanelInventorySortByName, not FCOIS.settingsVars.settings.showFCOISAdditionalInventoriesButton)
    	elseif buttonName == "FCOImprovementContextMenu" then
			--Add or hide a button to the crafting station improvement sort bar
			AddButton(ZO_SmithingTopLevelImprovementPanelInventorySortBy, FCOIS.invAdditionalButtonVars.smithingTopLevelImprovementPanelInventoryButtonAdditionalOptions, showContextMenuForAdditionalInvButtons, onContextMenuButtonMouseUp, 2, nil, nil, FCOIS.localizationVars.fcois_loc["button_context_menu_tooltip"], RIGHT, "/esoui/art/ava/tabicon_bg_score_inactive.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", 32, 32, -208, 26, BOTTOM, TOP, ZO_SmithingTopLevelImprovementPanelInventorySortByName, not FCOIS.settingsVars.settings.showFCOISAdditionalInventoriesButton)
    	elseif buttonName == "FCOEnchantingContextMenu" then
			--Add or hide a button to the crafting station improvement sort bar
			AddButton(ZO_EnchantingTopLevelInventorySortBy, FCOIS.invAdditionalButtonVars.enchantingTopLevelInventoryButtonAdditionalOptions, showContextMenuForAdditionalInvButtons, onContextMenuButtonMouseUp, 2, nil, nil, FCOIS.localizationVars.fcois_loc["button_context_menu_tooltip"], RIGHT, "/esoui/art/ava/tabicon_bg_score_inactive.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", 32, 32, -208, 26, BOTTOM, TOP, ZO_EnchantingTopLevelInventorySortByName, not FCOIS.settingsVars.settings.showFCOISAdditionalInventoriesButton)
    	elseif buttonName == "FCOCraftBagContextMenu" then
			--Add or hide a button to the crafting station improvement sort bar
			AddButton(ZO_CraftBagSortBy, FCOIS.invAdditionalButtonVars.craftBagInventoryButtonAdditionalOptions, showContextMenuForAdditionalInvButtons, onContextMenuButtonMouseUp, 2, nil, nil, FCOIS.localizationVars.fcois_loc["button_context_menu_tooltip"], RIGHT, "/esoui/art/ava/tabicon_bg_score_inactive.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", "/esoui/art/ava/tabicon_bg_score_disabled.dds", 32, 32, -233, 26, BOTTOM, TOP, ZO_CraftBagSortByName, not FCOIS.settingsVars.settings.showFCOISAdditionalInventoriesButton)
		end
    end
end

--Map the texture path to the texture ID
local function GetFCOTextureId(texturePath)
	if texturePath == nil or texturePath == "" then return 0 end
	for textureId, texturePathString in pairs(FCOIS.textureVars.MARKER_TEXTURES) do
    	if	texturePathString == texturePath then
        	return textureId
        end
    end
    return 0
end

--==============================================================================
--=========================== BEGIN SETTINGS ===================================
--==============================================================================
-- Build the options menu
local function BuildAddonMenu()
	local panelData = {
		type 				= 'panel',
		name 				= FCOIS.addonVars.addonNameMenu,
		displayName 		= FCOIS.addonVars.addonNameMenuDisplay,
		author 				= FCOIS.addonVars.addonAuthor,
		version 			= FCOIS.addonVars.addonVersionOptions,
		registerForRefresh 	= true,
		registerForDefaults = true,
		slashCommand = "/fcoiss",
	}

	-- Build options menu parts
	local texturesList = {
		FCOIS.localizationVars.fcois_loc["options_lock"],
		FCOIS.localizationVars.fcois_loc["options_gear"],
		FCOIS.localizationVars.fcois_loc["options_research"],
		FCOIS.localizationVars.fcois_loc["options_coins"],
        FCOIS.localizationVars.fcois_loc["options_star"],
        FCOIS.localizationVars.fcois_loc["options_flag"],
        FCOIS.localizationVars.fcois_loc["options_boxstar"],
        FCOIS.localizationVars.fcois_loc["options_medic"],
        FCOIS.localizationVars.fcois_loc["options_timer"],
        FCOIS.localizationVars.fcois_loc["options_flask"],
		FCOIS.localizationVars.fcois_loc["options_cancel_up"],
	    FCOIS.localizationVars.fcois_loc["options_info_up"],
	    FCOIS.localizationVars.fcois_loc["options_pinned_normal"],
	    FCOIS.localizationVars.fcois_loc["options_cadwell_indexicon_gold_up"],
	    FCOIS.localizationVars.fcois_loc["options_cadwell_indexicon_silver_up"],
	    FCOIS.localizationVars.fcois_loc["options_campaignbonus_keepicon"],
	    FCOIS.localizationVars.fcois_loc["options_campaignbonus_scrollicon"],
	    FCOIS.localizationVars.fcois_loc["options_campaignbrowser_columnheader_ad"],
	    FCOIS.localizationVars.fcois_loc["options_campaignbrowser_columnheader_dc"],
	    FCOIS.localizationVars.fcois_loc["options_campaignbrowser_columnheader_ep"],
	    FCOIS.localizationVars.fcois_loc["options_campaignbrowser_guild"],
	    FCOIS.localizationVars.fcois_loc["options_campaignbrowser_indexicon_normal_up"],
	    FCOIS.localizationVars.fcois_loc["options_overview_indexicon_scoring_up"],
	    FCOIS.localizationVars.fcois_loc["options_charactercreate_bodyicon_up"],
	    FCOIS.localizationVars.fcois_loc["options_gearslot_offhand"],
	    FCOIS.localizationVars.fcois_loc["options_gearslot_mainhand"],
	    FCOIS.localizationVars.fcois_loc["options_gearslot_costume"],
	    FCOIS.localizationVars.fcois_loc["options_chat_mail_up"],
	    FCOIS.localizationVars.fcois_loc["options_chat_notification_up"],
	    FCOIS.localizationVars.fcois_loc["options_alchemy_tabicon_reagent_up"],
	    FCOIS.localizationVars.fcois_loc["options_smithing_tabicon_refine_up"],
	    FCOIS.localizationVars.fcois_loc["options_deathrecap_killingblow_icon"],
		FCOIS.localizationVars.fcois_loc["options_bait_emptyslot"],
		FCOIS.localizationVars.fcois_loc["options_guildhistory_indexicon_guildbank_up"],
		FCOIS.localizationVars.fcois_loc["options_guild_indexicon_member_up"],
		FCOIS.localizationVars.fcois_loc["options_tabicon_roster_up"],
		FCOIS.localizationVars.fcois_loc["options_poi_dungeon_complete"],
		FCOIS.localizationVars.fcois_loc["options_poi_groupinstance_complete"],
		FCOIS.localizationVars.fcois_loc["options_servicepin_magesguild"],
		FCOIS.localizationVars.fcois_loc["options_servicepin_fightersguild"],
		FCOIS.localizationVars.fcois_loc["options_lfg_dps_up"],
		FCOIS.localizationVars.fcois_loc["options_lfg_leader_icon"],
		FCOIS.localizationVars.fcois_loc["options_lfg_tank_up"],
		FCOIS.localizationVars.fcois_loc["options_lfg_veterandungeon_up"],
		FCOIS.localizationVars.fcois_loc["options_lfg_normaldungeon_up"],
		FCOIS.localizationVars.fcois_loc["options_icon_dualwield"],
		FCOIS.localizationVars.fcois_loc["options_icon_firestaff"],
		FCOIS.localizationVars.fcois_loc["options_icon_bows"],
		FCOIS.localizationVars.fcois_loc["options_icon_2handed"],
		FCOIS.localizationVars.fcois_loc["options_icon_1handed"],
		FCOIS.localizationVars.fcois_loc["options_progression_tabicon_backup_inactive"],
		FCOIS.localizationVars.fcois_loc["options_inventory_tabicon_repair_disabled"],
		FCOIS.localizationVars.fcois_loc["options_selectedquesthighlight"],
		FCOIS.localizationVars.fcois_loc["options_writ"],
		FCOIS.localizationVars.fcois_loc["options_deconstruction"],
		FCOIS.localizationVars.fcois_loc["options_improvement"],
        FCOIS.localizationVars.fcois_loc["options_withdran_money"],
        FCOIS.localizationVars.fcois_loc["options_gold"],
        FCOIS.localizationVars.fcois_loc["options_guild_bank_access"],
        FCOIS.localizationVars.fcois_loc["options_intricate"],
	}

-- !!! RU Patch Section START
	local function nvl(val) if val == nil then return "..." end return val end
	local LV_Cur = FCOIS.localizationVars.fcois_loc
	local LV_Eng = FCOIS.localizationVars.localizationAll[1]
	local languageOptions = {}
	for i=1, FCOIS.numVars.languageCount do
		local s="options_language_dropdown_selection"..i
		if LV_Cur==LV_Eng then
			languageOptions[i] = nvl(LV_Cur[s])
		else
			languageOptions[i] = nvl(LV_Cur[s]) .. " (" .. nvl(LV_Eng[s]) .. ")"
		end
	end
-- !!! RU Patch Section END

    local savedVariablesOptions = {
    	[1] = FCOIS.localizationVars.fcois_loc["options_savedVariables_dropdown_selection1"],
        [2] = FCOIS.localizationVars.fcois_loc["options_savedVariables_dropdown_selection2"],
    }

	--Build the list of colored qualities for the settings
	local colorMagic = GetItemQualityColor(ITEM_QUALITY_MAGIC)
	local colorArcane = GetItemQualityColor(ITEM_QUALITY_ARCANE)
	local colorArtifact = GetItemQualityColor(ITEM_QUALITY_ARTIFACT)
	local colorLegendary = GetItemQualityColor(ITEM_QUALITY_LEGENDARY)
    local qualityList = {
		--[ITEM_QUALITY_TRASH] = FCOIS.localizationVars.fcois_loc["options_quality_trash"],
		--[ITEM_QUALITY_NORMAL] = FCOIS.localizationVars.fcois_loc["options_quality_normal"],
		[1] = FCOIS.localizationVars.fcois_loc["options_quality_OFF"],
		[ITEM_QUALITY_MAGIC] 	 = colorMagic:Colorize(FCOIS.localizationVars.fcois_loc["options_quality_magic"]),
		[ITEM_QUALITY_ARCANE] 	 = colorArcane:Colorize(FCOIS.localizationVars.fcois_loc["options_quality_arcane"]),
		[ITEM_QUALITY_ARTIFACT]  = colorArtifact:Colorize(FCOIS.localizationVars.fcois_loc["options_quality_artifact"]),
		[ITEM_QUALITY_LEGENDARY] = colorLegendary:Colorize(FCOIS.localizationVars.fcois_loc["options_quality_legendary"]),
	}

	--Build the icons index list
	FCOIS.iconVars.iconsIndexList = buildIconsIndexList()

	--Build the icon lists for the options
	local function buildIconsList(type)
    	type = type or 'standard'
		local iconsList = {}
        if type == 'standard' then
			for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				local locNameStr = FCOIS.localizationVars.iconEndStrArray[i]
               	iconsList[i] = FCOIS.localizationVars.fcois_loc["options_icon" .. tostring(i) .. "_" .. locNameStr]
            end
        elseif type == 'keybinds' then
			--Check for each icon if it is enabled in the settings
			for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				local locNameStr = FCOIS.localizationVars.iconEndStrArray[i]
               	iconsList[i] = FCOIS.localizationVars.fcois_loc["options_icon" .. tostring(i) .. "_" .. locNameStr]
--[[
				if FCOIS.settingsVars.settings.isIconEnabled[i] then
					local locNameStr = FCOIS.localizationVars.iconEndStrArray[i]
                    local textStr = FCOIS.localizationVars.fcois_loc["options_icon" .. tostring(i) .. "_" .. locNameStr]
                	table.insert(iconsList, textStr)
                end
]]
            end
        elseif type == 'gearSets' then
			local gearCounter = 1
			for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
                --Check if icon is a gear set icon and if it's enabled
                if FCOIS.mappingVars.iconIsGear[i] and FCOIS.settingsVars.settings.isIconEnabled[i] then
	            	local locNameStr = FCOIS.localizationVars.iconEndStrArray[i]
	               	iconsList[gearCounter] = FCOIS.localizationVars.fcois_loc["options_icon" .. tostring(i) .. "_" .. locNameStr]
                    gearCounter = gearCounter + 1
                end
            end
        end
        return iconsList
    end
	--Build the icons list and the keybindings icons list
    local iconsList = buildIconsList('standard')
    local iconsListStandardIconOnKeybind = buildIconsList('keybinds')
	--Build the gear sets icon dropdown for the automatic marking
	--local iconsListGearSets = buildIconsList('gearSets')

	--The LibAddonMenu2.0 settings panel reference variable
	FCOIS.FCOSettingsPanel = LAM:RegisterAddonPanel(FCOIS.addonVars.gAddonName, panelData)

	--LAM 2.0 callback function if the panel was created
	local FCOLAMPanelCreated = function(panel)
		if panel == FCOIS.FCOSettingsPanel then
			--Icon 1
		    FCOItemSaver_Settings_Filter1Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[1].color))
		    FCOItemSaver_Settings_Filter1Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[1].size)
			FCOItemSaver_Settings_Filter1Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon1_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[1].texture])
			--Icon 2
		    FCOItemSaver_Settings_Filter2Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[2].color))
		    FCOItemSaver_Settings_Filter2Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[2].size)
			FCOItemSaver_Settings_Filter2Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon2_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[2].texture])
			--Icon 3
		    FCOItemSaver_Settings_Filter3Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[3].color))
		    FCOItemSaver_Settings_Filter3Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[3].size)
			FCOItemSaver_Settings_Filter3Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon3_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[3].texture])
			--Icon 4
		    FCOItemSaver_Settings_Filter4Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[4].color))
		    FCOItemSaver_Settings_Filter4Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[4].size)
			FCOItemSaver_Settings_Filter4Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon4_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[4].texture])
			--Icon 5
		    FCOItemSaver_Settings_Filter5Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[5].color))
		    FCOItemSaver_Settings_Filter5Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[5].size)
			FCOItemSaver_Settings_Filter5Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon5_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[5].texture])
			--Icon 6
		    FCOItemSaver_Settings_Filter6Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[6].color))
		    FCOItemSaver_Settings_Filter6Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[6].size)
			FCOItemSaver_Settings_Filter6Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon6_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[6].texture])
			--Icon 7
		    FCOItemSaver_Settings_Filter7Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[7].color))
		    FCOItemSaver_Settings_Filter7Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[7].size)
			FCOItemSaver_Settings_Filter7Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon7_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[7].texture])
			--Icon 8
		    FCOItemSaver_Settings_Filter8Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[8].color))
		    FCOItemSaver_Settings_Filter8Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[8].size)
			FCOItemSaver_Settings_Filter8Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon8_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[8].texture])
			--Icon 9
		    FCOItemSaver_Settings_Filter9Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[9].color))
		    FCOItemSaver_Settings_Filter9Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[9].size)
			FCOItemSaver_Settings_Filter9Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon9_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[9].texture])
			--Icon 10
		    FCOItemSaver_Settings_Filter10Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[10].color))
		    FCOItemSaver_Settings_Filter10Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[10].size)
			FCOItemSaver_Settings_Filter10Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon10_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[10].texture])
			--Icon 11
		    FCOItemSaver_Settings_Filter11Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[11].color))
		    FCOItemSaver_Settings_Filter11Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[11].size)
			FCOItemSaver_Settings_Filter11Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon11_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[11].texture])
			--Icon 12
		    FCOItemSaver_Settings_Filter12Preview_Select:SetColor(ZO_ColorDef:New(FCOIS.settingsVars.settings.icon[12].color))
		    FCOItemSaver_Settings_Filter12Preview_Select:SetIconSize(FCOIS.settingsVars.settings.icon[12].size)
			FCOItemSaver_Settings_Filter12Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon12_texture"] .. ": " .. texturesList[FCOIS.settingsVars.settings.icon[12].texture])

	        CALLBACK_MANAGER:UnregisterCallback("LAM-RefreshPanel", FCOLAMPanelCreated)
        end
    end
	CALLBACK_MANAGER:RegisterCallback("LAM-PanelControlsCreated", FCOLAMPanelCreated)

	--The option controls for the LAM 2.0 panel
	local optionsTable =
    {	-- BEGIN OF OPTIONS TABLE

		{
			type = 'description',
			text = FCOIS.localizationVars.fcois_loc["options_description"],
		},

--==============================================================================
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header1"],
			controls = {
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_language"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_language_tooltip"],
					choices = languageOptions,
		            getFunc = function() return languageOptions[FCOIS.settingsVars.defaultSettings.language] end,
		            setFunc = function(value)
		                for i,v in pairs(languageOptions) do
		                    if v == value then
		                        debugMessage("[Settings language] v: " .. tostring(v) .. ", i: " .. tostring(i), false)
		                    	FCOIS.settingsVars.defaultSettings.language = i
		                        --Tell the FCOIS.settingsVars.settings that you have manually chosen the language and want to keep it
		                        --Read in function Localization() after ReloadUI()
		                        FCOIS.settingsVars.settings.languageChoosen = true
								--FCOIS.localizationVars.fcois_loc			  	 = FCOIS.localizationVars.fcois_loc[i]
		                        ReloadUI()
		                    end
		                end
		            end,
		            warning = FCOIS.localizationVars.fcois_loc["options_language_description1"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_savedvariables"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_savedvariables_tooltip"],
					choices = savedVariablesOptions,
		            getFunc = function() return savedVariablesOptions[FCOIS.settingsVars.defaultSettings.saveMode] end,
		            setFunc = function(value)
		                for i,v in pairs(savedVariablesOptions) do
		                    if v == value then
		                        debugMessage("[Settings save mode] v: " .. tostring(v) .. ", i: " .. tostring(i), false)
		                        FCOIS.settingsVars.defaultSettings.saveMode = i
		                        ReloadUI()
		                    end
		                end
		            end,
		            warning = FCOIS.localizationVars.fcois_loc["options_language_description1"],
				},
            }
		},
--==============================================================================
-- vvv OTHER ICONS vvv
--==============================================================================
		{
			type = "header",
			name = FCOIS.localizationVars.fcois_loc["options_header_color"]
		},
--==============================================================================
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_icons_non_gear"],
			controls =
			{
--==============================================================================
                {
	            	type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icon1_color"],
					reference = "FCOIS_OPTIONS_" .. FCOIS.localizationVars.fcois_loc["options_icon1_color"].."_submenu",
					controls =
					{
						{
							type = "colorpicker",
							name = FCOIS.localizationVars.fcois_loc["options_icon1_color"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon1_color_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[1].color.r, FCOIS.settingsVars.settings.icon[1].color.g, FCOIS.settingsVars.settings.icon[1].color.b, FCOIS.settingsVars.settings.icon[1].color.a end,
				            setFunc = function(r,g,b,a)
				            	FCOIS.settingsVars.settings.icon[1].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter1Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
								--Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
							end,
				            width="half",
				            default = FCOIS.settingsVars.settings.icon[1].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[1] end,
						},
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon1_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon1_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[1].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[1].texture = textureId
									FCOItemSaver_Settings_Filter1Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon1_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton1", "")
									UpdateButtonColorsAndTextures(1, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[1].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[1].texture],
						    reference = "FCOItemSaver_Settings_Filter1Preview_Select",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[1] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon1_tooltip"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon1_tooltip_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[1] end,
							setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[1] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[1],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[1] end,
						},
				 		{
							type = "slider",
							name = FCOIS.localizationVars.fcois_loc["options_icon1_size"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon1_size_tooltip"],
							min = 12,
							max = 48,
							getFunc = function() return FCOIS.settingsVars.settings.icon[1].size end,
							setFunc = function(size)
									FCOIS.settingsVars.settings.icon[1].size = size
									FCOItemSaver_Settings_Filter1Preview_Select:SetIconSize(size)
				 				end,
				            width="half",
							default = FCOIS.settingsVars.settings.icon[1].size,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[1] end,
						},
					} -- controls icon 1
	        	}, -- submenu icon 1
--==============================================================================
                {
					type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icon3_color"],
					reference = "FCOIS_OPTIONS_" .. FCOIS.localizationVars.fcois_loc["options_icon3_color"].."_submenu",
					controls =
					{
						{
							type = "colorpicker",
							name = FCOIS.localizationVars.fcois_loc["options_icon3_color"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon3_color_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[3].color.r, FCOIS.settingsVars.settings.icon[3].color.g, FCOIS.settingsVars.settings.icon[3].color.b, FCOIS.settingsVars.settings.icon[3].color.a end,
							setFunc = function(r,g,b,a)
								FCOIS.settingsVars.settings.icon[3].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter3Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
								--Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
								updateContextMenuFilterButtons(3, false, -1, "ResDecImp")
							end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[3].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[3] end,
						},
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon3_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon3_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[3].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[3].texture = textureId
									FCOItemSaver_Settings_Filter3Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon3_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton3", "")
									UpdateButtonColorsAndTextures(3, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
									updateContextMenuFilterButtons(3, false, -1, "ResDecImp")
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[3].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[3].texture],
						    reference = "FCOItemSaver_Settings_Filter3Preview_Select",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[3] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon3_tooltip"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon3_tooltip_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[3] end,
							setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[3] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[3],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[3] end,
						},
						{
							type = "slider",
							name = FCOIS.localizationVars.fcois_loc["options_icon3_size"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon3_size_tooltip"],
							min = 12,
							max = 48,
							getFunc = function() return FCOIS.settingsVars.settings.icon[3].size end,
							setFunc = function(size)
									FCOIS.settingsVars.settings.icon[3].size = size
				                    FCOItemSaver_Settings_Filter3Preview_Select:SetIconSize(size)
									--Set global variable to update the marker colors and textures
					                FCOIS.preventerVars.gUpdateMarkersNow = true
								end,
							default = FCOIS.settingsVars.settings.icon[3].size,
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[3] end,
						},
		            } -- controls icon 3
				}, -- submenu icon 3
--==============================================================================
                {
					type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icon5_color"],
					reference = "FCOIS_OPTIONS_" .. FCOIS.localizationVars.fcois_loc["options_icon5_color"].."_submenu",
					controls =
					{
						{
							type = "colorpicker",
							name = FCOIS.localizationVars.fcois_loc["options_icon5_color"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon5_color_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[5].color.r, FCOIS.settingsVars.settings.icon[5].color.g, FCOIS.settingsVars.settings.icon[5].color.b, FCOIS.settingsVars.settings.icon[5].color.a end,
							setFunc = function(r,g,b,a)
								FCOIS.settingsVars.settings.icon[5].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter5Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
								--Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
							end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[5].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[5] end,
						},
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon5_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon5_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[5].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[5].texture = textureId
									FCOItemSaver_Settings_Filter5Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon5_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton4", "")
									UpdateButtonColorsAndTextures(5, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
									updateContextMenuFilterButtons(5, false, -1, "SellGuildInt")
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[5].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[5].texture],
						    reference = "FCOItemSaver_Settings_Filter5Preview_Select",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[5] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon5_tooltip"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon5_tooltip_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[5] end,
							setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[5] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[5],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[5] end,
						},
						{
							type = "slider",
							name = FCOIS.localizationVars.fcois_loc["options_icon5_size"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon5_size_tooltip"],
							min = 12,
							max = 48,
							getFunc = function() return FCOIS.settingsVars.settings.icon[5].size end,
							setFunc = function(size)
									FCOIS.settingsVars.settings.icon[5].size = size
				                    FCOItemSaver_Settings_Filter5Preview_Select:SetIconSize(size)
									--Set global variable to update the marker colors and textures
					                FCOIS.preventerVars.gUpdateMarkersNow = true
								end,
							default = FCOIS.settingsVars.settings.icon[5].size,
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[5] end,
						},
		            } -- controls icon 5
				}, -- submenu icon 5
--==============================================================================
                {
					type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icon9_color"],
					reference = "FCOIS_OPTIONS_" .. FCOIS.localizationVars.fcois_loc["options_icon9_color"].."_submenu",
					controls =
					{
						{
							type = "colorpicker",
							name = FCOIS.localizationVars.fcois_loc["options_icon9_color"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon9_color_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[9].color.r, FCOIS.settingsVars.settings.icon[9].color.g, FCOIS.settingsVars.settings.icon[9].color.b, FCOIS.settingsVars.settings.icon[9].color.a end,
							setFunc = function(r,g,b,a)
								FCOIS.settingsVars.settings.icon[9].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter9Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
								--Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
								updateContextMenuFilterButtons(9, false, -1, "ResDecImp")
							end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[9].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[9] end,
						},
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon9_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon9_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[9].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[9].texture = textureId
									FCOItemSaver_Settings_Filter9Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon9_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton3", "")
									UpdateButtonColorsAndTextures(9, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
									updateContextMenuFilterButtons(9, false, -1, "ResDecImp")
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[9].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[9].texture],
						    reference = "FCOItemSaver_Settings_Filter9Preview_Select",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[9] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon9_tooltip"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon9_tooltip_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[9] end,
							setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[9] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[9],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[9] end,
						},
						{
							type = "slider",
							name = FCOIS.localizationVars.fcois_loc["options_icon9_size"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon9_size_tooltip"],
							min = 12,
							max = 48,
							getFunc = function() return FCOIS.settingsVars.settings.icon[9].size end,
							setFunc = function(size)
									FCOIS.settingsVars.settings.icon[9].size = size
				                    FCOItemSaver_Settings_Filter9Preview_Select:SetIconSize(size)
									--Set global variable to update the marker colors and textures
					                FCOIS.preventerVars.gUpdateMarkersNow = true
								end,
							default = FCOIS.settingsVars.settings.icon[9].size,
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[9] end,
						},
		            } -- controls icon 9
				}, -- submenu icon 9
--==============================================================================
                {
					type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icon10_color"],
					reference = "FCOIS_OPTIONS_" .. FCOIS.localizationVars.fcois_loc["options_icon10_color"].."_submenu",
					controls =
					{
						{
							type = "colorpicker",
							name = FCOIS.localizationVars.fcois_loc["options_icon10_color"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon10_color_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[10].color.r, FCOIS.settingsVars.settings.icon[10].color.g, FCOIS.settingsVars.settings.icon[10].color.b, FCOIS.settingsVars.settings.icon[10].color.a end,
							setFunc = function(r,g,b,a)
								FCOIS.settingsVars.settings.icon[10].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter10Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
								--Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
								updateContextMenuFilterButtons(10, false, -1, "ResDecImp")
							end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[10].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[10] end,
						},
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon10_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon10_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[10].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[10].texture = textureId
									FCOItemSaver_Settings_Filter10Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon10_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton3", "")
									UpdateButtonColorsAndTextures(10, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
									updateContextMenuFilterButtons(10, false, -1, "ResDecImp")
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[10].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[10].texture],
						    reference = "FCOItemSaver_Settings_Filter10Preview_Select",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[10] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon10_tooltip"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon10_tooltip_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[10] end,
							setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[10] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[10],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[10] end,
						},
						{
							type = "slider",
							name = FCOIS.localizationVars.fcois_loc["options_icon10_size"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon10_size_tooltip"],
							min = 12,
							max = 48,
							getFunc = function() return FCOIS.settingsVars.settings.icon[10].size end,
							setFunc = function(size)
									FCOIS.settingsVars.settings.icon[10].size = size
				                    FCOItemSaver_Settings_Filter10Preview_Select:SetIconSize(size)
									--Set global variable to update the marker colors and textures
					                FCOIS.preventerVars.gUpdateMarkersNow = true
								end,
							default = FCOIS.settingsVars.settings.icon[10].size,
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[10] end,
						},
		            } -- controls icon 10
				}, -- submenu icon 10
--==============================================================================
                {
					type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icon11_color"],
					reference = "FCOIS_OPTIONS_" .. FCOIS.localizationVars.fcois_loc["options_icon11_color"].."_submenu",
					controls =
					{
				        {
				            type = "colorpicker",
				            name = FCOIS.localizationVars.fcois_loc["options_icon11_color"],
				            tooltip = FCOIS.localizationVars.fcois_loc["options_icon11_color_tooltip"],
				            getFunc = function() return FCOIS.settingsVars.settings.icon[11].color.r, FCOIS.settingsVars.settings.icon[11].color.g, FCOIS.settingsVars.settings.icon[11].color.b, FCOIS.settingsVars.settings.icon[11].color.a end,
				            setFunc = function(r,g,b,a)
				                FCOIS.settingsVars.settings.icon[11].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter11Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
				                --Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            width = "half",
				            default = FCOIS.settingsVars.settings.icon[11].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[11] end,
				        },
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon11_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon11_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[11].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[11].texture = textureId
									FCOItemSaver_Settings_Filter11Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon11_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton4", "")
									UpdateButtonColorsAndTextures(11, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
									updateContextMenuFilterButtons(11, false, -1, "SellGuildInt")
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[11].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[11].texture],
						    reference = "FCOItemSaver_Settings_Filter11Preview_Select",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[11] end,
						},
				        {
				            type = "checkbox",
				            name = FCOIS.localizationVars.fcois_loc["options_icon11_tooltip"],
				            tooltip = FCOIS.localizationVars.fcois_loc["options_icon11_tooltip_tooltip"],
				            getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[11] end,
				            setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[11] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[11],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[11] end,
				        },
				        {
				            type = "slider",
				            name = FCOIS.localizationVars.fcois_loc["options_icon11_size"],
				            tooltip = FCOIS.localizationVars.fcois_loc["options_icon11_size_tooltip"],
				            min = 12,
				            max = 48,
				            getFunc = function() return FCOIS.settingsVars.settings.icon[11].size end,
				            setFunc = function(size)
				                FCOIS.settingsVars.settings.icon[11].size = size
				                FCOItemSaver_Settings_Filter11Preview_Select:SetIconSize(size)
				                --Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.icon[11].size,
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[11] end,
				        },
		            } -- controls icon 11
				}, -- submenu icon 11
--==============================================================================
                {
					type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icon12_color"],
					reference = "FCOIS_OPTIONS_" .. FCOIS.localizationVars.fcois_loc["options_icon12_color"].."_submenu",
					controls =
					{
				        {
				            type = "colorpicker",
				            name = FCOIS.localizationVars.fcois_loc["options_icon12_color"],
				            tooltip = FCOIS.localizationVars.fcois_loc["options_icon12_color_tooltip"],
				            getFunc = function() return FCOIS.settingsVars.settings.icon[12].color.r, FCOIS.settingsVars.settings.icon[12].color.g, FCOIS.settingsVars.settings.icon[12].color.b, FCOIS.settingsVars.settings.icon[12].color.a end,
				            setFunc = function(r,g,b,a)
				                FCOIS.settingsVars.settings.icon[12].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter12Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
				                --Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            width = "half",
				            default = FCOIS.settingsVars.settings.icon[12].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[12] end,
				        },
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon12_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon12_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[12].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[12].texture = textureId
									FCOItemSaver_Settings_Filter12Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon12_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton4", "")
									UpdateButtonColorsAndTextures(12, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
									updateContextMenuFilterButtons(12, false, -1, "SellGuildInt")
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[12].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[12].texture],
						    reference = "FCOItemSaver_Settings_Filter12Preview_Select",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[12] end,
						},
				        {
				            type = "checkbox",
				            name = FCOIS.localizationVars.fcois_loc["options_icon12_tooltip"],
				            tooltip = FCOIS.localizationVars.fcois_loc["options_icon12_tooltip_tooltip"],
				            getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[12] end,
				            setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[12] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[12],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[12] end,
				        },
				        {
				            type = "slider",
				            name = FCOIS.localizationVars.fcois_loc["options_icon12_size"],
				            tooltip = FCOIS.localizationVars.fcois_loc["options_icon12_size_tooltip"],
				            min = 12,
				            max = 48,
				            getFunc = function() return FCOIS.settingsVars.settings.icon[12].size end,
				            setFunc = function(size)
				                FCOIS.settingsVars.settings.icon[12].size = size
				                FCOItemSaver_Settings_Filter12Preview_Select:SetIconSize(size)
				                --Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.icon[12].size,
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[12] end,
				        },
		            } -- controls icon 12
				}, -- submenu icon 12
--==============================================================================
			} -- controls normal icons
		}, -- submenu normal icons
			{
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon1_activate_text"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon_activate_text_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[1] end,
				setFunc = function(value) FCOIS.settingsVars.settings.isIconEnabled[1] = value
	            	FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries
	                updateContextMenuInventoryButtons(1, false, -1)
	            end,
	            default = FCOIS.settingsVars.settings.isIconEnabled[1],
	            width="full",
			},
			{
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon3_activate_text"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon_activate_text_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[3] end,
				setFunc = function(value) FCOIS.settingsVars.settings.isIconEnabled[3] = value
	            	FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries
	                updateContextMenuInventoryButtons(3, false, -1)
					updateContextMenuFilterButtons(3, false, -1, "ResDecImp")
	            end,
	            default = FCOIS.settingsVars.settings.isIconEnabled[3],
	            width="full",
			},
			{
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon5_activate_text"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon_activate_text_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[5] end,
				setFunc = function(value) FCOIS.settingsVars.settings.isIconEnabled[5] = value
	            	FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries
	                updateContextMenuInventoryButtons(5, false, -1)
					updateContextMenuFilterButtons(5, false, -1, "SellGuildInt")
	            end,
	            default = FCOIS.settingsVars.settings.isIconEnabled[5],
	            width="full",
			},
			{
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon9_activate_text"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon_activate_text_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[9] end,
				setFunc = function(value) FCOIS.settingsVars.settings.isIconEnabled[9] = value
	            	FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries
	                updateContextMenuInventoryButtons(9, false, -1)
					updateContextMenuFilterButtons(9, false, -1, "ResDecImp")
	            end,
	            default = FCOIS.settingsVars.settings.isIconEnabled[9],
	            width="full",
			},
			{
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon10_activate_text"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon_activate_text_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[10] end,
				setFunc = function(value) FCOIS.settingsVars.settings.isIconEnabled[10] = value
	            	FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries
	                updateContextMenuInventoryButtons(10, false, -1)
					updateContextMenuFilterButtons(10, false, -1, "ResDecImp")
	            end,
	            default = FCOIS.settingsVars.settings.isIconEnabled[10],
	            width="full",
			},
			{
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon11_activate_text"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon_activate_text_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[11] end,
				setFunc = function(value) FCOIS.settingsVars.settings.isIconEnabled[11] = value
	            	FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries
	                updateContextMenuInventoryButtons(11, false, -1)
					updateContextMenuFilterButtons(11, false, -1, "SellGuildInt")
	            end,
	            default = FCOIS.settingsVars.settings.isIconEnabled[11],
	            width="full",
			},
			{
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon12_activate_text"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon_activate_text_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[12] end,
				setFunc = function(value) FCOIS.settingsVars.settings.isIconEnabled[12] = value
	            	FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries
	                updateContextMenuInventoryButtons(12, false, -1)
					updateContextMenuFilterButtons(12, false, -1, "SellGuildInt")
					--Update the combobox choices of the standard keybind icon now
                    --updateComboBoxChoices(FCOItemSaver_Standard_Icon_On_Keybind_Dropdown, "keybinds")
	            end,
	            default = FCOIS.settingsVars.settings.isIconEnabled[12],
	            width="full",
			},

--==============================================================================
-- ^^^ OTHER ICONS ^^^
--
-- vvv GEAR SETS vvv
--==============================================================================
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_icons_gears"],
			controls =
            {
				{
					type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icons_gear1"],
					controls =
			   		{
				   		{
							type = "editbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon2_name"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon2_name_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[2].name end,
							setFunc = function(newValue)
				            	FCOIS.settingsVars.settings.icon[2].name = newValue
								changeEquipmentGearMenuTexts(1)
				            end,
							width = "half",
							default = FCOIS.localizationVars.fcois_loc["options_icon2_name"],
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[1]] end,
						},
						{
							type = "colorpicker",
							name = FCOIS.localizationVars.fcois_loc["options_icon2_color"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon2_color_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[2].color.r, FCOIS.settingsVars.settings.icon[2].color.g, FCOIS.settingsVars.settings.icon[2].color.b, FCOIS.settingsVars.settings.icon[2].color.a end,
							setFunc = function(r,g,b,a)
								FCOIS.settingsVars.settings.icon[2].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter2Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
								updateContextMenuFilterButtons(1, false, -1, "gear")
								--Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
								updateContextMenuFilterButtons(1, false, -1, "gear")
							end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[2].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[1]] end,
						},
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon2_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon2_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[2].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[2].texture = textureId
									FCOItemSaver_Settings_Filter2Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon2_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton2", "")
									UpdateButtonColorsAndTextures(2, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
									updateContextMenuFilterButtons(1, false, -1, "gear")
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[2].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[2].texture],
						    reference = "FCOItemSaver_Settings_Filter2Preview_Select",
							disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[1]] end,
						},
						{
							type = "slider",
							name = FCOIS.localizationVars.fcois_loc["options_icon2_size"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon2_size_tooltip"],
							min = 12,
							max = 48,
							getFunc = function() return FCOIS.settingsVars.settings.icon[2].size end,
							setFunc = function(size)
									FCOIS.settingsVars.settings.icon[2].size = size
				                    FCOItemSaver_Settings_Filter2Preview_Select:SetIconSize(size)
									--Set global variable to update the marker colors and textures
					                FCOIS.preventerVars.gUpdateMarkersNow = true
								end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[2].size,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[1]] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon2_tooltip"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon2_tooltip_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[2] end,
							setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[2] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[2],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[1]] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_gear_disable_research_check"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_gear_disable_research_check_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.disableResearchCheck[2] end,
							setFunc = function(value) FCOIS.settingsVars.settings.disableResearchCheck[2] = value
				            end,
				            default = FCOIS.settingsVars.settings.disableResearchCheck[2],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[1]] end,
						},
					} -- controls gear 1
		        }, -- submenu gear 1

		--==============================================================================
				{
					type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icons_gear2"],
					controls =
			   		{
				   		{
							type = "editbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon4_name"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon4_name_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[4].name end,
							setFunc = function(newValue)
				            	FCOIS.settingsVars.settings.icon[4].name = newValue
								changeEquipmentGearMenuTexts(2)
							end,
							width = "half",
							default = FCOIS.localizationVars.fcois_loc["options_icon4_name"],
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[2]] end,
						},
						{
							type = "colorpicker",
							name = FCOIS.localizationVars.fcois_loc["options_icon4_color"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon4_color_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[4].color.r, FCOIS.settingsVars.settings.icon[4].color.g, FCOIS.settingsVars.settings.icon[4].color.b, FCOIS.settingsVars.settings.icon[4].color.a end,
							setFunc = function(r,g,b,a)
								FCOIS.settingsVars.settings.icon[4].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter4Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
								updateContextMenuFilterButtons(2, false, -1, "gear")
								--Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
							end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[4].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[2]] end,
						},
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon4_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon4_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[4].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[4].texture = textureId
									FCOItemSaver_Settings_Filter4Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon4_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton2", "")
									UpdateButtonColorsAndTextures(4, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
									updateContextMenuFilterButtons(2, false, -1, "gear")
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[4].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[4].texture],
						    reference = "FCOItemSaver_Settings_Filter4Preview_Select",
							disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[2]] end,
						},
						{
							type = "slider",
							name = FCOIS.localizationVars.fcois_loc["options_icon4_size"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon4_size_tooltip"],
							min = 12,
							max = 48,
							getFunc = function() return FCOIS.settingsVars.settings.icon[4].size end,
							setFunc = function(size)
									FCOIS.settingsVars.settings.icon[4].size = size
				                    FCOItemSaver_Settings_Filter4Preview_Select:SetIconSize(size)
									--Set global variable to update the marker colors and textures
					                FCOIS.preventerVars.gUpdateMarkersNow = true
								end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[4].size,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[2]] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon4_tooltip"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon4_tooltip_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[4] end,
							setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[4] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[4],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[2]] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_gear_disable_research_check"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_gear_disable_research_check_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.disableResearchCheck[4] end,
							setFunc = function(value) FCOIS.settingsVars.settings.disableResearchCheck[4] = value
				            end,
				            default = FCOIS.settingsVars.settings.disableResearchCheck[4],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[2]] end,
						},
					} -- controls gear 2
		        }, -- submenu gear 2

		--==============================================================================
				{
					type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icons_gear3"],
					controls =
			   		{
				   		{
							type = "editbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon6_name"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon6_name_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[6].name end,
							setFunc = function(newValue)
				            	FCOIS.settingsVars.settings.icon[6].name = newValue
								changeEquipmentGearMenuTexts(3)
							end,
							width = "half",
							default = FCOIS.localizationVars.fcois_loc["options_icon6_name"],
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[3]] end,
						},
						{
							type = "colorpicker",
							name = FCOIS.localizationVars.fcois_loc["options_icon6_color"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon6_color_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[6].color.r, FCOIS.settingsVars.settings.icon[6].color.g, FCOIS.settingsVars.settings.icon[6].color.b, FCOIS.settingsVars.settings.icon[6].color.a end,
							setFunc = function(r,g,b,a)
								FCOIS.settingsVars.settings.icon[6].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter6Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
								updateContextMenuFilterButtons(3, false, -1, "gear")
								--Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
							end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[6].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[3]] end,
						},
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon6_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon6_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[6].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[6].texture = textureId
									FCOItemSaver_Settings_Filter6Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon6_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton2", "")
									UpdateButtonColorsAndTextures(6, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
									updateContextMenuFilterButtons(3, false, -1, "gear")
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[6].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[6].texture],
						    reference = "FCOItemSaver_Settings_Filter6Preview_Select",
							disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[3]] end,
						},
						{
							type = "slider",
							name = FCOIS.localizationVars.fcois_loc["options_icon6_size"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon6_size_tooltip"],
							min = 12,
							max = 48,
							getFunc = function() return FCOIS.settingsVars.settings.icon[6].size end,
							setFunc = function(size)
									FCOIS.settingsVars.settings.icon[6].size = size
				                    FCOItemSaver_Settings_Filter6Preview_Select:SetIconSize(size)
									--Set global variable to update the marker colors and textures
					                FCOIS.preventerVars.gUpdateMarkersNow = true
								end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[6].size,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[3]] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon6_tooltip"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon6_tooltip_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[6] end,
							setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[6] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[6],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[3]] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_gear_disable_research_check"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_gear_disable_research_check_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.disableResearchCheck[6] end,
							setFunc = function(value) FCOIS.settingsVars.settings.disableResearchCheck[6] = value
				            end,
				            default = FCOIS.settingsVars.settings.disableResearchCheck[6],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[3]] end,
						},
					} -- controls gear 3
		        }, -- submenu gear 3
		--==============================================================================
				{
					type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icons_gear4"],
					controls =
			   		{
				   		{
							type = "editbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon7_name"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon7_name_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[7].name end,
							setFunc = function(newValue)
				            	FCOIS.settingsVars.settings.icon[7].name = newValue
								changeEquipmentGearMenuTexts(4)
							end,
							width = "half",
							default = FCOIS.localizationVars.fcois_loc["options_icon7_name"],
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[4]] end,
						},
						{
							type = "colorpicker",
							name = FCOIS.localizationVars.fcois_loc["options_icon7_color"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon7_color_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[7].color.r, FCOIS.settingsVars.settings.icon[7].color.g, FCOIS.settingsVars.settings.icon[7].color.b, FCOIS.settingsVars.settings.icon[7].color.a end,
							setFunc = function(r,g,b,a)
								FCOIS.settingsVars.settings.icon[7].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter7Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
								updateContextMenuFilterButtons(4, false, -1, "gear")
								--Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
							end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[7].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[4]] end,
						},
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon7_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon7_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[7].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[7].texture = textureId
									FCOItemSaver_Settings_Filter7Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon7_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton2", "")
									UpdateButtonColorsAndTextures(7, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
									updateContextMenuFilterButtons(4, false, -1, "gear")
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[7].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[7].texture],
						    reference = "FCOItemSaver_Settings_Filter7Preview_Select",
							disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[4]] end,
						},
						{
							type = "slider",
							name = FCOIS.localizationVars.fcois_loc["options_icon7_size"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon7_size_tooltip"],
							min = 12,
							max = 48,
							getFunc = function() return FCOIS.settingsVars.settings.icon[7].size end,
							setFunc = function(size)
									FCOIS.settingsVars.settings.icon[7].size = size
				                    FCOItemSaver_Settings_Filter7Preview_Select:SetIconSize(size)
									--Set global variable to update the marker colors and textures
					                FCOIS.preventerVars.gUpdateMarkersNow = true
								end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[7].size,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[4]] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon7_tooltip"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon7_tooltip_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[7] end,
							setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[7] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[7],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[4]] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_gear_disable_research_check"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_gear_disable_research_check_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.disableResearchCheck[7] end,
							setFunc = function(value) FCOIS.settingsVars.settings.disableResearchCheck[7] = value
				            end,
				            default = FCOIS.settingsVars.settings.disableResearchCheck[7],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[4]] end,
						},
					} -- controls gear 4
		        }, -- submenu gear 4
		--==============================================================================
				{
					type = "submenu",
					name = FCOIS.localizationVars.fcois_loc["options_icons_gear5"],
					controls =
		            {
				   		{
							type = "editbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon8_name"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon8_name_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[8].name end,
							setFunc = function(newValue)
				            	FCOIS.settingsVars.settings.icon[8].name = newValue
								changeEquipmentGearMenuTexts(5)
							end,
							width = "half",
							default = FCOIS.localizationVars.fcois_loc["options_icon8_name"],
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[5]] end,
						},
						{
							type = "colorpicker",
							name = FCOIS.localizationVars.fcois_loc["options_icon8_color"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon8_color_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.icon[8].color.r, FCOIS.settingsVars.settings.icon[8].color.g, FCOIS.settingsVars.settings.icon[8].color.b, FCOIS.settingsVars.settings.icon[8].color.a end,
							setFunc = function(r,g,b,a)
								FCOIS.settingsVars.settings.icon[8].color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
				                FCOItemSaver_Settings_Filter8Preview_Select:SetColor(ZO_ColorDef:New(r,g,b,a))
								updateContextMenuFilterButtons(5, false, -1, "gear")
								--Set global variable to update the marker colors and textures
				                FCOIS.preventerVars.gUpdateMarkersNow = true
							end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[8].color,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[5]] end,
						},
						{
						    type = "iconpicker",
						    name = FCOIS.localizationVars.fcois_loc["options_icon8_texture"],
						    tooltip = FCOIS.localizationVars.fcois_loc["options_icon8_texture_tooltip"],
						    choices = FCOIS.textureVars.MARKER_TEXTURES,
							choicesTooltips = texturesList,
						    getFunc = function() return FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[8].texture] end,
						    setFunc = function(texturePath)
						        local textureId = GetFCOTextureId(texturePath)
								if textureId ~= 0 then
									FCOIS.settingsVars.settings.icon[8].texture = textureId
									FCOItemSaver_Settings_Filter8Preview_Select.label:SetText(FCOIS.localizationVars.fcois_loc["options_icon8_texture"] .. ": " .. texturesList[textureId])
									local p_button = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton2", "")
									UpdateButtonColorsAndTextures(8, p_button, -999)
									--Set global variable to update the marker colors and textures
									FCOIS.preventerVars.gUpdateMarkersNow = true
									updateContextMenuFilterButtons(8, false, -1, "gear")
						        end
						    end,
						   	maxColumns = 6,
						    visibleRows = 5,
						    iconSize = FCOIS.settingsVars.settings.icon[8].size,
						    width = "half",
						    default = FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[8].texture],
						    reference = "FCOItemSaver_Settings_Filter8Preview_Select",
							disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[5]] end,
						},
						{
							type = "slider",
							name = FCOIS.localizationVars.fcois_loc["options_icon8_size"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon8_size_tooltip"],
							min = 12,
							max = 48,
							getFunc = function() return FCOIS.settingsVars.settings.icon[8].size end,
							setFunc = function(size)
									FCOIS.settingsVars.settings.icon[8].size = size
				                    FCOItemSaver_Settings_Filter8Preview_Select:SetIconSize(size)
									--Set global variable to update the marker colors and textures
					                FCOIS.preventerVars.gUpdateMarkersNow = true
								end,
				            width = "half",
							default = FCOIS.settingsVars.settings.icon[8].size,
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[5]] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_icon8_tooltip"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_icon8_tooltip_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.showMarkerTooltip[8] end,
							setFunc = function(value) FCOIS.settingsVars.settings.showMarkerTooltip[8] = value
				            	FCOIS.preventerVars.gUpdateMarkersNow = true
				            end,
				            default = FCOIS.settingsVars.settings.showMarkerTooltip[8],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[5]] end,
						},
						{
							type = "checkbox",
							name = FCOIS.localizationVars.fcois_loc["options_gear_disable_research_check"],
							tooltip = FCOIS.localizationVars.fcois_loc["options_gear_disable_research_check_tooltip"],
							getFunc = function() return FCOIS.settingsVars.settings.disableResearchCheck[8] end,
							setFunc = function(value) FCOIS.settingsVars.settings.disableResearchCheck[8] = value
				            end,
				            default = FCOIS.settingsVars.settings.disableResearchCheck[8],
				            width="half",
				            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[5]] end,
						},
			        } -- controls gear 5
			    }, -- submenu gear 5
	        } -- controls gear sets
	}, -- submenu gear sets
			{
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon2_enabled"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon2_enabled_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[1]] end,
				setFunc = function(value)
	            	FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[1]] = value
					--Hide the textures for gear 1
	            	if not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[1]] then
	                    --Character equipment
	                	RefreshEquipmentControl(nil, false, 2)
	                    FilterBasics(true)
	                else
	                    --Character equipment, create if not yet created
	                	RefreshEquipmentControl(nil, true, 2)
	                    FilterBasics(true)
	                end
	                FCOIS.preventerVars.gChangedGears 	= true
	                FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries for the gear
	                updateContextMenuInventoryButtons(FCOIS.mappingVars.gearToIcon[1], false, -1)
					updateContextMenuFilterButtons(1, false, -1, "gear")
	            end,
	            width="full",
			},
			{
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon4_enabled"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon4_enabled_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[2]] end,
				setFunc = function(value) FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[2]] = value
					--Hide the textures for gear 2
	            	if not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[2]] then
	                    --Character equipment
	                	RefreshEquipmentControl(nil, false, 4)
	                    FilterBasics(true)
	                else
	                    --Character equipment, create if not yet created
	                	RefreshEquipmentControl(nil, true, 4)
	                    FilterBasics(true)
	                end
	                FCOIS.preventerVars.gChangedGears 	= true
	                FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries for the gear
	                updateContextMenuInventoryButtons(FCOIS.mappingVars.gearToIcon[2], false, -1)
					updateContextMenuFilterButtons(2, false, -1, "gear")
	            end,
	            width="full",
			},
			{
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon6_enabled"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon6_enabled_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[3]] end,
				setFunc = function(value) FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[3]] = value
					--Hide the textures for gear 3
	            	if not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[3]] then
	                    --Character equipment
	                	RefreshEquipmentControl(nil, false, 6)
	                    FilterBasics(true)
	                else
	                    --Character equipment, create if not yet created
	                	RefreshEquipmentControl(nil, true, 6)
	                    FilterBasics(true)
	                end
	                FCOIS.preventerVars.gChangedGears 	= true
	                FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries for the gear
	                updateContextMenuInventoryButtons(FCOIS.mappingVars.gearToIcon[3], false, -1)
					updateContextMenuFilterButtons(3, false, -1, "gear")
	            end,
	            width="full",
			},
	        {
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon7_enabled"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon7_enabled_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[4]] end,
				setFunc = function(value) FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[4]] = value
					--Hide the textures for gear 4
	            	if not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[4]] then
	                    --Character equipment
	                	RefreshEquipmentControl(nil, false, 7)
	                    FilterBasics(true)
	                else
	                    --Character equipment, create if not yet created
	                	RefreshEquipmentControl(nil, true, 7)
	                    FilterBasics(true)
	                end
	                FCOIS.preventerVars.gChangedGears 	= true
	                FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries for the gear
	                updateContextMenuInventoryButtons(FCOIS.mappingVars.gearToIcon[4], false, -1)
					updateContextMenuFilterButtons(4, false, -1, "gear")
	            end,
	            width="full",
			},
	        {
				type = "checkbox",
				name = FCOIS.localizationVars.fcois_loc["options_icon8_enabled"],
				tooltip = FCOIS.localizationVars.fcois_loc["options_icon8_enabled_tooltip"],
				getFunc = function() return FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[5]] end,
				setFunc = function(value) FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[5]] = value
					--Hide the textures for gear 5
	            	if not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[5]] then
	                    --Character equipment
	                	RefreshEquipmentControl(nil, false, 8)
	                    FilterBasics(true)
	                else
	                    --Character equipment, create if not yet created
	                	RefreshEquipmentControl(nil, true, 8)
	                    FilterBasics(true)
	                end
	                FCOIS.preventerVars.gChangedGears 	= true
	                FCOIS.preventerVars.gUpdateMarkersNow = true
	                --Update the context menu button entries for the gear
	                updateContextMenuInventoryButtons(FCOIS.mappingVars.gearToIcon[5], false, -1)
					updateContextMenuFilterButtons(5, false, -1, "gear")
	            end,
	            width="full",
			},
--==============================================================================
-- ^^^ GEAR SETS ^^^
--==============================================================================

--=============================================================================================
-- DEACTIVATED ICONS OPTIONS
--=============================================================================================
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_deactivated_symbols"],
            controls =
            {
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_deactivated_symbols_apply_anti_checks"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_deactivated_symbols_apply_anti_checks_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.checkDeactivatedIcons end,
					setFunc = function(value) FCOIS.settingsVars.settings.checkDeactivatedIcons = value
		            end,
				},
            } -- controls deactivated items
		}, -- submenu deactivated items

--========= ICON SORT OPTIONS ==================================================
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_sort_order"],
            controls = {
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_1"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[1]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 1
		                        FCOIS.settingsVars.settings.iconSortOrder[1] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[1]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_1",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_2"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[2]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 2
		                        FCOIS.settingsVars.settings.iconSortOrder[2] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[2]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_2",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_3"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[3]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 3
		                        FCOIS.settingsVars.settings.iconSortOrder[3] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[3]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_3",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_4"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[4]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 4
		                        FCOIS.settingsVars.settings.iconSortOrder[4] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[4]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_4",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_5"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[5]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 5
		                        FCOIS.settingsVars.settings.iconSortOrder[5] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[5]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_5",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_6"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[6]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 6
		                        FCOIS.settingsVars.settings.iconSortOrder[6] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[6]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_6",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_7"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[7]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 7
		                        FCOIS.settingsVars.settings.iconSortOrder[7] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[7]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_7",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_8"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[8]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 8
		                        FCOIS.settingsVars.settings.iconSortOrder[8] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[8]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_8",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_9"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[9]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 9
		                        FCOIS.settingsVars.settings.iconSortOrder[9] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[9]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_9",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_10"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[10]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 10
		                        FCOIS.settingsVars.settings.iconSortOrder[10] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[10]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_10",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_11"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[11]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 11
		                        FCOIS.settingsVars.settings.iconSortOrder[11] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[11]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_11",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_sort_12"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_tooltip"],
					choices = iconsList,
		            getFunc = function() return iconsList[FCOIS.settingsVars.settings.iconSortOrder[12]] end,
		            setFunc = function(value)
		                for i,v in pairs(iconsList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.icon[i].sortOrder = 12
		                        FCOIS.settingsVars.settings.iconSortOrder[12] = i
		                    end
		                end
		            end,
		            default = iconsList[FCOIS.settingsVars.settings.iconSortOrder[12]],
		            reference = "FCOItemSaver_Icon_Sort_Dropdown_12",
		            warning = FCOIS.localizationVars.fcois_loc["options_icon_sort_order_warning"],
				},
			} -- controls sort order
		}, -- submenu sort order
--==============================================================================
-- ICON POSITIONS
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_pos"],
            controls =
            {
				{
					type = "slider",
					name = FCOIS.localizationVars.fcois_loc["options_pos_inventories"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_pos_inventories_tooltip"],
					min = -10,
					max = 540,
					getFunc = function() return FCOIS.settingsVars.settings.iconPosition.x end,
					setFunc = function(offset)
							FCOIS.settingsVars.settings.iconPosition.x = offset
							--Set global variable to update the marker colors and textures
			                FCOIS.preventerVars.gUpdateMarkersNow = true
						end,
					default = FCOIS.settingsVars.settings.iconPosition.x,
		            width="full",
				},
				{
					type = "slider",
					name = FCOIS.localizationVars.fcois_loc["options_pos_crafting"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_pos_crafting_tooltip"],
					min = -10,
					max = 540,
					getFunc = function() return FCOIS.settingsVars.settings.iconPositionCrafting.x end,
					setFunc = function(offset)
							FCOIS.settingsVars.settings.iconPositionCrafting.x = offset
							--Set global variable to update the marker colors and textures
			                FCOIS.preventerVars.gUpdateMarkersNow = true
						end,
					default = FCOIS.settingsVars.settings.iconPositionCrafting.x,
		            width="full",
				},
				{
					type = "slider",
					name = FCOIS.localizationVars.fcois_loc["options_pos_character_x"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_pos_character_x_tooltip"],
					min = -15,
					max = 40,
					getFunc = function() return FCOIS.settingsVars.settings.iconPositionCharacter.x end,
					setFunc = function(offset)
							FCOIS.settingsVars.settings.iconPositionCharacter.x = offset
							--Set global variable to update the marker colors and textures
			                FCOIS.preventerVars.gUpdateMarkersNow = true
						end,
					default = FCOIS.settingsVars.settings.iconPositionCharacter.x,
		            width="half",
				},
				{
					type = "slider",
					name = FCOIS.localizationVars.fcois_loc["options_pos_character_y"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_pos_character_y_tooltip"],
					min = -40,
					max = 15,
					getFunc = function() return FCOIS.settingsVars.settings.iconPositionCharacter.y end,
					setFunc = function(offset)
							FCOIS.settingsVars.settings.iconPositionCharacter.y = offset
							--Set global variable to update the marker colors and textures
			                FCOIS.preventerVars.gUpdateMarkersNow = true
						end,
					default = FCOIS.settingsVars.settings.iconPositionCharacter.y,
		            width="half",
				},
            } -- controls positions
		}, -- submenu positions

--==============================================================================
-- ITEM AUOMATIC MARKING
--==============================================================================
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_items"],
            controls =
            {
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_ornate_items"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_ornate_items_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkOrnate end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.autoMarkOrnate = value
		                if (FCOIS.settingsVars.settings.autoMarkOrnate == true) then
		                	scanInventoryForOrnate()
		                end
		            end,
		            width = "half",
		            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[5] end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_ornate_items_in_chat"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_ornate_items_in_chat_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showOrnateItemsInChat end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.showOrnateItemsInChat = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.autoMarkOrnate end,
		            width = "half",
		            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[5] end,
				},
		        {
		            type = "checkbox",
		            name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_intricate_items"],
		            tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_intricate_items_tooltip"],
		            getFunc = function() return FCOIS.settingsVars.settings.autoMarkIntricate end,
		            setFunc = function(value)
		                FCOIS.settingsVars.settings.autoMarkIntricate = value
		                if (FCOIS.settingsVars.settings.autoMarkIntricate == true) then
		                    scanInventoryForIntricate()
		                end
		            end,
		            width = "half",
		            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[12] end,
		        },
		        {
		            type = "checkbox",
		            name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_intricate_items_in_chat"],
		            tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_intricate_items_in_chat_tooltip"],
		            getFunc = function() return FCOIS.settingsVars.settings.showIntricateItemsInChat end,
		            setFunc = function(value)
		                FCOIS.settingsVars.settings.showIntricateItemsInChat = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.autoMarkIntricate or not FCOIS.settingsVars.settings.isIconEnabled[12] end,
		            width = "half",
		        },
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_research_items"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_research_items_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkResearch end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.autoMarkResearch = value
		                if (FCOIS.settingsVars.settings.autoMarkResearch == true and FCOIS.otherAddons.researchAssistantActive == true) then
			            	scanInventoryForResearch()
		                end
		            end,
		            disabled = function() return not FCOIS.otherAddons.researchAssistantActive or not FCOIS.settingsVars.settings.isIconEnabled[3] end,
		            warning = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_research_items_hint"],
		            width = "half",
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_research_items_in_chat"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_research_items_in_chat_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showResearchItemsInChat end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.showResearchItemsInChat = value
		            end,
		            disabled = function() return not FCOIS.otherAddons.researchAssistantActive or not FCOIS.settingsVars.settings.isIconEnabled[3] or not FCOIS.settingsVars.settings.autoMarkResearch end,
		            warning = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_research_items_hint"],
		            width = "half",
				},

				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_sets"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_sets_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkSets end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.autoMarkSets = value
		                if (FCOIS.settingsVars.settings.autoMarkSets == true) then
			            	scanInventoryForSets()
		                end
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.settingsVars.settings.autoMarkSetsGearIconNr] end,
		            width = "half",
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_auto_mark_sets_icon"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_mark_sets_icon_tooltip"],
					choices = iconsListStandardIconOnKeybind,
		            getFunc = function() return iconsListStandardIconOnKeybind[FCOIS.settingsVars.settings.autoMarkSetsGearIconNr]
                    end,
		            setFunc = function(value)
		                for i,v in pairs(iconsListStandardIconOnKeybind) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.autoMarkSetsGearIconNr = i
		                    end
		                end
		            end,
		            default = iconsListStandardIconOnKeybind[FCOIS.settingsVars.settings.autoMarkSetsGearIconNr],
		            reference = "FCOItemSaver_Icon_On_Automatic_Set_Part_Dropdown",
		            disabled = function() return not FCOIS.settingsVars.settings.autoMarkSets end,
                    width = "half",
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_sets_all_gear_marker_icons"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_sets_all_gear_marker_icons_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkSetsCheckAllGearIcons end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.autoMarkSetsCheckAllGearIcons = value
		                if (FCOIS.settingsVars.settings.autoMarkSetsCheckAllGearIcons == true) then
			            	scanInventoryForSets()
		                end
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.settingsVars.settings.autoMarkSetsGearIconNr] or not FCOIS.settingsVars.settings.autoMarkSets end,
		            width = "half",
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_sets_in_chat"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_sets_in_chat_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showSetsInChat end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.showSetsInChat = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[FCOIS.settingsVars.settings.autoMarkSetsGearIconNr] or not FCOIS.settingsVars.settings.autoMarkSets end,
		            width = "half",
				},

				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_recipes"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_recipes_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkRecipes end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.autoMarkRecipes = value
		                if (FCOIS.settingsVars.settings.autoMarkRecipes == true and FCOIS.otherAddons.sousChefActive == true and SousChef.settings.showAltKnowledge) then
			            	scanInventoryForRecipes()
		                end
		            end,
		            disabled = function() return not FCOIS.otherAddons.sousChefActive or not SousChef.settings.showAltKnowledge or not FCOIS.settingsVars.settings.isIconEnabled[1] end,
		            warning = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_recipes_hint"],
		            width = "half",
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_recipes_in_chat"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_recipes_in_chat_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showRecipesInChat end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.showRecipesInChat = value
		            end,
		            disabled = function() return not FCOIS.otherAddons.sousChefActive or not SousChef.settings.showAltKnowledge or not FCOIS.settingsVars.settings.isIconEnabled[1] or not FCOIS.settingsVars.settings.autoMarkRecipes end,
		            warning = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_recipes_hint"],
		            width = "half",
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_mark_recipes_this_char"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_mark_recipes_this_char_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkRecipesOnlyThisChar end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.autoMarkRecipesOnlyThisChar = value
		                if (FCOIS.settingsVars.settings.autoMarkRecipes == true and FCOIS.otherAddons.sousChefActive == true and SousChef.settings.showAltKnowledge) then
			            	scanInventoryForRecipes()
		                end
		            end,
		            disabled = function() return not FCOIS.otherAddons.sousChefActive or not SousChef.settings.showAltKnowledge or not FCOIS.settingsVars.settings.isIconEnabled[1] end,
		            width = "full",
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_mark_crafted_items"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_mark_crafted_items_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkCraftedItems end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.autoMarkCraftedItems = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[1] end,
		            width = "half",
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_mark_crafted_items_sets"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_mark_crafted_items_sets_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkCraftedItemsSets end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.autoMarkCraftedItemsSets = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.autoMarkCraftedItems or not FCOIS.settingsVars.settings.isIconEnabled[1] end,
		            width = "half",
				},
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_quality_items"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_quality_items_tooltip"],
					choices = qualityList,
		            getFunc = function() return qualityList[FCOIS.settingsVars.settings.autoMarkQuality] end,
		            setFunc = function(value)
		                for i,v in pairs(qualityList) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.autoMarkQuality = i
		                        if i ~= 1 then scanInventoryForQuality() end
		                        break
		                    end
		                end
		            end,
		            default = FCOIS.settingsVars.settings.autoMarkQuality,
		            width = "half",
                    disabled = function() return not FCOIS.settingsVars.settings.isIconEnabled[1] end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_quality_items_in_chat"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_quality_items_in_chat_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showQualityItemsInChat end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.showQualityItemsInChat = value
		            end,
		            disabled = function() return FCOIS.settingsVars.settings.autoMarkQuality == 1 end,
		            width = "half",
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_higher_quality_items"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_auto_mark_higher_quality_items_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkHigherQuality end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.autoMarkHigherQuality = value
		                if FCOIS.settingsVars.settings.autoMarkHigherQuality and FCOIS.settingsVars.settings.autoMarkQuality ~= 1 and FCOIS.settingsVars.settings.autoMarkQuality ~= ITEM_QUALITY_LEGENDARY then
		                	scanInventoryForQuality()
		                end
		            end,
		            disabled = function() return FCOIS.settingsVars.settings.autoMarkQuality == 1 or FCOIS.settingsVars.settings.autoMarkQuality == ITEM_QUALITY_LEGENDARY end,
		            width = "full",
				},
--==============================================================================
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_equipment"],
                },
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_equipment_markall_gear"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_equipment_markall_gear_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkAllEquipment end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoMarkAllEquipment = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_equipment_markall_gear_add_weapons"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_equipment_markall_gear_add_weapons_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkAllWeapon end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoMarkAllWeapon = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.autoMarkAllEquipment end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_equipment_markall_gear_add_jewelry"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_equipment_markall_gear_add_jewelry_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoMarkAllJewelry end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoMarkAllJewelry = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.autoMarkAllEquipment end,
				},
            } -- controls marking
		}, -- submenu marking

--==============================================================================
-- ITEM AUOMATIC DE-MARKING
--==============================================================================
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_items_demark"],
            controls =
            {
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_demark_all_selling"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_demark_all_selling_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoDeMarkSell end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoDeMarkSell = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_demark_all_selling_guild_store"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_demark_all_selling_guild_store_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoDeMarkSellInGuildStore end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoDeMarkSellInGuildStore = value
		            end,
				},
            } -- controls de-marking
		}, -- submenu de-marking

--==============================================================================
-- CONTEXT MENUs IN INVENTORY
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_context_menu"],
            controls =
            {
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_use_subcontextmenu"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_use_subcontextmenu_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.useSubContextMenu end,
					setFunc = function(value) FCOIS.settingsVars.settings.useSubContextMenu = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_show_contextmenu_divider"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_show_contextmenu_divider_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showContextMenuDivider end,
					setFunc = function(value) FCOIS.settingsVars.settings.showContextMenuDivider = value
		            end,
		            disabled = function() return FCOIS.settingsVars.settings.useSubContextMenu end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_contextmenu_divider_opens_settings"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_contextmenu_divider_opens_settings_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.contextMenuDividerShowsSettings end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.contextMenuDividerShowsSettings = value
						if FCOIS.settingsVars.settings.contextMenuDividerShowsSettings then
		                	FCOIS.settingsVars.settings.contextMenuDividerClearsMarkers = false
		                end
		            end,
		            disabled = function()
		            	if FCOIS.settingsVars.settings.useSubContextMenu then
		                	return true
		                else
		                	if FCOIS.settingsVars.settings.showContextMenuDivider then
		                    	return FCOIS.settingsVars.settings.contextMenuDividerClearsMarkers
		                    else
		                    	return true
		                    end
		                end
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_contextmenu_divider_clears_all_markers"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_contextmenu_divider_clears_all_markers_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.contextMenuDividerClearsMarkers end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.contextMenuDividerClearsMarkers = value
						if FCOIS.settingsVars.settings.contextMenuDividerClearsMarkers then
		                	FCOIS.settingsVars.settings.contextMenuDividerShowsSettings = false
		                end
		            end,
		            disabled = function()
		            	if FCOIS.settingsVars.settings.useSubContextMenu then
		                	return true
		                else
		                	if FCOIS.settingsVars.settings.showContextMenuDivider then
		                        return FCOIS.settingsVars.settings.contextMenuDividerShowsSettings
		                    else
		                    	return true
		                    end
		                end
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_contextmenu_use_custom_marked_normal_color"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_contextmenu_use_custom_marked_normal_color_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.useContextMenuCustomMarkedNormalColor end,
					setFunc = function(value) FCOIS.settingsVars.settings.useContextMenuCustomMarkedNormalColor = value
		            end,
					default = FCOIS.settingsVars.settings.useContextMenuCustomMarkedNormalColor,
		            width = "half",
				},
		        {
		            type = "colorpicker",
		            name = FCOIS.localizationVars.fcois_loc["options_contextmenu_custom_marked_normal_color"],
		            tooltip = FCOIS.localizationVars.fcois_loc["options_contextmenu_custom_marked_normal_color_tooltip"],
		            getFunc = function() return FCOIS.settingsVars.settings.contextMenuCustomMarkedNormalColor.r, FCOIS.settingsVars.settings.contextMenuCustomMarkedNormalColor.g, FCOIS.settingsVars.settings.contextMenuCustomMarkedNormalColor.b, FCOIS.settingsVars.settings.contextMenuCustomMarkedNormalColor.a end,
		            setFunc = function(r,g,b,a)
		                FCOIS.settingsVars.settings.contextMenuCustomMarkedNormalColor = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
		            end,
		            width = "half",
		            disabled = function() return not FCOIS.settingsVars.settings.useContextMenuCustomMarkedNormalColor end,
		            default = FCOIS.settingsVars.settings.contextMenuCustomMarkedNormalColor,
		        },
            } -- controls context menu
		}, -- submenu context menu

--==============================================================================
-- vvv Keybinds vvv
--==============================================================================
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_keybind_options"],
            controls = {
--[[
		        {
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_icon_cycle_on_keybind"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_cycle_on_keybind_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.cycleMarkerSymbolOnKeybind end,
					setFunc = function(value) FCOIS.settingsVars.settings.cycleMarkerSymbolOnKeybind = value
		            end,
		            width="full",
				},
]]
				{
					type = 'dropdown',
					name = FCOIS.localizationVars.fcois_loc["options_icon_standard_on_keybind"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_icon_standard_on_keybind_tooltip"],
					choices = iconsListStandardIconOnKeybind,
		            getFunc = function() return iconsListStandardIconOnKeybind[FCOIS.settingsVars.settings.standardIconOnKeybind]
                    end,
		            setFunc = function(value)
		                for i,v in pairs(iconsListStandardIconOnKeybind) do
		                    if v == value then
		                    	FCOIS.settingsVars.settings.standardIconOnKeybind = i
		                    end
		                end
		            end,
		            default = iconsListStandardIconOnKeybind[FCOIS.settingsVars.settings.standardIconOnKeybind],
		            reference = "FCOItemSaver_Standard_Icon_On_Keybind_Dropdown",
		            disabled = function()
                    	return FCOIS.settingsVars.settings.cycleMarkerSymbolOnKeybind
                    end
				},
			},  -- controls keybinds
        },  -- submenu keybinds
--==============================================================================
-- ^^^ Keybinds ^^^
--==============================================================================

--==============================================================================
--		FILTERs
--==============================================================================
        {
            type = "submenu",
            name = FCOIS.localizationVars.fcois_loc["options_header_filters"],
            controls =
            {
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_filters"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_inventory"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_inventory_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowInventoryFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowInventoryFilter = value
				        --Hide the filter buttons at the filter panel Id
						updateFilterButtonsInInv(-1)
		                --Unregister and reregister the inventory filter LF_INVENTORY
		            	EnableFilters(-100)
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_craftbag"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_craftbag_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowCraftBagFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowCraftBagFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_bank"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_bank_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowBankFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowBankFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_guildbank"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_guildbank_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowGuildBankFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowGuildBankFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_tradinghouse"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_tradinghouse_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowTradinghouseFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowTradinghouseFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_trade"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_trade_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowTradeFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowTradeFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_mail"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_mail_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowMailFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowMailFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_store"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_store_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowVendorFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowVendorFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_fence"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_fence_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowFenceFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowFenceFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_launder"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_launder_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowLaunderFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowLaunderFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_deconstruction"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_deconstruction_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowDeconstructionFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowDeconstructionFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_improvement"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_improvement_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowImprovementFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowImprovementFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_enchanting"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_enchanting_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowEnchantingFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowEnchantingFilter = value
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_alchemy"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_filter_in_alchemy_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowAlchemyFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowAlchemyFilter = value
		            end,
				},
		--==============================================================================
				{
		        	type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_filter_chat"],
			    },
			    {
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_chat_filter_status"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_chat_filter_status_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showFilterStatusInChat end,
					setFunc = function(value) FCOIS.settingsVars.settings.showFilterStatusInChat = value
		            end,
				},
            } -- controls filters
        }, -- submenu filters


--==============================================================================
--		FILTER BUTTONS
--==============================================================================
		{
        	type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_filter_buttons"],
            controls =
            {
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_split_filters"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_split_filters"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_split_filters_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.splitFilters end,
					setFunc = function(value)
		            	FCOIS.settingsVars.settings.splitFilters = value
		                --Set the global flag to override the function's unregisterFilters() unregister method
		                --because it will check for FCOIS.settingsVars.settings.SplitFilters and is called from function doFilter() at
		                --function EnableFilters(-100) below
		                FCOIS.overrideVars.gSplitFilterOverride = true
				        --Hide the filter buttons at the filter panel Id
						updateFilterButtonsInInv(-1)
		                --Unregister and reregister the inventory filter LF_INVENTORY
						EnableFilters(-100)
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_split_gearsets_filter"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_split_gearsets_filter_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.splitGearSetsFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.splitGearSetsFilter = value
				        --Change the gear sets filter context-menu button's texture
		                local gearSetSplitFilterContextMenuButton = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton2", "")
						if gearSetSplitFilterContextMenuButton ~= nil then
			                UpdateButtonColorsAndTextures(2, gearSetSplitFilterContextMenuButton, nil, LF_INVENTORY)
		                    FilterBasics(true)
		                end
		            end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_split_resdecimp_filter"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_split_resdecimp_filter_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter end,
					setFunc = function(value) FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter = value
				        --Change the gear sets filter context-menu button's texture
		                local resDecSplitFilterContextMenuButton = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton3", "")
						if resDecSplitFilterContextMenuButton ~= nil then
			                UpdateButtonColorsAndTextures(3, resDecSplitFilterContextMenuButton, nil, LF_INVENTORY)
		                    FilterBasics(true)
		                end
		            end,
				},
		        {
		            type = "checkbox",
		            name = FCOIS.localizationVars.fcois_loc["options_split_sellguildint_filter"],
		            tooltip = FCOIS.localizationVars.fcois_loc["options_split_sellguildint_filter_tooltip"],
		            getFunc = function() return FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter end,
		            setFunc = function(value) FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter = value
			            --Change the gear sets filter context-menu button's texture
			            local sellGuildIntSplitFilterContextMenuButton = WINDOW_MANAGER:GetControlByName("ZO_PlayerInventory_FilterButton4", "")
			            if sellGuildIntSplitFilterContextMenuButton ~= nil then
			                UpdateButtonColorsAndTextures(4, sellGuildIntSplitFilterContextMenuButton, nil, LF_INVENTORY)
			                FilterBasics(true)
			            end
		            end,
		        },

				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_filter_buttons_context_menu_show_tooltip"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_filter_buttons_context_menu_show_tooltip_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showFilterButtonContextTooltip end,
					setFunc = function(value) FCOIS.settingsVars.settings.showFilterButtonContextTooltip = value
		            end,
					disabled = function() if FCOIS.settingsVars.settings.splitGearSetsFilter == false and FCOIS.settingsVars.settings.splitResearchDeconstructionImprovementFilter == false and FCOIS.settingsVars.settings.splitSellGuildSellIntricateFilter == false then
		            		return true
		                else
		                	return false
		                end
		            end,
				},
		--==============================================================================
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_filter_buttons_position"],
				},
		        {
					type = "editbox",
					name = FCOIS.localizationVars.fcois_loc["options_filter_button1_left"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_filter_button1_left_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.filterButtonLeft[1] end,
					setFunc = function(newValue)
			           		FCOIS.settingsVars.settings.filterButtonLeft[1] = newValue
			                        updateFilterButtonsInInv(1)
					end,
					width = "half",
					default = FCOIS.settingsVars.settings.filterButtonLeft[1],
				},
				{
					type = "editbox",
					name = FCOIS.localizationVars.fcois_loc["options_filter_button1_top"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_filter_button1_top_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.filterButtonTop[1] end,
					setFunc = function(newValue)
			           		FCOIS.settingsVars.settings.filterButtonTop[1] = newValue
			                        updateFilterButtonsInInv(1)
					end,
					width = "half",
					default = FCOIS.settingsVars.settings.filterButtonTop[1],
				},
				{
					type = "editbox",
					name = FCOIS.localizationVars.fcois_loc["options_filter_button2_left"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_filter_button2_left_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.filterButtonLeft[2] end,
					setFunc = function(newValue)
			           		FCOIS.settingsVars.settings.filterButtonLeft[2] = newValue
			                        updateFilterButtonsInInv(2)
					end,
					width = "half",
					default = FCOIS.settingsVars.settings.filterButtonLeft[2],
				},
		        {
					type = "editbox",
					name = FCOIS.localizationVars.fcois_loc["options_filter_button2_top"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_filter_button2_top_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.filterButtonTop[2] end,
					setFunc = function(newValue)
			           		FCOIS.settingsVars.settings.filterButtonTop[2] = newValue
			                        updateFilterButtonsInInv(2)
					end,
					width = "half",
					default = FCOIS.settingsVars.settings.filterButtonTop[2],
				},
				{
					type = "editbox",
					name = FCOIS.localizationVars.fcois_loc["options_filter_button3_left"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_filter_button3_left_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.filterButtonLeft[3] end,
					setFunc = function(newValue)
			           		FCOIS.settingsVars.settings.filterButtonLeft[3] = newValue
			                        updateFilterButtonsInInv(3)
					end,
					width = "half",
					default = FCOIS.settingsVars.settings.filterButtonLeft[3],
				},
		        {
					type = "editbox",
					name = FCOIS.localizationVars.fcois_loc["options_filter_button3_top"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_filter_button3_top_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.filterButtonTop[3] end,
					setFunc = function(newValue)
			           		FCOIS.settingsVars.settings.filterButtonTop[3] = newValue
			                        updateFilterButtonsInInv(3)
					end,
					width = "half",
					default = FCOIS.settingsVars.settings.filterButtonTop[3],
				},
				{
					type = "editbox",
					name = FCOIS.localizationVars.fcois_loc["options_filter_button4_left"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_filter_button4_left_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.filterButtonLeft[4] end,
					setFunc = function(newValue)
			           		FCOIS.settingsVars.settings.filterButtonLeft[4] = newValue
			                        updateFilterButtonsInInv(4)
					end,
					width = "half",
					default = FCOIS.settingsVars.settings.filterButtonLeft[4],
				},
				{
					type = "editbox",
					name = FCOIS.localizationVars.fcois_loc["options_filter_button4_top"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_filter_button4_top_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.filterButtonTop[4] end,
					setFunc = function(newValue)
			           		FCOIS.settingsVars.settings.filterButtonTop[4] = newValue
			                        updateFilterButtonsInInv(4)
					end,
					width = "half",
					default = FCOIS.settingsVars.settings.filterButtonTop[4],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_filter_buttons_show_tooltip"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_filter_buttons_show_tooltip_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showFilterButtonTooltip end,
					setFunc = function(value) FCOIS.settingsVars.settings.showFilterButtonTooltip = value
		            end,
				},
            } -- controls filter buttons
	    }, -- submenu filter buttons

--==============================================================================
-- ANTI DESTROY
--==============================================================================
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_anti_destroy"],
            controls =
            {
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_destroy"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_destroying"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_destroying_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockDestroying end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockDestroying = value
                           	FCOIS.settingsVars.settings.autoReenable_blockDestroying = value
			           end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_destroying"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_destroying_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockDestroying end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockDestroying = value
			           end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockDestroying end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_deconstruction"],
				},
			    {
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_deconstruction"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_deconstruction_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockDeconstruction end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockDeconstruction = value
                           	FCOIS.settingsVars.settings.autoReenable_blockDeconstruction = value
			           end,
				},
			    {
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_deconstruction"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_deconstruction_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockDeconstruction end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockDeconstruction = value
			           end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockDeconstruction end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_deconstruction_exception_intricate"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_deconstruction_exception_intricate_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowDeconstructIntricate end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowDeconstructIntricate = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockDeconstruction end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_deconstruction_exception_deconstruction"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_deconstruction_exception_deconstruction_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowDeconstructDeconstruction end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowDeconstructDeconstruction = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockDeconstruction end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_improvement"],
				},
			    {
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_improvement"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_improvement_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockImprovement end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockImprovement = value
                           	FCOIS.settingsVars.settings.autoReenable_blockImprovement = value
			           end,
				},
			    {
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_improvement"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_improvement_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockImprovement end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockImprovement = value
			           end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockImprovement end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_improvement_exception_improvement"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_improvement_exception_improvement_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowImproveImprovement end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowImproveImprovement = value
		             end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockImprovement end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_research"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_research_filter"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_research_filter_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowResearch end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowResearch = value
		             end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_repair"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_allow_marked_repair"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_allow_marked_repair_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockMarkedRepairKits end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockMarkedRepairKits = value
		             end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_rune_creation"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_creation"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_creation_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockEnchantingCreation end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockEnchantingCreation = value
                           	FCOIS.settingsVars.settings.autoReenable_blockEnchantingCreation = value
			           end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_creation"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_creation_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockEnchantingCreation end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockEnchantingCreation = value
			           end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockEnchantingCreation end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_rune_extraction"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_extraction"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_extraction_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockEnchantingExtraction end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockEnchantingExtraction = value
                           	FCOIS.settingsVars.settings.autoReenable_blockEnchantingExtraction = value
			           end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_extraction"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_extraction_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockEnchantingExtraction end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockEnchantingExtraction = value
			           end,
					disabled = function() return not FCOIS.settingsVars.settings.blockEnchantingExtraction end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_sell"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_selling"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_selling_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockSelling end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockSelling = value
                           	FCOIS.settingsVars.settings.autoReenable_blockSelling = value
			           end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_selling"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_selling_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockSelling end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockSelling = value
			           end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockSelling end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_block_selling_exception"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_block_selling_exception_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowSellingForBlocked end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowSellingForBlocked = value
			           end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockSelling end,
				},
		        {
		            type = "checkbox",
		            name = FCOIS.localizationVars.fcois_loc["options_block_selling_exception_intricate"],
		            tooltip = FCOIS.localizationVars.fcois_loc["options_block_selling_exception_intricate_tooltip"],
		            getFunc = function() return FCOIS.settingsVars.settings.allowSellingForBlockedIntricate end,
		            setFunc = function(value) FCOIS.settingsVars.settings.allowSellingForBlockedIntricate = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockSelling and not FCOIS.settingsVars.settings.blockSellingGuildStore end,
		        },
		        {
		            type = "checkbox",
		            name = FCOIS.localizationVars.fcois_loc["options_block_selling_exception_ornate"],
		            tooltip = FCOIS.localizationVars.fcois_loc["options_block_selling_exception_ornate_tooltip"],
		            getFunc = function() return FCOIS.settingsVars.settings.allowSellingForBlockedOrnate end,
		            setFunc = function(value) FCOIS.settingsVars.settings.allowSellingForBlockedOrnate = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockSelling and not FCOIS.settingsVars.settings.blockSellingGuildStore end,
		        },
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_sell_at_guild_store"],
				},
		        {
		            type = "checkbox",
		            name = FCOIS.localizationVars.fcois_loc["options_enable_block_selling_guild_store"],
		            tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_selling_guild_store_tooltip"],
		            getFunc = function() return FCOIS.settingsVars.settings.blockSellingGuildStore end,
		            setFunc = function(value) FCOIS.settingsVars.settings.blockSellingGuildStore = value
                           	FCOIS.settingsVars.settings.autoReenable_blockSellingGuildStore = value
		            end,
		        },
		        {
		            type = "checkbox",
		            name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_selling_guild_store"],
		            tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_selling_guild_store_tooltip"],
		            getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockSellingGuildStore end,
		            setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockSellingGuildStore = value
		            end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockSellingGuildStore end,
		        },
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_block_selling_exception_guild_store"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_block_selling_exception_guild_store_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.allowSellingInGuildStoreForBlocked end,
					setFunc = function(value) FCOIS.settingsVars.settings.allowSellingInGuildStoreForBlocked = value
			           end,
					disabled = function() return not FCOIS.settingsVars.settings.blockSellingGuildStore end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_fence"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_fence_selling"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_fence_selling_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockFence end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockFence = value
                           	FCOIS.settingsVars.settings.autoReenable_blockFenceSelling = value
			           end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_fence_selling"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_fence_selling_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockFenceSelling end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockFenceSelling = value
			           end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockFence end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_launder"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_launder_selling"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_launder_selling_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockLaunder end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockLaunder = value
                           	FCOIS.settingsVars.settings.autoReenable_blockLaunderSelling = value
			           end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_launder_selling"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_launder_selling_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockLaunderSelling end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockLaunderSelling = value
			           end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockLaunder end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_trade"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_trading"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_trading_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockTrading end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockTrading = value
                           	FCOIS.settingsVars.settings.autoReenable_blockTrading = value
			           end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_trading"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_trading_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockTrading end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockTrading = value
			           end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockTrading end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_mail"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_sending_mail"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_sending_mail_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockSendingByMail end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockSendingByMail = value
                           	FCOIS.settingsVars.settings.autoReenable_blockSendingByMail = value
			           end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_sending_mail"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_sending_mail_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockSendingByMail end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockSendingByMail = value
			           end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockSendingByMail end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_alchemy_destroy"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_alchemy_destroy"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_alchemy_destroy_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockAlchemyDestroy end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockAlchemyDestroy = value
                           	FCOIS.settingsVars.settings.autoReenable_blockAlchemyDestroy = value
			           end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_alchemy_destroy"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_auto_reenable_block_alchemy_destroy_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.autoReenable_blockAlchemyDestroy end,
					setFunc = function(value) FCOIS.settingsVars.settings.autoReenable_blockAlchemyDestroy = value
			           end,
		            disabled = function() return not FCOIS.settingsVars.settings.blockAlchemyDestroy end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_containers"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_autoloot_container"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_autoloot_container_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockAutoLootContainer end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockAutoLootContainer = value
			           end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_recipes"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_marked_recipes"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_marked_recipes_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockMarkedRecipes end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockMarkedRecipes = value
			           end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_motifs"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_motifs"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_motifs_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockMarkedMotifs end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockMarkedMotifs = value
			           end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_food"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_potions"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_potions_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockMarkedPotions end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockMarkedPotions = value
			           end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_food"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_food_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockMarkedFood end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockMarkedFood = value
			           end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_guild_bank"],
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_enable_block_guild_bank_without_withdraw"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_enable_block_guild_bank_without_withdraw_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.blockGuildBankWithoutWithdraw end,
					setFunc = function(value) FCOIS.settingsVars.settings.blockGuildBankWithoutWithdraw = value
			           end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_anti_output_options"],
				},
			    {
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["show_anti_messages_in_chat"],
					tooltip = FCOIS.localizationVars.fcois_loc["show_anti_messages_in_chat_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showAntiMessageInChat end,
					setFunc = function(value) FCOIS.settingsVars.settings.showAntiMessageInChat = value
			           end,
				},
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["show_anti_messages_as_alert"],
					tooltip = FCOIS.localizationVars.fcois_loc["show_anti_messages_as_alert_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showAntiMessageAsAlert end,
					setFunc = function(value) FCOIS.settingsVars.settings.showAntiMessageAsAlert = value
			           end,
				},
            } -- controls anti destroy
		}, -- submenu anti destroy

--==============================================================================
-- ANTI EQUIP
		{
        	type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_anti_equip"],
            controls =
            {
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_askBeforeEquipBoundItems"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_askBeforeEquipBoundItems_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.askBeforeEquipBoundItems end,
					setFunc = function(value) FCOIS.settingsVars.settings.askBeforeEquipBoundItems = value
		            end,
		            default = FCOIS.settingsVars.settings.askBeforeEquipBoundItems
				},
            }, -- controls anti equip
        }, -- submenu anti equip

--==============================================================================
-- ADDITIONAL BUTTONS
		{
        	type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_additional_buttons"],
            controls =
            {
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_additional_buttons_FCOIS_settings"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_additional_buttons_FCOIS_settings_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showFCOISMenuBarButton end,
					setFunc = function(value) FCOIS.settingsVars.settings.showFCOISMenuBarButton = value
			            --AddAdditionalButtons("FCOSettings")
		            end,
		            default = FCOIS.settingsVars.settings.showFCOISMenuBarButton
				},
		        {
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_additional_buttons_FCOIS_additional_options"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_additional_buttons_FCOIS_additional_options_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showFCOISAdditionalInventoriesButton end,
					setFunc = function(value) FCOIS.settingsVars.settings.showFCOISAdditionalInventoriesButton = value
                        if value == false then
					   		--Change the button color of the context menu invoker
							changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)
                        end
			            AddAdditionalButtons("FCOInventoriesContextMenuButtons")
		            end,
		            default = FCOIS.settingsVars.settings.showFCOISAdditionalInventoriesButton
				},
		        {
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_additional_buttons_FCOIS_additional_options_colorize"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_additional_buttons_FCOIS_additional_options_colorize_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.colorizeFCOISAdditionalInventoriesButton end,
					setFunc = function(value) FCOIS.settingsVars.settings.colorizeFCOISAdditionalInventoriesButton = value
				   		--Change the button color of the context menu invoker
						changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)
		            end,
                    disabled = function() return not FCOIS.settingsVars.settings.showFCOISAdditionalInventoriesButton end,
		            default = FCOIS.settingsVars.settings.colorizeFCOISAdditionalInventoriesButton
				},
            } -- controls additional buttons
        }, -- submenu  additional buttons
--==============================================================================
-- CHARACTER
		{
			type = "submenu",
			name = FCOIS.localizationVars.fcois_loc["options_header_character"],
            controls =
            {
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_tooltipatchar"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_tooltipatchar_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showIconTooltipAtCharacter end,
					setFunc = function(value) FCOIS.settingsVars.settings.showIconTooltipAtCharacter = value
		            end,
				},
				{
					type = "header",
					name = FCOIS.localizationVars.fcois_loc["options_header_character_armor_type"],
                },
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_show_armor_type_icon"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_show_armor_type_icon_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter end,
					setFunc = function(value) FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter = value
		            end,
				},
				{
					type = "slider",
					name = FCOIS.localizationVars.fcois_loc["options_armor_type_icon_character_pos_x"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_armor_type_icon_character_pos_x_tooltip"],
					min = -15,
					max = 40,
					getFunc = function() return FCOIS.settingsVars.settings.armorTypeIconAtCharacterX end,
					setFunc = function(offset)
							FCOIS.settingsVars.settings.armorTypeIconAtCharacterX = offset
							countAndUpdateEquippedArmorTypes()
						end,
					default = FCOIS.settingsVars.settings.armorTypeIconAtCharacterX,
		            disabled = function() return not FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter end,
		            width="half",
				},
				{
					type = "slider",
					name = FCOIS.localizationVars.fcois_loc["options_armor_type_icon_character_pos_y"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_armor_type_icon_character_pos_y_tooltip"],
					min = -15,
					max = 40,
					getFunc = function() return FCOIS.settingsVars.settings.armorTypeIconAtCharacterY end,
					setFunc = function(offset)
							FCOIS.settingsVars.settings.armorTypeIconAtCharacterY = offset
							countAndUpdateEquippedArmorTypes()
						end,
					default = FCOIS.settingsVars.settings.armorTypeIconAtCharacterY,
		            disabled = function() return not FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter end,
		            width="half",
				},
		        {
		            type = "colorpicker",
		            name = FCOIS.localizationVars.fcois_loc["options_armor_type_icon_character_light_color"],
		            tooltip = FCOIS.localizationVars.fcois_loc["options_armor_type_icon_character_light_color_tooltip"],
		            getFunc = function() return FCOIS.settingsVars.settings.armorTypeIconAtCharacterLightColor.r, FCOIS.settingsVars.settings.armorTypeIconAtCharacterLightColor.g, FCOIS.settingsVars.settings.armorTypeIconAtCharacterLightColor.b, FCOIS.settingsVars.settings.armorTypeIconAtCharacterLightColor.a end,
		            setFunc = function(r,g,b,a)
		                FCOIS.settingsVars.settings.armorTypeIconAtCharacterLightColor = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
						countAndUpdateEquippedArmorTypes()
		            end,
		            width = "full",
		            disabled = function() return not FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter end,
		            default = FCOIS.settingsVars.settings.armorTypeIconAtCharacterLightColor,
		        },
		        {
		            type = "colorpicker",
		            name = FCOIS.localizationVars.fcois_loc["options_armor_type_icon_character_medium_color"],
		            tooltip = FCOIS.localizationVars.fcois_loc["options_armor_type_icon_character_medium_color_tooltip"],
		            getFunc = function() return FCOIS.settingsVars.settings.armorTypeIconAtCharacterMediumColor.r, FCOIS.settingsVars.settings.armorTypeIconAtCharacterMediumColor.g, FCOIS.settingsVars.settings.armorTypeIconAtCharacterMediumColor.b, FCOIS.settingsVars.settings.armorTypeIconAtCharacterMediumColor.a end,
		            setFunc = function(r,g,b,a)
		                FCOIS.settingsVars.settings.armorTypeIconAtCharacterMediumColor = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
						countAndUpdateEquippedArmorTypes()
		            end,
		            width = "full",
		            disabled = function() return not FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter end,
		            default = FCOIS.settingsVars.settings.armorTypeIconAtCharacterMediumColor,
		        },
		        {
		            type = "colorpicker",
		            name = FCOIS.localizationVars.fcois_loc["options_armor_type_icon_character_heavy_color"],
		            tooltip = FCOIS.localizationVars.fcois_loc["options_armor_type_icon_character_heavy_color_tooltip"],
		            getFunc = function() return FCOIS.settingsVars.settings.armorTypeIconAtCharacterHeavyColor.r, FCOIS.settingsVars.settings.armorTypeIconAtCharacterHeavyColor.g, FCOIS.settingsVars.settings.armorTypeIconAtCharacterHeavyColor.b, FCOIS.settingsVars.settings.armorTypeIconAtCharacterHeavyColor.a end,
		            setFunc = function(r,g,b,a)
		                FCOIS.settingsVars.settings.armorTypeIconAtCharacterHeavyColor = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
						countAndUpdateEquippedArmorTypes()
		            end,
		            width = "full",
		            disabled = function() return not FCOIS.settingsVars.settings.showArmorTypeIconAtCharacter end,
		            default = FCOIS.settingsVars.settings.armorTypeIconAtCharacterHeavyColor,
		        },
				{
					type = "checkbox",
					name = FCOIS.localizationVars.fcois_loc["options_show_armor_type_header_text"],
					tooltip = FCOIS.localizationVars.fcois_loc["options_show_armor_type_header_text_tooltip"],
					getFunc = function() return FCOIS.settingsVars.settings.showArmorTypeHeaderTextAtCharacter end,
					setFunc = function(value) FCOIS.settingsVars.settings.showArmorTypeHeaderTextAtCharacter = value
		            end,
				},
            } -- controls character
		}, -- submenu chracter
--=============================================================================================

	} -- END OF OPTIONS TABLE

	LAM:RegisterOptionControls(FCOIS.addonVars.gAddonName, optionsTable)
end

--==============================================================================
--============================== END SETTINGS ==================================
--==============================================================================

--Check if the 4 filter buttons exist at the currently shown panel and create them if they are missing
--Update the color and texture of the buttons too
local function CheckFilterButtonsAtCurrentPanel(doUpdateLists, comingFrom)
	debugMessage("[CheckFilterButtonsAtCurrentPanel] Start - Coming from panel ID: " ..tostring(comingFrom), false)

	--Should we update the marker textures, size and color?
    checkMarker(-1)

	--Get the currently shown panel and update FCOIS.gFilterWhere
	local buttonsParentCtrl = checkActivePanel(comingFrom)

	--Is an inventory found?
    if buttonsParentCtrl ~= nil then
		debugMessage("[CheckFilterButtonsAtCurrentPanel] " .. buttonsParentCtrl:GetName() .. ", FilterPanelId: " .. tostring(FCOIS.gFilterWhere) .. ", UseFilters: " .. tostring(FCOIS.settingsVars.settings.atPanelEnabled[FCOIS.gFilterWhere]["filters"]), true)

		--Is the setting enabled to use the filters at this panel?
	    if FCOIS.settingsVars.settings.atPanelEnabled[FCOIS.gFilterWhere]["filters"] == true then

			local filterBtn
			local isFilterActivated

			--Change the filter buttons & callback functions
			for buttonNr=1, FCOIS.numVars.gFCONumFilters, 1 do
				filterBtn = AddOrChangeFilterButton(buttonsParentCtrl, buttonNr, FCOIS.filterButtonVars.gFilterButtonWidth, FCOIS.filterButtonVars.gFilterButtonHeight, FCOIS.settingsVars.settings.filterButtonLeft[buttonNr], FCOIS.settingsVars.settings.filterButtonTop[buttonNr], false, FCOIS.gFilterWhere)
	            if (filterBtn ~= nil) then
					--Get the filter's state
  					isFilterActivated = getSettingsIsFilterOn(buttonNr, FCOIS.gFilterWhere)
				    --Update the button's color
				    UpdateButtonColorsAndTextures(buttonNr, filterBtn, isFilterActivated, FCOIS.gFilterWhere)
					--(Re)register the filter (for the given panel)?
                    if isFilterActivated == false then
                        unregisterFilters(buttonNr, false, FCOIS.gFilterWhere)
                    else
	                    registerFilters(buttonNr, false, FCOIS.gFilterWhere)
                    end
	            end
			end

		else -- if FCOIS.settingsVars.settings.atPanelEnabled...

			--Change the filter buttons & callback functions
			for buttonNr=1, FCOIS.numVars.gFCONumFilters, 1 do
				--Hide the filter buttons
				AddOrChangeFilterButton(buttonsParentCtrl, buttonNr, FCOIS.filterButtonVars.gFilterButtonWidth, FCOIS.filterButtonVars.gFilterButtonHeight, FCOIS.settingsVars.settings.filterButtonLeft[buttonNr], FCOIS.settingsVars.settings.filterButtonTop[buttonNr], true, FCOIS.gFilterWhere)
			end
        	--Unregister the filter as the FCOIS.settingsVars.settings don't want a button and filter here
            unregisterFilters(-1, false, FCOIS.gFilterWhere)

        end -- if FCOIS.settingsVars.settings.atPanelEnabled...

		--Should the curent panel's iventory list be updated?
		if (doUpdateLists == true and FCOIS.settingsVars.settings.atPanelEnabled[FCOIS.gFilterWhere]["filters"] == true) then
	    	FilterBasics(false)
	    end
	end
end

--Check for other addons and react on them
local function CheckIfOtherAddonsActive()
    if (FCOIS.otherAddons.inventoryGridViewActive == false) then
		--Check if Inventory Gridview is active
	    local gridViewControlName = WINDOW_MANAGER:GetControlByName(FCOIS.otherAddons.GRIDVIEWBUTTON, "")
	    if gridViewControlName ~=  nil then
	    	debugMessage("Addon Inventory Gridview is active", false)
	    	FCOIS.otherAddons.inventoryGridViewActive = true
	    end
    end

	--Check if Chat Merchant is active
	if ( FCOIS.otherAddons.chatMerchantActive == false) then
	    FCOIS.otherAddons.chatMerchantActive = false
	    local chatMerchantControlName = WINDOW_MANAGER:GetControlByName(FCOIS.otherAddons.CHATMERCHANTBUTTON, "")
	    if chatMerchantControlName ~=  nil then
	    	debugMessage("Addon ChatMerchant is active", false)
	    	FCOIS.otherAddons.chatMerchantActive = true
        end
    end
	--Was ChatMerchant addon's control found now?
	if ( FCOIS.otherAddons.chatMerchantActive == true) then
		if chatMerchantControlName ~=  nil then
			chatMerchantControlName:ClearAnchors()
	        if (FCOIS.otherAddons.inventoryGridViewActive == true) then
		        -- With Inventory Grid View activated
		        chatMerchantControlName:SetAnchor(TOP, ZO_PlayerInventory, BOTTOM, -18, 6)
	        else
		        -- Without Inventory Grid View activated
		        chatMerchantControlName:SetAnchor(TOP, ZO_PlayerInventory, BOTTOM, -10, 6)
	        end
	    end
    end
end

--==============================================================================
--==================== START EVENT CALLBACK FUNCTIONS===========================
--==============================================================================

--PreHook function for the buttons at the top menus of banks, crafting stations, mail panel, trading, etc.
local function FCOItemSaver_PreHookButtonHandler(comingFrom)
   	FCOIS.preventerVars.gActiveFilterPanel = true
	debugMessage("~~~~~~~~~~~~~~~~~~~~~~~~", true)
	debugMessage("[PreHookButtonHandler] Coming from panel ID: " ..tostring(comingFrom), true)

	--Hide the context menu at last active panel
    hideContextMenu(comingFrom)

	--Alchemy
    if comingFrom ~= INV_ALVHEMY then
		--Change the button color of the context menu invoker
		changeContextMenuInvokerButtonColorByPanelId(comingFrom)
	end

	--Check the filter buttons and create them if they are not there
	CheckFilterButtonsAtCurrentPanel(true, comingFrom)

	--Return false to call the normal callback handler of the button afterwards
    return false
end

--Event upon closing of a crafting station
local function FCOItemSaver_End_Crafting_Interact(...)
	debugMessage("[EVENT] End crafting", true)

	--Hide the context menu at last active panel
    hideContextMenu(FCOIS.gFilterWhere)

	if FCOIS.preventerVars.gNoCloseEvent == false then
	    --Update the inventory filter buttons
	    updateFilterButtonsInInv(-1)
		--Update the 4 inventory button's color
		UpdateButtonColorsAndTextures(-1, nil, -1, LF_INVENTORY)
		--Change the button color of the context menu invoker
		changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)
    end
	FCOIS.preventerVars.gNoCloseEvent      = false
    FCOIS.preventerVars.gActiveFilterPanel = false

	--Check if the Anti-* functions need to be enabled again
    autoReenableAntiSettingsCheck("CRAFTING_STATION")
end

--Event upon opening of a vendor store
local function FCOItemSaver_Open_Store(p_storeIndicator)
    FCOIS.preventerVars.gActiveFilterPanel = true

	p_storeIndicator = p_storeIndicator or "vendor"
	debugMessage("[EVENT] Open store: " .. p_storeIndicator, true)

	--Check the filter buttons and create them if they are not there. Update the inventory afterwards too
	if p_storeIndicator == "vendor" then
		--Change the button color of the context menu invoker
		changeContextMenuInvokerButtonColorByPanelId(LF_VENDOR_SELL)
		CheckFilterButtonsAtCurrentPanel(true, LF_VENDOR_SELL)
    end
end

--Event upon closing of a vendor store
local function FCOItemSaver_Close_Store()
	debugMessage("[EVENT] Close store", true)

	--Hide the context menu at last active panel
    hideContextMenu(FCOIS.gFilterWhere)

	if FCOIS.preventerVars.gNoCloseEvent == false then
	    --Update the inventory filter buttons
	    updateFilterButtonsInInv(-1)
		--Update the 4 inventory button's color
		UpdateButtonColorsAndTextures(-1, nil, -1, LF_INVENTORY)
		--Change the button color of the context menu invoker
		changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)
	end
	FCOIS.preventerVars.gNoCloseEvent 	 = false
    FCOIS.preventerVars.gActiveFilterPanel = false

	--Check, if the Anti-* checks need to be enabled again
	autoReenableAntiSettingsCheck("STORE")
end

--Event upon opening of a guild store
local function FCOItemSaver_Open_Trading_House()
    FCOIS.preventerVars.gActiveFilterPanel = true
	debugMessage("[EVENT] Open trading house", true)

	--Change the button color of the context menu invoker
	changeContextMenuInvokerButtonColorByPanelId(LF_GUILDSTORE_SELL)
	--Check the filter buttons and create them if they are not there. Update the inventory afterwards too
	CheckFilterButtonsAtCurrentPanel(true, LF_GUILDSTORE_SELL)
end

--Event upon closing of a guild store
local function FCOItemSaver_Close_Trading_House()
	debugMessage("[EVENT] Close trading house", true)

	--Hide the context menu at last active panel
    hideContextMenu(FCOIS.gFilterWhere)

	if FCOIS.preventerVars.gNoCloseEvent == false then
	    --Update the inventory filter buttons
	    updateFilterButtonsInInv(-1)
		--Update the 4 inventory button's color
		UpdateButtonColorsAndTextures(-1, nil, -1, LF_INVENTORY)
		--Change the button color of the context menu invoker
		changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)
    end
	FCOIS.preventerVars.gNoCloseEvent 	 = false
    FCOIS.preventerVars.gActiveFilterPanel = false

	--Check, if the Anti-* checks need to be enabled again
	autoReenableAntiSettingsCheck("GUILD_STORE")
end

--Event upon opening of a guild bank
local function FCOItemSaver_Open_Guild_Bank()
    FCOIS.preventerVars.gActiveFilterPanel = true
	debugMessage("[EVENT] Open guild bank", true)
	--Reset the last clicked guild bank button as it will always be the withdraw tab if you open the guild bank, and if the
    --deposit button was the last one clicked it won't change the filter buttons as it thinks it is still active
    FCOIS.lastVars.gLastGuildBankButton = FCOIS.ZOControlVars.GUILD_BANK_MENUBAR_BUTTON_WITHDRAW

	--Change the button color of the context menu invoker
	changeContextMenuInvokerButtonColorByPanelId(LF_GUILDBANK_WITHDRAW)
	--Check the filter buttons and create them if they are not there. Update the inventory afterwards too
    CheckFilterButtonsAtCurrentPanel(true, LF_GUILDBANK_WITHDRAW)
end

--Event upon closing of a guild bank
local function FCOItemSaver_Close_Guild_Bank()
	debugMessage("[EVENT] Close guild bank", true)

	--Hide the context menu at last active panel
    hideContextMenu(FCOIS.gFilterWhere)

	if FCOIS.preventerVars.gNoCloseEvent == false then
	    --Update the inventory filter buttons
	    updateFilterButtonsInInv(-1)
		--Update the 4 inventory button's color
		UpdateButtonColorsAndTextures(-1, nil, -1, LF_INVENTORY)
		--Change the button color of the context menu invoker
		changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)
    end
	FCOIS.preventerVars.gNoCloseEvent 	 = false
    FCOIS.preventerVars.gActiveFilterPanel = false

	--Check, if the Anti-* checks need to be enabled again
	autoReenableAntiSettingsCheck("DESTROY")
end

--Event upon opening of a player bank
local function FCOItemSaver_Open_Player_Bank()
    FCOIS.preventerVars.gActiveFilterPanel = true
	debugMessage("[EVENT] Open bank", true)
	--Reset the last clicked bank button as it will always be the withdraw tab if you open the bank, and if the
    --deposit button was the last one clicked it won't change the filter buttons as it thinks it is still active
	FCOIS.lastVars.gLastBankButton = FCOIS.ZOControlVars.BANK_MENUBAR_BUTTON_WITHDRAW

	--Change the button color of the context menu invoker
	changeContextMenuInvokerButtonColorByPanelId(LF_BANK_WITHDRAW)
	--Check the filter buttons and create them if they are not there. Update the inventory afterwards too
	CheckFilterButtonsAtCurrentPanel(true, LF_BANK_WITHDRAW)
end

--Event upon closing of a player bank
local function FCOItemSaver_Close_Player_Bank()
	debugMessage("[EVENT] Close bank", true)

	--Hide the context menu at last active panel
    hideContextMenu(FCOIS.gFilterWhere)

	if FCOIS.preventerVars.gNoCloseEvent == false then
	    --Update the inventory filter buttons
	    updateFilterButtonsInInv(-1)
		--Update the 4 inventory button's color
		UpdateButtonColorsAndTextures(-1, nil, -1, LF_INVENTORY)
		--Change the button color of the context menu invoker
		changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)
    end
	FCOIS.preventerVars.gNoCloseEvent 	 = false
    FCOIS.preventerVars.gActiveFilterPanel = false

	--Check, if the Anti-* checks need to be enabled again
	autoReenableAntiSettingsCheck("DESTROY")
end

--Event upon opening of the trade panel
local function FCOItemSaver_Open_Trade_Panel()
   	FCOIS.preventerVars.gActiveFilterPanel = true
	debugMessage("[EVENT] Start trading", true)

	--Change the button color of the context menu invoker
	changeContextMenuInvokerButtonColorByPanelId(LF_TRADE)
	--Check the filter buttons and create them if they are not there. Update the inventory afterwards too
	CheckFilterButtonsAtCurrentPanel(true, LF_TRADE)
end

--Event upon closing of the trade panel
local function FCOItemSaver_Close_Trade_Panel()
	debugMessage("[EVENT] End trading", true)

	--Hide the context menu at last active panel
    hideContextMenu(FCOIS.gFilterWhere)

	if FCOIS.preventerVars.gNoCloseEvent == false then
	    --Update the inventory filter buttons
	    updateFilterButtonsInInv(-1)
		--Update the 4 inventory button's color
		UpdateButtonColorsAndTextures(-1, nil, -1, LF_INVENTORY)
		--Change the button color of the context menu invoker
		changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)
    end
	FCOIS.preventerVars.gNoCloseEvent 	 = false
   	FCOIS.preventerVars.gActiveFilterPanel = false

	--Check, if the Anti-* checks need to be enabled again
	autoReenableAntiSettingsCheck("TRADE")
end

-- Fires each time after addons were loaded and player is ready to move (after each zone change too)
local function FCOItemSaver_Player_Activated(...)
	--Prevent this event to be fired again and again upon each zone change
	EVENT_MANAGER:UnregisterForEvent(FCOIS.addonVars.gAddonName, EVENT_PLAYER_ACTIVATED)

	--Disable this addon if we are in GamePad mode
    if not FCOItemSaver_CheckGamePadMode() then
	    debugMessage("[EVENT] Player activated", true)

		--Check if other Addons active
	    CheckIfOtherAddonsActive()

		--Add/update the filter buttons, but only if not done already in addon initialization
	    if FCOIS.addonVars.gAddonLoaded == false then
		    updateFilterButtonsInInv(-1)
	    end
	    FCOIS.addonVars.gAddonLoaded = false

		--Overwrite the localized texts for the equipment gears, if changed in the FCOIS.settingsVars.settings
		changeEquipmentGearMenuTexts(-1)

   		--Change the button color of the context menu invoker
		changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)

	    FCOIS.addonVars.gPlayerActivated = true
	end
end

--event handler for ACTION LAYER POPPED
local function FCOItemsaver_OnActionLayerPopped(layerIndex, activeLayerIndex)
	debugMessage("[Action layer popped] LayerIndex: " .. tostring(layerIndex) .. ", ActiveLayerIndex: " .. tostring(activeLayerIndex), false)

    --LayerIndex 2 will be opened in most cases
    if activeLayerIndex == 2 then
	    --Hide the context menu at last active panel
	    hideContextMenu(FCOIS.gFilterWhere)
    end
end

--event handler for GAME_CAMERA_UI_MODE_CHANGED
local function FCOItemsaver_OnGameCameraUIModeChanged()
    if not IsGameCameraUIModeActive() then
        --Hide the contxt menu if still open
        hideContextMenu(FCOIS.gFilterWhere)
    end
end

--event handler for EVENT_CRAFT_STARTED
local function FCOItemSaver_Craft_Started(eventCode, craftSkill)
	local allowedCraftSkills = {
		[CRAFTING_TYPE_ALCHEMY] 		= false,
		[CRAFTING_TYPE_BLACKSMITHING] 	= true,
		[CRAFTING_TYPE_CLOTHIER] 		= true,
		[CRAFTING_TYPE_ENCHANTING] 		= false,
		[CRAFTING_TYPE_INVALID] 		= false,
		[CRAFTING_TYPE_PROVISIONING] 	= false,
		[CRAFTING_TYPE_WOODWORKING] 	= true,
    }

	--Mark new crafted item with the lock icon?
    --Are we creating an item, is the setting for automark enabled and is the current crafting station allowed?
    FCOIS.preventerVars.newItemCrafted = false
    local creatingItem = not ZO_SmithingTopLevelCreationPanel:IsHidden()
	local allowedCraftingSkill = allowedCraftSkills[craftSkill] or false
    if creatingItem and allowedCraftingSkill and FCOIS.settingsVars.settings.autoMarkCraftedItems then
    	--Set the variable to know if an item is getting into our bag after crafting complete
        FCOIS.preventerVars.newItemCrafted = true
	end

	--Only at smithing improvement
	if ZO_SmithingTopLevelImprovementPanelInventory:IsHidden() then return false end
	if FCOIS.ZOControlVars.IMPROVEMENT_SLOT == nil then return false end
    FCOIS.improvementVars.improvementBagId		= FCOIS.ZOControlVars.IMPROVEMENT_SLOT.bagId
    FCOIS.improvementVars.improvementSlotIndex	= FCOIS.ZOControlVars.IMPROVEMENT_SLOT.slotIndex

	--Check if the item is marked with several icons
	FCOIS.improvementVars.improvementMarkedIcons = {}
    local improvementMarkedIcons = {}
	local _, improvementMarkedIcons = FCOIsMarked(GetItemInstanceId(FCOIS.improvementVars.improvementBagId, FCOIS.improvementVars.improvementSlotIndex), -1)
    FCOIS.improvementVars.improvementMarkedIcons = improvementMarkedIcons
end

--event handler for EVENT_CRAFT_COMPLETED
local function FCOItemSaver_Craft_Completed(eventCode, craftSkill)
   	--Reset the variable to know if an item is getting into our bag after crafting complete
      FCOIS.preventerVars.newItemCrafted = false

	--Only at smithing improvement
	if ZO_SmithingTopLevelImprovementPanelInventory:IsHidden() then return false end
	if FCOIS.improvementVars.improvementBagId == nil or FCOIS.improvementVars.improvementSlotIndex == nil then return false end
	--For each marked icon of the currently improved item:
    --Set the icons/markers of the previous item again
	if FCOIS.improvementVars.improvementMarkedIcons and #FCOIS.improvementVars.improvementMarkedIcons > 0 then
    	for iconId, iconIsMarked in pairs(FCOIS.improvementVars.improvementMarkedIcons) do
			if iconIsMarked then
				--Inventory update will be automatcally done after each improvement of an item
				FCOMarkItem(FCOIS.improvementVars.improvementBagId, FCOIS.improvementVars.improvementSlotIndex, iconId, true, false, nil)
			end
        end
    end
    FCOIS.improvementVars.improvementBagId		= nil
    FCOIS.improvementVars.improvementSlotIndex	= nil
end

--==============================================================================
--===================== END EVENT CALLBACK FUNCTIONS============================
--==============================================================================

--==============================================================================
--			Anti - Destruction & Destroy & Send by Mail & Trade Methods
--==============================================================================

--Modify the destroy dialog
local function overrideDialogYesButton(dialog)
	--Get the "yes" button control of the destroy popup window
FCOdialog = dialog
	local button1 = dialog:GetNamedChild("Button1")
    if button1 == nil then return false end

	if dialog.info == FCOIS.ZOControlVars.DestroyItemDialog then
		--Use standard behaviour of "YES" button in destroy dialog
		button1:SetText(GetString(SI_YES))
		button1:SetClickSound(SOUNDS.DIALOG_ACCEPT)
		button1.m_callback = FCOIS.ZOControlVars.DestroyItemDialog.buttons[1].callback
		button1:SetEnabled(true)
		button1:SetMouseEnabled(true)
		button1:SetHidden(false)
		button1:SetKeybindEnabled(true)

        if not FCOIS.preventerVars.gAllowDestroyItem then
            --Set global variable so the next time the menu is opened the "YES" button of the dialog ZO_Dialog1 gets reset
        	FCOIS.preventerVars.wasDestroyDone = true

        	--Use own behaviour of "YES" button in destroy dialog:
            --Button's text will change to "NO"
            --Button will be set invisible and callback method changes so that
            --using a keybind will press ESC key instead
            button1:SetText(GetString(SI_NO))
            button1:SetClickSound(SOUNDS.DIALOG_DECLINE)
            button1.m_callback = FCOIS.ZOControlVars.DestroyItemDialog.noChoiceCallback
           	button1:SetMouseEnabled(false)
			button1:SetHidden(true)
			button1:SetKeybindEnabled(false)
        end
    else
	    --All other dialogs
        ZO_Dialog1:SetKeyboardEnabled(false)
		button1:SetEnabled(true)
		button1:SetMouseEnabled(true)
		button1:SetHidden(false)
		button1:SetKeybindEnabled(true)
    end
end

-- fired when user selects an item to destroy
-- warns user if the item is marked with any of the filter icons
local function DestroySelectionHandler(bag, slot, echo, parentControl)
    debugMessage("[DestroySelectionHandler] Bag: " .. tostring(bag) .. ", Slot: " .. tostring(slot), true)
	--Are we coming from the character window?
	if bag == BAG_WORN and parentControl ~= nil then
    	FCOIS.preventerVars.gCheckEquipmentSlots = true
    end

	-- if not active then abort checking
	if( not FCOIS.settingsVars.settings.blockDestroying ) then return nil end

	-- get instance id of the item, this value is persistant across all game
    local id = MyGetItemInstanceIdNoControl(bag, slot)

	-- if item is in any protection list, warn user
	for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
		if( checkIfItemIsProtected(i, id) ) then
            if (echo == true) then
				if FCOIS.settingsVars.settings.showAntiMessageInChat == true or FCOIS.settingsVars.settings.showAntiMessageAsAlert == true then
	            	local antiDestroyMsg = FCOIS.localizationVars.fcois_loc["destroying_not_allowed"] .. " [" .. zo_strformat(SI_TOOLTIP_ITEM_NAME, GetItemName(bag, slot)) .. "]"
					if FCOIS.settingsVars.settings.showAntiMessageInChat == true then
            			d(antiDestroyMsg)
    				end
					if FCOIS.settingsVars.settings.showAntiMessageAsAlert == true then
                		showAlert(antiDestroyMsg)
					end
				end
            end
			return true
		end
	end
	return false
end

-- fired when user selects an item by drag&drop, doubleclick etc.
-- warns user if the item is marked with any of the filter icons
local function ItemSelectionHandler(bag, slot, echo)
    debugMessage("[ItemSelectionHandler] Bag: " .. tostring(bag) .. ", Slot: " .. tostring(slot) .. ", Echo: " .. tostring(echo), true)
	local whereAreWe = 1

	--Inside mail panel?
  	if (not FCOIS.ZOControlVars.MAIL_SEND:IsHidden() or FCOIS.gFilterWhere == LF_MAIL_SEND) then
		-- if not active then abort checking
        whereAreWe = 2
		if( not FCOIS.settingsVars.settings.blockSendingByMail) then return false end

	--Inside trading player 2 player panel?
    elseif (not FCOIS.ZOControlVars.PLAYER_TRADE:IsHidden() or FCOIS.gFilterWhere == LF_TRADE) then
	    whereAreWe = 3
		-- if not active then abort checking
		if( not FCOIS.settingsVars.settings.blockTrading ) then return false end

	--Inside vendor sell?
    elseif (not FCOIS.ZOControlVars.VENDOR_SELL:IsHidden() or FCOIS.gFilterWhere == LF_VENDOR_SELL) then
	    whereAreWe = 4
		-- if not active then abort checking
		if( not FCOIS.settingsVars.settings.blockSelling ) then return false end

	--Inside fence sell?
    elseif (FCOIS.gFilterWhere == LF_FENCE_SELL) then
	    whereAreWe = 10
		-- if not active then abort checking
		if( not FCOIS.settingsVars.settings.blockFence ) then return false end

	--Inside launder sell?
    elseif (FCOIS.gFilterWhere == LF_FENCE_LAUNDER) then
	    whereAreWe = 11
		-- if not active then abort checking
		if( not FCOIS.settingsVars.settings.blockLaunder ) then return false end

	--Inside crafting station improvement
    elseif (not FCOIS.ZOControlVars.IMPROVEMENT:IsHidden() or FCOIS.gFilterWhere == LF_SMITHING_IMPROVEMENT) then
	    whereAreWe = 5
		-- if not active then abort checking
		if( not FCOIS.settingsVars.settings.blockImprovement ) then return false end

	--Inside crafting station deconstruction
    elseif (not FCOIS.ZOControlVars.DECONSTRUCTION:IsHidden() or FCOIS.gFilterWhere == LF_SMITHING_DECONSTRUCT) then
	    whereAreWe = 6
		-- if not active then abort checking
		if( not FCOIS.settingsVars.settings.blockDeconstruction ) then return false end

	--Inside enchanting station
    elseif (not FCOIS.ZOControlVars.ENCHANTING_STATION:IsHidden()) then
		--Enchanting Extraction panel?
    	if FCOIS.gFilterWhere == LF_ENCHANTING_EXTRACTION or ENCHANTING.enchantingMode == 2 then
		    whereAreWe = 7
			-- if not active then abort checking
			if( not FCOIS.settingsVars.settings.blockEnchantingExtraction ) then return false end
		--Enchanting Creation panel?
    	elseif FCOIS.gFilterWhere == LF_ENCHANTING_CREATION or ENCHANTING.enchantingMode == 1 then
		    whereAreWe = 8
			-- if not active then abort checking
			if( not FCOIS.settingsVars.settings.blockEnchantingCreation ) then return false end
		end

	--Inside guild store selling?
    elseif (not FCOIS.ZOControlVars.GUILD_STORE:IsHidden() or FCOIS.gFilterWhere == LF_GUILDSTORE_SELL) then
	    whereAreWe = 9
		-- if not active then abort checking
		if( not FCOIS.settingsVars.settings.blockSellingGuildStore ) then return false end

	--Are we at the alchemy station?
    elseif (not FCOIS.ZOControlVars.ALCHEMY_STATION:IsHidden() or FCOIS.gFilterWhere == LF_ALCHEMY_CREATION) then
        whereAreWe = 12
 		-- if not active then abort checking
		if( not FCOIS.settingsVars.settings.blockAlchemyDestroy ) then return false end

	--Are we at a bank/inventory and trying to withdraw/deposit some items by double clicking it?
    elseif (not FCOIS.ZOControlVars.BANK:IsHidden() or FCOIS.gFilterWhere == LF_BANK_WITHDRAW) then
       	return false

	--Are we at a guild bank/inventory and trying to withdraw/deposit some items by double clicking it?
    elseif (not FCOIS.ZOControlVars.GUILD_BANK:IsHidden() or FCOIS.gFilterWhere == LF_GUILDBANK_WITHDRAW) then
       	return false

	--Are we at the inventory and trying to use/equip an item?
    elseif (not FCOIS.ZOControlVars.BACKPACK:IsHidden() or FCOIS.gFilterWhere == LF_INVENTORY or FCOIS.gFilterWhere == LF_BANK_DEPOSIT or FCOIS.gFilterWhere == LF_GUILDBANK_DEPOSIT) then

		--Check if player or guild bank is active by checking current scene in scene manager
        local currentSceneName = SCENE_MANAGER.currentScene.name
        if currentSceneName ~= nil and (currentSceneName == FCOIS.ZOControlVars.bankSceneName or currentSceneName == FCOIS.ZOControlVars.guildBankSceneName) then
		--If bank/guild bank deposit tab is active
        	if FCOIS.ZOControlVars.BANK:IsHidden() and FCOIS.ZOControlVars.GUILD_BANK:IsHidden() then
				--If the item is double clicked + marked deposit it, instead of blocking the deposition
        		return false
            end
		end

		--Are we trying to open a container with autoloot on?
		if (isAutolootContainer(bag, slot)) then
			whereAreWe = 13
 			-- if not active then abort checking
    		if( not FCOIS.settingsVars.settings.blockAutoLootContainer ) then return false end
		elseif (isItemType(bag, slot, ITEMTYPE_RECIPE)) then
			whereAreWe = 14
 			-- if not active then abort checking
    		if( not FCOIS.settingsVars.settings.blockMarkedRecipes ) then return false end
		elseif (isItemType(bag, slot, ITEMTYPE_RACIAL_STYLE_MOTIF)) then
			whereAreWe = 15
 			-- if not active then abort checking
    		if( not FCOIS.settingsVars.settings.blockMarkedMotifs ) then return false end
		elseif (isItemType(bag, slot, ITEMTYPE_POTION)) then
			whereAreWe = 16
 			-- if not active then abort checking
    		if( not FCOIS.settingsVars.settings.blockMarkedPotions ) then return false end
		elseif (isItemType(bag, slot, ITEMTYPE_FOOD)) then
			whereAreWe = 17
 			-- if not active then abort checking
    		if( not FCOIS.settingsVars.settings.blockMarkedFood ) then return false end
        else
	       	return false
        end

	else
		whereAreWe = 1
		-- if not active then abort checking
		if( not FCOIS.settingsVars.settings.blockDestroying ) then return false end
	end

	-- get instance id of the item, this value is persistant across all game
    local id = MyGetItemInstanceIdNoControl(bag, slot)
    local retVal = true

	-- if item is in any protection list, warn user
	for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
		if( checkIfItemIsProtected(i, id) ) then
			--Reset the return variable for each icon again!
			retVal = true
			--If current checked panel = sell or guild store sell or fence sell
			if (whereAreWe == 4 or whereAreWe == 9 or whereAreWe == 10) then
--d("!: " .. whereAreWe .. ", i: " .. i)
				--If current checked panel = sell and the item is marked for selling
				if (i==5) then

					--If the settings to allow selling of marked items is enabled -> Abort here
					if FCOIS.settingsVars.settings.allowSellingForBlocked == true then
                    	retVal = false

	                --if filter Id equals "Ornate" and the item is marked for selling,
					--and the settings to allow selling of marked ornate items is enabled -> Abort here
					elseif FCOIS.settingsVars.settings.allowSellingForBlockedOrnate == true then
                    	retVal = false
					end

				--If current checked panel = guild store sell and the filterId equals = sell in guild store and the item is marked for guild store selling,
				--and the settings to allow selling of marked guild store items is enabled -> Abort here
				elseif (i==11 and whereAreWe==9 and FCOIS.settingsVars.settings.allowSellingInGuildStoreForBlocked == true) then
	               	retVal = false

				--If current checked panel = sell or guild store sell or fence sell and filter Id equals "Intricate" and the item is marked as intricate,
				--and the settings to allow selling of marked ornate items is enabled -> Abort here
				elseif (i==12 and FCOIS.settingsVars.settings.allowSellingForBlockedIntricate == true) then
	               	retVal = false
                end

			--Improved items and settings allow them to be improved?
            elseif (whereAreWe == 5 and i==10 and FCOIS.settingsVars.settings.allowImproveImprovement == true) then
	           	retVal = false

			--Extraction of glyphs with deconstruction item
            elseif (whereAreWe == 7 and i==9 and FCOIS.settingsVars.settings.allowDeconstructDeconstruction == true) then
            	retVal = false
			end

            --Abort here as at least one marker icon was set
            if retVal == true then
	            --Show text in chat or alert message now?
	            if echo == true then
					if FCOIS.settingsVars.settings.showAntiMessageInChat == true or FCOIS.settingsVars.settings.showAntiMessageAsAlert == true then
						local itemMsg
                        local formattedItemName = zo_strformat(SI_TOOLTIP_ITEM_NAME, GetItemName(bag, slot))
		                if (whereAreWe == 2) then
			            	itemMsg = FCOIS.localizationVars.fcois_loc["sendbymail_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 3) then
			            	itemMsg = FCOIS.localizationVars.fcois_loc["trading_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 4) then
		                	itemMsg = FCOIS.localizationVars.fcois_loc["selling_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 5) then
			            	itemMsg = FCOIS.localizationVars.fcois_loc["improvement_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 6) then
			            	itemMsg = FCOIS.localizationVars.fcois_loc["deconstruction_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 7) then
			            	itemMsg = FCOIS.localizationVars.fcois_loc["enchanting_extraction_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 8) then
			            	itemMsg = FCOIS.localizationVars.fcois_loc["enchanting_creation_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 9) then
			            	itemMsg = FCOIS.localizationVars.fcois_loc["guild_store_sell_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 10) then
		                	itemMsg = FCOIS.localizationVars.fcois_loc["fence_selling_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 11) then
		                	itemMsg = FCOIS.localizationVars.fcois_loc["launder_selling_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 12) then
		                	itemMsg = FCOIS.localizationVars.fcois_loc["alchemy_destroy_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 13) then
		                	itemMsg = FCOIS.localizationVars.fcois_loc["container_autoloot_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 14) then
		                	itemMsg = FCOIS.localizationVars.fcois_loc["recipe_usage_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 15) then
		                	itemMsg = FCOIS.localizationVars.fcois_loc["motif_usage_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 16) then
		                	itemMsg = FCOIS.localizationVars.fcois_loc["potion_usage_not_allowed"] .. " [" .. formattedItemName .. "]"
		                elseif (whereAreWe == 17) then
		                	itemMsg = FCOIS.localizationVars.fcois_loc["food_usage_not_allowed"] .. " [" .. formattedItemName .. "]"
		                else
		                	itemMsg = FCOIS.localizationVars.fcois_loc["destroying_not_allowed"] .. " [" .. formattedItemName .. "]"
		                end
						if FCOIS.settingsVars.settings.showAntiMessageInChat == true then
							d(itemMsg)
						end
						if FCOIS.settingsVars.settings.showAntiMessageAsAlert == true then
		                    showAlert(itemMsg)
	                    end
	                end
	            end

				--Abort here as at least 1 icon is marked for the current item
				return true
    		end -- if retVal == true

		end -- item is marked?
	end -- for
	return false
end

-- fired when user selects an item to deconstruct
-- warns user if the item is marked with any of the filter icons
local function DeconstructionSelectionHandler(bag, slot, echo)
    debugMessage("[DeconstructionSelectionHandler] Bag: " .. tostring(bag) .. ", Slot: " .. tostring(slot).. ", Echo: " .. tostring(echo), true)

	--> GOT HERE FROM CONTEXT MENU ENTRY FOR "ADD ITEM TO CRAFT" e.g.
	-- Is user not at the deconstruction panel?
	if( not FCOIS.ZOControlVars.DECONSTRUCTION_BAG or FCOIS.ZOControlVars.DECONSTRUCTION_BAG:IsHidden() ) then
        if (not FCOIS.ZOControlVars.IMPROVEMENT:IsHidden() or FCOIS.gFilterWhere == LF_SMITHING_IMPROVEMENT)
        or (not FCOIS.ZOControlVars.ENCHANTING_STATION:IsHidden() or FCOIS.gFilterWhere == LF_ENCHANTING_CREATION or FCOIS.gFilterWhere == LF_ENCHANTING_EXTRACTION)
		or (not FCOIS.ZOControlVars.ALCHEMY_STATION:IsHidden() or FCOIS.gFilterWhere == LF_ALCHEMY_CREATION) then
        	return ItemSelectionHandler(bag, slot, echo)
        else
    		return false
        end
    end

	-- if not active then abort checking
	if( not FCOIS.settingsVars.settings.blockDeconstruction ) then return false end

	-- get instance id of the item, this value is persistant across all game
    local id = MyGetItemInstanceIdNoControl(bag, slot)
    if id ~= nil then
		-- if item is in any protection list, warn user
		for i=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
			if( checkIfItemIsProtected(i, id) ) then
				--Icon for deconstruction, and settings allow deconstruction?
            	if i == 9 and FCOIS.settingsVars.settings.allowDeconstructDeconstruction then
                	--do nothing here
				--Icon for intricate, and settings allows deconstruction?
                elseif i==12 and FCOIS.settingsVars.settings.allowDeconstructIntricate then
                	--do nothing here
                else
	            	if (echo == true) then
	                    if FCOIS.settingsVars.settings.showAntiMessageInChat == true or FCOIS.settingsVars.settings.showAntiMessageAsAlert == true then
		                	local antiDeconstructMsg = FCOIS.localizationVars.fcois_loc["deconstruction_not_allowed"] .. " [" .. zo_strformat(SI_TOOLTIP_ITEM_NAME, GetItemName(bag, slot)) .. "]"
							if FCOIS.settingsVars.settings.showAntiMessageInChat == true then
			               		d(antiDeconstructMsg)
	                        end
							if FCOIS.settingsVars.settings.showAntiMessageAsAlert == true then
			                	showAlert(antiDeconstructMsg)
	                        end
	                    end
		            end
					return true
				end
			end
		end
    end
	return false
end

--==============================================================================
--			Dialog functions
--==============================================================================

--The callback function for the "Accept" button at the FCO custom dialogs
local function FCOAcceptDialogChanges(dialog)
    --globDialog = dialog
	if dialog.data.bag ~= nil and dialog.data.slot ~= nil then
		--Set global var to allow equip of item
    	FCOIS.preventerVars.askBeforeEquipDialogRetVal = true
        --Equip the item now
--d("Equipping item now!")
		EquipItem(dialog.data.bag, dialog.data.slot)
		--reset the global var again
		FCOIS.preventerVars.askBeforeEquipDialogRetVal = false
    end
end

--Format the text for the dialog
local function GetFormattedDialogText(text, params)
    if(text)
    then
        if(params and #params > 0)
        then
            text = zo_strformat(text, unpack(params))
        elseif(type(text) == "number") then
            text = GetString(text)
        end
    else
        text = ""
    end
    return text
end

--function to initialize the ask before binding an item dialog
local function FCOAskBeforeBindDialogInitialize(control)
	local content   = GetControl(control, "Content")
	local acceptBtn = GetControl(control, "Accept")
	local cancelBtn = GetControl(control, "Cancel")
	local descLabel = GetControl(content, "Text")

	ZO_Dialogs_RegisterCustomDialog("FCO_ASK_BEFORE_BIND_DIALOG", {
		customControl = control,
	    title = { text = FCOIS.localizationVars.fcois_loc["options_header_anti_equip"]  },
		mainText = { text = FCOIS.localizationVars.fcois_loc["options_anti_equip_question"] },
		setup = function(dialog, data)
        	FCOIS.preventerVars.askBeforeEquipDialogRetVal = false
            --Format the dialog text: Show the item's name inside
            local itemLink = GetItemLink(data.bag, data.slot)
			local params = {itemLink}
			local formattedText = GetFormattedDialogText(FCOIS.localizationVars.fcois_loc["options_anti_equip_question"], params)
		    descLabel:SetText(formattedText)
        end,
        noChoiceCallback = function(dialog)
            --Simulate the button "cancel" click
        	FCOIS.preventerVars.askBeforeEquipDialogRetVal = false
        end,
        buttons =
        {
            {
                control = acceptBtn,
                text = SI_DIALOG_ACCEPT,
                keybind = "DIALOG_PRIMARY",
                callback = function(dialog) FCOAcceptDialogChanges(dialog) end,
            },
            {
                control = cancelBtn,
                text = SI_DIALOG_CANCEL,
                keybind = "DIALOG_NEGATIVE",
                callback = function(dialog)
                	FCOIS.preventerVars.askBeforeEquipDialogRetVal = false
				end,
            },
        },
	})
end

--Function to check if an item is bindable and show an dialog to ask the user if the item should be bound to the account, if enabled
local function FCOAskBeforeBindDialog(bagId, slotIndex, contextMenuActive)
	if FCOIS.settingsVars.settings.askBeforeEquipBoundItems then
		if contextMenuActive then
            local itemBindable = isItemBindable(bagId, slotIndex)
--d("[FCOAskBeforeBindDialog] ItemBindable: " .. tostring(itemBindable))
        	if itemBindable then
            	local dialog = ZO_Dialogs_ShowDialog("FCO_ASK_BEFORE_BIND_DIALOG", {bag=bagId,slot=slotIndex})
				return true --to prevent function EquipItem(bag, slot)
			else
				return false
		    end
	    else
	        return false
	    end
	else
		FCOIS.preventerVars.askBeforeEquipDialogRetVal = true
    	return false
    end
end

--Function to check if item is bindable and show a question dialog then
local function CheckBindableItems(bagId, slotIndex)
--d("[CheckBindableItems] settings: " .. tostring(FCOIS.settingsVars.settings.askBeforeEquipBoundItems) .. ", FCOIS.preventerVars: " .. tostring(FCOIS.preventerVars.askBeforeEquipDialogRetVal))
	if FCOIS.settingsVars.settings.askBeforeEquipBoundItems == true and FCOIS.preventerVars.askBeforeEquipDialogRetVal == false then
		local retVal = FCOAskBeforeBindDialog(bagId, slotIndex, true)
		--Returning true will skip the "real" EquipItem(bag, slot) function!
		return retVal
	else
		return false
	end
end

--==============================================================================
--			Right-click /context menu functions
--==============================================================================

--AddContextMenuEntry
--Pre Hook function for adding right click/context menu entries.
--Attention: Will be executed on mouse enter on a slot in inventories
--AND if you open the context menu by a mouse righ-click
--To distinguish these two "events" you can use: if self.m_contextMenuMode == true then
--true: Context-menu is open, false: Only mouse hoevered over the item
local function FCOItemSaver_AddSlotAction(self, actionStringId, ...)
	--Is the ZOs player lock item functionality enabled?
    if FCOIS.preventerVars.ZOsPlayerItemLockEnabled then
	    --Only execute if context menu is visible?
	    if self.m_contextMenuMode then
	    	--Hide the context menu entry ZOs added to lock/unlock items
	        if actionStringId == SI_ITEM_ACTION_MARK_AS_LOCKED or actionStringId == SI_ITEM_ACTION_UNMARK_AS_LOCKED then
                return true
		    end
        end
    end

	local parentControl
	--Chracter equipment, or normal slot?
    if (not FCOIS.ZOControlVars.CHARACTER:IsHidden()) then
		parentControl = self.m_inventorySlot
    else
		parentControl = self.m_inventorySlot:GetParent()
    end
	--No parent found? Abort
	if parentControl == nil then return false end

    --debugMessage("[FCOItemSaver_AddSlotAction] Parent: " .. parentControl:GetName(), true)
    local bag
    local slotIndex
    bag, slotIndex = MyGetItemDetails(parentControl)

	--Hide the button FCOIS.contextMenu if shown and if we right clicked another item
	if self.m_contextMenuMode == true then
		--Hide the context menu at last active panel
	    hideContextMenu(FCOIS.gFilterWhere)
    end

	--Use item?
    if        actionStringId == SI_ITEM_ACTION_USE then
		--Is the item protected with any icon?
		local marked, _ = FCOIsMarked(GetItemInstanceId(bag, slotIndex), -1)
		if marked and IsItemUsable(bag, slotIndex) then
			--If mail send or player trade panel is activated
	        local isCurrentlyShowingMailSend 	= not FCOIS.ZOControlVars.MAIL_SEND:IsHidden() and FCOIS.settingsVars.settings.blockSendingByMail
	        local isCurrentlyShowingPlayerTrade = not FCOIS.ZOControlVars.PLAYER_TRADE:IsHidden() and FCOIS.settingsVars.settings.blockTrading
	        local isContainerWithAutoLootEnabled= isAutolootContainer(bag, slotIndex) and FCOIS.settingsVars.settings.blockAutoLootContainer
	        local isARecipe		 				= isItemType(bag, slotIndex, ITEMTYPE_RECIPE) and FCOIS.settingsVars.settings.blockMarkedRecipes
	        local isAStyleMotif					= isItemType(bag, slotIndex, ITEMTYPE_RACIAL_STYLE_MOTIF) and FCOIS.settingsVars.settings.blockMarkedMotifs
	        local isAPotion					    = isItemType(bag, slotIndex, ITEMTYPE_POTION) and FCOIS.settingsVars.settings.blockMarkedPotions
	        local isAFood					    = isItemType(bag, slotIndex, ITEMTYPE_FOOD) and FCOIS.settingsVars.settings.blockMarkedFood
			--local isARepairKit				  = isItemType(bag, slot, ITEMTYPE_TOOL)

			--Is any of the settings to protect enabled
	        if    isCurrentlyShowingMailSend or isCurrentlyShowingPlayerTrade
               or isContainerWithAutoLootEnabled
               or isARecipe
               or isAStyleMotif
               or isAPotion
               or isAFood
            then
				--remove the context-menu entry for "use" (and the keybinding)
				return true
	        end
		end
	--Destroy item
    elseif    actionStringId == SI_ITEM_ACTION_DESTROY then

	    --Only execute if context menu is visible?
	    if self.m_contextMenuMode == false then
			return
        end

		--Abort if parent control cannot be found
        --Is item marked with any of the FCOItemSaver icons? Then don't show the actionStringId in the contextmenu
    	return DestroySelectionHandler(bag, slotIndex, false, parentControl)

	--Add item to crafting station, improvement, enchanting table
	elseif actionStringId == SI_ITEM_ACTION_ADD_TO_CRAFT or actionStringId == SI_ITEM_ACTION_RESEARCH then
    	--Is item marked with any of the FCOItemSaver icons? Then don't show the actionStringId in the contextmenu
    	return DeconstructionSelectionHandler(bag, slotIndex, false)

	--Trade an item, mark item as junk, attach item to mail, sell item, launder item, add to trading house listing (sell there) or add to crafting station
    elseif actionStringId == SI_ITEM_ACTION_TRADE_ADD or actionStringId == SI_ITEM_ACTION_MARK_AS_JUNK
       or actionStringId == SI_ITEM_ACTION_MAIL_ATTACH or actionStringId == SI_ITEM_ACTION_SELL
       or actionStringId == SI_ITEM_ACTION_LAUNDER
	   --Anbieten / sell in guild shop
       --or actionStringId == 3763 then
       or actionStringId == SI_TRADING_HOUSE_ADD_ITEM_TO_LISTING then
        --Is item marked with any of the FCOItemSaver icons? Then don't show the actionStringId in the contextmenu
        return ItemSelectionHandler(bag, slotIndex, false)

   --Anziehen
    elseif actionStringId == SI_ITEM_ACTION_EQUIP then
        local isCurrentlyShowingGuildStore = not FCOIS.ZOControlVars.GUILD_STORE:IsHidden()
        if isCurrentlyShowingGuildStore and FCOIS.settingsVars.settings.blockSellingGuildStore then
            --Is the item protected with any icon?
            local marked, _ = FCOIsMarked(GetItemInstanceId(bag, slotIndex), -1)
            if marked then
                --remove the context-menu entry for "equip" (and the keybinding)
                return true
            end
        end

    --Bank deposit
    elseif actionStringId == SI_ITEM_ACTION_BANK_DEPOSIT then
		--Are we at the guild bank and is the protection setting for "non-withdrawable items" enabled?
        if FCOIS.settingsVars.settings.blockGuildBankWithoutWithdraw and (SCENE_MANAGER:GetCurrentScene():GetName() == "guildBank" or SCENE_MANAGER:GetCurrentScene():GetName() == "gamepad_guild_bank") then
            if FCOIS.guildBankVars.guildBankId == 0 then return true end
        	return not checkIfGuildBankWithdrawAllowed(FCOIS.guildBankVars.guildBankId)
    	end

    --else
    	--d("actionStringId: " .. actionStringId .. " = " .. GetString(actionStringId))
    end
end


--==============================================================================
--			ON EVENT Implementation Methods
--==============================================================================

-- handler function for EVENT_INVENTORY_SLOT_UNLOCKED global event
-- will be fired (after EVENT_INVENTORY_SLOT_LOCKED) if you have pickuped an item (e.g. by drag&drop) and drop it again
local function FCOItemSaver_OnInventorySlotUnLocked(self, bag, slot)
	debugMessage("[Event] OnInventorySlotUnLocked: bag: " .. tostring(bag) .. ", slot: " .. tostring(slot), true)

	if bag == BAG_WORN and FCOIS.preventerVars.gItemSlotIsLocked == true then
	    --If item was unequipped: Remove the armor type marker if necessary
		zo_callLater(function()
			removeArmorTypeMarker(bag, slot)
		end, 100)

		--Update 2hd weapon backup slot icons etc.
		zo_callLater(function()
        	checkForEmptyOrTwoHandedWeaponMarker(slot, false)
	        FCOIS.preventerVars.gCheckEquipmentSlots = false
        end, 200)
    end
    FCOIS.preventerVars.gItemSlotIsLocked = false
end

-- handler function for EVENT_INVENTORY_SLOT_LOCKED global event
-- will be fired (before EVENT_CURSOR_PICKUP) if you pickup an item (e.g. by drag&drop)
local function FCOItemSaver_OnInventorySlotLocked(self, bag, slot)
	debugMessage("[Event] OnInventorySlotLocked: bag: " .. tostring(bag) .. ", slot: " .. tostring(slot), true)

    FCOIS.preventerVars.gItemSlotIsLocked = true

	if(not FCOIS.ZOControlVars.DECONSTRUCTION_BAG:IsHidden() ) then
		-- check if deconstruction is forbidden
		-- if so, clear item hold by cursor
		if( DeconstructionSelectionHandler(bag, slot, true) ) then
			--Remove the picked item from drag&drop cursor
	    	ClearCursor()
            return false
	    end
	else
        local doShowItemBindDialog = false
		-- check if destroying, improvement, sending or trading is forbidden
		-- and check if item is bindable (above)
		-- if so, clear item hold by cursor
		if( doShowItemBindDialog or ItemSelectionHandler(bag, slot, true) ) then
			--Remove the picked item from drag&drop cursor
			ClearCursor()
			return false
		else
			return false
		end
    end
end

-- handler function for character window item controls' OnMouseDoubleClick event
local function FCOItemSaver_CharacterItem_OnMouseDoubleClick(self, ...)
	--Hide the context menu at last active panel
    hideContextMenu(FCOIS.gFilterWhere)

    local bagId, slotId

	-- make sure control contains an item
	if( self.dataEntry and self.dataEntry.data ) then
		bagId = self.dataEntry.data.bagId
		slotId = self.dataEntry.data.slotIndex
    else
		if( self.slotIndex and self.bagId ) then
			bagId  = self.bagId
			slotId = self.slotIndex
		end
    end

	-- call the original handler function
	local func = GetEventHandler("OnMouseDoubleClick", self:GetName())
	if ( not func ) then return false end

	return func(self, ...)
end


-- handler function for inventory item controls' OnMouseDoubleClick event
local function FCOItemSaver_InventoryItem_OnMouseDoubleClick(self, ...)
--d("[FCOItemSaver_InventoryItem_OnMouseDoubleClick]")
	--Hide the context menu at last active panel
    hideContextMenu(FCOIS.gFilterWhere)

    local bagId, slotId
	-- make sure control contains an item
	if( self.dataEntry and self.dataEntry.data ) then
		bagId = self.dataEntry.data.bagId
		slotId = self.dataEntry.data.slotIndex
    else
		if( self.slotIndex and self.bagId ) then
			bagId  = self.bagId
			slotId = self.slotIndex
		end
    end

	if( bagId ~= nil and slotId ~= nil ) then

	    -- Inside deconstruction?
		if(not FCOIS.ZOControlVars.DECONSTRUCTION_BAG:IsHidden() ) then
			-- check if deconstruction is forbidden
			-- if so, return false to prevent selection of the item
			if( DeconstructionSelectionHandler(bagId, slotId, true) ) then
                ClearCursor()
            	return false
            end
        --Others
        else
			--check if item interaction is forbidden
            if( ItemSelectionHandler(bagId, slotId, true) ) then
                ClearCursor()
				-- item is not allowed to work with return false to prevent selection of the item
            	return false
            end
		end

	end

	-- call the original handler function
	local func = GetEventHandler("OnMouseDoubleClick", self:GetName())
	if ( not func ) then return false end

	return func(self, ...)
end

--==============================================================================
--			ON EVENT Methods
--==============================================================================

-- handler function for character equipment double click -> OnEffectivelyShown function
local function FCOItemSaver_CharacterOnEffectivelyShown(self, ...)
	if ( not self ) then return false end

    local equipmentSlotName
	for i = 1, self:GetNumChildren() do
		-- override OnMouseDoubleClick event of character window item controls, for each row (children)
    	equipmentSlotName = FCOIS.ZOControlVars.CHARACTER:GetChild(i):GetName()
        if equipmentSlotName ~= nil then
			if(string.find(equipmentSlotName, "ZO_CharacterEquipmentSlots")) then
				if( self:GetChild(i):GetHandler("OnMouseDoubleClick") ~= FCOItemSaver_CharacterItem_OnMouseDoubleClick ) then
					PreHookHandler( "OnMouseDoubleClick", self:GetChild(i), FCOItemSaver_CharacterItem_OnMouseDoubleClick )
				end
	        end
        end
	end
	-- call the original handler function
	local func = GetEventHandler("OnEffectivelyShown", self:GetName())
	if ( not func ) then return false end
	return func(self, ...)
end

-- handler function for bag's double click -> OnEffectivelyShown function
local function FCOItemSaver_OnEffectivelyShown(self, ...)
--d("[FCOItemSaver_OnEffectivelyShown]: " .. self:GetName())
	--Should we update the marker textures, size and color?
    checkMarker(-1)

	if ( not self ) then return false end

	for i = 1, self:GetNumChildren() do
		-- override OnMouseDoubleClick event of inventory item controls, for each row (children)
		if( self:GetChild(i):GetHandler("OnMouseDoubleClick") ~= FCOItemSaver_InventoryItem_OnMouseDoubleClick ) then
			PreHookHandler( "OnMouseDoubleClick", self:GetChild(i), FCOItemSaver_InventoryItem_OnMouseDoubleClick )
		end
	end
	-- call the original handler function
	local func = GetEventHandler("OnEffectivelyShown", self:GetName())
	if ( not func ) then return false end
	return func(self, ...)
end

--Executed if item should be destroyed manually
local function FCOItemSaver_OnMouseRequestDestroyItem(eventCode, bagId, slotIndex, itemCount, name, needsConfirm)
	--Hide the context menu at last active panel
    hideContextMenu(FCOIS.gFilterWhere)

    if not needsConfirm then
		--Only react if anti destroy setting is enabled
	    if (not FCOIS.settingsVars.settings.blockDestroying) then
	        FCOIS.preventerVars.gAllowDestroyItem = true
        	return nil
        end

		FCOIS.preventerVars.gAllowDestroyItem = false

		if( bagId and slotIndex ) then
            FCOIS.preventerVars.gAllowDestroyItem = not DestroySelectionHandler(bagId, slotIndex, true)
            --Change the "YES" button of the destroy dialog and disable keybind
	        overrideDialogYesButton(ZO_Dialog1)
		end
	end
end

--Callback function for start a new drag&drop operation
local function FCOItemSaver_OnDragStart(inventorySlot)
	if inventorySlot == nil then return end
	local cursorContentType = GetCursorContentType()
--d("[OnDragStart] cursorContentType: " .. tostring(cursorContentType) .. "/" .. tostring(MOUSE_CONTENT_INVENTORY_ITEM))
	--if(cursorContentType == MOUSE_CONTENT_EMPTY) then return end

    local bag
	local slot
    if inventorySlot:GetParent() == ZO_Character then
		bag		= inventorySlot.bagId
        slot	= inventorySlot.slotIndex
	else
	    if inventorySlot.dataEntry.data ~= nil then
			bag		= inventorySlot.dataEntry.data.bagId
	        slot	= inventorySlot.dataEntry.data.slotIndex
		end
	end
	FCOIS.dragAndDropVars.bag = nil
	FCOIS.dragAndDropVars.slot = nil
    if bag == nil or slot == nil then return end
	FCOIS.dragAndDropVars.bag = bag
	FCOIS.dragAndDropVars.slot = slot
end

--Callback function for receive a dragged inventory item
local function FCOItemSaver_OnReceiveDrag(inventorySlot)
FCOinvs = inventorySlot
	local cursorContentType = GetCursorContentType()
	debugMessage("[OnReceiveDrag] cursorContentType: " .. tostring(cursorContentType) .. "/" .. tostring(MOUSE_CONTENT_INVENTORY_ITEM) .. ", invSlotType: " .. tostring(inventorySlot.slotType) .. "/" .. tostring(SLOT_TYPE_EQUIPMENT), true)

    -- if there is an inventory item on the cursor:
    if cursorContentType ~= MOUSE_CONTENT_INVENTORY_ITEM and cursorContentType ~= MOUSE_CONTENT_EQUIPPED_ITEM then return end

    -- and the slot type we're dropping it on is an equip slot:
    if inventorySlot.slotType == SLOT_TYPE_EQUIPMENT then
		local bag
        local slot
    	--Was the drag started with another item then the dropped item slot?
        if FCOIS.dragAndDropVars.bag ~= nil and FCOIS.dragAndDropVars.slot ~= nil then
			bag			= FCOIS.dragAndDropVars.bag
	        slot		= FCOIS.dragAndDropVars.slot
            receiveSlot = inventorySlot.slotIndex
        else
        --get bagid and SlotIndex from receiving inventorySlot -> Makes no sense as the bind dialog shows the wrong item then!
			--bag   = inventorySlot.bagId
	        --slot	= inventorySlot.slotIndex
			--Abort here as it makes no sense to try to equip an item that you are already wearing :-D
			--Clear the old values from drag start now
			FCOIS.dragAndDropVars.bag = nil
			FCOIS.dragAndDropVars.slot = nil
		end
        if bag == nil or slot == nil then return end
        --Is item bindable and equipable?
        local doShowItemBindDialog = false
        if ( not FCOIS.ZOControlVars.CHARACTER:IsHidden() ) then
			local equipSucceeds, _ = IsEquipable(bag, slot)
	        if equipSucceeds and CheckBindableItems(bag, slot) then
            	doShowItemBindDialog = true
	        end
	    end

		--Clear the old values from drag start now
		FCOIS.dragAndDropVars.bag = nil
		FCOIS.dragAndDropVars.slot = nil
		-- check if destroying, improvement, sending or trading is forbidden
		-- and check if item is bindable (above)
		-- if so, clear item hold by cursor
		if( doShowItemBindDialog ) then
			--Remove the picked item from drag&drop cursor
			ClearCursor()
			return false
		else
			return false
		end

    end
end

--==============================================================================
--==============================================================================

--Event upon opening of a crafting station
local function FCOItemSaver_Crafting_Interact(craftSkill, sameStation)
    FCOIS.preventerVars.gActiveFilterPanel = true
	--EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_END_CRAFTING_STATION_INTERACT, FCOItemSaver_End_Crafting_Interact)

    --Abort if crafting station type is invalid
	if craftSkill == 0 then return end
	debugMessage("[EVENT] Crafting Interact: Craft skill: ".. tostring(craftSkill), true)

	--ALCHEMY
	if craftSkill == CRAFTING_TYPE_ALCHEMY then
		debugMessage("[ALCHEMY Crafting station opened]", true)
   		--Hide the context menu at last active panel
	    hideContextMenu(FCOIS.gFilterWhere)

		--If the addon PotionMaker is activated and got it's own Alchemy button activated too
        if FCOIS.otherAddons.potionMakerActive then
--d("PotionMaker active and button exists -> Resetting last clicked button now")
	        --Reset the variable for the last pressed button
        	FCOIS.lastVars.gLastAlchemyButton = FCOIS.ZOControlVars.ALCHEMY_STATION_MENUBAR_BUTTON_POTIONMAKER
        end
		--Show the filter buttons at the alchemy station
		FCOItemSaver_PreHookButtonHandler(LF_ALCHEMY_CREATION)

	else
--d("[FCOItemSaver_Crafting_Interact] FCOIS.gFilterWhere: " .. FCOIS.gFilterWhere)
		--Change the button color of the context menu invoker
		changeContextMenuInvokerButtonColorByPanelId(FCOIS.gFilterWhere)
	end
end

--Inventory slot gets updated function
function FCOItemSaver_Inv_Update( eventCode, bagId, slotId, isNewItem, itemSoundCategory, updateReason )
	--Scan new items in the player inventory and add markers OR update equipped/unequipped item markers
--d("[FCOItemSaver_Inv_Update] eventCode: " .. tostring(eventCode) .. ", bagId: " .. bagId .. ", slot: " .. slotId..", isNewItem: " .. tostring(isNewItem)..", updateReason: " .. tostring(updateReason))
	-- ===== Do some abort checks first =====

	--Mark new crafted item with the lock icon?
    if bagId ~= nil and slotId ~= nil and FCOIS.settingsVars.settings.autoMarkCraftedItems and FCOIS.preventerVars.newItemCrafted and isNewItem then
		local itemLink = GetItemLink(bagId, slotId)
		if itemLink ~= "" then
			-- item is a recipe
			local itemType = GetItemLinkItemType(itemLink)
            if itemType ~= nil then
				local allowedItemTypes = FCOIS.checkVars.allowedSetItemTypes
            	local allowed = allowedItemTypes[itemType] or false
                if allowed then
					--Check slightly delayed if the crafted item should be marked
	                zo_callLater(function()
						local allowed = true
						--Only mark crafted set parts?
						--Only do this if not already set parts, which get "new" into your inventory, will get marked automatically
		                if not FCOIS.settingsVars.settings.autoMarkSets and FCOIS.settingsVars.settings.autoMarkCraftedItemsSets then
							--Check if item is a set part
							allowed = isItemSetPartNoControl(bagId, slotId)
	                    end
						--Mark item now?
				        if allowed then
					        FCOIS.preventerVars.newItemCrafted = false
					    	FCOMarkItem(bagId, slotId, 1, true, true, nil)
				        end
	                end, 500)
                end
			end
		end
    end

	--ignore durability/dye update
	if updateReason ~= INVENTORY_UPDATE_REASON_DEFAULT then return end
	--Abort here if we are arrested by a guard (thief system) as it will scan our inventory for stolen items and destroy them.
    --We don't need to scan it with our functions too at this case
    if IsUnderArrest() then return end
	--Do not execute if horse is changed
	if SCENE_MANAGER:GetCurrentScene() == STABLES_SCENE then return end
    --Check if item in slot is still there
	if GetItemType(bagId, slotId) == ITEMTYPE_NONE then return end

    --All bags except the equipment
    if bagId ~= BAG_WORN then
	    --Abort if not new item is added to inventory
		if (not isNewItem) then return end
		--support for Roomba
		if Roomba and Roomba.WorkInProgress and Roomba.WorkInProgress() then return end

	    --Only check for normal player inventory
	    if (bagId == BAG_BACKPACK) then
			debugMessage("==============", true)
			debugMessage("Event 'Single inventory slot updated' raised! NewItem=" .. tostring(isNewItem) .. ", bagId=" .. bagId .. ", slotIndex=" .. slotId .. ", updateReason=" .. tostring(updateReason), true)
			--if(FCOIS.preventerVars.canUpdateInv == true) then
				--FCOIS.preventerVars.canUpdateInv = false
				zo_callLater(function()
	   				debugMessage("Event 'Single inventory slot updated' executed now! bagId=" .. bagId .. ", slotIndex=" .. slotId, true)
	                scanInventory(bagId, slotId)
	            end, 250)
				--FCOIS.preventerVars.canUpdateInv = true
			--end
	    end

	--Equipment bag:  BAG_WORN (character equipment)
    else
    	if slotId ~= nil then
			--Update the equipment slot control's markers
	        updateEquipmentSlotMarker(slotId, 50)
        end
    end
end

--Callback function for scenes to hide the context menu of inventory buttons
local function sceneCallbackHideContextMenu(oldState, newState)
	--When the mail inbox panel is showing up
	if newState == SCENE_SHOWING then
		--Hide the context menu at last active panel
		hideContextMenu(FCOIS.gFilterWhere)
	end
end

--Function to update the on mouse double click (OnEffectivelyShown) functions for new visible inventory rows
local function UpdateOnEffectivelyShownRows(delay)
	delay = delay or 0
	--Update the mouse double click handler OnEffectivelyShown() for the crafting stations, as new inventory rows could have been added
	if not FCOIS.ZOControlVars.DECONSTRUCTION_BAG:IsHidden() then
		zo_callLater(function() FCOItemSaver_OnEffectivelyShown(FCOIS.ZOControlVars.DECONSTRUCTION_BAG) end, delay)
	elseif not FCOIS.ZOControlVars.IMPROVEMENT_BAG:IsHidden() then
		zo_callLater(function() FCOItemSaver_OnEffectivelyShown(FCOIS.ZOControlVars.IMPROVEMENT_BAG) end, delay)
	elseif not FCOIS.ZOControlVars.ENCHANTING_STATION_BAG:IsHidden() then
		zo_callLater(function() FCOItemSaver_OnEffectivelyShown(FCOIS.ZOControlVars.ENCHANTING_STATION_BAG) end, delay)
	elseif not FCOIS.ZOControlVars.ALCHEMY_STATION_BAG:IsHidden() then
		zo_callLater(function() FCOItemSaver_OnEffectivelyShown(FCOIS.ZOControlVars.ALCHEMY_STATION_BAG) end, delay)
	end
end

--Check if the user specified a valid context menu ordering
local function checkIfUserContextMenuOrderValid()
--d("[checkIfUserContextMenuOrderValid]")
	local resultVar = true
    local checkDuplicateTable = {}
	--check each iconId if there are duplicates in the custom user sort order for the context menu
	for i = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
        if not checkDuplicateTable[FCOIS.settingsVars.settings.icon[i].sortOrder] then
--d("> added sortOrder " .. i)
			checkDuplicateTable[FCOIS.settingsVars.settings.icon[i].sortOrder] = true
		else
--d(">>> Sort order " .. FCOIS.settingsVars.settings.icon[i].sortOrder .. " is duplicate! Abort here!!!")
        	resultVar = false
            break -- end the loop here
        end
	end
	return resultVar
end

--Event callback function if a guild bank was swapped
local function FCOItemsaver_SelectGuildBank(_, guildBankId)
	--Store the current guild Id
	FCOIS.guildBankVars.guildBankId = guildBankId
end

--Function to reset the ANTI settings for the normal inventory
local function resetInventoryAntiSettings(currentSceneState)
	if currentSceneState == SCENE_SHOWING then
		autoReenableAntiSettingsCheck("DESTROY")
	end
end

--==============================================================================
--===== HOOKS BEGIN ============================================================
--==============================================================================
--Create the hooks & pre-hooks
local function FCOItemSaver_CreateHooks()
	local parentControl

--========= INVENTORY SLOT - SHOW CONTEXT MENU =================================
   	-- Hook functions for the inventory/store contextmenus
	ZO_PreHook("ZO_InventorySlot_ShowContextMenu", function(rowControl)
		--Clear the sub context menu entries
        FCOIS.customMenuVars.customMenuSubEntries = {}
		FCOIS.customMenuVars.customMenuCurrentCounter = 0

    	--Call a little bit later so the context menu is already created
		zo_callLater(function()
	        local noGear
	        local gearId

	  		local parentControl = rowControl:GetParent()
			local FCOcontextMenu = nil
            FCOcontextMenu = {}

			--Check if the user set ordering is valid, else use the standard sorting
            local userOrderValid = checkIfUserContextMenuOrderValid()

            local contextMenuEntriesAdded = 0
			--check each iconId and build a sorted context menu then
			for iconId = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				--Check if the icon (including gear sets) is enabled
				if FCOIS.settingsVars.settings.isIconEnabled[iconId] then
	                --Re-order the context menu entries by defaults, or with user settings
	                local newOrderId = 0
	                if userOrderValid then
	                	newOrderId = FCOIS.settingsVars.settings.icon[iconId].sortOrder
	                else
	                    newOrderId = FCOIS.settingsVars.defaults.icon[iconId].sortOrder
	                end
	                if newOrderId > 0 and newOrderId <= FCOIS.numVars.gFCONumFilterIcons then
						--Initialize the context menu entry at the new index
	            		FCOcontextMenu[newOrderId] = nil
	            		FCOcontextMenu[newOrderId] = {}
						--Is the current control an equipment control?
						local isEquipControl = (parentControl == FCOIS.ZOControlVars.CHARACTER)
			        	if(isEquipControl) then
            				FCOcontextMenu[newOrderId].control		= rowControl
						else
            				FCOcontextMenu[newOrderId].control		= parentControl
						end
           				FCOcontextMenu[newOrderId].iconId		= iconId
           				FCOcontextMenu[newOrderId].refreshPopup	= false
           				FCOcontextMenu[newOrderId].isEquip		= isEquipControl
           				FCOcontextMenu[newOrderId].useSubMenu	= FCOIS.settingsVars.settings.useSubContextMenu
                        --Increase the counter for added context menu entries
                        contextMenuEntriesAdded = contextMenuEntriesAdded + 1
	                end -- if newOrderId > 0 and newOrderId <= FCOIS.numVars.gFCONumFilterIcons then
				end -- if FCOIS.settingsVars.settings.isIconEnabled[iconId] then
        	end -- for

			--Are there andy context menu entries?
            if contextMenuEntriesAdded > 0 then
				for j = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
                    if FCOcontextMenu[j] ~= nil then
                        --AddMark(rowControl, markId, isEquipmentSlot, refreshPopupDialog, useSubMenu)
                        --Increase the global counter for the added context menu entries so the function AddMark can react on it
                        FCOIS.customMenuVars.customMenuCurrentCounter = FCOIS.customMenuVars.customMenuCurrentCounter + 1
                    	AddMark(FCOcontextMenu[j].control, FCOcontextMenu[j].iconId, FCOcontextMenu[j].isEquip, FCOcontextMenu[j].refreshPopup, FCOcontextMenu[j].useSubMenu)
                    end
                end

				--As the sub menu entries were build, show them now
		        if FCOIS.settingsVars.settings.useSubContextMenu then
					zo_callLater(function()
						--ClearMenu()
						if FCOIS.customMenuVars.customMenuSubEntries ~= nil and #FCOIS.customMenuVars.customMenuSubEntries > 0 then
							AddCustomSubMenuItem("|c22DD22FCO|r ItemSaver", FCOIS.customMenuVars.customMenuSubEntries)
		                end
						ShowMenu()
					end, 30)
		        end
            end -- if contextMenuEntriesAdded > 0 then
		end, 30) -- zo_callLater
	end)

--========= ZO_DIALOG1 / DESTROY DIALOG ========================================
	ZO_PreHook(FCOIS.ZOControlVars.DestroyItemDialog.buttons[2], "callback", function()
        --Get the "YES" button of the destroy dialog
    	local button1 = ZO_Dialog1:GetNamedChild("Button1")
	    if button1 == nil then return false end
		--Reset the "YES" button of the dialog again after a few seconds
        zo_callLater(function()
			if ZO_Dialog1 ~= nil and button1 ~= nil then
		        ZO_Dialog1:SetKeyboardEnabled(false)
				button1:SetText(GetString(SI_YES))
				button1:SetClickSound(SOUNDS.DIALOG_ACCEPT)
				button1:SetEnabled(true)
				button1:SetMouseEnabled(true)
				button1:SetHidden(false)
				button1:SetKeybindEnabled(true)
            end
		end, 100)
        return false
	end)

--========= START DRAG FROM INVENTORY ==========================================
	ZO_PreHook("ZO_InventorySlot_OnDragStart", FCOItemSaver_OnDragStart)

--========= DROP ITEM AT CHARACTER SLOT ========================================
	ZO_PreHook("ZO_InventorySlot_OnReceiveDrag", FCOItemSaver_OnReceiveDrag)

--========= EQUIP ITEM =========================================================
	ZO_PreHook("EquipItem", function(bagId, slotIndex)
        if bagId ~= nil and slotIndex ~= nil then
	    	debugMessage("[EquipItem] bagId: " .. bagId .. ", slotIndex: " .. slotIndex, true)
			--Check if the item is bound on equip and show dialog to acceppt the binding before (if enabled in the settings)
	    	return CheckBindableItems(bagId, slotIndex)
		end
    end)

--========= UNEQUIP ITEM =======================================================
	ZO_PreHook("UnequipItem", function(equipSlot)
        if equipSlot ~= nil then
			debugMessage("[UnequipItem] slotIndex: " .. equipSlot, true)
			--If item was unequipped: Remove the armor type marker if necessary
			zo_callLater(function() removeArmorTypeMarker(BAG_WORN, equipSlot) end, 100)
			zo_callLater(function() checkForEmptyOrTwoHandedWeaponMarker(equipSlot, false) end, 200)
	       	--Update the marker control of the new equipped item
			updateEquipmentSlotMarker(equipSlot, 1000)
		end
    end)

--========= MENU BARS ==========================================================
	--Preehook the menu bar shown event to update the character equipment section if it is shown
	ZO_PreHookHandler(ZO_MainMenuCategoryBar, "OnShow", function()
		debugMessage("[Hook] Main Menu Category Bar: OnShow")

        --Hide the context menu
        hideContextMenu(FCOIS.gFilterWhere)

        --Update the character's equipment markers, if the character screen is shown
		if not FCOIS.ZOControlVars.CHARACTER:IsHidden() then
			RefreshEquipmentControl()
		end

        --Update the dialog button 1 to show and respond again, if an item was tried to destroyed
       	--Get the "yes" button control of the destroy popup window
		if FCOIS.preventerVars.wasDestroyDone then
            local button1 = ZO_Dialog1:GetNamedChild("Button1")
            if button1 then
                button1:SetEnabled(true)
                button1:SetMouseEnabled(true)
                button1:SetHidden(false)
                button1:SetKeybindEnabled(true)
                ZO_Dialog1:SetKeyboardEnabled(false)
                FCOIS.preventerVars.wasDestroyDone = false
            end
        end

		--Check, if the Anti-* checks need to be enabled again
		autoReenableAntiSettingsCheck("DESTROY")
	end)

--========= REFINEMENT =========================================================
	local hookedFunctions = FCOIS.ZOControlVars.REFINEMENT.dataTypes[1].setupCallback
	FCOIS.ZOControlVars.REFINEMENT.dataTypes[1].setupCallback = function(rowControl, slot)
			hookedFunctions(rowControl, slot)
            -- for all filters: Create/Update the icons
			for i = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				CreateMarkerControl(rowControl, i, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[i].texture])
            end
			--Add additional FCO point to the dataEntry.data slot
			--FCOItemSaver_AddInfoToData(rowControl)
		end

--========= DECONSTRUCTION =====================================================
	local hookedFunctions = FCOIS.ZOControlVars.DECONSTRUCTION.dataTypes[1].setupCallback
	FCOIS.ZOControlVars.DECONSTRUCTION.dataTypes[1].setupCallback = function(rowControl, slot)
			hookedFunctions(rowControl, slot)
            -- for all filters: Create/Update the icons
			for i = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				CreateMarkerControl(rowControl, i, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[i].texture])
            end
			--Add additional FCO point to the dataEntry.data slot
			--FCOItemSaver_AddInfoToData(rowControl)
	        --Update the mouse double click handler OnEffectivelyShown() for the crafting stations, as new inventory rows could have been added
	        UpdateOnEffectivelyShownRows()
		end

--========= IMPROVEMENT ========================================================
	local hookedFunctions = FCOIS.ZOControlVars.IMPROVEMENT.dataTypes[1].setupCallback
	FCOIS.ZOControlVars.IMPROVEMENT.dataTypes[1].setupCallback = function(rowControl, slot)
			hookedFunctions(rowControl, slot)
            -- for all filters: Create/Update the icons
			for i = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				CreateMarkerControl(rowControl, i, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[i].texture])
            end
			--Add additional FCO point to the dataEntry.data slot
			--FCOItemSaver_AddInfoToData(rowControl)
	        --Update the mouse double click handler OnEffectivelyShown() for the crafting stations, as new inventory rows could have been added
	        UpdateOnEffectivelyShownRows()
		end

--========= ENCHANTING =========================================================
	local hookedFunctions = FCOIS.ZOControlVars.ENCHANTING_STATION.dataTypes[1].setupCallback
	FCOIS.ZOControlVars.ENCHANTING_STATION.dataTypes[1].setupCallback = function(rowControl, slot)
			hookedFunctions(rowControl, slot)
            -- for all filters: Create/Update the icons
			for i = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				CreateMarkerControl(rowControl, i, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[i].texture])
            end
			--Add additional FCO point to the dataEntry.data slot
			--FCOItemSaver_AddInfoToData(rowControl)
	        --Update the mouse double click handler OnEffectivelyShown() for the crafting stations, as new inventory rows could have been added
	        UpdateOnEffectivelyShownRows()
		end

--========= ALCHEMY ============================================================
	--Solvents
	local hookedFunctions = FCOIS.ZOControlVars.ALCHEMY_STATION.dataTypes[1].setupCallback
	FCOIS.ZOControlVars.ALCHEMY_STATION.dataTypes[1].setupCallback = function(rowControl, slot)
			hookedFunctions(rowControl, slot)
            -- for all filters: Create/Update the icons
			for i = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				CreateMarkerControl(rowControl, i, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[i].texture])
            end
			--Add additional FCO point to the dataEntry.data slot
			--FCOItemSaver_AddInfoToData(rowControl)
	        --Update the mouse double click handler OnEffectivelyShown() for the crafting stations, as new inventory rows could have been added
	        UpdateOnEffectivelyShownRows()
		end
	--Reagents
	local hookedFunctions = FCOIS.ZOControlVars.ALCHEMY_STATION.dataTypes[2].setupCallback
	FCOIS.ZOControlVars.ALCHEMY_STATION.dataTypes[2].setupCallback = function(rowControl, slot)
			hookedFunctions(rowControl, slot)
            -- for all filters: Create/Update the icons
			for i = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
				CreateMarkerControl(rowControl, i, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.settingsVars.settings.icon[i].size or FCOIS.iconVars.gIconWidth, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[i].texture])
            end
			--Add additional FCO point to the dataEntry.data slot
			--FCOItemSaver_AddInfoToData(rowControl)
	        --Update the mouse double click handler OnEffectivelyShown() for the crafting stations, as new inventory rows could have been added
	        UpdateOnEffectivelyShownRows()
		end

--========= RESEARCH LIST / ListDialog (also repair, enchant, charge, etc.) ======================================================
	local hookedFunctions = FCOIS.ZOControlVars.LIST_DIALOG.dataTypes[1].setupCallback
	FCOIS.ZOControlVars.LIST_DIALOG.dataTypes[1].setupCallback = function(rowControl, slot)
		hookedFunctions(rowControl, slot)

   		local data = rowControl.dataEntry.data

		--Check if rowControl is a soulgem
		--GLOBALOL = data
		local isSoulGem = false
		if(data and GetSoulGemItemInfo(data.bag, data.index) > 0) then
			isSoulGem = true
		end

        local myItemInstanceIdOfControl = MyGetItemInstanceId(rowControl)
        local disableControl = false

		--Suspend the source code here a bit as the list dialog name etc. will be given in a few milliseconds
        zo_callLater(function()

			--Current dialog is the repair item dialog?
			local isRepairDialog = ZO_Dialogs_IsShowing(FCOIS.ZOControlVars.RepairItemDialogName)

			-- Check the rowControl if the item is marked and update the OnMouseUp functions and the color of the item row then
			for iconId = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
	            local iconIsProtected = checkIfItemIsProtected(iconId, myItemInstanceIdOfControl)

				--Special research icon handling
				if iconId == 3 then
	                if(not isSoulGem and iconIsProtected) then
		            	--Not inside the repair dialog and settings allow research of "marked for research" items?
						if not isRepairDialog and not FCOIS.settingsVars.settings.allowResearch then
							disableControl = true
							break -- leave for ... do loop
	                    else
							if not disableControl then
								disableControl = false
							end
	                    end
					else
						if not disableControl then
							disableControl = false
						end
	                end

				--All other icons
	            else
		           if(not isSoulGem and iconIsProtected) then
	                    if (isRepairDialog and FCOIS.settingsVars.settings.blockMarkedRepairKits) or (not isRepairDialog) then
		               		disableControl = true
			                break -- leave for ... do loop
	                    else
							if not disableControl then
								disableControl = false
							end
	                    end
	               else
						if not disableControl then
							disableControl = false
						end
		            end
	            end

	        end -- for j = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do

			--Get here after for loop is left by a "break" and item is not researchable
	        if disableControl == true then
				--Change the color of the item to red
				rowControl:GetNamedChild("Name"):SetColor(0.75, 0, 0, 1)
	        end

		end, 150) -- zo_callLater(function()...)

		--Pre-Hook the list dialog's rows

        --Pre-Hook the handler "OnMouseUp" event for the rowControl to disable the researching of the item,
        --but still enable the right click/context menu:
		--Show context menu at mouse button 2, but keep the normal OnMouseUp handler as well
		ZO_PreHookHandler(rowControl, "OnMouseUp", function(control, button, upInside)
			debugMessage("Clicked: " ..control:GetName() .. ", MouseButton: " .. tostring(button), true)
			--button 1= left mouse button / 2= right mouse button

			--Right click/mouse button 2 context menu hook part:
			if button == 2 and upInside then
                local FCOcontextMenu = nil
                local FCOcontextMenu = {}
				--Check if the user set ordeirng is valid, else use the standard sorting
	            local userOrderValid = checkIfUserContextMenuOrderValid()
	            local contextMenuEntriesAdded = 0
				--check each iconId and build a sorted context menu then
				for iconId = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
					--Check if the icon (including gear sets) is enabled
					if FCOIS.settingsVars.settings.isIconEnabled[iconId] then
		                --Re-order the context menu entries by defaults, or with user settings
		                local newOrderId = 0
		                if userOrderValid then
		                	newOrderId = FCOIS.settingsVars.settings.icon[iconId].sortOrder
		                else
		                    newOrderId = FCOIS.settingsVars.defaults.icon[iconId].sortOrder
		                end
		                if newOrderId > 0 and newOrderId <= FCOIS.numVars.gFCONumFilterIcons then
							--Initialize the context menu entry at the new index
		            		FCOcontextMenu[newOrderId] = nil
		            		FCOcontextMenu[newOrderId] = {}
            				FCOcontextMenu[newOrderId].control		= rowControl
	           				FCOcontextMenu[newOrderId].iconId		= iconId
	           				FCOcontextMenu[newOrderId].refreshPopup	= true
	           				FCOcontextMenu[newOrderId].isEquip		= false
	           				FCOcontextMenu[newOrderId].useSubMenu	= false
	                        --Increase the counter for added context menu entries
	                        contextMenuEntriesAdded = contextMenuEntriesAdded + 1
		                end -- if newOrderId > 0 and newOrderId <= FCOIS.numVars.gFCONumFilterIcons then
					end -- if FCOIS.settingsVars.settings.isIconEnabled[iconId] then
	        	end -- for

				--Are there any context menu entries?
	            if contextMenuEntriesAdded > 0 then
					--Reset the counter for the AddMark function
					FCOIS.customMenuVars.customMenuCurrentCounter = 0
					--Clear the menu
		            ClearMenu()
					--Add the context menu entries now
					for j = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
	                    if FCOcontextMenu[j] ~= nil then
	                        --AddMark(rowControl, markId, isEquipmentSlot, refreshPopupDialog, useSubMenu)
	                        --Increase the global counter for the added context menu entries so the function AddMark can react on it
	                        FCOIS.customMenuVars.customMenuCurrentCounter = FCOIS.customMenuVars.customMenuCurrentCounter + 1
	                    	AddMark(FCOcontextMenu[j].control, FCOcontextMenu[j].iconId, FCOcontextMenu[j].isEquip, FCOcontextMenu[j].refreshPopup, FCOcontextMenu[j].useSubMenu)
	                    end
	                end
				end

			end -- if button == 2 and upInside then

			--Disable the usage/research/etc. of this item
			if disableControl == true then
				--Do nothing (true tells the handler function that everything was achieved already in this function
				--and the normal "hooked" functions don't need to be run afterwards)
                -- -> All handling will be done in function MarkMe() as the dialog list will be refreshed!
				return true

			else -- if disableControl == false
                --Is the row selected?
            	if FCOIS.ZOControlVars.LIST_DIALOG.selectedControl ~= nil then
					--Enable the "use" button again, as it will be disabled before if the item won't be usable
	  				WINDOW_MANAGER:GetControlByName("ZO_ListDialog1Button1", ""):SetEnabled(true)
                end
			end -- if disableControl == true

		end) -- ZO_PreHookHandler(rowControl, "OnMouseUp"...

        -- Create/Update all the icons for the current dialog row
		for iconNumb = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
			CreateMarkerControl(rowControl, iconNumb, FCOIS.settingsVars.settings.icon[iconNumb].size or FCOIS.iconVars.gIconWidth, FCOIS.settingsVars.settings.icon[iconNumb].size or FCOIS.iconVars.gIconWidth, FCOIS.textureVars.MARKER_TEXTURES[FCOIS.settingsVars.settings.icon[iconNumb].texture])
		end -- for i = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do

	end -- list dialog 1 pre-hook (e.g. research, repair item, enchant, charge, etc.)

--======== INVENTORY ===========================================================
	--Pre Hook the inventory for prevention methods
	PreHookHandler( "OnEffectivelyShown",      FCOIS.ZOControlVars.BACKPACK_BAG, FCOItemSaver_OnEffectivelyShown )

	--ONLY if the craftbag is active
    if HasCraftBagAccess() then
	    --Pre Hook the 2 menubar button's (items and crafting bag) handler at the inventory
		ZO_PreHookHandler(FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_ITEMS, "OnMouseUp", function(control, button, upInside)
	--d("inv button 1, button: " .. button .. ", upInside: " .. tostring(upInside) .. ", lastButton: " .. FCOIS.lastVars.gLastInvButton:GetName())
	        if (button == 1 and upInside and FCOIS.lastVars.gLastInvButton~=FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_ITEMS) then
	            FCOIS.lastVars.gLastInvButton = FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_ITEMS
                hideContextMenu(LF_CRAFTBAG)
	            zo_callLater(function() FCOItemSaver_PreHookButtonHandler(LF_INVENTORY) end, 50)
	        end
	    end)
		ZO_PreHookHandler(FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_CRAFTBAG, "OnMouseUp", function(control, button, upInside)
	--d("inv button 2, button: " .. button .. ", upInside: " .. tostring(upInside) .. ", lastButton: " .. FCOIS.lastVars.gLastInvButton:GetName())
	        if (button == 1 and upInside and FCOIS.lastVars.gLastInvButton~=FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_CRAFTBAG) then
	            FCOIS.lastVars.gLastInvButton = FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_CRAFTBAG
                hideContextMenu(LF_INVENTORY)
	            zo_callLater(function() FCOItemSaver_PreHookButtonHandler(LF_CRAFTBAG) end, 50)
	        end
	    end)
    end

--======== CHARACTER ===========================================================
	--Pre Hook the character window for prevention methods
	PreHookHandler( "OnEffectivelyShown",      FCOIS.ZOControlVars.CHARACTER, FCOItemSaver_CharacterOnEffectivelyShown )

--======== RIGHT CLICK / CONTEXT MENU ==========================================
	--Pre Hook the right click/context menu addition of items
    ZO_PreHook(ZO_InventorySlotActions, "AddSlotAction", FCOItemSaver_AddSlotAction)

--======== CURRENCIES (in inventory) ===========================================
    --Pre Hook the 3rd menubar button (Currencies) handler at the player inventory
	ZO_PreHookHandler(FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_CURRENCIES, "OnMouseUp", function(control, button, upInside)
        if (button == 1 and upInside) then
			zo_callLater(function()
		       	hideContextMenu(LF_INVENTORY)
    			if HasCraftBagAccess() then
			       	hideContextMenu(LF_CRAFTBAG)
				end
            end, 50)
        end
    end)

--======== QUICK SLOTS =========================================================
    --Pre Hook the 4th menubar button (Quickslots) handler at the player inventory
	ZO_PreHookHandler(FCOIS.ZOControlVars.INV_MENUBAR_BUTTON_QUICKSLOTS, "OnMouseUp", function(control, button, upInside)
        if (button == 1 and upInside) then
			zo_callLater(function()
				if not FCOIS.ZOControlVars.QUICKSLOT_CIRCLE:IsHidden() then
			       	hideContextMenu(LF_INVENTORY)
	    			if HasCraftBagAccess() then
				       	hideContextMenu(LF_CRAFTBAG)
					end
				end
            end, 50)
        end
    end)

--======== FENCE & LAUNDER =====================================================
	--Pre Hook the fence and launder "enter" and "fence closed" functions
    ZO_PreHook(FENCE_MANAGER, "OnEnterSell", function(...)
        zo_callLater(function() FCOItemSaver_PreHookButtonHandler(LF_FENCE_SELL) end, 50)
	end)
    ZO_PreHook(FENCE_MANAGER, "OnEnterLaunder", function(...)
        zo_callLater(function() FCOItemSaver_PreHookButtonHandler(LF_FENCE_LAUNDER) end, 50)
    end)
    ZO_PreHook(FENCE_MANAGER, "OnFenceClosed", function(...)
		debugMessage("[FENCE_MANAGER:OnFenceClosed]", true)
		--Avoid the filter panel ID change if the fence_manager is called from a normal vendor, which closes the store:
        --If you directly open the mail panel at the vendor the current panel ID will be reset to LF_INVENTORY and this would be not true!
		if FCOIS.preventerVars.gNoCloseEvent == false then
	    	FCOIS.gFilterWhere = LF_INVENTORY
			--Change the button color of the context menu invoker
			changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)
		end
    end)

--======== BANK ================================================================
    --Pre Hook the 2 menubar button's (take and deposit) handler at the bank
	ZO_PreHookHandler(FCOIS.ZOControlVars.BANK_MENUBAR_BUTTON_WITHDRAW, "OnMouseUp", function(control, button, upInside)
--d("bank button 1, button: " .. button .. ", upInside: " .. tostring(upInside) .. ", lastButton: " .. FCOIS.lastVars.gLastBankButton:GetName())
        if (button == 1 and upInside and FCOIS.lastVars.gLastBankButton~=FCOIS.ZOControlVars.BANK_MENUBAR_BUTTON_WITHDRAW) then
            FCOIS.lastVars.gLastBankButton = FCOIS.ZOControlVars.BANK_MENUBAR_BUTTON_WITHDRAW
            zo_callLater(function() FCOItemSaver_PreHookButtonHandler(LF_BANK_WITHDRAW) end, 50)
        end
    end)
	ZO_PreHookHandler(FCOIS.ZOControlVars.BANK_MENUBAR_BUTTON_DEPOSIT, "OnMouseUp", function(control, button, upInside)
--d("bank button 2, button: " .. button .. ", upInside: " .. tostring(upInside) .. ", lastButton: " .. FCOIS.lastVars.gLastBankButton:GetName())
        if (button == 1 and upInside and FCOIS.lastVars.gLastBankButton~=FCOIS.ZOControlVars.BANK_MENUBAR_BUTTON_DEPOSIT) then
            FCOIS.lastVars.gLastBankButton = FCOIS.ZOControlVars.BANK_MENUBAR_BUTTON_DEPOSIT
            zo_callLater(function() FCOItemSaver_PreHookButtonHandler(LF_BANK_DEPOSIT) end, 50)
        end
    end)

--======== GUILD BANK ==========================================================
    --Pre Hook the 2 menubar button's (take and deposit) handler at the guild bank
	ZO_PreHookHandler(FCOIS.ZOControlVars.GUILD_BANK_MENUBAR_BUTTON_WITHDRAW, "OnMouseUp", function(control, button, upInside)
--d("guild bank button 1, button: " .. button .. ", upInside: " .. tostring(upInside) .. ", lastButton: " .. FCOIS.lastVars.gLastGuildBankButton:GetName())
        if (button == 1 and upInside and FCOIS.lastVars.gLastGuildBankButton~=FCOIS.ZOControlVars.GUILD_BANK_MENUBAR_BUTTON_WITHDRAW) then
            FCOIS.lastVars.gLastGuildBankButton = FCOIS.ZOControlVars.GUILD_BANK_MENUBAR_BUTTON_WITHDRAW
            zo_callLater(function() FCOItemSaver_PreHookButtonHandler(LF_GUILDBANK_WITHDRAW) end, 50)
        end
    end)
	ZO_PreHookHandler(FCOIS.ZOControlVars.GUILD_BANK_MENUBAR_BUTTON_DEPOSIT, "OnMouseUp", function(control, button, upInside)
--d("guild bank button 2, button: " .. button .. ", upInside: " .. tostring(upInside) .. ", lastButton: " .. FCOIS.lastVars.gLastGuildBankButton:GetName())
        if (button == 1 and upInside and FCOIS.lastVars.gLastGuildBankButton~=FCOIS.ZOControlVars.GUILD_BANK_MENUBAR_BUTTON_DEPOSIT) then
            FCOIS.lastVars.gLastGuildBankButton = FCOIS.ZOControlVars.GUILD_BANK_MENUBAR_BUTTON_DEPOSIT
            zo_callLater(function() FCOItemSaver_PreHookButtonHandler(LF_GUILDBANK_DEPOSIT) end, 50)
        end
    end)

--======== SMITHING=============================================================
--Prehook the smithing function SetMode() which gets executed as the smithing tabs are changed
	ZO_PreHook(SMITHING, "SetMode", function(smithing_obj, mode)
		--Hide the context menu at last active panel
	    hideContextMenu(FCOIS.gFilterWhere)

		debugMessage("[SMITHING:SetMode] Mode: " .. tostring(mode), true)
        --Deconstruction
        if     mode == SMITHING_MODE_DECONSTRUCTION then
	        FCOItemSaver_PreHookButtonHandler(LF_SMITHING_DECONSTRUCT)
            --zo_callLater(function() FCOItemSaver_OnEffectivelyShown(FCOIS.ZOControlVars.DECONSTRUCTION_BAG) end, 100)
		--Improvement
        elseif mode == SMITHING_MODE_IMPROVEMENT then
	        FCOItemSaver_PreHookButtonHandler(LF_SMITHING_IMPROVEMENT)
            --zo_callLater(function() FCOItemSaver_OnEffectivelyShown(FCOIS.ZOControlVars.IMPROVEMENT_BAG) end, 100)
        end
        --Go on with original function
        return false
	end)

--======== ENCHANTING ==========================================================
--Prehook the enchanting function SetEnchantingMode() which gets executed as the enchanting tabs are changed
	ZO_PreHook(ENCHANTING, "SetEnchantingMode", function(enchanting_obj, enchantingMode)
		--Hide the context menu at last active panel
	    hideContextMenu(FCOIS.gFilterWhere)

		debugMessage("[ENCHANTING:SetEnchantingMode] EnchantingMode: " .. tostring(enchantingMode), true)
    	--[[ enchantingMode could be:
			ENCHANTING_MODE_CREATION
			ENCHANTING_MODE_EXTRACTION
        ]]
        --Creation
        if     enchantingMode == ENCHANTING_MODE_CREATION then
	        FCOItemSaver_PreHookButtonHandler(LF_ENCHANTING_CREATION)
            --zo_callLater(function() FCOItemSaver_OnEffectivelyShown(FCOIS.ZOControlVars.ENCHANTING_STATION_BAG) end, 100)
		--Extraction
        elseif enchantingMode == ENCHANTING_MODE_EXTRACTION then
	        FCOItemSaver_PreHookButtonHandler(LF_ENCHANTING_EXTRACTION)
            --zo_callLater(function() FCOItemSaver_OnEffectivelyShown(FCOIS.ZOControlVars.ENCHANTING_STATION_BAG) end, 100)
        end
        --Go on with original function
        return false
	end)

--======== ALCHEMY =============================================================
	--Prehook the alchemy function which gets executed as the alchemy tabs are changed
	ZO_PreHookHandler(FCOIS.ZOControlVars.ALCHEMY_STATION_MENUBAR_BUTTON_CREATION, "OnMouseUp", function(control, button, upInside)
--d("Alchemy button 1, button: " .. button .. ", upInside: " .. tostring(upInside) .. ", lastButton: " .. FCOIS.lastVars.gLastAlchemyButton:GetName())
        if (button == 1 and upInside) then
        	if (FCOIS.otherAddons.potionMakerActive and FCOIS.lastVars.gLastAlchemyButton~=FCOIS.ZOControlVars.ALCHEMY_STATION_MENUBAR_BUTTON_CREATION) then
            	FCOIS.lastVars.gLastAlchemyButton = FCOIS.ZOControlVars.ALCHEMY_STATION_MENUBAR_BUTTON_CREATION
	            --zo_callLater(function() FCOItemSaver_PreHookButtonHandler(LF_ALCHEMY_CREATION) end, 50)
			else
	            --zo_callLater(function() FCOItemSaver_PreHookButtonHandler(LF_ALCHEMY_CREATION) end, 50)
			end
        end
    end)

	--Another Prehook will be done at the event callback function for the crafting station interact, when the alchemy station
    --gets opened as the PotionMaker addon button will be created then

--======== MAIL ================================================================
	--Register a callback function for the mail inbox scene
	MAIL_INBOX_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[MAIL_INBOX_SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

	--Register a callback function for the mail send scene
	MAIL_SEND_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[MAIL_SEND_SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)

    	--When the mail send panel is showing up
		if newState == SCENE_SHOWING then
			--Update the current filter panel ID to "Mail"
	    	FCOIS.gFilterWhere = LF_MAIL_SEND

        	--Check if another filter panel was already opened and we are coming form there before the CLOSE EVENT function was called
			if FCOIS.preventerVars.gActiveFilterPanel == true then
				--Set the "No Close Event" flag so the called close event won't override gFilterWhere and update the filter button colors and callback handlers
				FCOIS.preventerVars.gNoCloseEvent = true
            end
			--Change the button color of the context menu invoker
			changeContextMenuInvokerButtonColorByPanelId(LF_MAIL_SEND)
			--Check the filter buttons and create them if they are not there. Update the inventory afterwards too
			CheckFilterButtonsAtCurrentPanel(true, LF_MAIL_SEND)

		--When the mail send panel is hiding
        elseif newState == SCENE_HIDING then
			--Hide the context menu at mail panel
		    hideContextMenu(LF_MAIL_SEND)

		--When the mail send panel is hidden
		elseif newState == SCENE_HIDDEN then
		    --Update the inventory filter buttons
		    updateFilterButtonsInInv(-1)
			--Update the 4 inventory button's color
			UpdateButtonColorsAndTextures(-1, nil, -1, LF_INVENTORY)

			FCOIS.preventerVars.gActiveFilterPanel = false
			FCOIS.preventerVars.gNoCloseEvent = false

			--Change the button color of the context menu invoker
			changeContextMenuInvokerButtonColorByPanelId(LF_INVENTORY)
			--Check, if the Anti-* checks need to be enabled again
			autoReenableAntiSettingsCheck("MAIL")
        end
	end)

--======== QUEST JOURNAL ================================================================
	--Register a callback function for the quest journal scene
	QUEST_JOURNAL_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[QUEST JOURNAL SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== GROUP LIST ================================================================
	--Register a callback function for the group list scene
	local groupScene
	--> API 100013
	if FCOIS.APIversion <= 100013 then
		groupScene = GROUP_LIST_SCENE
	--> API 100014
	else
		groupScene = KEYBOARD_GROUP_MENU_SCENE
	end
	groupScene:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[KEYBOARD_GROUP_MENU_SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== GROUPING TOOLS ================================================================
	if FCOIS.APIversion <= 100013 then
		--Register a callback function for the grouping list scene
		GROUPING_TOOLS_SCENE:RegisterCallback("StateChange", function(oldState, newState)
			debugMessage("[GROUPING TOOLS SCENE] State: " .. tostring(newState), true)

	        sceneCallbackHideContextMenu(oldState, newState)

			--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
	        --So do this here now:
			resetInventoryAntiSettings(newState)
		end)
	end

--======== LORE LIBRARY ================================================================
	--Register a callback function for the lore library scene
	LORE_LIBRARY_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[LORE LIBRARY SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== LORE READER INVENTORY ================================================================
	--Register a callback function for the lore reader inventory scene
	LORE_READER_INVENTORY_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[LORE READER INVENTORY SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== LORE READER LORE LIBRARY ================================================================
	--Register a callback function for the lore library scene
	LORE_READER_LORE_LIBRARY_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[LORE READER LORE LIBRARY SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== LORE READER INTERACTION ================================================================
	--Register a callback function for the lore reader interaction scene
	LORE_READER_INTERACTION_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[LORE READER INTERACTION SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)
	end)

--======== TREASURE MAP INVENTORY ================================================================
	--Register a callback function for the treasure map inventory scene
	TREASURE_MAP_INVENTORY_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[TREASURE MAP INVENTORY SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)
	end)

--======== TREASURE MAP QUICK SLOT ================================================================
	--Register a callback function for the treasure map quick slot scene
	TREASURE_MAP_QUICK_SLOT_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[TREASURE MAP QUICK SLOT SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)
	end)

--======== GAME MENU ================================================================
	--Register a callback function for the game menu scene
	GAME_MENU_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[GAME MENU SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== LEADERBOARD ================================================================
	--Register a callback function for the leaderboard scene
	LEADERBOARDS_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[LEADERBOARDS SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== WORLD MAP ================================================================
	--Register a callback function for the wolrd map scene
	WORLD_MAP_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[WORLD MAP SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== HELP CUSTOMER SUPPORT ================================================================
	--Register a callback function for the help customer support scene
	HELP_CUSTOMER_SUPPORT_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[HELP CUSTOMER SUPPORT SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== FRIENDS LIST ================================================================
	--Register a callback function for the friends list scene
	FRIENDS_LIST_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[FRIENDS LIST SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== IGNORE LIST ================================================================
	--Register a callback function for the ignore list scene
	IGNORE_LIST_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[IGNORE LIST SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== GUILD HOME ================================================================
	--Register a callback function for the guild hom scene
	GUILD_HOME_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[GUILD HOME SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)
	end)

--======== GUILD ROSTER ================================================================
	--Register a callback function for the guild roster scene
	GUILD_ROSTER_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[GUILD ROSTER SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== GUILD RANKS ================================================================
	--Register a callback function for the guild ranks scene
	GUILD_RANKS_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[GUILD RANKS SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== GUILD HERALDRY ================================================================
	--Register a callback function for the guild heraldry scene
	GUILD_HERALDRY_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[GUILD HERALDRY SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== GUILD HISTORY ================================================================
	--Register a callback function for the guild history scene
	GUILD_HISTORY_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[GUILD HISTORY SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== GUILD CREATE ================================================================
	--Register a callback function for the guild create scene
	GUILD_CREATE_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[GUILD CREATE SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== NOTIFICATIONS ================================================================
	--Register a callback function for the notifications scene
	NOTIFICATIONS_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[NOTIFICATIONS SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== CAMPAIGN BROWSER ================================================================
	--Register a callback function for the campaign browser scene
	CAMPAIGN_BROWSER_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[CAMPAIGN BROWSER SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== CAMPAIGN OVERVIEW ================================================================
	--Register a callback function for the campaign overview scene
	CAMPAIGN_OVERVIEW_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[CAMPAIGN OVERVIEW SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== STATS ================================================================
	--Register a callback function for the stats scene
	STATS_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[STATS SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)
	end)

--======== SIEGE BAR ================================================================
	--Register a callback function for the siege bar scene
	SIEGE_BAR_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[SIEGE BAR SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
	end)

--======== CHAMPION PERKS ================================================================
    CHAMPION_PERKS_SCENE:RegisterCallback("StateChange", function(oldState, newState)
		debugMessage("[CHAMPION PERKS SCENE] State: " .. tostring(newState), true)

        sceneCallbackHideContextMenu(oldState, newState)

		--If the inventory was shown at last and the mail panel was opened directly with shown inventory the settings for the anti-destroy won't be updted!
        --So do this here now:
		resetInventoryAntiSettings(newState)
    end)
end
--==============================================================================
--===== HOOKS END ==============================================================
--==============================================================================


--Localized texts etc.
local function Localization()
--d("[FCOIS] Localization - Start, keybindings: " .. tostring(FCOIS.preventerVars.KeyBindingTexts))
	--Was localization already done during keybindings? Then abort here
 	if FCOIS.preventerVars.KeyBindingTexts == true and FCOIS.preventerVars.gLocalizationDone == true then return end
	--Was a language chosen already?
    --Fallback to english variable
    local fallbackToEnglish = false
    if not FCOIS.settingsVars.settings.languageChosen then
--d("[FCOIS] Localization: Fallback to english. Keybindings: " .. tostring(FCOIS.preventerVars.KeyBindingTexts) .. ", language chosen: " .. tostring(FCOIS.settingsVars.settings.languageChosen))
		if FCOIS.settingsVars.defaultSettings.language == nil then
--d("[FCOIS] Localization: defaultSettings.language is NIL -> Fallback to english now")
	    	fallbackToEnglish = true
	    else
			if FCOIS.numVars.languageCount ~= nil and FCOIS.numVars.languageCount >= 1 then
				for i=1, FCOIS.numVars.languageCount do
				    if FCOIS.settingsVars.defaultSettings.language ~= i then
			        	fallbackToEnglish = true
--d("[FCOIS] Localization: defaultSettings.language is ~= " .. i .. " -> Fallback to english now")
			            break
				    end
			    end
			end
	    end
	end
--d("[FCOIS] localization, fallBackToEnglish: " .. tostring(fallbackToEnglish))
	--Fallback to english language now
    if (fallbackToEnglish) then FCOIS.settingsVars.defaultSettings.language = 1 end
	--Is the standard language english set?
    if (FCOIS.preventerVars.KeyBindingTexts == true or (FCOIS.settingsVars.defaultSettings.language == 1 and FCOIS.settingsVars.settings.languageChosen == false)) then
--d("[FCOIS] localization: Language chosen is false!")
		local lang = GetCVar("language.2")
		--Check for supported languages
		if(lang == "de") then
	    	FCOIS.settingsVars.defaultSettings.language = 2
	    elseif (lang == "en") then
	    	FCOIS.settingsVars.defaultSettings.language = 1
	    elseif (lang == "fr") then
	    	FCOIS.settingsVars.defaultSettings.language = 3
	    elseif (lang == "es") then
	    	FCOIS.settingsVars.defaultSettings.language = 4
	    elseif (lang == "it") then
	    	FCOIS.settingsVars.defaultSettings.language = 5
	    elseif (lang == "jp") then
	    	FCOIS.settingsVars.defaultSettings.language = 6
	    elseif (lang == "ru") then
	    	FCOIS.settingsVars.defaultSettings.language = 7
		else
	    	FCOIS.settingsVars.defaultSettings.language = 1
	    end
	end
	debugMessage("[Localization] default FCOIS.settingsVars.settings, language: " .. tostring(FCOIS.settingsVars.defaultSettings.language), false)
    --Get the localized texts from the localization file
    FCOIS.localizationVars.fcois_loc = FCOIS.localizationVars.localizationAll[FCOIS.settingsVars.defaultSettings.language]

	--The localization end string array
	FCOIS.localizationVars.iconEndStrArray = {
    	[1]  = "color",
    	[2]  = "name",
    	[3]  = "color",
    	[4]  = "name",
    	[5]  = "color",
    	[6]  = "name",
    	[7]  = "name",
    	[8]  = "name",
    	[9]  = "color",
    	[10] = "color",
    	[11] = "color",
    	[12] = "color",
    }

    FCOIS.preventerVars.gLocalizationDone = true
	--Abort here if we only needed the keybinding texts
   	if FCOIS.preventerVars.KeyBindingTexts == true then return end

	--Initialize arrays
	FCOIS.localizationVars.contextEntries.menu_add_all_text			= {}
	FCOIS.localizationVars.contextEntries.menu_remove_all_text		= {}
    FCOIS.localizationVars.contextEntries.menu_add_gear_text			= {}
    FCOIS.localizationVars.contextEntries.menu_remove_gear_text		= {}
    FCOIS.localizationVars.contextEntries.menu_add_all_gear_text		= {}
    FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text	= {}

    --Prepare the texts for the right click menus
    FCOIS.localizationVars.contextEntries.menu_add_lock_text  	   	= FCOIS.localizationVars.fcois_loc["rightclick_menu_lock"]
	FCOIS.localizationVars.contextEntries.menu_add_gear_text[1]	  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_gear1"]
	FCOIS.localizationVars.contextEntries.menu_add_research_text     	= FCOIS.localizationVars.fcois_loc["rightclick_menu_mark_analysis"]
	FCOIS.localizationVars.contextEntries.menu_add_gear_text[2]	  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_gear2"]
	FCOIS.localizationVars.contextEntries.menu_add_sell_text 	 	  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_mark_sell"]
	FCOIS.localizationVars.contextEntries.menu_add_gear_text[3] 	  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_gear3"]
	FCOIS.localizationVars.contextEntries.menu_add_gear_text[4] 	  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_gear4"]
	FCOIS.localizationVars.contextEntries.menu_add_gear_text[5] 	  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_gear5"]
    FCOIS.localizationVars.contextEntries.menu_add_deconstruction_text = FCOIS.localizationVars.fcois_loc["rightclick_menu_mark_deconstruction"]
    FCOIS.localizationVars.contextEntries.menu_add_improvement_text   = FCOIS.localizationVars.fcois_loc["rightclick_menu_mark_improvement"]
    FCOIS.localizationVars.contextEntries.menu_add_sell_to_guild_text = FCOIS.localizationVars.fcois_loc["rightclick_menu_mark_sell_in_guild_store"]
    FCOIS.localizationVars.contextEntries.menu_add_intricate_text     = FCOIS.localizationVars.fcois_loc["rightclick_menu_mark_intricate"]

	FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[1]  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_all_gear1"]
	FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[2]  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_all_gear2"]
	FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[3]  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_all_gear3"]
	FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[4]  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_all_gear4"]
	FCOIS.localizationVars.contextEntries.menu_add_all_gear_text[5]  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_add_all_gear5"]

    FCOIS.localizationVars.contextEntries.menu_remove_lock_text  	  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_unlock"]
	FCOIS.localizationVars.contextEntries.menu_remove_gear_text[1]	= FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_gear1"]
	FCOIS.localizationVars.contextEntries.menu_remove_research_text  	= FCOIS.localizationVars.fcois_loc["rightclick_menu_demark_analysis"]
	FCOIS.localizationVars.contextEntries.menu_remove_gear_text[2]	= FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_gear2"]
	FCOIS.localizationVars.contextEntries.menu_remove_sell_text 	 	= FCOIS.localizationVars.fcois_loc["rightclick_menu_demark_sell"]
	FCOIS.localizationVars.contextEntries.menu_remove_gear_text[3]   	= FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_gear3"]
	FCOIS.localizationVars.contextEntries.menu_remove_gear_text[4]   	= FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_gear4"]
	FCOIS.localizationVars.contextEntries.menu_remove_gear_text[5]   	= FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_gear5"]
    FCOIS.localizationVars.contextEntries.menu_remove_deconstruction_text = FCOIS.localizationVars.fcois_loc["rightclick_menu_demark_deconstruction"]
    FCOIS.localizationVars.contextEntries.menu_remove_improvement_text = FCOIS.localizationVars.fcois_loc["rightclick_menu_demark_improvement"]
    FCOIS.localizationVars.contextEntries.menu_remove_sell_to_guild_text = FCOIS.localizationVars.fcois_loc["rightclick_menu_demark_sell_in_guild_store"]
    FCOIS.localizationVars.contextEntries.menu_remove_intricate_text  = FCOIS.localizationVars.fcois_loc["rightclick_menu_demark_intricate"]

	FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[1] = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_all_gear1"]
	FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[2] = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_all_gear2"]
	FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[3] = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_all_gear3"]
	FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[4] = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_all_gear4"]
	FCOIS.localizationVars.contextEntries.menu_remove_all_gear_text[5] = FCOIS.localizationVars.fcois_loc["rightclick_menu_remove_all_gear5"]

	FCOIS.localizationVars.contextEntries.menu_add_all_text[1] 		= FCOIS.localizationVars.fcois_loc["button_context_menu_lock_all"]
	FCOIS.localizationVars.contextEntries.menu_remove_all_text[1] 	= FCOIS.localizationVars.fcois_loc["button_context_menu_unlock_all"]
	FCOIS.localizationVars.contextEntries.menu_add_all_text[3] 		= FCOIS.localizationVars.fcois_loc["button_context_menu_research_all"]
	FCOIS.localizationVars.contextEntries.menu_remove_all_text[3] 	= FCOIS.localizationVars.fcois_loc["button_context_menu_dont_research_all"]
	FCOIS.localizationVars.contextEntries.menu_add_all_text[5] 		= FCOIS.localizationVars.fcois_loc["button_context_menu_sell_all"]
	FCOIS.localizationVars.contextEntries.menu_remove_all_text[5] 	= FCOIS.localizationVars.fcois_loc["button_context_menu_dont_sell_all"]
	FCOIS.localizationVars.contextEntries.menu_add_all_text[9] 		= FCOIS.localizationVars.fcois_loc["button_context_menu_deconstruct_all"]
	FCOIS.localizationVars.contextEntries.menu_remove_all_text[9] 	= FCOIS.localizationVars.fcois_loc["button_context_menu_dont_deconstruct_all"]
	FCOIS.localizationVars.contextEntries.menu_add_all_text[10] 		= FCOIS.localizationVars.fcois_loc["button_context_menu_improve_all"]
	FCOIS.localizationVars.contextEntries.menu_remove_all_text[10] 	= FCOIS.localizationVars.fcois_loc["button_context_menu_dont_improve_all"]
	FCOIS.localizationVars.contextEntries.menu_add_all_text[11] 		= FCOIS.localizationVars.fcois_loc["button_context_menu_sell_all_in_guild_store"]
	FCOIS.localizationVars.contextEntries.menu_remove_all_text[11] 	= FCOIS.localizationVars.fcois_loc["button_context_menu_dont_sell_all_in_guild_store"]
	FCOIS.localizationVars.contextEntries.menu_add_all_text[12] 		= FCOIS.localizationVars.fcois_loc["button_context_menu_mark_all_as_intricate"]
	FCOIS.localizationVars.contextEntries.menu_remove_all_text[12] 	= FCOIS.localizationVars.fcois_loc["button_context_menu_unmark_all_as_intricate"]
end

--Register the slash commands
local function RegisterSlashCommands()
    -- Register slash commands
	SLASH_COMMANDS["/fcoitemsaver"] = command_handler
	SLASH_COMMANDS["/fcois"] 		= command_handler
end

--=============================================================================================================
-- CURRENT NEEDED WORKAROUNDS & FIXES
--=============================================================================================================
--Apply current needed fixes and workarounds because of other addons/libraries etc.
local function FCOItemSaver_LoadWorkarounds()
    --FCOIS v0.7.8
	--The array to map the marker icon offsets for each filter panel ID
	FCOIS.mappingVars.filterPanelIdToIconOffset = {
		[LF_INVENTORY] 					= FCOIS.settingsVars.settings.iconPosition,
		[LF_SMITHING_DECONSTRUCT]		= FCOIS.settingsVars.settings.iconPositionCrafting,
		[LF_SMITHING_IMPROVEMENT] 			= FCOIS.settingsVars.settings.iconPositionCrafting,
		[LF_VENDOR_SELL] 				= FCOIS.settingsVars.settings.iconPosition,
		[LF_GUILDBANK_WITHDRAW] 			= FCOIS.settingsVars.settings.iconPosition,
		[LF_GUILDBANK_DEPOSIT]		= FCOIS.settingsVars.settings.iconPosition,
		[LF_GUILDSTORE_SELL] 			= FCOIS.settingsVars.settings.iconPosition,
		[LF_BANK_WITHDRAW] 					= FCOIS.settingsVars.settings.iconPosition,
		[LF_BANK_DEPOSIT] 			= FCOIS.settingsVars.settings.iconPosition,
		[LF_ENCHANTING_EXTRACTION] = FCOIS.settingsVars.settings.iconPositionCrafting,
		[LF_ENCHANTING_CREATION] 	= FCOIS.settingsVars.settings.iconPositionCrafting,
		[LF_MAIL_SEND] 					= FCOIS.settingsVars.settings.iconPosition,
		[LF_TRADE] 				= FCOIS.settingsVars.settings.iconPosition,
		[LF_FENCE_SELL] 				= FCOIS.settingsVars.settings.iconPosition,
		[LF_FENCE_LAUNDER] 				= FCOIS.settingsVars.settings.iconPosition,
		[LF_ALCHEMY_CREATION] 				= FCOIS.settingsVars.settings.iconPositionCrafting,
		[LF_CRAFTBAG] 				= FCOIS.settingsVars.settings.iconPosition, -- Workaround: Craftbag, added with API 100015
	}

    --FCOIS v0.7.8b
	--Workaround to update the icon enabled array for the gear sets, by help of the old gearEnabled array
    --so the users keep their settings
    if FCOIS.settingsVars.settings.isGearEnabled ~= nil then
	    for gearId = 1, FCOIS.numVars.gFCONumGearSets, 1 do
			if FCOIS.settingsVars.settings.isGearEnabled[gearId] ~= nil then
				--Overwrite the current setting of isIconEnabled with the old isGearEnabled settings
				FCOIS.settingsVars.settings.isIconEnabled[FCOIS.mappingVars.gearToIcon[gearId]] = FCOIS.settingsVars.settings.isGearEnabled[gearId]
				--Remove the old used variable now
				FCOIS.settingsVars.settings.isGearEnabled[gearId] = nil
			end
		end
	    --Remove the old array
	    FCOIS.settingsVars.settings.isGearEnabled = nil
	end
end
--=============================================================================================================
--	LOAD USER SETTINGS
--=============================================================================================================
--Load the SavedVariables now
local function LoadUserSettings()
	if not FCOIS.addonVars.gSettingsLoaded then

	    --The default values for the language and save mode
	    FCOIS.settingsVars.firstRunSettings = {
	        language 	 		    = 1, --Standard: English
	        saveMode     		    = 2, --Standard: Account wide FCOIS.settingsVars.settings
	    }

	    --Pre-set the deafult values
	    FCOIS.settingsVars.defaults = {
	        --language 	 		    	= 1, --Standard: English
	        --saveMode     		    	= 2, --Standard: Account wide FCOIS.settingsVars.settings
	        languageChoosen				= false,
	    	markedItems	 		    	= {},
	        icon		 		    	= {},
	        iconPosition				= {},
			iconPositionCrafting		= {},
	        iconPositionCharacter		= {},
            iconSortOrder				= {},
	        filterButtonLeft			= {},
	        filterButtonTop				= {},
	        splitFilters				= true,
	        isFilterOn   		   	 	= {false, false, false, false},
	        isFilterPanelOn				= {},
			atPanelEnabled				= {},
            isIconEnabled				= {},
            lastGearFilterIconId		 = {},
            lastResDecImpFilterIconId	 = {},
            lastSellGuildIntFilterIconId = {},
	        allowResearch 		    	= false,
            allowDeconstructIntricate	= true,
            allowDeconstructDeconstruction = true,
	        allowInventoryFilter    	= true,
			allowCraftBagFilter			= true,
	        allowVendorFilter	    	= true,
            allowFenceFilter			= true,
            allowLaunderFilter			= true,
	        allowGuildBankFilter    	= true,
	        allowBankFilter 	    	= true,
			allowTradinghouseFilter		= true,
			allowTradeFilter	   	 	= true,
	        allowMailFilter		    	= true,
	        allowEnchantingFilter		= true,
	        allowDeconstructionFilter	= true,
	        allowImprovementFilter  	= true,
            allowAlchemyFilter			= true,
            blockMarkedRepairKits		= false,
	        blockDestroying				= true,
	        blockDeconstruction			= true,
	        blockImprovement			= true,
	        blockSendingByMail			= true,
			blockTrading				= true,
	        blockSelling				= true,
            blockSellingGuildStore      = true,
            blockFence					= true,
            blockLaunder				= true,
			allowSellingForBlocked		= false,
	        allowSellingForBlockedOrnate = true,
	        allowSellingForBlockedIntricate = false,
            allowImproveImprovement		= false,
			blockEnchantingCreation		= true,
			blockEnchantingExtraction	= true,
            blockAlchemyDestroy			= true,
            blockAutoLootContainer		= false,
            blockMarkedRecipes			= true,
            blockMarkedMotifs			= true,
            blockMarkedPotions			= false,
            blockMarkedFood			    = false,
            blockGuildBankWithoutWithdraw = true,
            checkDeactivatedIcons		= false,
	        autoReenable_blockDestroying			= true,
	        autoReenable_blockDeconstruction		= true,
	        autoReenable_blockImprovement			= true,
	        autoReenable_blockSendingByMail			= true,
			autoReenable_blockTrading				= true,
	        autoReenable_blockSelling				= true,
            autoReenable_blockSellingGuildStore		= true,
	        autoReenable_blockFenceSelling			= true,
	        autoReenable_blockLaunderSelling		= true,
			autoReenable_blockEnchantingCreation	= true,
			autoReenable_blockEnchantingExtraction	= true,
            autoReenable_blockAlchemyDestroy		= true,
			autoMarkOrnate 		    	= false,
            autoMarkIntricate           = false,
	        autoMarkResearch			= false,
			autoMarkQuality				= 1,
			autoMarkHigherQuality		= false,
	        autoMarkAllEquipment		= false,
	        autoMarkAllWeapon			= false,
	        autoMarkAllJewelry			= false,
            autoMarkRecipes 			= true,
            autoMarkRecipesOnlyThisChar = false,
            autoMarkCraftedItems		= false,
            autoMarkCraftedItemsSets	= false,
			autoMarkSets				= false,
            autoMarkSetsGearIconNr		= 1,
			autoMarkSetsCheckAllGearIcons = true,
			autoDeMarkSell				= false,
            autoDeMarkSellInGuildStore	= false,
			showFilterStatusInChat  	= true,
	        showOrnateItemsInChat   	= false,
            showIntricateItemInChat     = false,
	        showResearchItemsInChat 	= false,
	        showSetsInChat 				= false,
			showQualityItemsInChat		= false,
            showRecipesInChat			= false,
	        showFCOISMenuBarButton		= true,
			showFCOISAdditionalInventoriesButton = true,
            colorizeFCOISAdditionalInventoriesButton = true,
	        showFilterButtonTooltip		= true,
            showFilterButtonContextTooltip = true,
	        showAntiMessageInChat		= true,
	        showAntiMessageAsAlert		= true,
	        showMarkerTooltip			= {false, false, false, false, false, false, false, false},
	        askBeforeEquipBoundItems	= true,
	        splitGearSetsFilter			= false,
	        splitResearchDeconstructionImprovementFilter = false,
            splitSellGuildSellIntricateFilter = false,
	        showIconTooltipAtCharacter	= true,
            useSubContextMenu			= true,
            showContextMenuDivider		= false,
            contextMenuDividerShowsSettings = false,
			contextMenuDividerClearsMarkers = false,
            useContextMenuCustomMarkedNormalColor = true,
            contextMenuCustomMarkedNormalColor = {["r"] = 1,["g"] = 0,["b"] = 0,["a"] = 1},
            showArmorTypeIconAtCharacter = true,
            armorTypeIconAtCharacterX	 = 15,
            armorTypeIconAtCharacterY	 = 15,
            armorTypeIconAtCharacterLightColor  = {["r"] = 1,["g"] = 1,["b"] = 1,["a"] = 1},
            armorTypeIconAtCharacterMediumColor = {["r"] = 1,["g"] = 1,["b"] = 1,["a"] = 1},
            armorTypeIconAtCharacterHeavyColor  = {["r"] = 1,["g"] = 1,["b"] = 1,["a"] = 1},
            showArmorTypeHeaderTextAtCharacter = true,
            disableResearchCheck 		= {},
            cycleMarkerSymbolOnKeybind	= false,
            standardIconOnKeybind		= 1,
	        debug						= false,
	        deepDebug					= false,
	    }

	    for helper_inv = 1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
			if FCOIS.mappingVars.activeFilterPanelIds[helper_inv] == true then
				--Create 2-dimensional arrays for the filters
			    FCOIS.settingsVars.defaults.isFilterPanelOn[helper_inv] = {false, false, false, false}

			    --Create 2-dimensional array for the "enabled" setings (Filters, Anti-Destroy, Anti-Deconstruction, Anti-Sell, Anti-Trade, Anti-Mail, etc.)
			    FCOIS.settingsVars.defaults.atPanelEnabled[helper_inv]	= {
			        ["filters"] 		 = false,
	--			    ["anti-destroy"] 	 = false,
	--			    ["anti-deconstruct"] = false,
	--				["anti-improve"]	 = false,
	--				["anti-sell"]	 	 = false,
	--				["anti-sell-exception"] = false,
	--				["anti-trade"] 		 = false,
	--				["anti-mail"] 		 = false,
	--				["anti-fence"] 		 = false,
	--				["anti-launder"]	 = false,
		        }

				--Create the helper arays for the filter button context menus
	            FCOIS.settingsVars.defaults.lastGearFilterIconId[helper_inv] 		= -1
	            FCOIS.settingsVars.defaults.lastResDecImpFilterIconId[helper_inv] 	= -1
	            FCOIS.settingsVars.defaults.lastSellGuildIntFilterIconId[helper_inv]= -1
			end
	    end

		--Create 2-dimensional arrays for the icons
		for helper = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
            --Marker icons in inventories
        	FCOIS.settingsVars.defaults.markedItems[helper] 	= {}
            --General icon information
	        FCOIS.settingsVars.defaults.icon[helper] 		  	= {}

			--Defaults for filter button offsets
			FCOIS.settingsVars.defaults.filterButtonTop[helper]  = FCOIS.filterButtonVars.gFilterButtonTop
			FCOIS.settingsVars.defaults.filterButtonLeft[helper] = FCOIS.filterButtonVars.gFilterButtonLeft[helper]

            --Defaults for research check
            FCOIS.settingsVars.defaults.disableResearchCheck[helper] = false

            --Defaults for the enabling/disabling of the icons
            FCOIS.settingsVars.defaults.isIconEnabled[helper] = true
	    end

	--[[ DEFAULT CONTEXT MENU SORT ORDER
		The default sort order is:
		Icon 1: Lock
		Icon 3: Research
		Icon 5: Sell
		Icon 9: Deconstruction
		Icon 10: Improvement
		Icon 11: Sell at guild store
		Icon 12: Intricate
		Icon 2: Gear 1
		Icon 4: Gear 2
		Icon 6: Gear 3
		Icon 7: Gear 4
		Icon 8: Gear 5
    ]]

		--Preset the default icon colors, textures and sort orders
	    FCOIS.settingsVars.defaults.icon[1].color   = {["r"] = 1,["g"] = 0,["b"] = 0,["a"] = 1}
		FCOIS.settingsVars.defaults.icon[1].texture = 1
		FCOIS.settingsVars.defaults.icon[1].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[1].sortOrder = 1
        FCOIS.settingsVars.defaults.iconSortOrder[1] = 1

		FCOIS.settingsVars.defaults.icon[2].color   = {["r"] = 0,["g"] = 1,["b"] = 0,["a"] = 1}
		FCOIS.settingsVars.defaults.icon[2].texture = 2
		FCOIS.settingsVars.defaults.icon[2].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[2].sortOrder = 8
        FCOIS.settingsVars.defaults.iconSortOrder[2] = 3

		FCOIS.settingsVars.defaults.icon[3].color   = {["r"] = 1,["g"] = 1,["b"] = 1,["a"] = 1}
		FCOIS.settingsVars.defaults.icon[3].texture = 3
		FCOIS.settingsVars.defaults.icon[3].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[3].sortOrder = 2
        FCOIS.settingsVars.defaults.iconSortOrder[3] = 5

		FCOIS.settingsVars.defaults.icon[4].color   = {["r"] = 1,["g"] = 0,["b"] = 1,["a"] = 1}
		FCOIS.settingsVars.defaults.icon[4].texture = 2
		FCOIS.settingsVars.defaults.icon[4].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[4].sortOrder = 9
        FCOIS.settingsVars.defaults.iconSortOrder[4] = 9

		FCOIS.settingsVars.defaults.icon[5].color   = {["r"] = 1,["g"] = 1,["b"] = 0,["a"] = 1}
		FCOIS.settingsVars.defaults.icon[5].texture = 4
		FCOIS.settingsVars.defaults.icon[5].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[5].sortOrder = 3
        FCOIS.settingsVars.defaults.iconSortOrder[5] = 10

		FCOIS.settingsVars.defaults.icon[6].color   = {["r"] = 1,["g"] = 1,["b"] = 0,["a"] = 1}
		FCOIS.settingsVars.defaults.icon[6].texture = 2
		FCOIS.settingsVars.defaults.icon[6].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[6].sortOrder = 10
        FCOIS.settingsVars.defaults.iconSortOrder[6] = 11

		FCOIS.settingsVars.defaults.icon[7].color   = {["r"] = 1,["g"] = 1,["b"] = 0,["a"] = 1}
		FCOIS.settingsVars.defaults.icon[7].texture = 2
		FCOIS.settingsVars.defaults.icon[7].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[7].sortOrder = 11
        FCOIS.settingsVars.defaults.iconSortOrder[7] = 12

		FCOIS.settingsVars.defaults.icon[8].color   = {["r"] = 1,["g"] = 1,["b"] = 0,["a"] = 1}
		FCOIS.settingsVars.defaults.icon[8].texture = 2
		FCOIS.settingsVars.defaults.icon[8].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[8].sortOrder = 12
        FCOIS.settingsVars.defaults.iconSortOrder[8] = 2

		FCOIS.settingsVars.defaults.icon[9].color   = {["r"] = 1,["g"] = 1,["b"] = 1,["a"] = 1}
		FCOIS.settingsVars.defaults.icon[9].texture = 55
		FCOIS.settingsVars.defaults.icon[9].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[9].sortOrder = 4
        FCOIS.settingsVars.defaults.iconSortOrder[9] = 4

		FCOIS.settingsVars.defaults.icon[10].color   = {["r"] = 1,["g"] = 1,["b"] = 1,["a"] = 1}
		FCOIS.settingsVars.defaults.icon[10].texture = 56
		FCOIS.settingsVars.defaults.icon[10].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[10].sortOrder = 5
        FCOIS.settingsVars.defaults.iconSortOrder[10] = 6

        FCOIS.settingsVars.defaults.icon[11].color   = {["r"] = 1,["g"] = 1,["b"] = 0,["a"] = 1}
        FCOIS.settingsVars.defaults.icon[11].texture = 58
        FCOIS.settingsVars.defaults.icon[11].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[11].sortOrder = 6
        FCOIS.settingsVars.defaults.iconSortOrder[11] = 7

        FCOIS.settingsVars.defaults.icon[12].color   = {["r"] = 1,["g"] = 1,["b"] = 1,["a"] = 1}
        FCOIS.settingsVars.defaults.icon[12].texture = 60
        FCOIS.settingsVars.defaults.icon[12].size    = FCOIS.iconVars.gIconWidth
        FCOIS.settingsVars.defaults.icon[12].sortOrder = 7
        FCOIS.settingsVars.defaults.iconSortOrder[12] = 8

		--Preset the default icon positions (inventories and character)
	    FCOIS.settingsVars.defaults.iconPosition.x		 	=	0
	    FCOIS.settingsVars.defaults.iconPosition.y		 	=	0
	    FCOIS.settingsVars.defaults.iconPositionCrafting.x 	=	-5
	    FCOIS.settingsVars.defaults.iconPositionCrafting.y 	=	0
	    FCOIS.settingsVars.defaults.iconPositionCharacter.x	=   0
	    FCOIS.settingsVars.defaults.iconPositionCharacter.y	=	0

		--Update the "Gear sets" texts depending on locaization
		FCOIS.settingsVars.defaults.icon[2].name    = "Gear 1"
		FCOIS.settingsVars.defaults.icon[4].name    = "Gear 2"
		FCOIS.settingsVars.defaults.icon[6].name    = "Gear 3"
		FCOIS.settingsVars.defaults.icon[7].name    = "Gear 4"
		FCOIS.settingsVars.defaults.icon[8].name    = "Gear 5"

--=========== BEGIN - SAVED VARIABLES ==========================================
	    --Load the user's settings from SavedVariables file -> Account wide of basic version 999 at first
		FCOIS.settingsVars.defaultSettings = ZO_SavedVars:NewAccountWide("FCOItemSaver_Settings", 999, "SettingsForAll", FCOIS.settingsVars.firstRunSettings)

		--Check, by help of basic version 999 FCOIS.settingsVars.settings, if the settings should be loaded for each character or account wide
	    --Use the current addon version to read the FCOIS.settingsVars.settings now
		if (FCOIS.settingsVars.defaultSettings.saveMode == 1) then
	    	FCOIS.settingsVars.settings = ZO_SavedVars:New("FCOItemSaver_Settings", FCOIS.addonVars.addonVersion , "Settings", FCOIS.settingsVars.defaults)
		elseif (FCOIS.settingsVars.defaultSettings.saveMode == 2) then
			FCOIS.settingsVars.settings = ZO_SavedVars:NewAccountWide("FCOItemSaver_Settings", FCOIS.addonVars.addonVersion, "Settings", FCOIS.settingsVars.defaults)
		else
			FCOIS.settingsVars.settings = ZO_SavedVars:NewAccountWide("FCOItemSaver_Settings", FCOIS.addonVars.addonVersion, "Settings", FCOIS.settingsVars.defaults)
		end
--=========== END - SAVED VARIABLES ============================================

		--Load the current needed workarounds/fixes
        FCOItemSaver_LoadWorkarounds()

        --Set settings = loaded
        FCOIS.addonVars.gSettingsLoaded = true
    end
--=============================================================================================================
end

--Addon is now loading and building up
local function FCOItemSaver_Loaded(eventCode, addOnName)
	--Check if gamepad mode is deactivated?
	if not FCOItemSaver_CheckGamePadMode() then

		--Check if addon "Research Assistant" is active
		if(addOnName == "ResearchAssistant") then
	        FCOIS.otherAddons.researchAssistantActive = true
	        debugMessage("Addon Research Assistant is active", false)
	    end
		--Check if addon "InventoryGridView" is active
	    if(addOnName == "InventoryGridView") then
	    	FCOIS.otherAddons.inventoryGridViewActive = true
	    end
		--Check if addon "ChatMerchant" is active
	    if(addOnName == "ChatMerchant") then
	    	FCOIS.otherAddons.chatMerchantActive = true
	    end
		--Check if addon "PotionMaker" is active
	    if(addOnName == "PotionMaker") then
	    	FCOIS.otherAddons.potionMakerActive = true
	    end
	    --Check if addon "Votans Settings Menu" is active
	    if(addOnName == "VotansSettingsMenu" or VOTANS_MENU_SETTINGS) then
	    	FCOIS.otherAddons.votansSettingsMenuActive = true
	    end
        --Check if addon "SousChef" is active
        if(addOnName == "SousChef" or SousChef ~= nil) then
        	FCOIS.otherAddons.sousChefActive = true
        end
		--Is this addon found?
		if(addOnName ~= FCOIS.addonVars.gAddonName) then
	        return
	    end
		--Unregister this event again so it isn't fired again after this addon has beend recognized
	    EVENT_MANAGER:UnregisterForEvent(FCOIS.addonVars.gAddonName, EVENT_ADD_ON_LOADED)

	    debugMessage("[Addon loading begins...]", true)
		FCOIS.addonVars.gAddonLoaded = false

	    -- Standard filtering: Inside player inventory
	    FCOIS.gFilterWhere = 1

	--=============================================================================================================
	--	LOAD USER SETTINGS
	--=============================================================================================================
		LoadUserSettings()

	    --Preset global variable for item destroying
	    FCOIS.preventerVars.gAllowDestroyItem = not FCOIS.settingsVars.settings.blockDestroying

	    -- Get the marked items for each filter from the settings (or defaults, if not set yet)
		for markedItemsId = 1, FCOIS.numVars.gFCONumFilterIcons, 1 do
		    FCOIS.markedItems[markedItemsId] = FCOIS.settingsVars.settings.markedItems[markedItemsId]
	    end

		-- Set Localization
		FCOIS.preventerVars.KeyBindingTexts = false
	    Localization()

		--Update the texts depending on localization
		FCOIS.settingsVars.defaults.icon[2].name    = FCOIS.localizationVars.fcois_loc["options_icon2_name"]
		FCOIS.settingsVars.defaults.icon[4].name    = FCOIS.localizationVars.fcois_loc["options_icon4_name"]
		FCOIS.settingsVars.defaults.icon[6].name    = FCOIS.localizationVars.fcois_loc["options_icon6_name"]
		FCOIS.settingsVars.defaults.icon[7].name    = FCOIS.localizationVars.fcois_loc["options_icon7_name"]
		FCOIS.settingsVars.defaults.icon[8].name    = FCOIS.localizationVars.fcois_loc["options_icon8_name"]
	    if (FCOIS.settingsVars.settings.icon[2].name == "") then
			FCOIS.settingsVars.settings.icon[2].name = FCOIS.settingsVars.defaults.icon[2].name
	    end
	    if (FCOIS.settingsVars.settings.icon[4].name == "") then
			FCOIS.settingsVars.settings.icon[4].name = FCOIS.settingsVars.defaults.icon[4].name
	    end
	    if (FCOIS.settingsVars.settings.icon[6].name == "") then
			FCOIS.settingsVars.settings.icon[6].name = FCOIS.settingsVars.defaults.icon[6].name
	    end
	    if (FCOIS.settingsVars.settings.icon[7].name == "") then
			FCOIS.settingsVars.settings.icon[7].name = FCOIS.settingsVars.defaults.icon[7].name
	    end
	    if (FCOIS.settingsVars.settings.icon[8].name == "") then
			FCOIS.settingsVars.settings.icon[8].name = FCOIS.settingsVars.defaults.icon[8].name
	    end
		--Was the standard icon name for the keybinding changed before, or not?
        if FCOIS.settingsVars.settings.standardIconNameOnKeybind == "" then
        	--It wasn't changed so update it according to the localized language
			local locIconNameStr = FCOIS.localizationVars.fcois_loc["options_icon" .. tostring(1) .. "_" .. FCOIS.localizationVars.iconEndStrArray[1]]
            FCOIS.settingsVars.settings.standardIconNameOnKeybind = locIconNameStr
        end

		--Show the menu
		BuildAddonMenu()

	    --Create the icon textures
	    CreateTextures(-1)

		--Create the hooks
	    FCOItemSaver_CreateHooks()

	    --Check inventory for ornate and/or researchable items and mark them to be sold, if activated in the FCOIS.settingsVars.settings
		scanInventory()

		--Build the inventory filter buttons and add them to the panels
		updateFilterButtonsInInv(-1)

		--Initialize the filters
	    EnableFilters(-100)

	    -- Register slash commands
	    RegisterSlashCommands()

		--Add the additional buttons, controlled by the FCOIS settings
        --e.g. quick access settings button to the main menu, or the context menu invoker buttons at the inventories
		AddAdditionalButtons(-1)

		--Initialize the custom dialogs
		FCOAskBeforeBindDialogInitialize(FCOISAskBeforeBindDialog)

		--Check the auto reenable Anti-* settings and react on them
		autoReenableAntiSettingsCheck("-ALL-")

	    debugMessage("[Addon loading finished. Have fun!]", true)
	    FCOIS.addonVars.gAddonLoaded = true

		-- Registers addon to loadedAddon library
		LIBLA:RegisterAddon(FCOIS.addonVars.gAddonName, FCOIS.addonVars.addonVersionOptionsNumber)
	end --gamepad active check
end

--Check the current API and if patches need to be applied, do so here
local function CheckNeededAPIChanges()
	--Switch variable name of ZOs inventory for API version 100015
	if(FCOIS.APIversion == 100015) then
		--New since 1000015
		FCOIS.ZOControlVars.BACKPACK_BAG 				= ZO_PlayerInventoryListContents
	    --Disable the ZOs "player item lock" possibilities
        --by hiding the "lock" context menu entry
        FCOIS.preventerVars.ZOsPlayerItemLockEnabled = true
	end

end

-- Register the event "addon loaded" for this addon
local function FCOItemSaver_Initialized()
	--Disable this addon if we are in GamePad mode
    if not FCOItemSaver_CheckGamePadMode() then
		--Check and apply needed API changes for cross API compatibility of this addon
		CheckNeededAPIChanges()

        --Register the addon's loaded callback function
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_ADD_ON_LOADED, FCOItemSaver_Loaded)
		--Register for the zone change/player ready event
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_PLAYER_ACTIVATED, FCOItemSaver_Player_Activated)
		--Register for Crafting stations opened & closed
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_CRAFTING_STATION_INTERACT, function(eventCode, craftSkill, sameStation) FCOItemSaver_Crafting_Interact(craftSkill, sameStation) end)
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_END_CRAFTING_STATION_INTERACT, FCOItemSaver_End_Crafting_Interact)
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_CRAFT_STARTED, FCOItemSaver_Craft_Started)
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_CRAFT_COMPLETED, FCOItemSaver_Craft_Completed)
	    --Register for Store opened & closed
	    EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_OPEN_STORE, function() FCOItemSaver_Open_Store("vendor") end)
	    EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_CLOSE_STORE, FCOItemSaver_Close_Store)
		--Register for Trading house (guild store) opened & closed
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_OPEN_TRADING_HOUSE, FCOItemSaver_Open_Trading_House)
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_CLOSE_TRADING_HOUSE, FCOItemSaver_Close_Trading_House)
	    --Register for Guild Bank opened & closed
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_OPEN_GUILD_BANK, FCOItemSaver_Open_Guild_Bank)
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_CLOSE_GUILD_BANK, FCOItemSaver_Close_Guild_Bank)
	    --Register for Player's Bank opened & closed
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_OPEN_BANK, FCOItemSaver_Open_Player_Bank)
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_CLOSE_BANK, FCOItemSaver_Close_Player_Bank)
	    --Register for Trade panel opened & closed
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_TRADE_INVITE_ACCEPTED, FCOItemSaver_Open_Trade_Panel)
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_TRADE_CANCELED, FCOItemSaver_Close_Trade_Panel)
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_TRADE_SUCCEEDED, FCOItemSaver_Close_Trade_Panel)
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_TRADE_FAILED, FCOItemSaver_Close_Trade_Panel)
	    --Register for player inventory slot update
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, FCOItemSaver_Inv_Update)
        --Register the callback function for an update of the inventory slots
		--SHARED_INVENTORY:RegisterCallback("SingleSlotInventoryUpdate", FCOItemSaver_OnSharedSingleSlotUpdate)
		--Events for destruction & destroy prevention
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_INVENTORY_SLOT_LOCKED, FCOItemSaver_OnInventorySlotLocked)
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_INVENTORY_SLOT_UNLOCKED, FCOItemSaver_OnInventorySlotUnLocked)
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_MOUSE_REQUEST_DESTROY_ITEM, FCOItemSaver_OnMouseRequestDestroyItem)
		--Event if an action layer changes
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_ACTION_LAYER_POPPED, FCOItemsaver_OnActionLayerPopped)
		--EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_ACTION_LAYER_PUSHED, FCOItemsaver_OnActionLayerPushed)
	    EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_GAME_CAMERA_UI_MODE_CHANGED, FCOItemsaver_OnGameCameraUIModeChanged)
		--Guild bank is selected
		EVENT_MANAGER:RegisterForEvent(FCOIS.addonVars.gAddonName, EVENT_GUILD_BANK_SELECTED, FCOItemsaver_SelectGuildBank)
    end
end

--==========================================================================================================================================
--==========================================================================================================================================
--==========================================================================================================================================
-- 															GLOBAL FUNCTIONS
--==========================================================================================================================================
--==========================================================================================================================================
--==========================================================================================================================================

--Global function to return boolean value, if an item is filtered
--The item's instance ID and the filter number are needed to check, if the item is marked. Filter can be 1, 2, 3, 4 or -1 for all
--Instance: gives the current item's instance ID, determined by function GetItemInstanceId(bagId, slotIndex)
--filterId: Specifies the filter for which the item is marked. The variable FCOIS.numVars.gFCONumFilters gives the maximum filter number.
--			Special: Filter 2 controls icons 2, 4, 6, 7 and 8 (Gear sets)
--					 Filter 3 controls icons 3, 9 and 10 (Research, deconstruction & improvement)
--					 Filter 4 controls icon 5, 11 and 12 (Sell, sell in guild store & intricate)
--filterPanelId: The panel where the filter is activated. Possible values are:
function FCOIsFiltered(instance, filterId, filterPanelId)
	if (instance ~= nil and filterId ~= nil) then
        local offs  = 0
        --Workaround for empty filterPanelId
    	if (filterPanelId == nil or filterPanelId == '') then
            --Fallback solution: FilterPanelId will be deconstruction panel
        	filterPanelId = LF_SMITHING_DECONSTRUCT
        end

	    local filterStatusVar = {}
		--Create 2-dimensional arrays for the filters
	    for h_inv = 1, FCOIS.numVars.gFCONumFilterInventoryTypes, 1 do
			if FCOIS.mappingVars.activeFilterPanelIds[h_inv] == true then
			    filterStatusVar[h_inv] = {false, false, false, false}
			end
	    end

	    --Check all filters, silently (chat output: disabled)
	    filterStatusVar = filterStatus(-1, true, true)

		if (filterId ~= -1) then
			if (filterId == 2) then
				if (checkIfItemIsProtected(2, instance) == true or
					checkIfItemIsProtected(4, instance) == true or
					checkIfItemIsProtected(6, instance) == true or
					checkIfItemIsProtected(7, instance) == true or
					checkIfItemIsProtected(8, instance) == true    ) then
					--Split filtes, or old behaviour?
                    if (FCOIS.settingsVars.settings.splitFilters == true) then
						--Is the deconstruction filter activated?
	                    if (filterStatusVar[filterPanelId][2] == true) then
		                    return true
	                    end
                    else
	                    if (filterStatusVar[1][2] == true) then
		                    return true
	                    end
                   	end
				end
			elseif (filterId == 3) then
				if (checkIfItemIsProtected(3, instance) == true or
					checkIfItemIsProtected(9, instance) == true or
					checkIfItemIsProtected(10, instance) == true    ) then
					--Split filtes, or old behaviour?
                    if (FCOIS.settingsVars.settings.splitFilters == true) then
						--Is the research filter activated?
	                    if (filterStatusVar[filterPanelId][3] == true) then
		                    return true
	                    end
                    else
	                    if (filterStatusVar[1][3] == true) then
		                    return true
	                    end
                   	end
				end
            elseif (filterId == 4) then
                if (checkIfItemIsProtected(5, instance) == true or
					checkIfItemIsProtected(11, instance) == true or
					checkIfItemIsProtected(12, instance) == true    ) then
                    --Split filtes, or old behaviour?
                    if (FCOIS.settingsVars.settings.splitFilters == true) then
                        --Is the sell filter activated?
                        if (filterStatusVar[filterPanelId][5] == true) then
                            return true
                        end
                    else
                        if (filterStatusVar[1][5] == true) then
                            return true
                        end
                    end
                end
			else
				if (FCOIS.markedItems[filterId] ~= nil and checkIfItemIsProtected(filterId, instance)) then
					--Split filtes, or old behaviour?
                    if (FCOIS.settingsVars.settings.splitFilters == true) then
						--Is the deconstruction filter activated?
	                    if (filterStatusVar[filterPanelId][filterId] == true) then
		                    return true
	                    end
                    else
	                    if (filterStatusVar[1][filterId] == true) then
		                    return true
	                    end
                   	end
                end
			end
        else
			--Check for all filters if the item is marked. return true, if any filter applies
			for filtId = 1, FCOIS.numVars.gFCONumFilters, 1 do
            	if (filtId == 2) then
					if (checkIfItemIsProtected(2, instance) == true or
                    	checkIfItemIsProtected(4, instance) == true or
                    	checkIfItemIsProtected(6, instance) == true or
                    	checkIfItemIsProtected(7, instance) == true or
                    	checkIfItemIsProtected(8, instance) == true    ) then
						--Split filtes, or old behaviour?
	                    if (FCOIS.settingsVars.settings.splitFilters == true) then
							--Is the deconstruction filter activated?
		                    if (filterStatusVar[filterPanelId][2] == true) then
			                    return true
		                    end
	                    else
		                    if (filterStatusVar[1][2] == true) then
			                    return true
		                    end
	                   	end
	                end
            	elseif (filtId == 3) then
					if (checkIfItemIsProtected(3, instance) == true or
                    	checkIfItemIsProtected(9, instance) == true or
                    	checkIfItemIsProtected(10, instance) == true    ) then
						--Split filtes, or old behaviour?
	                    if (FCOIS.settingsVars.settings.splitFilters == true) then
							--Is the research filter activated?
		                    if (filterStatusVar[filterPanelId][3] == true) then
			                    return true
		                    end
	                    else
		                    if (filterStatusVar[1][3] == true) then
			                    return true
		                    end
	                   	end
	                end
                elseif (filtId == 4) then
                    if (checkIfItemIsProtected(5, instance) == true or
                        checkIfItemIsProtected(11, instance) == true or
                        checkIfItemIsProtected(12, instance) == true    ) then
                        --Split filtes, or old behaviour?
                        if (FCOIS.settingsVars.settings.splitFilters == true) then
                            --Is the sell filter activated?
                            if (filterStatusVar[filterPanelId][5] == true) then
                                return true
                            end
                        else
                            if (filterStatusVar[1][5] == true) then
                                return true
                            end
                        end
                    end
                else
                	if (checkIfItemIsProtected(filtId, instance) == true) then
						--Split filtes, or old behaviour?
	                    if (FCOIS.settingsVars.settings.splitFilters == true) then
							--Is the deconstruction filter activated?
		                    if (filterStatusVar[filterPanelId][filtId] == true) then
			                    return true
		                    end
	                    else
		                    if (filterStatusVar[1][filtId] == true) then
			                    return true
		                    end
	                   	end
	                end
                end
			end
            return false
        end
    else
        --Current function parameters are missing. Return nil
    	return nil
    end
end

--Global function to mark an item with an icon
function FCOMarkItem(bag, slot, iconId, showIcon, updateInventories, itemInstanceId)
--d("[FCOMarkItem] bag " .. bag .. ", slot: " .. slot .. ", iconId: " .. iconId .. ", show: " .. tostring(showIcon) .. ", updateInv: " .. tostring(updateInventories))
	if ((bag == nil or slot == nil) and (itemInstanceId == nil)) then return false end
    if ((iconId > FCOIS.numVars.gFCONumFilterIcons) or (iconId < 1 and iconId ~= -1)) then return false end
    if showIcon == nil then showIcon = true end
	updateInventories = updateInventories or false

    local recurRetValTotal = true
    --Recursively set/remove markers for all icons?
    if iconId == -1 then
		local doUpdateInvNow = false
        local recurRetVal = false
    	for iconNr=1, FCOIS.numVars.gFCONumFilterIcons, 1 do
			if updateInventories and iconNr == FCOIS.numVars.gFCONumFilterIcons then
	            doUpdateInvNow = true
            end
            --Recursively call this function again to change each marker icon at the item
	    	recurRetVal = FCOMarkItem(bag, slot, iconNr, showIcon, doUpdateInvNow, itemInstanceId)
            if not recurRetVal then
            	recurRetValTotal = false
            end
        end
	end -- if iconId == -1 then

	--Only run this code if not all icons should be checked
	if iconId ~= -1 then
		--Allow the update of the marker here, and change it later if no update is needed
		local doUpdateMarkerNow = true
		--Get/use the (given) item instance id
	    local itemId = nil
		if itemInstanceId == nil then
			--Get the itemId of the bag, slot combination
			itemId = MyGetItemInstanceIdNoControl(bag, slot)
		else
			--Get the itemId of the bag, slot combination
			itemId = itemInstanceId
	    end
	    if itemId ~= nil then
	        local researchableItem = false
			--Set the marker here now
	        local itemIsMarked = showIcon
	        if itemIsMarked == nil then itemIsMarked = false end
	        --Item is already un/marked -> No need to change it
	        if checkIfItemIsProtected(iconId, itemId) == itemIsMarked then
	        	doUpdateMarkerNow = false
	        else
				--Check if the item is a researchable one, but only if icon should be shown and bag + slot are given
				-- Equipment gear 1, 2, 3, 4, 5, Research, Improve, Deconstruct or Intricate
				if (showIcon and FCOIS.mappingVars.iconIsResearchable[iconId] and (bag ~= nil and slot ~= nil)) then
	                researchableItem = true
	            	-- Check if item is researchable (as only researchable items can work as equipment too)
					if (not isItemResearchableNoControl(bag, slot, iconId)) then
		            	doUpdateMarkerNow = false
	                    recurRetValTotal = false
					end
				end
			end

	        --Change the marker now?
			if doUpdateMarkerNow then
				--Un/Mark the item now
				FCOIS.markedItems[iconId][SignItemId(itemId)] = itemIsMarked
	        end
	    end --if itemId ~= nil

		--Update inventories or character equipment, but only needed if marker was changed
		if ( (updateInventories and doUpdateMarkerNow) or (FCOIS.preventerVars.gOverrideInvUpdateAfterMarkItem) ) then
			FCOIS.preventerVars.gOverrideInvUpdateAfterMarkItem = false

			if bag == BAG_WORN and not FCOIS.ZOControlVars.CHARACTER:IsHidden() then
				RefreshEquipmentControl(nil, showIcon, iconId)
			elseif bag == BAG_BACKPACK or bag == BAG_VIRTUAL then
				FilterBasics(false)
			elseif bag == BAG_BANK then
				FilterBasics(false)
			elseif bag == BAG_GUILDBANK then
				FilterBasics(false)
			end
		end -- if updateInventories ...
	end -- if iconId ~= -1

	--Return value if icon was added/removed
	return recurRetValTotal
end

--Global function to change a filter at the given panel Id
function FCOChangeFilter(filterId)
	--Valid filterId?
	if filterId == nil or filterId <= 0 or filterId > FCOIS.numVars.gFCONumFilters then return end
    --Valid filterPanelId?
    if FCOIS.gFilterWhere == nil or FCOIS.gFilterWhere <= 0
     or FCOIS.gFilterWhere > FCOIS.numVars.gFCONumFilterInventoryTypes then return end
	--Is filtering at the current panel enabled?
	if not FCOIS.settingsVars.settings.atPanelEnabled[FCOIS.gFilterWhere]["filters"] then return end
    --is the filterPanelId visible?
    if FCOIS.mappingVars.gFilterPanelIdToInv[FCOIS.gFilterWhere]:IsHidden() then return end

	debugMessage("[FCOChangeFilter] FilterId: " .. tostring(filterId) .. ", FilterPanelId: " .. tostring(FCOIS.gFilterWhere) .. ", InventoryName: " .. FCOIS.mappingVars.gFilterPanelIdToInv[FCOIS.gFilterWhere]:GetName(), true)
    --Use the chat command handler now to emulate a filter change
    command_handler("filter" .. tostring(filterId) .. " " .. tostring(FCOIS.gFilterWhere))
end

--Global function to get text for the keybindings etc.
function FCOGetLocText(textName, isKeybindingText)
	isKeybindingText = isKeybindingText or false

    FCOIS.preventerVars.KeyBindingTexts = isKeybindingText

	--Do the localization now
   	Localization()

	if textName == nil or FCOIS.localizationVars.fcois_loc == nil or FCOIS.localizationVars.fcois_loc[textName] == nil then return "" end
   	return FCOIS.localizationVars.fcois_loc[textName]
end

--Global function to get the number of possible different gear sets and their icon IDs
--Returns the number of gear sets as 1st value, and an array for the mapping from gear to the icon ID as 2nd value
function FCOGetGearSetInfo()
	if #FCOIS.mappingVars.gearToIcon <= 0 then return nil end
    return #FCOIS.mappingVars.gearToIcon, FCOIS.mappingVars.gearToIcon
end

--Global function to get the gear set text for a given gear set's iconId (2, 4, 6, 7 or 8)
function FCOGetGearSetText(gearSetIconId)
	--Load the user settings, if not done already
	LoadUserSettings()

	if gearSetIconId ~= nil and FCOIS.settingsVars.settings.icon ~= nil and
       FCOIS.settingsVars.settings.icon[gearSetIconId] ~= nil and FCOIS.settingsVars.settings.icon[gearSetIconId].name ~= nil and FCOIS.settingsVars.settings.icon[gearSetIconId].name ~= "" then
	   	return FCOIS.settingsVars.settings.icon[gearSetIconId].name
    end
end

--Mark the icon from a keybind
function FCOMarkItemByKeybind(iconId)
	if iconId == nil then return false end
	local bagId
    local slotIndex
    bagId, slotIndex = FCOGetBagAndSlotFromControlUnderMouse()
	--bag and slot could be retrieved?
	if bagId ~= nil and slotIndex ~= nil then
		debugMessage("[FCOMarkItemByKeybind] Bag: " .. bagId .. ", slot: " .. slotIndex, true)

		--Check if the item is currently marked with this icon, or not
		--Get the itemId of the bag, slot combination
	    local itemId = MyGetItemInstanceIdNoControl(bagId, slotIndex)
	    if itemId ~= nil then
			--Set the marker here now
	        --Item is already un/marked?
	        local itemIsMarked = checkIfItemIsProtected(iconId, itemId)
            itemIsMarked = not itemIsMarked
			--Mark/UnMark the item now, and update inventory afterwards
	        FCOMarkItem(bagId, slotIndex, iconId, itemIsMarked, true, itemId)
        end
	else
    	return false
	end
end

--Cycle through the item marks by help of a keybind (cyclable items are set in the settings)
function FCOMarkItemCycle(direction)
	direction = direction or "next"
    if direction ~= "next" and direction ~= "prev" then return false end

	--Is the marker symbol cycling enabled in the settings
--===========================
--!!! DISABLED with 1 == 2!!!
--===========================
	if 1==2 and FCOIS.settingsVars.settings.cycleMarkerSymbolOnKeybind then
--d("Item cycling on keybind")
		local bagId
	    local slotIndex
	    bagId, slotIndex = FCOGetBagAndSlotFromControlUnderMouse()
        if bagId ~= nil and slotIndex ~= nil then
	        local _, markedIcons = FCOIsMarked(GetItemInstanceId(bagId, slotIndex), -1)
            if markedIcons ~= nil and #markedIcons > 0 then
--d("marked icons found")
				local lastMarkedIconId = -1
                local newMarkerIconId = -1
				for iconId = FCOIS.numVars.gFCONumFilterIcons, 1, -1 do
					if markedIcons[iconId] then
                        if FCOIS.settingsVars.settings.isIconEnabled[iconId] then
	                    	lastMarkedIconId = iconId
							break -- end the for ... loop
                        end
					end
				end
--d("last marked enabled icon: " .. lastMarkedIconId)

				--Mark with next icon
		    	if direction == "next" then
                    if lastMarkedIconId ~= -1 then
                    	if lastMarkedIconId == FCOIS.numVars.gFCONumFilterIcons then
                        	newMarkerIconId = 1
                        else
                        	newMarkerIconId = lastMarkedIconId + 1
                        end
                    else
                    	newMarkerIconId = 1
                    end

				--Mark with previous icon
		    	elseif direction == "prev" then
                    if lastMarkedIconId ~= -1 then
                    	if lastMarkedIconId == 1 then
                        	newMarkerIconId = FCOIS.numVars.gFCONumFilterIcons
                        else
                        	newMarkerIconId = lastMarkedIconId - 1
                        end
                    else
                    	newMarkerIconId = FCOIS.numVars.gFCONumFilterIcons
                    end
		        end

                if newMarkerIconId ~= -1 then
--d("Checking if " .. newMarkerIconId .. " is a valid iconId for this item...")
					--Repeat until the next iconId to set is valid for the current item (non-researchable items cannot get the research, gear sets, deconstruct, improve or intricate icons
                    local validNextIconId = -1
                    while validNextIconId == -1 do
	                    --Check if the item is a researchable one, but only if icon should be shown and bag + slot are given
						-- Equipment gear 1, 2, 3, 4, 5, Research, Improve, Deconstruct or Intricate
						if (FCOIS.mappingVars.iconIsResearchable[newMarkerIconId]) then
			            	-- Check if item is researchable (as only researchable items can work as equipment too)
							if (isItemResearchableNoControl(bag, slot, newMarkerIconId)) then
								--IconId = researchable and item is researchable -> Allowed!
                            	validNextIconId = newMarkerIconId
                                break
							else
                            	--iconId = researchable and item is not researchable -> Disallowed! Try next iconId
                                if direction == "next" then
									local incVal = 1
                                    local compVal = newMarkerIconId+1
                                    if compVal > FCOIS.numVars.gFCONumFilterIcons then compVal = 1 end
                                    if (compVal) == lastMarkedIconId then
                                    	incVal = 2
                                    end
                                	newMarkerIconId = newMarkerIconId + incVal
                                    if newMarkerIconId > FCOIS.numVars.gFCONumFilterIcons then newMarkerIconId = 1 end
                                elseif direction == "prev" then
	                                local decVal = 1
                                    local compVal = newMarkerIconId-1
                                    if compVal < 1 then compVal = FCOIS.numVars.gFCONumFilterIcons end
                                    if (compVal) == lastMarkedIconId then
                                    	decVal = 2
                                    end
									newMarkerIconId = newMarkerIconId - decVal
                                    if newMarkerIconId < 1 then newMarkerIconId = FCOIS.numVars.gFCONumFilterIcons end
                                end
							end
                        else
                            --Non researchable iconId -> Allowed!
                        	validNextIconId = newMarkerIconId
							break
                        end
                    end
					--Unmark all iconIds now and mark with the new one
                    if validNextIconId ~= -1 then
--f("Unmarking all icons now and remark it with iconId: " .. validNextIconId)
						--UnMark all marker icons first
				        --FCOMarkItem(bagId, slotIndex, -1, false, false, nil)
						--Mark with the new icon now
				        --FCOMarkItem(bagId, slotIndex, validNextIconId, true, true, nil)
					end
				end
			end
		end
	else -- if 1==2 and FCOIS.settingsVars.settings.cycleMarkerSymbolOnKeybind then

		if FCOIS.settingsVars.settings.standardIconOnKeybind ~= nil and FCOIS.settingsVars.settings.standardIconOnKeybind > 0 and FCOIS.settingsVars.settings.standardIconOnKeybind <= FCOIS.numVars.gFCONumFilterIcons then
--d("Mark item with standard markersymbol from settings: " .. FCOIS.settingsVars.settings.standardIconOnKeybind)
			local iconId = FCOIS.settingsVars.settings.standardIconOnKeybind
            if not FCOIS.settingsVars.settings.isIconEnabled[iconId] then
--d(">> IconId was not enabled in the settings! Taking iconId 1")
            	iconId = 1 --the lock symbol will be always enabled!
            end
			--Mark/Unmark the item now
			FCOMarkItemByKeybind(iconId)
		end

	end -- if 1==2 and FCOIS.settingsVars.settings.cycleMarkerSymbolOnKeybind then
end

--------------------------------------------------------------------------------
--- Call the start function for this addon to register events etc.
--------------------------------------------------------------------------------
FCOItemSaver_Initialized()
